@@ -37,6 +37,7 @@
 #include <assert.h> /* assert() */
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL wchar_t */
+#include <stdint.h> /* uintptr_t */
 #include <stdlib.h> /* EXIT_FAILURE _Exit() free() malloc() */
 #include <string.h> /* memcpy() */
 
@@ -113,15 +114,15 @@ static void job_check(bg_job_t *const job);
 static void job_free(bg_job_t *const job);
 #ifndef _WIN32
 static void * error_thread(void *p);
-static int update_job_list(bg_job_t **jobs, fd_set *set);
+static int update_error_jobs(bg_job_t **jobs, fd_set *set);
+static void free_drained_jobs(bg_job_t **jobs);
+static void import_error_jobs(bg_job_t **jobs);
+static int make_ready_list(bg_job_t **jobs, fd_set *set);
 static void report_error_msg(const char title[], const char text[]);
 static void append_error_msg(bg_job_t *job, const char err_msg[]);
-static bg_job_t * add_background_job(pid_t pid, const char cmd[], int fd,
-		BgJobType type);
-#else
-static bg_job_t * add_background_job(pid_t pid, const char cmd[],
-		HANDLE hprocess, BgJobType type);
 #endif
+static bg_job_t * add_background_job(pid_t pid, const char cmd[],
+		uintptr_t data, BgJobType type);
 static void * background_task_bootstrap(void *arg);
 static void set_current_job(bg_job_t *job);
 static void make_current_job_key(void);
@@ -468,7 +469,7 @@ error_thread(void *p)
 	while(1)
 	{
 		fd_set active, ready;
-		const int max_fd = update_job_list(&jobs, &active);
+		const int max_fd = update_error_jobs(&jobs, &active);
 		struct timeval timeout = { .tv_sec = 0, .tv_usec = 250*1000 }, ts = timeout;
 
 		while(select(max_fd + 1, memcpy(&ready, &active, sizeof(active)), NULL,
@@ -492,16 +493,18 @@ error_thread(void *p)
 				if(nread < 0)
 				{
 					need_update_list = 1;
+					j->drained = 1;
 					goto next_job;
 				}
 
 				if(nread == 0)
 				{
-					/* Reached EOF, cut this job out of our list and allow its
-					 * deletion. */
+					/* Reached EOF, exclude corresponding file descriptor from the set,
+					 * cut the job out of our list and allow its deletion. */
+					FD_CLR(j->fd, &active);
+					*job = j->err_next;
 					pthread_spin_lock(&j->status_lock);
 					j->in_use = 0;
-					*job = j->err_next;
 					pthread_spin_unlock(&j->status_lock);
 					continue;
 				}
@@ -531,13 +534,47 @@ error_thread(void *p)
 }
 
 /* Updates *jobs by removing finished tasks and adding new ones.  Initializes
- * *set in the process.  Returns max file descriptor number of the set. */
+ * *set set.  Returns max file descriptor number of the set. */
 static int
-update_job_list(bg_job_t **jobs, fd_set *set)
+update_error_jobs(bg_job_t **jobs, fd_set *set)
+{
+	free_drained_jobs(jobs);
+	import_error_jobs(jobs);
+	return make_ready_list(jobs, set);
+}
+
+/* Updates *jobs by removing finished tasks. */
+static void
+free_drained_jobs(bg_job_t **jobs)
+{
+	bg_job_t **job = jobs;
+	while(*job != NULL)
+	{
+		bg_job_t *const j = *job;
+
+		if(j->drained)
+		{
+			pthread_spin_lock(&j->status_lock);
+			/* If finished, reset in_use mark and drop it from the list. */
+			if(!j->running)
+			{
+				j->in_use = 0;
+				*job = j->err_next;
+				pthread_spin_unlock(&j->status_lock);
+				continue;
+			}
+			pthread_spin_unlock(&j->status_lock);
+		}
+
+		job = &j->err_next;
+	}
+}
+
+/* Updates *jobs by adding new tasks. */
+static void
+import_error_jobs(bg_job_t **jobs)
 {
-	int max_fd;
 	bg_job_t *new_jobs;
-	bg_job_t **job;
 
 	/* Add new tasks to internal list, wait if there are no jobs. */
 	pthread_mutex_lock(&new_err_jobs_lock);
@@ -558,29 +595,28 @@ update_job_list(bg_job_t **jobs, fd_set *set)
 		assert(new_job->type == BJT_COMMAND &&
 				"Only external commands should be here.");
 
+		/* Mark a this job as an interesting one to avoid it being killed until we
+		 * have a chance to read error stream. */
+		new_job->drained = 0;
+
 		new_job->err_next = *jobs;
 		*jobs = new_job;
 	}
+}
+
+/* Initializes *set set.  Returns max file descriptor number of the set. */
+static int
+make_ready_list(bg_job_t **jobs, fd_set *set)
+{
+	int max_fd = 0;
+	bg_job_t **job = jobs;
 
 	FD_ZERO(set);
 
-	max_fd = 0;
-	job = jobs;
 	while(*job != NULL)
 	{
 		bg_job_t *const j = *job;
 
-		pthread_spin_lock(&j->status_lock);
-		/* If finished, reset in_use mark and drop it from the list. */
-		if(!j->running)
-		{
-			j->in_use = 0;
-			*job = j->err_next;
-			pthread_spin_unlock(&j->status_lock);
-			continue;
-		}
-		pthread_spin_unlock(&j->status_lock);
-
 		FD_SET(j->fd, set);
 		if(j->fd > max_fd)
 		{
@@ -685,31 +721,31 @@ static pid_t
 background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 		int out_pipe[2], int err_pipe[2])
 {
-	wchar_t *args[4];
-	char cwd[PATH_MAX];
+	const wchar_t *args[4];
+	char *cwd;
 	int code;
 	wchar_t *final_wide_cmd;
 	wchar_t *wide_sh = NULL;
+	const int use_cmd = (!user_sh || curr_stats.shell_type == ST_CMD);
 
 	if(_dup2(out_pipe[1], _fileno(stdout)) != 0)
 		return (pid_t)-1;
 	if(_dup2(err_pipe[1], _fileno(stderr)) != 0)
 		return (pid_t)-1;
 
-	cwd[0] = '\0';
-	if(get_cwd(cwd, sizeof(cwd)) != NULL)
+	/* At least cmd.exe is incapable of handling UNC paths. */
+	cwd = save_cwd();
+	if(cwd != NULL && is_unc_path(cwd))
 	{
-		if(is_unc_path(cwd))
-		{
-			(void)chdir(get_tmpdir());
-		}
+		(void)chdir(get_tmpdir());
 	}
 
-	final_wide_cmd = to_wide(cmd);
+	wide_sh = to_wide(use_cmd ? "cmd" : cfg.shell);
 
-	wide_sh = to_wide(user_sh ? cfg.shell : "cmd");
-	if(!user_sh || curr_stats.shell_type == ST_CMD)
+	if(use_cmd)
 	{
+		final_wide_cmd = to_wide(cmd);
+
 		args[0] = wide_sh;
 		args[1] = L"/C";
 		args[2] = final_wide_cmd;
@@ -717,25 +753,23 @@ background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 	}
 	else
 	{
-		args[0] = wide_sh;
-		args[1] = L"-c";
-		args[2] = final_wide_cmd;
-		args[3] = NULL;
+		/* Nobody cares that there is an array of arguments, all arguments just get
+		 * concatenated anyway...  Therefore we need to take care of escaping stuff
+		 * ourselves. */
+		char *const modified_cmd = win_make_sh_cmd(cmd);
+		final_wide_cmd = to_wide(modified_cmd);
+		free(modified_cmd);
+
+		args[0] = final_wide_cmd;
+		args[1] = NULL;
 	}
 
-	/* XXX: using unicode version of spawn breaks commands like `:!echo hi%S` when
-	 *      'shell' is "bash".  Windows version of bash seems to process some
-	 *      strings in unicode as ASCII.  Using narrow version on the other hand
-	 *      will break use of non-ASCII paths on English version of Windows... */
-	code = _wspawnvp(P_NOWAIT, args[0], (const wchar_t **)args);
+	code = _wspawnvp(P_NOWAIT, wide_sh, args);
 
 	free(wide_sh);
 	free(final_wide_cmd);
 
-	if(is_unc_path(cwd))
-	{
-		(void)chdir(cwd);
-	}
+	restore_cwd(cwd);
 
 	if(code == 0)
 	{
@@ -867,7 +901,8 @@ bg_run_external(const char cmd[], int skip_errors)
 		/* Close write end of pipe. */
 		close(error_pipe[1]);
 
-		job = add_background_job(pid, command, error_pipe[0], BJT_COMMAND);
+		job = add_background_job(pid, command, (uintptr_t)error_pipe[0],
+				BJT_COMMAND);
 		if(job == NULL)
 		{
 			free(command);
@@ -901,8 +936,8 @@ bg_run_external(const char cmd[], int skip_errors)
 	{
 		CloseHandle(pinfo.hThread);
 
-		job = add_background_job(pinfo.dwProcessId, sh_cmd, pinfo.hProcess,
-				BJT_COMMAND);
+		job = add_background_job(pinfo.dwProcessId, sh_cmd,
+				(uintptr_t)pinfo.hProcess, BJT_COMMAND);
 		if(job == NULL)
 		{
 			free(sh_cmd);
@@ -938,7 +973,7 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 
 	task_args->func = task_func;
 	task_args->args = args;
-	task_args->job = add_background_job(WRONG_PID, descr, NO_JOB_ID,
+	task_args->job = add_background_job(WRONG_PID, descr, (uintptr_t)NO_JOB_ID,
 			important ? BJT_OPERATION : BJT_TASK);
 
 	if(task_args->job == NULL)
@@ -973,13 +1008,8 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 
 /* Creates structure that describes background job and registers it in the list
  * of jobs. */
-#ifndef _WIN32
 static bg_job_t *
-add_background_job(pid_t pid, const char cmd[], int fd, BgJobType type)
-#else
-static bg_job_t *
-add_background_job(pid_t pid, const char cmd[], HANDLE hprocess, BgJobType type)
-#endif
+add_background_job(pid_t pid, const char cmd[], uintptr_t data, BgJobType type)
 {
 	bg_job_t *new = malloc(sizeof(*new));
 	if(new == NULL)
@@ -1005,8 +1035,8 @@ add_background_job(pid_t pid, const char cmd[], HANDLE hprocess, BgJobType type)
 	new->exit_code = -1;
 
 #ifndef _WIN32
-	new->fd = fd;
-	if(fd != -1)
+	new->fd = (int)data;
+	if(new->fd != -1)
 	{
 		pthread_mutex_lock(&new_err_jobs_lock);
 		new->err_next = new_err_jobs;
@@ -1015,7 +1045,7 @@ add_background_job(pid_t pid, const char cmd[], HANDLE hprocess, BgJobType type)
 		pthread_cond_signal(&new_err_jobs_cond);
 	}
 #else
-	new->hprocess = hprocess;
+	new->hprocess = (HANDLE)data;
 #endif
 
 	if(type != BJT_COMMAND)
