@@ -47,6 +47,7 @@ static var_t has_builtin(const call_info_t *call_info);
 static var_t layoutis_builtin(const call_info_t *call_info);
 static var_t paneisat_builtin(const call_info_t *call_info);
 static var_t system_builtin(const call_info_t *call_info);
+static var_t term_builtin(const call_info_t *call_info);
 
 static const function_t functions[] = {
 	/* Name          Description                 Argc  Handler  */
@@ -59,6 +60,7 @@ static const function_t functions[] = {
 	{ "layoutis",    "query current layout",       1, &layoutis_builtin },
 	{ "paneisat",    "query pane location",        1, &paneisat_builtin },
 	{ "system",      "execute external command",   1, &system_builtin },
+	{ "term",        "run interactive command",    1, &term_builtin },
 };
 
 void
@@ -186,7 +188,7 @@ get_fnum(const char position[])
 static var_t
 getpanetype_builtin(const call_info_t *call_info)
 {
-	FileView *const view = curr_view;
+	view_t *const view = curr_view;
 	var_val_t var_val = { .string = "UNKNOWN" };
 
 	if(flist_custom_active(view))
@@ -195,6 +197,7 @@ getpanetype_builtin(const call_info_t *call_info)
 		{
 			case CV_REGULAR: var_val.string = "custom";      break;
 			case CV_VERY:    var_val.string = "very-custom"; break;
+			case CV_CUSTOM_TREE:
 			case CV_TREE:    var_val.string = "tree";        break;
 			case CV_DIFF:
 			case CV_COMPARE: var_val.string = "compare";     break;
@@ -347,5 +350,15 @@ system_builtin(const call_info_t *call_info)
 	return result;
 }
 
+/* Runs interactive command in shell and returns its output (joined standard
+ * output and standard error streams).  All trailing newline characters are
+ * stripped to allow easy appending to command output.  Returns the output. */
+static var_t
+term_builtin(const call_info_t *call_info)
+{
+	ui_shutdown();
+	return system_builtin(call_info);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
