@@ -22,7 +22,8 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
 #include <stddef.h> /* NULL size_t */
-#include <stdio.h> /* fgets() fprintf() fputc() fscanf() snprintf() */
+#include <stdio.h> /* FILE fpos_t fclose() fgetpos() fgets() fprintf() fputc()
+                      fread() fscanf() fsetpos() fwrite() snprintf() */
 #include <stdlib.h> /* abs() free() */
 #include <string.h> /* memcpy() memset() strtol() strcmp() strchr() strlen() */
 
@@ -37,8 +38,10 @@
 #include "../utils/filemon.h"
 #include "../utils/filter.h"
 #include "../utils/fs.h"
+#include "../utils/hist.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
+#include "../utils/matcher.h"
 #include "../utils/matchers.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
@@ -57,20 +60,20 @@
 #include "../status.h"
 #include "../trash.h"
 #include "config.h"
-#include "hist.h"
 #include "info_chars.h"
 
-static void get_sort_info(FileView *view, const char line[]);
+static void get_sort_info(view_t *view, const char line[]);
 static void append_to_history(hist_t *hist, void (*saver)(const char[]),
 		const char item[]);
 static void ensure_history_not_full(hist_t *hist);
-static void get_history(FileView *view, int reread, const char dir[],
+static void get_history(view_t *view, int reread, const char dir[],
 		const char file[], int rel_pos);
-static void set_view_property(FileView *view, char type, const char value[]);
+static void set_manual_filter(view_t *view, const char value[]);
+static void set_view_property(view_t *view, char type, const char value[]);
 static int copy_file(const char src[], const char dst[]);
 static int copy_file_internal(FILE *const src, FILE *const dst);
 static void update_info_file(const char filename[], int merge);
-static void process_hist_entry(FileView *view, const char dir[],
+static void process_hist_entry(view_t *view, const char dir[],
 		const char file[], int pos, char ***lh, int *nlh, int **lhp, size_t *nlhp);
 static char * convert_old_trash_path(const char trash_path[]);
 static void write_options(FILE *const fp);
@@ -86,7 +89,7 @@ static void write_bmarks(FILE *const fp, char *bmarks[], const int timestamps[],
 static void write_bmark(const char path[], const char tags[], time_t timestamp,
 		void *arg);
 static void write_tui_state(FILE *const fp);
-static void write_view_history(FILE *fp, FileView *view, const char str[],
+static void write_view_history(FILE *fp, view_t *view, const char str[],
 		char mark, int prev_count, char *prev[], int pos[]);
 static void write_history(FILE *fp, const char str[], char mark, int prev_count,
 		char *prev[], const hist_t *hist);
@@ -98,7 +101,7 @@ static void write_general_state(FILE *const fp);
 static char * read_vifminfo_line(FILE *fp, char buffer[]);
 static void remove_leading_whitespace(char line[]);
 static const char * escape_spaces(const char *str);
-static void put_sort_info(FILE *fp, char leading_char, const FileView *view);
+static void put_sort_info(FILE *fp, char leading_char, const view_t *view);
 static int read_optional_number(FILE *f);
 static int read_number(const char line[], long *value);
 static size_t add_to_int_array(int **array, size_t len, int what);
@@ -134,7 +137,7 @@ read_info_file(int reread)
 		{
 			if(line_val[0] == '[' || line_val[0] == ']')
 			{
-				FileView *v = curr_view;
+				view_t *v = curr_view;
 				curr_view = (line_val[0] == '[') ? &lwin : &rwin;
 				process_set_args(line_val + 1, 1, 1);
 				curr_view = v;
@@ -237,7 +240,7 @@ read_info_file(int reread)
 		else if(type == LINE_TYPE_QUICK_VIEW_STATE)
 		{
 			const int i = atoi(line_val);
-			curr_stats.view = (i == 1);
+			curr_stats.preview.on = (i == 1);
 		}
 		else if(type == LINE_TYPE_WIN_COUNT)
 		{
@@ -265,7 +268,7 @@ read_info_file(int reread)
 		}
 		else if(type == LINE_TYPE_LWIN_HIST || type == LINE_TYPE_RWIN_HIST)
 		{
-			FileView *const view = (type == LINE_TYPE_LWIN_HIST) ? &lwin : &rwin;
+			view_t *const view = (type == LINE_TYPE_LWIN_HIST) ? &lwin : &rwin;
 			if(line_val[0] == '\0')
 			{
 				if(!reread && view->history_num > 0)
@@ -277,24 +280,24 @@ read_info_file(int reread)
 			else if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 			{
 				const int rel_pos = read_optional_number(fp);
-				get_history(view, reread, line_val, line2, rel_pos);
+				get_history(view, reread, line_val, line2, rel_pos < 0 ? 0 : rel_pos);
 			}
 		}
 		else if(type == LINE_TYPE_CMDLINE_HIST)
 		{
-			append_to_history(&cfg.cmd_hist, cfg_save_command_history, line_val);
+			append_to_history(&curr_stats.cmd_hist, &hists_commands_save, line_val);
 		}
 		else if(type == LINE_TYPE_SEARCH_HIST)
 		{
-			append_to_history(&cfg.search_hist, cfg_save_search_history, line_val);
+			append_to_history(&curr_stats.search_hist, &hists_search_save, line_val);
 		}
 		else if(type == LINE_TYPE_PROMPT_HIST)
 		{
-			append_to_history(&cfg.prompt_hist, cfg_save_prompt_history, line_val);
+			append_to_history(&curr_stats.prompt_hist, &hists_prompt_save, line_val);
 		}
 		else if(type == LINE_TYPE_FILTER_HIST)
 		{
-			append_to_history(&cfg.filter_hist, cfg_save_filter_history, line_val);
+			append_to_history(&curr_stats.filter_hist, &hists_filter_save, line_val);
 		}
 		else if(type == LINE_TYPE_DIR_STACK)
 		{
@@ -324,13 +327,11 @@ read_info_file(int reread)
 		}
 		else if(type == LINE_TYPE_LWIN_FILT)
 		{
-			(void)replace_string(&lwin.prev_manual_filter, line_val);
-			(void)filter_set(&lwin.manual_filter, line_val);
+			set_manual_filter(&lwin, line_val);
 		}
 		else if(type == LINE_TYPE_RWIN_FILT)
 		{
-			(void)replace_string(&rwin.prev_manual_filter, line_val);
-			(void)filter_set(&rwin.manual_filter, line_val);
+			set_manual_filter(&rwin, line_val);
 		}
 		else if(type == LINE_TYPE_LWIN_FILT_INV)
 		{
@@ -354,7 +355,7 @@ read_info_file(int reread)
 		}
 		else if(type == LINE_TYPE_LWIN_SPECIFIC || type == LINE_TYPE_RWIN_SPECIFIC)
 		{
-			FileView *view = (type == LINE_TYPE_LWIN_SPECIFIC) ? &lwin : &rwin;
+			view_t *view = (type == LINE_TYPE_LWIN_SPECIFIC) ? &lwin : &rwin;
 			set_view_property(view, line_val[0], line_val + 1);
 		}
 	}
@@ -370,7 +371,7 @@ read_info_file(int reread)
 
 /* Parses sort description line of the view and initialized its sort field. */
 static void
-get_sort_info(FileView *view, const char line[])
+get_sort_info(view_t *view, const char line[])
 {
 	char *const sort = curr_stats.restart_in_progress
 	                 ? ui_view_sort_list_get(view, view->sort)
@@ -425,7 +426,7 @@ ensure_history_not_full(hist_t *hist)
 
 /* Loads single history entry from vifminfo into the view. */
 static void
-get_history(FileView *view, int reread, const char dir[], const char file[],
+get_history(view_t *view, int reread, const char dir[], const char file[],
 		int rel_pos)
 {
 	const int list_rows = view->list_rows;
@@ -446,13 +447,37 @@ get_history(FileView *view, int reread, const char dir[], const char file[],
 	}
 }
 
+/* Sets manual filter of the view and its previous state to given value. */
+static void
+set_manual_filter(view_t *view, const char value[])
+{
+	char *error;
+	matcher_t *matcher;
+
+	(void)replace_string(&view->prev_manual_filter, value);
+	matcher = matcher_alloc(value, FILTER_DEF_CASE_SENSITIVITY, 0, "", &error);
+	free(error);
+
+	/* If setting filter value has failed, try to setup an empty value instead. */
+	if(matcher == NULL)
+	{
+		(void)replace_string(&view->prev_manual_filter, "");
+		matcher = matcher_alloc("", FILTER_DEF_CASE_SENSITIVITY, 0, "", &error);
+		free(error);
+		assert(matcher != NULL && "Can't init manual filter.");
+	}
+
+	matcher_free(view->manual_filter);
+	view->manual_filter = matcher;
+}
+
 /* Sets view property specified by the type to the value. */
 static void
-set_view_property(FileView *view, char type, const char value[])
+set_view_property(view_t *view, char type, const char value[])
 {
 	if(type == PROP_TYPE_DOTFILES)
 	{
-		set_dot_files_visible(view, !atoi(value));
+		dot_filter_set(view, !atoi(value));
 	}
 	else if(type == PROP_TYPE_AUTO_FILTER)
 	{
@@ -726,28 +751,28 @@ update_info_file(const char filename[], int merge)
 			}
 			else if(type == LINE_TYPE_CMDLINE_HIST)
 			{
-				if(!hist_contains(&cfg.cmd_hist, line_val))
+				if(!hist_contains(&curr_stats.cmd_hist, line_val))
 				{
 					ncmdh = add_to_string_array(&cmdh, ncmdh, 1, line_val);
 				}
 			}
 			else if(type == LINE_TYPE_SEARCH_HIST)
 			{
-				if(!hist_contains(&cfg.search_hist, line_val))
+				if(!hist_contains(&curr_stats.search_hist, line_val))
 				{
 					nsrch = add_to_string_array(&srch, nsrch, 1, line_val);
 				}
 			}
 			else if(type == LINE_TYPE_PROMPT_HIST)
 			{
-				if(!hist_contains(&cfg.prompt_hist, line_val))
+				if(!hist_contains(&curr_stats.prompt_hist, line_val))
 				{
 					nprompt = add_to_string_array(&prompt, nprompt, 1, line_val);
 				}
 			}
 			else if(type == LINE_TYPE_FILTER_HIST)
 			{
-				if(!hist_contains(&cfg.filter_hist, line_val))
+				if(!hist_contains(&curr_stats.filter_hist, line_val))
 				{
 					nfilter = add_to_string_array(&filter, nfilter, 1, line_val);
 				}
@@ -830,25 +855,26 @@ update_info_file(const char filename[], int merge)
 		if(cfg.vifm_info & VIFMINFO_CHISTORY)
 		{
 			write_history(fp, "Command line", LINE_TYPE_CMDLINE_HIST,
-					MIN(ncmdh, cfg.history_len - cfg.cmd_hist.pos), cmdh, &cfg.cmd_hist);
+					MIN(ncmdh, cfg.history_len - curr_stats.cmd_hist.pos), cmdh,
+					&curr_stats.cmd_hist);
 		}
 
 		if(cfg.vifm_info & VIFMINFO_SHISTORY)
 		{
 			write_history(fp, "Search", LINE_TYPE_SEARCH_HIST, nsrch, srch,
-					&cfg.search_hist);
+					&curr_stats.search_hist);
 		}
 
 		if(cfg.vifm_info & VIFMINFO_PHISTORY)
 		{
 			write_history(fp, "Prompt", LINE_TYPE_PROMPT_HIST, nprompt, prompt,
-					&cfg.prompt_hist);
+					&curr_stats.prompt_hist);
 		}
 
 		if(cfg.vifm_info & VIFMINFO_FHISTORY)
 		{
 			write_history(fp, "Local filter", LINE_TYPE_FILTER_HIST, nfilter, filter,
-					&cfg.filter_hist);
+					&curr_stats.filter_hist);
 		}
 
 		if(cfg.vifm_info & VIFMINFO_REGISTERS)
@@ -902,7 +928,7 @@ update_info_file(const char filename[], int merge)
 
 /* Handles single directory history entry, possibly skipping merging it in. */
 static void
-process_hist_entry(FileView *view, const char dir[], const char file[], int pos,
+process_hist_entry(view_t *view, const char dir[], const char file[], int pos,
 		char ***lh, int *nlh, int **lhp, size_t *nlhp)
 {
 	if(view->history_pos + *nlh/2 == cfg.history_len - 1 ||
@@ -993,9 +1019,10 @@ write_options(FILE *const fp)
 	fprintf(fp, "=timefmt=%s\n", escape_spaces(cfg.time_format + 1));
 	fprintf(fp, "=timeoutlen=%d\n", cfg.timeout_len);
 	fprintf(fp, "=%strash\n", cfg.use_trash ? "" : "no");
-	fprintf(fp, "=tuioptions=%s%s\n",
+	fprintf(fp, "=tuioptions=%s%s%s\n",
 			cfg.extra_padding ? "p" : "",
-			cfg.side_borders_visible ? "s" : "");
+			cfg.side_borders_visible ? "s" : "",
+			cfg.use_unicode_characters ? "u" : "");
 	fprintf(fp, "=undolevels=%d\n", cfg.undo_levels);
 	fprintf(fp, "=vicmd=%s%s\n", escape_spaces(cfg.vi_command),
 			cfg.vi_cmd_bg ? " &" : "");
@@ -1006,8 +1033,18 @@ write_options(FILE *const fp)
 	fprintf(fp, "=]viewcolumns=%s\n", escape_spaces(rwin.view_columns_g));
 	fprintf(fp, "=[sortgroups=%s\n", escape_spaces(lwin.sort_groups_g));
 	fprintf(fp, "=]sortgroups=%s\n", escape_spaces(rwin.sort_groups_g));
+	fprintf(fp, "=[lsoptions=%s\n", lwin.ls_transposed_g ? "transposed" : "");
+	fprintf(fp, "=]lsoptions=%s\n", rwin.ls_transposed_g ? "transposed" : "");
 	fprintf(fp, "=[%slsview\n", lwin.ls_view_g ? "" : "no");
 	fprintf(fp, "=]%slsview\n", rwin.ls_view_g ? "" : "no");
+	fprintf(fp, "=[milleroptions=lsize:%d,csize:%d,rsize:%d\n",
+			lwin.miller_ratios_g[0], lwin.miller_ratios_g[1],
+			lwin.miller_ratios_g[2]);
+	fprintf(fp, "=]milleroptions=lsize:%d,csize:%d,rsize:%d\n",
+			rwin.miller_ratios_g[0], rwin.miller_ratios_g[1],
+			rwin.miller_ratios_g[2]);
+	fprintf(fp, "=[%smillerview\n", lwin.miller_view_g ? "" : "no");
+	fprintf(fp, "=]%smillerview\n", rwin.miller_view_g ? "" : "no");
 	fprintf(fp, "=[%snumber\n", (lwin.num_type_g & NT_SEQ) ? "" : "no");
 	fprintf(fp, "=]%snumber\n", (rwin.num_type_g & NT_SEQ) ? "" : "no");
 	fprintf(fp, "=[numberwidth=%d\n", lwin.num_width_g);
@@ -1016,6 +1053,8 @@ write_options(FILE *const fp)
 	fprintf(fp, "=]%srelativenumber\n", (rwin.num_type_g & NT_REL) ? "" : "no");
 	fprintf(fp, "=[%sdotfiles\n", lwin.hide_dot_g ? "" : "no");
 	fprintf(fp, "=]%sdotfiles\n", rwin.hide_dot_g ? "" : "no");
+	fprintf(fp, "=[previewprg=%s\n", escape_spaces(lwin.preview_prg_g));
+	fprintf(fp, "=]previewprg=%s\n", escape_spaces(rwin.preview_prg_g));
 
 	fprintf(fp, "%s", "=confirm=");
 	if(cfg.confirm & CONFIRM_DELETE)
@@ -1049,6 +1088,8 @@ write_options(FILE *const fp)
 		fprintf(fp, "%s", "otherpane,");
 	if(cfg.sug.flags & SF_KEYS)
 		fprintf(fp, "%s", "keys,");
+	if(cfg.sug.flags & SF_FOLDSUBKEYS)
+		fprintf(fp, "%s", "foldsubkeys,");
 	if(cfg.sug.flags & SF_MARKS)
 		fprintf(fp, "%s", "marks,");
 	if(cfg.sug.flags & SF_DELAY)
@@ -1273,7 +1314,7 @@ write_tui_state(FILE *const fp)
 {
 	fputs("\n# TUI:\n", fp);
 	fprintf(fp, "a%c\n", (curr_view == &rwin) ? 'r' : 'l');
-	fprintf(fp, "q%d\n", curr_stats.view);
+	fprintf(fp, "q%u\n", curr_stats.preview.on);
 	fprintf(fp, "v%d\n", curr_stats.number_of_windows);
 	fprintf(fp, "o%c\n", (curr_stats.split == VSPLIT) ? 'v' : 'h');
 	fprintf(fp, "m%d\n", curr_stats.splitter_pos);
@@ -1284,7 +1325,7 @@ write_tui_state(FILE *const fp)
 
 /* Stores history of the view to the file. */
 static void
-write_view_history(FILE *fp, FileView *view, const char str[], char mark,
+write_view_history(FILE *fp, view_t *view, const char str[], char mark,
 		int prev_count, char *prev[], int pos[])
 {
 	int i;
@@ -1400,11 +1441,11 @@ static void
 write_general_state(FILE *const fp)
 {
 	fputs("\n# State:\n", fp);
-	fprintf(fp, "f%s\n", lwin.manual_filter.raw);
+	fprintf(fp, "f%s\n", matcher_get_expr(lwin.manual_filter));
 	fprintf(fp, "i%d\n", lwin.invert);
 	fprintf(fp, "[.%d\n", lwin.hide_dot);
 	fprintf(fp, "[F%s\n", lwin.auto_filter.raw);
-	fprintf(fp, "F%s\n", rwin.manual_filter.raw);
+	fprintf(fp, "F%s\n", matcher_get_expr(rwin.manual_filter));
 	fprintf(fp, "I%d\n", rwin.invert);
 	fprintf(fp, "].%d\n", rwin.hide_dot);
 	fprintf(fp, "]F%s\n", rwin.auto_filter.raw);
@@ -1460,7 +1501,7 @@ escape_spaces(const char str[])
 /* Writes sort description line of the view to the fp file prepending the
  * leading_char to it. */
 static void
-put_sort_info(FILE *fp, char leading_char, const FileView *view)
+put_sort_info(FILE *fp, char leading_char, const view_t *view)
 {
 	int i = -1;
 	const char *const sort = ui_view_sort_list_get(view, view->sort_g);
@@ -1479,18 +1520,33 @@ put_sort_info(FILE *fp, char leading_char, const FileView *view)
 static int
 read_optional_number(FILE *f)
 {
-	int num = -1;
-	const int c = getc(f);
+	int num;
+	int nread;
+	int c;
+	fpos_t pos;
 
-	if(c != EOF)
+	if(fgetpos(f, &pos) != 0)
 	{
-		ungetc(c, f);
-		if(isdigit(c) || c == '-' || c == '+')
-		{
-			const int nread = fscanf(f, "%30d\n", &num);
-			assert(nread == 1 && "Wrong number of read numbers.");
-			(void)nread;
-		}
+		return -1;
+	}
+
+	c = getc(f);
+	if(c == EOF)
+	{
+		return -1;
+	}
+	ungetc(c, f);
+
+	if(!isdigit(c) && c != '-' && c != '+')
+	{
+		return -1;
+	}
+
+	nread = fscanf(f, "%30d\n", &num);
+	if(nread != 1)
+	{
+		fsetpos(f, &pos);
+		return -1;
 	}
 
 	return num;
