@@ -42,6 +42,7 @@
                        strrchr() */
 
 #include "cfg/config.h"
+#include "compat/dtype.h"
 #include "compat/fs_limits.h"
 #include "compat/os.h"
 #include "engine/abbrevs.h"
@@ -62,6 +63,7 @@
 #include "utils/env.h"
 #include "utils/fs.h"
 #include "utils/macros.h"
+#include "utils/matchers.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/string_array.h"
@@ -83,7 +85,7 @@ static void complete_invert(const char str[]);
 static int complete_chown(const char *str);
 static void complete_filetype(const char *str);
 static void complete_progs(const char *str, assoc_records_t records);
-static void complete_highlight_groups(const char *str);
+static void complete_highlight_groups(const char str[], int file_hi_only);
 static int complete_highlight_arg(const char *str);
 static void complete_envvar(const char str[]);
 static void complete_winrun(const char str[]);
@@ -92,8 +94,8 @@ static void complete_from_string_list(const char str[], const char *items[][2],
 static void complete_command_name(const char beginning[]);
 static void filename_completion_in_dir(const char *path, const char *str,
 		CompletionType type);
-static void filename_completion_internal(DIR *dir, const char filename[],
-		CompletionType type);
+static void filename_completion_internal(DIR *dir, const char dir_path[],
+		const char filename[], CompletionType type);
 static int is_dirent_targets_exec(const struct dirent *d);
 #ifdef _WIN32
 static void complete_with_shared(const char *server, const char *file);
@@ -178,10 +180,22 @@ complete_args(int id, const cmd_info_t *cmd_info, int arg_pos, void *extra_arg)
 		complete_filetype(args);
 	else if(id == COM_HIGHLIGHT)
 	{
-		if(earg_num(argc, args) <= 1)
-			complete_highlight_groups(args);
+		const int arg_num = earg_num(argc, args);
+		if(arg_num <= 1)
+		{
+			complete_highlight_groups(args, 0);
+		}
+		else if(starts_with_lit(args, "clear "))
+		{
+			if(arg_num == 2)
+			{
+				complete_highlight_groups(arg, 1);
+			}
+		}
 		else
+		{
 			start += complete_highlight_arg(arg);
+		}
 	}
 	else if((id == COM_CD || id == COM_PUSHD || id == COM_EXECUTE ||
 			id == COM_SOURCE || id == COM_EDIT) && dollar != NULL && dollar > slash)
@@ -611,21 +625,37 @@ complete_progs(const char *str, assoc_records_t records)
 
 /* Completes highlight groups and subcommand "clear" for :highlight command. */
 static void
-complete_highlight_groups(const char *str)
+complete_highlight_groups(const char str[], int file_hi_only)
 {
 	int i;
 	const size_t len = strlen(str);
-	for(i = 0; i < MAXNUM_COLOR; ++i)
+
+	col_scheme_t *const cs = curr_stats.cs;
+	for(i = 0; i < cs->file_hi_count; ++i)
 	{
-		if(strncasecmp(str, HI_GROUPS[i], len) == 0)
+		const char *const expr = matchers_get_expr(cs->file_hi[i].matchers);
+		if(strncasecmp(str, expr, len) == 0)
 		{
-			vle_compl_add_match(HI_GROUPS[i], HI_GROUPS_DESCR[i]);
+			vle_compl_add_match(expr, "");
 		}
 	}
-	if(strncmp(str, "clear", len) == 0)
+
+	if(!file_hi_only)
 	{
-		vle_compl_add_match("clear", "restore default colors");
+		for(i = 0; i < MAXNUM_COLOR; ++i)
+		{
+			if(strncasecmp(str, HI_GROUPS[i], len) == 0)
+			{
+				vle_compl_add_match(HI_GROUPS[i], HI_GROUPS_DESCR[i]);
+			}
+		}
+
+		if(strncmp(str, "clear", len) == 0)
+		{
+			vle_compl_add_match("clear", "clear color rules");
+		}
 	}
+
 	vle_compl_finish_group();
 	vle_compl_add_last_match(str);
 }
@@ -819,7 +849,7 @@ fast_run_complete(const char cmd[])
 
 		if(result == NULL)
 		{
-			status_bar_error("Command beginning is ambiguous");
+			ui_sb_err("Command beginning is ambiguous");
 		}
 	}
 	else
@@ -859,10 +889,10 @@ static void
 filename_completion_in_dir(const char *path, const char *str,
 		CompletionType type)
 {
-	char buf[PATH_MAX];
+	char buf[PATH_MAX + 1];
 	if(is_root_dir(str))
 	{
-		snprintf(buf, sizeof(buf), "%s", str);
+		copy_str(buf, sizeof(buf), str);
 	}
 	else
 	{
@@ -931,11 +961,11 @@ filename_completion(const char str[], CompletionType type,
 	{
 		char buf[PATH_MAX];
 		if(!is_unc_root(dirname))
-			snprintf(buf,
-					strchr(curr_view->curr_dir + 2, '/') - curr_view->curr_dir + 1, "%s",
+			copy_str(buf,
+					strchr(curr_view->curr_dir + 2, '/') - curr_view->curr_dir + 1,
 					curr_view->curr_dir);
 		else
-			snprintf(buf, sizeof(buf), "%s", dirname);
+			copy_str(buf, sizeof(buf), dirname);
 
 		complete_with_shared(buf, filename);
 		free(filename);
@@ -946,11 +976,11 @@ filename_completion(const char str[], CompletionType type,
 	{
 		char buf[PATH_MAX];
 		if(is_path_absolute(dirname) && !is_unc_root(curr_view->curr_dir))
-			snprintf(buf,
-					strchr(curr_view->curr_dir + 2, '/') - curr_view->curr_dir + 2, "%s",
+			copy_str(buf,
+					strchr(curr_view->curr_dir + 2, '/') - curr_view->curr_dir + 2,
 					curr_view->curr_dir);
 		else
-			snprintf(buf, sizeof(buf), "%s", curr_view->curr_dir);
+			copy_str(buf, sizeof(buf), curr_view->curr_dir);
 		strcat(buf, dirname);
 		chosp(buf);
 		(void)replace_string(&dirname, buf);
@@ -967,7 +997,7 @@ filename_completion(const char str[], CompletionType type,
 	}
 	else
 	{
-		filename_completion_internal(dir, filename, type);
+		filename_completion_internal(dir, dirname, filename, type);
 		(void)vifm_chdir(flist_get_dir(curr_view));
 	}
 
@@ -982,29 +1012,35 @@ filename_completion(const char str[], CompletionType type,
 	restore_cwd(cwd);
 }
 
+/* The file completion core of filename_completion(). */
 static void
-filename_completion_internal(DIR *dir, const char filename[],
-		CompletionType type)
+filename_completion_internal(DIR *dir, const char dir_path[],
+		const char filename[], CompletionType type)
 {
+	/* It's OK to use relative paths here, because filename_completion()
+	 * guarantees that we are in correct directory. */
 	struct dirent *d;
 
 	size_t filename_len = strlen(filename);
 	while((d = os_readdir(dir)) != NULL)
 	{
+		int is_dir;
+
 		if(filename[0] == '\0' && d->d_name[0] == '.')
 			continue;
 		if(!file_matches(d->d_name, filename, filename_len))
 			continue;
 
-		if(type == CT_DIRONLY && !is_dirent_targets_dir(d))
+		is_dir = is_dirent_targets_dir(d->d_name, d);
+
+		if(type == CT_DIRONLY && !is_dir)
 			continue;
 		else if(type == CT_EXECONLY && !is_dirent_targets_exec(d))
 			continue;
-		else if(type == CT_DIREXEC && !is_dirent_targets_dir(d) &&
-				!is_dirent_targets_exec(d))
+		else if(type == CT_DIREXEC && !is_dir && !is_dirent_targets_exec(d))
 			continue;
 
-		if(is_dirent_targets_dir(d) && type != CT_ALL_WOS)
+		if(is_dir && type != CT_ALL_WOS)
 		{
 			vle_compl_put_path_match(format_str("%s/", d->d_name));
 		}
@@ -1026,10 +1062,13 @@ filename_completion_internal(DIR *dir, const char filename[],
 static int
 is_dirent_targets_exec(const struct dirent *d)
 {
+	/* It's OK to use relative paths here, because filename_completion()
+	 * guarantees that we are in correct directory. */
 #ifndef _WIN32
-	if(d->d_type == DT_DIR)
+	const unsigned char type = get_dirent_type(d, d->d_name);
+	if(type == DT_DIR)
 		return 0;
-	if(d->d_type == DT_LNK && get_symlink_type(d->d_name) != SLT_UNKNOWN)
+	if(type == DT_LNK && get_symlink_type(d->d_name) != SLT_UNKNOWN)
 		return 0;
 	return os_access(d->d_name, X_OK) == 0;
 #else
@@ -1040,7 +1079,7 @@ is_dirent_targets_exec(const struct dirent *d)
 #ifndef _WIN32
 
 void
-complete_user_name(const char *str)
+complete_user_name(const char str[])
 {
 	struct passwd *pw;
 	size_t len;
