@@ -34,7 +34,6 @@
 #include <string.h> /* strcmp() strcpy() strlen() */
 
 #include "cfg/config.h"
-#include "cfg/hist.h"
 #include "compat/fs_limits.h"
 #include "compat/os.h"
 #include "engine/autocmds.h"
@@ -58,6 +57,7 @@
 #include "utils/file_streams.h"
 #include "utils/filter.h"
 #include "utils/fs.h"
+#include "utils/hist.h"
 #include "utils/int_stack.h"
 #include "utils/path.h"
 #include "utils/str.h"
@@ -119,7 +119,7 @@ TSTATIC int line_pos(const char begin[], const char end[], char sep,
 		int rquoting);
 static CmdArgsType get_cmd_args_type(const char cmd[]);
 static char * skip_to_cmd_name(const char cmd[]);
-static int repeat_command(FileView *view, CmdInputType type);
+static int repeat_command(view_t *view, CmdInputType type);
 static int is_at_scope_bottom(const int_stack_t *scope_stack);
 TSTATIC char * eval_arglist(const char args[], const char **stop_ptr);
 
@@ -151,11 +151,11 @@ swap_range(void)
 static int
 resolve_mark(char mark)
 {
-	int result;
-
-	result = check_mark_directory(curr_view, mark);
+	const int result = check_mark_directory(curr_view, mark);
 	if(result < 0)
-		status_bar_errorf("Trying to use an invalid mark: '%c", mark);
+	{
+		ui_sb_errf("Trying to use an invalid mark: '%c", mark);
+	}
 	return result;
 }
 
@@ -267,14 +267,14 @@ history_by_type(CmdInputType type)
 	switch(type)
 	{
 		case CIT_COMMAND:
-			return &cfg.cmd_hist;
+			return &curr_stats.cmd_hist;
 		case CIT_PROMPT_INPUT:
-			return &cfg.prompt_hist;
+			return &curr_stats.prompt_hist;
 		case CIT_FILTER_PATTERN:
-			return &cfg.filter_hist;
+			return &curr_stats.filter_hist;
 
 		default:
-			return &cfg.search_hist;
+			return &curr_stats.search_hist;
 	}
 }
 
@@ -314,11 +314,11 @@ save_extcmd(const char command[], CmdInputType type)
 {
 	if(type == CIT_COMMAND)
 	{
-		cfg_save_command_history(command);
+		hists_commands_save(command);
 	}
 	else
 	{
-		cfg_save_search_history(command);
+		hists_search_save(command);
 	}
 }
 
@@ -436,7 +436,7 @@ pattern_expand_hook(const char pattern[])
 
 /* Returns negative value in case of error */
 static int
-execute_command(FileView *view, const char command[], int menu)
+execute_command(view_t *view, const char command[], int menu)
 {
 	int id;
 	int result;
@@ -493,58 +493,58 @@ execute_command(FileView *view, const char command[], int menu)
 	switch(result)
 	{
 		case CMDS_ERR_LOOP:
-			status_bar_error("Loop in commands");
+			ui_sb_err("Loop in commands");
 			break;
 		case CMDS_ERR_NO_MEM:
-			status_bar_error("Unable to allocate enough memory");
+			ui_sb_err("Unable to allocate enough memory");
 			break;
 		case CMDS_ERR_TOO_FEW_ARGS:
-			status_bar_error("Too few arguments");
+			ui_sb_err("Too few arguments");
 			break;
 		case CMDS_ERR_TRAILING_CHARS:
-			status_bar_error("Trailing characters");
+			ui_sb_err("Trailing characters");
 			break;
 		case CMDS_ERR_INCORRECT_NAME:
-			status_bar_error("Incorrect command name");
+			ui_sb_err("Incorrect command name");
 			break;
 		case CMDS_ERR_NEED_BANG:
-			status_bar_error("Add bang to force");
+			ui_sb_err("Add bang to force");
 			break;
 		case CMDS_ERR_NO_BUILTIN_REDEFINE:
-			status_bar_error("Can't redefine builtin command");
+			ui_sb_err("Can't redefine builtin command");
 			break;
 		case CMDS_ERR_INVALID_CMD:
-			status_bar_error("Invalid command name");
+			ui_sb_err("Invalid command name");
 			break;
 		case CMDS_ERR_NO_BANG_ALLOWED:
-			status_bar_error("No ! is allowed");
+			ui_sb_err("No ! is allowed");
 			break;
 		case CMDS_ERR_NO_RANGE_ALLOWED:
-			status_bar_error("No range is allowed");
+			ui_sb_err("No range is allowed");
 			break;
 		case CMDS_ERR_NO_QMARK_ALLOWED:
-			status_bar_error("No ? is allowed");
+			ui_sb_err("No ? is allowed");
 			break;
 		case CMDS_ERR_INVALID_RANGE:
-			/* message dialog is enough */
+			ui_sb_err("Invalid range");
 			break;
 		case CMDS_ERR_NO_SUCH_UDF:
-			status_bar_error("No such user defined command");
+			ui_sb_err("No such user defined command");
 			break;
 		case CMDS_ERR_UDF_IS_AMBIGUOUS:
-			status_bar_error("Ambiguous use of user-defined command");
+			ui_sb_err("Ambiguous use of user-defined command");
 			break;
 		case CMDS_ERR_ZERO_COUNT:
-			status_bar_error("Zero count");
+			ui_sb_err("Zero count");
 			break;
 		case CMDS_ERR_INVALID_ARG:
-			status_bar_error("Invalid argument");
+			ui_sb_err("Invalid argument");
 			break;
 		case CMDS_ERR_CUSTOM:
 			/* error message is posted by command handler */
 			break;
 		default:
-			status_bar_error("Unknown error");
+			ui_sb_err("Unknown error");
 			break;
 	}
 
@@ -730,7 +730,7 @@ line_pos(const char begin[], const char end[], char sep, int rquoting)
 }
 
 int
-exec_commands(const char cmdline[], FileView *view, CmdInputType type)
+exec_commands(const char cmdline[], view_t *view, CmdInputType type)
 {
 	int save_msg = 0;
 	char **cmds = break_cmdline(cmdline, type == CIT_MENU_COMMAND);
@@ -1020,7 +1020,7 @@ skip_to_cmd_name(const char cmd[])
 }
 
 int
-exec_command(const char cmd[], FileView *view, CmdInputType type)
+exec_command(const char cmd[], view_t *view, CmdInputType type)
 {
 	int menu;
 	int backward;
@@ -1044,7 +1044,7 @@ exec_command(const char cmd[], FileView *view, CmdInputType type)
 
 		case CIT_VWBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VWFSEARCH_PATTERN:
-			return find_vwpattern(cmd, backward);
+			return view_find_pattern(cmd, backward);
 
 		case CIT_MENU_COMMAND: menu = 1; /* Fall through. */
 		case CIT_COMMAND:
@@ -1071,22 +1071,22 @@ exec_command(const char cmd[], FileView *view, CmdInputType type)
  * message should be saved in the status bar, > 0 to save message on successful
  * execution and < 0 in case of error with error message. */
 static int
-repeat_command(FileView *view, CmdInputType type)
+repeat_command(view_t *view, CmdInputType type)
 {
 	int backward = 0;
 	switch(type)
 	{
 		case CIT_BSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_FSEARCH_PATTERN:
-			return find_npattern(view, cfg_get_last_search_pattern(), backward, 1);
+			return find_npattern(view, hists_search_last(), backward, 1);
 
 		case CIT_VBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VFSEARCH_PATTERN:
-			return find_vpattern(view, cfg_get_last_search_pattern(), backward, 1);
+			return find_vpattern(view, hists_search_last(), backward, 1);
 
 		case CIT_VWBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VWFSEARCH_PATTERN:
-			return find_vwpattern(NULL, backward);
+			return view_find_pattern(NULL, backward);
 
 		case CIT_COMMAND:
 			return execute_command(view, NULL, 0);
@@ -1112,7 +1112,7 @@ commands_scope_finish(void)
 {
 	if(!is_at_scope_bottom(&if_levels))
 	{
-		status_bar_error("Missing :endif");
+		ui_sb_err("Missing :endif");
 		int_stack_pop_seq(&if_levels, IF_SCOPE_GUARD);
 		return 1;
 	}
