@@ -66,15 +66,14 @@ compare_record_t;
 
 static void make_unique_lists(entries_t curr, entries_t other);
 static void leave_only_dups(entries_t *curr, entries_t *other);
-static int is_not_duplicate(FileView *view, const dir_entry_t *entry,
-		void *arg);
+static int is_not_duplicate(view_t *view, const dir_entry_t *entry, void *arg);
 static void fill_side_by_side(entries_t curr, entries_t other, int group_paths);
 static int id_sorter(const void *first, const void *second);
-static void put_or_free(FileView *view, dir_entry_t *entry, int id, int take);
-static entries_t make_diff_list(trie_t *trie, FileView *view, int *next_id,
+static void put_or_free(view_t *view, dir_entry_t *entry, int id, int take);
+static entries_t make_diff_list(trie_t *trie, view_t *view, int *next_id,
 		CompareType ct, int skip_empty, int dups_only);
-static void list_view_entries(const FileView *view, strlist_t *list);
-static void append_valid_nodes(const char name[], int valid,
+static void list_view_entries(const view_t *view, strlist_t *list);
+static int append_valid_nodes(const char name[], int valid,
 		const void *parent_data, void *data, void *arg);
 static void list_files_recursively(const char path[], int skip_dot_files,
 		strlist_t *list);
@@ -113,7 +112,7 @@ compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 	{
 		free_dir_entries(curr_view, &curr.entries, &curr.nentries);
 		free_dir_entries(other_view, &other.entries, &other.nentries);
-		status_bar_message("Comparison has been cancelled");
+		ui_sb_msg("Comparison has been cancelled");
 		return 1;
 	}
 
@@ -279,7 +278,7 @@ leave_only_dups(entries_t *curr, entries_t *other)
 /* zap_entries() filter to filter-out files marked for removal.  Returns
  * non-zero if entry is to be kept and zero otherwise. */
 static int
-is_not_duplicate(FileView *view, const dir_entry_t *entry, void *arg)
+is_not_duplicate(view_t *view, const dir_entry_t *entry, void *arg)
 {
 	return entry->id != -1;
 }
@@ -366,10 +365,10 @@ fill_side_by_side(entries_t curr, entries_t other, int group_paths)
 }
 
 int
-compare_one_pane(FileView *view, CompareType ct, ListType lt, int skip_empty)
+compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
 {
 	int i, dup_id;
-	FileView *other = (view == curr_view) ? other_view : curr_view;
+	view_t *other = (view == curr_view) ? other_view : curr_view;
 	const char *const title = (lt == LT_ALL)  ? "compare"
 	                        : (lt == LT_DUPS) ? "dups" : "nondups";
 
@@ -391,12 +390,14 @@ compare_one_pane(FileView *view, CompareType ct, ListType lt, int skip_empty)
 	if(ui_cancellation_requested())
 	{
 		free_dir_entries(view, &curr.entries, &curr.nentries);
-		status_bar_message("Comparison has been cancelled");
+		ui_sb_msg("Comparison has been cancelled");
 		return 1;
 	}
 
-	if(curr.entries != NULL)
+	if(curr.nentries > 0)
 	{
+		/* When there are no entries, qsort() might be called with a NULL parameter,
+		 * which isn't allowed by the standard. */
 		qsort(curr.entries, curr.nentries, sizeof(*curr.entries), &id_sorter);
 	}
 
@@ -469,7 +470,7 @@ id_sorter(const void *first, const void *second)
 /* Either puts the entry into the view or frees it (depends on the take
  * argument). */
 static void
-put_or_free(FileView *view, dir_entry_t *entry, int id, int take)
+put_or_free(view_t *view, dir_entry_t *entry, int id, int take)
 {
 	if(take)
 	{
@@ -486,7 +487,7 @@ put_or_free(FileView *view, dir_entry_t *entry, int id, int take)
  * identical files.  With non-zero dups_only, new files aren't added to the
  * trie. */
 static entries_t
-make_diff_list(trie_t *trie, FileView *view, int *next_id, CompareType ct,
+make_diff_list(trie_t *trie, view_t *view, int *next_id, CompareType ct,
 		int skip_empty, int dups_only)
 {
 	int i;
@@ -570,7 +571,7 @@ make_diff_list(trie_t *trie, FileView *view, int *next_id, CompareType ct,
 /* Fills the list with entries of the view in hierarchical order (pre-order tree
  * traversal). */
 static void
-list_view_entries(const FileView *view, strlist_t *list)
+list_view_entries(const view_t *view, strlist_t *list)
 {
 	int i;
 
@@ -593,8 +594,8 @@ list_view_entries(const FileView *view, strlist_t *list)
 }
 
 /* fsdata_traverse() callback that collects names of existing files into a
- * list. */
-static void
+ * list.  Should return non-zero to stop traverser. */
+static int
 append_valid_nodes(const char name[], int valid, const void *parent_data,
 		void *data, void *arg)
 {
@@ -608,6 +609,7 @@ append_valid_nodes(const char name[], int valid, const void *parent_data,
 		list->nitems = add_to_string_array(&list->items, list->nitems, 1,
 				full_path);
 	}
+	return 0;
 }
 
 /* Collects files under specified file system tree. */
@@ -862,7 +864,7 @@ free_compare_records(void *ptr)
 }
 
 int
-compare_move(FileView *from, FileView *to)
+compare_move(view_t *from, view_t *to)
 {
 	char from_path[PATH_MAX], to_path[PATH_MAX];
 	char *from_fingerprint, *to_fingerprint;
@@ -874,7 +876,7 @@ compare_move(FileView *from, FileView *to)
 
 	if(from->custom.type != CV_DIFF || !from->custom.diff_path_group)
 	{
-		status_bar_error("Not in diff mode with path grouping");
+		ui_sb_err("Not in diff mode with path grouping");
 		return 1;
 	}
 
