@@ -37,8 +37,9 @@ typedef var_t (*function_ptr_t)(const call_info_t *call_info);
 /* Function description. */
 typedef struct
 {
-	const char *name; /* Name of a function. */
-	size_t arg_count; /* Required number of arguments. */
+	const char *name;   /* Name of a function. */
+	const char *descr;  /* Brief description. */
+	size_t arg_count;   /* Required number of arguments. */
 	function_ptr_t ptr; /* Pointer to function implementation. */
 }
 function_t;
@@ -46,6 +47,10 @@ function_t;
 /* Registers new function.  Returns non-zero on error. */
 int function_register(const function_t *func_info);
 
+/* Checks whether function with specified name exists or not.  Returns non-zero
+ * if function with specified name is already registered. */
+int function_registered(const char func_name[]);
+
 /* Calls function.  Returns its result or variable of type VTYPE_ERROR in case
  * of error. */
 var_t function_call(const char func_name[], const call_info_t *call_info);
@@ -69,4 +74,4 @@ void function_call_info_free(call_info_t *call_info);
 #endif /* VIFM__ENGINE__FUNCTIONS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
