@@ -178,9 +178,11 @@ size_t vle_keys_counter(void);
 int vle_keys_inside_mapping(void);
 
 /* Invokes cb for each possible keys continuation.  Intended to be used on
- * KEYS_WAIT and KEYS_WAIT_SHORT returns. */
+ * KEYS_WAIT and KEYS_WAIT_SHORT returns.  The custom_only flag limits
+ * suggestions to those generated by invoking key_conf_t::suggest.  The
+ * fold_subkeys enables folding of multiple keys with common prefix. */
 void vle_keys_suggest(const wchar_t keys[], vle_keys_list_cb cb,
-		int custom_only);
+		int custom_only, int fold_subkeys);
 
 #endif /* VIFM__ENGINE__KEYS_H__ */
 
