@@ -58,10 +58,10 @@
 static int ensure_term_is_ready(void);
 static int get_char_async_loop(WINDOW *win, wint_t *c, int timeout);
 static void process_scheduled_updates(void);
-TSTATIC int process_scheduled_updates_of_view(FileView *view);
+TSTATIC int process_scheduled_updates_of_view(view_t *view);
 static void update_hardware_cursor(void);
 static int should_check_views_for_changes(void);
-static void check_view_for_changes(FileView *view);
+static void check_view_for_changes(view_t *view);
 static void reset_input_buf(wchar_t curr_input_buf[],
 		size_t *curr_input_buf_pos);
 static void display_suggestion_box(const wchar_t input[]);
@@ -182,8 +182,7 @@ event_loop(const int *quit)
 
 			if(c == WC_C_z)
 			{
-				def_prog_mode();
-				endwin();
+				ui_shutdown();
 				stop_process();
 				continue;
 			}
@@ -277,7 +276,7 @@ event_loop(const int *quit)
 		reset_input_buf(input_buf, &input_buf_pos);
 		clear_input_bar();
 
-		if(is_status_bar_multiline())
+		if(ui_sb_multiline())
 		{
 			wait_for_enter = 1;
 			update_all_windows();
@@ -322,7 +321,7 @@ ensure_term_is_ready(void)
 		wtimeout(status_bar, cfg.timeout_len);
 
 		curr_stats.save_msg = 0;
-		status_bar_message("");
+		ui_sb_msg("");
 	}
 
 	return 1;
@@ -362,7 +361,7 @@ get_char_async_loop(WINDOW *win, wint_t *c, int timeout)
 		{
 			int result;
 
-			ipc_check();
+			ipc_check(curr_stats.ipc);
 			wtimeout(win, delay_slice);
 			timeout -= delay_slice;
 
@@ -404,24 +403,35 @@ process_scheduled_updates(void)
 
 	ui_stat_job_bar_check_for_updates();
 
+	if(vle_mode_is(CMDLINE_MODE))
+	{
+		/* Redrawing implicitly updates hardware cursor, so temporary disable it. */
+		curs_set(0);
+	}
+
 	if(vle_mode_get_primary() != MENU_MODE)
 	{
 		need_redraw += (process_scheduled_updates_of_view(curr_view) != 0);
 		need_redraw += (process_scheduled_updates_of_view(other_view) != 0);
 	}
 
-	need_redraw += (fetch_redraw_scheduled() != 0);
+	need_redraw += (stats_redraw_fetch() != 0);
 
 	if(need_redraw)
 	{
 		modes_redraw();
 	}
+
+	if(vle_mode_is(CMDLINE_MODE))
+	{
+		curs_set(1);
+	}
 }
 
 /* Performs postponed updates for the view, if any.  Returns non-zero if
  * something was indeed updated, and zero otherwise. */
 TSTATIC int
-process_scheduled_updates_of_view(FileView *view)
+process_scheduled_updates_of_view(view_t *view)
 {
 	if(!window_shows_dirlist(view))
 	{
@@ -434,23 +444,15 @@ process_scheduled_updates_of_view(FileView *view)
 			/* Nothing to do. */
 			return 0;
 		case UUE_REDRAW:
-			redraw_view_imm(view);
+			redraw_view(view);
 			return 1;
 		case UUE_RELOAD:
-			load_saving_pos(view, 1);
-			if(view == curr_view && !is_status_bar_multiline())
-			{
-				ui_ruler_update(view, 1);
-			}
+			load_saving_pos(view);
 			return 1;
-		case UUE_FULL_RELOAD:
-			load_saving_pos(view, 0);
-			return 1;
-
-		default:
-			assert(0 && "Unexpected type of scheduled UI event.");
-			return 0;
 	}
+
+	assert(0 && "Unexpected type of scheduled UI event.");
+	return 0;
 }
 
 /* Updates hardware cursor to be on currently active area of the interface,
@@ -458,7 +460,7 @@ process_scheduled_updates_of_view(FileView *view)
 static void
 update_hardware_cursor(void)
 {
-	if(is_status_bar_multiline())
+	if(ui_sb_multiline())
 	{
 		checked_wmove(status_bar, 0, 0);
 		wrefresh(status_bar);
@@ -491,18 +493,18 @@ update_hardware_cursor(void)
 static int
 should_check_views_for_changes(void)
 {
-	return !is_status_bar_multiline()
+	return !ui_sb_multiline()
 	    && !is_in_menu_like_mode()
 	    && NONE(vle_mode_is, CMDLINE_MODE, MSG_MODE);
 }
 
 /* Updates view in case directory it displays was changed externally. */
 static void
-check_view_for_changes(FileView *view)
+check_view_for_changes(view_t *view)
 {
 	if(window_shows_dirlist(view))
 	{
-		check_if_filelist_have_changed(view);
+		check_if_filelist_has_changed(view);
 	}
 }
 
@@ -545,7 +547,8 @@ display_suggestion_box(const wchar_t input[])
 
 	/* Fill completion list with suggestions of keys and marks. */
 	vle_compl_reset();
-	vle_keys_suggest(input, &process_suggestion, !(cfg.sug.flags & SF_KEYS));
+	vle_keys_suggest(input, &process_suggestion, !(cfg.sug.flags & SF_KEYS),
+				(cfg.sug.flags & SF_FOLDSUBKEYS));
 	/* Completion grouping removes duplicates.  Because user-defined keys are
 	 * reported first, this has an effect of leaving only them in the resulting
 	 * list, which is correct as they have higher priority. */
