@@ -20,11 +20,9 @@
 #ifndef VIFM__FILETYPE_H__
 #define VIFM__FILETYPE_H__
 
-#include "utils/test_helpers.h"
-
 #define VIFM_PSEUDO_CMD "vifm"
 
-struct matcher_t;
+struct matchers_t;
 
 /* Type of file association by it's source. */
 typedef enum
@@ -51,10 +49,11 @@ typedef struct
 }
 assoc_records_t;
 
+/* Single file association entry. */
 typedef struct
 {
-	struct matcher_t *matcher;
-	assoc_records_t records;
+	struct matchers_t *matchers; /* Matching mechanism. */
+	assoc_records_t records;     /* Associated programs. */
 }
 assoc_t;
 
@@ -96,8 +95,8 @@ assoc_records_t ft_get_all_programs(const char file[]);
 
 /* Associates list of comma separated patterns with each item in the list of
  * comma separated programs either for X or non-X associations and depending on
- * current execution environment. */
-void ft_set_programs(struct matcher_t *matcher, const char programs[],
+ * current execution environment.  Takes over ownership of the matchers. */
+void ft_set_programs(struct matchers_t *matchers, const char programs[],
 		int for_x, int in_x);
 
 /* Viewers. */
@@ -112,10 +111,15 @@ assoc_records_t ft_get_all_viewers(const char file[]);
 
 /* Associates list of comma separated patterns with each item in the list of
  * comma separated viewers. */
-void ft_set_viewers(struct matcher_t *matcher, const char viewers[]);
+void ft_set_viewers(struct matchers_t *matchers, const char viewers[]);
 
 /* Records managing. */
 
+/* Checks that given pair of pattern and command exists in specified list of
+ * associations.  Returns non-zero if so, otherwise zero is returned. */
+int ft_assoc_exists(const assoc_list_t *assocs, const char pattern[],
+		const char cmd[]);
+
 void ft_assoc_record_add(assoc_records_t *assocs, const char *command,
 		const char *description);
 
@@ -125,11 +129,7 @@ void ft_assoc_record_add_all(assoc_records_t *assocs,
 /* After this call the structure contains NULL values. */
 void ft_assoc_records_free(assoc_records_t *records);
 
-TSTATIC_DEFS(
-	void replace_double_comma(char cmd[], int put_null);
-)
-
 #endif /* VIFM__FILETYPE_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
