@@ -26,6 +26,7 @@
 #include "compat/reallocarray.h"
 #include "ui/ui.h"
 #include "utils/dynarray.h"
+#include "utils/matcher.h"
 #include "utils/path.h"
 #include "utils/regexp.h"
 #include "utils/str.h"
@@ -36,32 +37,31 @@
 #include "opt_handlers.h"
 
 static void reset_filter(filter_t *filter);
-static int is_newly_filtered(FileView *view, const dir_entry_t *entry,
+static int is_newly_filtered(view_t *view, const dir_entry_t *entry,
 		void *arg);
-static int file_is_filtered(FileView *view, const char filename[], int is_dir,
-		int apply_local_filter);
-static int get_unfiltered_pos(const FileView *const view, int pos);
-static int load_unfiltered_list(FileView *const view);
-static int list_is_incomplete(FileView *const view);
-static void store_local_filter_position(FileView *const view, int pos);
-static int update_filtering_lists(FileView *view, int add, int clear);
+static void replace_matcher(matcher_t **matcher, const char expr[]);
+static int get_unfiltered_pos(const view_t *const view, int pos);
+static int load_unfiltered_list(view_t *const view);
+static int list_is_incomplete(view_t *const view);
+static void store_local_filter_position(view_t *const view, int pos);
+static int update_filtering_lists(view_t *view, int add, int clear);
 static void reparent_tree_node(dir_entry_t *original, dir_entry_t *filtered);
-static void ensure_filtered_list_not_empty(FileView *view,
+static void ensure_filtered_list_not_empty(view_t *view,
 		dir_entry_t *parent_entry);
-static int extract_previously_selected_pos(FileView *const view);
-static void clear_local_filter_hist_after(FileView *const view, int pos);
-static int find_nearest_neighour(const FileView *const view);
-static void local_filter_finish(FileView *view);
+static int extract_previously_selected_pos(view_t *const view);
+static void clear_local_filter_hist_after(view_t *const view, int pos);
+static int find_nearest_neighour(const view_t *const view);
+static void local_filter_finish(view_t *view);
 static void append_slash(const char name[], char buf[], size_t buf_size);
 
 void
-filters_view_reset(FileView *view)
+filters_view_reset(view_t *view)
 {
 	view->invert = cfg.filter_inverted_by_default ? 1 : 0;
 	view->prev_invert = view->invert;
 
 	(void)replace_string(&view->prev_manual_filter, "");
-	reset_filter(&view->manual_filter);
+	replace_matcher(&view->manual_filter, "");
 	(void)replace_string(&view->prev_auto_filter, "");
 	reset_filter(&view->auto_filter);
 
@@ -88,7 +88,7 @@ reset_filter(filter_t *filter)
 }
 
 void
-set_dot_files_visible(FileView *view, int visible)
+dot_filter_set(view_t *view, int visible)
 {
 	view->hide_dot_g = view->hide_dot = !visible;
 	ui_view_schedule_reload(view);
@@ -98,7 +98,7 @@ set_dot_files_visible(FileView *view, int visible)
 	}
 	if(curr_stats.global_local_settings)
 	{
-		FileView *other = (view == curr_view) ? other_view : curr_view;
+		view_t *other = (view == curr_view) ? other_view : curr_view;
 		other->hide_dot_g = other->hide_dot = !visible;
 		ui_view_schedule_reload(other);
 		if(other == curr_view)
@@ -109,7 +109,7 @@ set_dot_files_visible(FileView *view, int visible)
 }
 
 void
-toggle_dot_files(FileView *view)
+dot_filter_toggle(view_t *view)
 {
 	view->hide_dot_g = view->hide_dot = !view->hide_dot;
 	ui_view_schedule_reload(view);
@@ -119,7 +119,7 @@ toggle_dot_files(FileView *view)
 	}
 	if(curr_stats.global_local_settings)
 	{
-		FileView *other = (view == curr_view) ? other_view : curr_view;
+		view_t *other = (view == curr_view) ? other_view : curr_view;
 		other->hide_dot_g = other->hide_dot = !other->hide_dot;
 		ui_view_schedule_reload(other);
 		if(other == curr_view)
@@ -130,7 +130,7 @@ toggle_dot_files(FileView *view)
 }
 
 void
-filter_selected_files(FileView *view)
+name_filters_add_selection(view_t *view)
 {
 	dir_entry_t *entry;
 	filter_t filter;
@@ -194,12 +194,11 @@ filter_selected_files(FileView *view)
 /* zap_entries() filter to filter-out files that match filter passed in the
  * arg. */
 static int
-is_newly_filtered(FileView *view, const dir_entry_t *entry, void *arg)
+is_newly_filtered(view_t *view, const dir_entry_t *entry, void *arg)
 {
 	filter_t *const filter = arg;
 
-	/* FIXME: some very long file names won't be matched against some
-	 * regexps. */
+	/* FIXME: some very long file names won't be matched against some regexps. */
 	char name_with_slash[NAME_MAX + 1 + 1];
 	const char *filename = entry->name;
 
@@ -213,54 +212,68 @@ is_newly_filtered(FileView *view, const dir_entry_t *entry, void *arg)
 }
 
 void
-remove_filename_filter(FileView *view)
+name_filters_remove(view_t *view)
 {
-	if(filename_filter_is_empty(view))
+	if(name_filters_empty(view))
 	{
 		return;
 	}
 
-	(void)replace_string(&view->prev_manual_filter, view->manual_filter.raw);
+	(void)replace_string(&view->prev_manual_filter,
+			matcher_get_expr(view->manual_filter));
 	(void)replace_string(&view->prev_auto_filter, view->auto_filter.raw);
 	view->prev_invert = view->invert;
 
-	filename_filter_clear(view);
+	name_filters_drop(view);
 	view->invert = cfg.filter_inverted_by_default ? 1 : 0;
 
-	ui_view_schedule_full_reload(view);
+	ui_view_schedule_reload(view);
 }
 
 int
-filename_filter_is_empty(FileView *view)
+name_filters_empty(view_t *view)
 {
-	return filter_is_empty(&view->manual_filter)
+	return matcher_is_empty(view->manual_filter)
 	    && filter_is_empty(&view->auto_filter);
 }
 
 void
-filename_filter_clear(FileView *view)
+name_filters_drop(view_t *view)
 {
 	filter_clear(&view->auto_filter);
-	filter_clear(&view->manual_filter);
+	replace_matcher(&view->manual_filter, "");
 	view->invert = 1;
 }
 
 void
-restore_filename_filter(FileView *view)
+name_filters_restore(view_t *view)
 {
 	if(view->prev_manual_filter[0] == '\0' && view->prev_auto_filter[0] == '\0')
 	{
 		return;
 	}
 
-	(void)filter_set(&view->manual_filter, view->prev_manual_filter);
+	replace_matcher(&view->manual_filter, view->prev_manual_filter);
+
 	(void)filter_set(&view->auto_filter, view->prev_auto_filter);
 	view->invert = view->prev_invert;
 	ui_view_schedule_reload(view);
 }
 
+/* Changes *matcher to have the value of the expr.  The operation is assumed to
+ * succeed, but it's not guaranteed. */
+static void
+replace_matcher(matcher_t **matcher, const char expr[])
+{
+	char *error;
+
+	matcher_free(*matcher);
+	*matcher = matcher_alloc(expr, FILTER_DEF_CASE_SENSITIVITY, 0, "", &error);
+	free(error);
+}
+
 void
-toggle_filter_inversion(FileView *view)
+filters_invert(view_t *view)
 {
 	view->invert = !view->invert;
 	load_dir_list(view, 1);
@@ -268,66 +281,56 @@ toggle_filter_inversion(FileView *view)
 }
 
 int
-filters_file_is_visible(FileView *view, const char filename[], int is_dir)
-{
-	return file_is_filtered(view, filename, is_dir, 1);
-}
-
-int
-filters_file_is_filtered(FileView *view, const char filename[], int is_dir)
-{
-	return file_is_filtered(view, filename, is_dir, 0);
-}
-
-/* Checks whether file/directory passes filename filters of the view.  Returns
- * non-zero if given filename passes filter and should be visible, otherwise
- * zero is returned, in which case the file should be hidden. */
-static int
-file_is_filtered(FileView *view, const char filename[], int is_dir,
-		int apply_local_filter)
+filters_file_is_visible(view_t *view, const char dir[], const char name[],
+		int is_dir, int apply_local_filter)
 {
 	/* FIXME: some very long file names won't be matched against some regexps. */
 	char name_with_slash[NAME_MAX + 1 + 1];
+	char path[PATH_MAX + sizeof(name_with_slash)];
+
 	if(is_dir)
 	{
-		append_slash(filename, name_with_slash, sizeof(name_with_slash));
-		filename = name_with_slash;
+		append_slash(name, name_with_slash, sizeof(name_with_slash));
+		name = name_with_slash;
 	}
 
-	if(filter_matches(&view->auto_filter, filename) > 0)
+	if(filter_matches(&view->auto_filter, name) > 0)
 	{
 		return 0;
 	}
 
 	if(apply_local_filter &&
-			filter_matches(&view->local_filter.filter, filename) == 0)
+			filter_matches(&view->local_filter.filter, name) == 0)
 	{
 		return 0;
 	}
 
-	if(filter_is_empty(&view->manual_filter))
+	if(matcher_is_empty(view->manual_filter))
 	{
 		return 1;
 	}
 
-	if(filter_matches(&view->manual_filter, filename) > 0)
+	if(matcher_is_full_path(view->manual_filter))
 	{
-		return !view->invert;
-	}
-	else
-	{
-		return view->invert;
+		const size_t nchars = copy_str(path, sizeof(path) - 1, dir);
+		path[nchars - 1U] = '/';
+		copy_str(path + nchars, sizeof(path) - nchars, name);
+		name = path;
 	}
+
+	return matcher_matches(view->manual_filter, name)
+	     ? !view->invert
+	     : view->invert;
 }
 
 void
-filters_dir_updated(FileView *view)
+filters_dir_updated(view_t *view)
 {
 	filter_clear(&view->local_filter.filter);
 }
 
 void
-filter_temporary_nodes(FileView *view, dir_entry_t *list)
+filters_drop_temporaries(view_t *view, dir_entry_t entries[])
 {
 	/* This is basically a simplified version of update_filtering_lists().  Not
 	 * sure if it's worth merging them. */
@@ -340,8 +343,8 @@ filter_temporary_nodes(FileView *view, dir_entry_t *list)
 		dir_entry_t *new_entry;
 		dir_entry_t *const entry = &view->dir_entry[i];
 
-		/* tag links to position of nodes passed through filter in list of
-		 * visible files.  Removed nodes have -1. */
+		/* The tag field links to position of nodes passed through filter in the
+		 * list of visible files.  Removed nodes have -1. */
 		entry->tag = -1;
 
 		if(entry->temporary)
@@ -350,7 +353,7 @@ filter_temporary_nodes(FileView *view, dir_entry_t *list)
 			continue;
 		}
 
-		new_entry = add_dir_entry(&list, &list_size, entry);
+		new_entry = add_dir_entry(&entries, &list_size, entry);
 		if(new_entry != NULL)
 		{
 			entry->tag = list_size - 1U;
@@ -361,12 +364,12 @@ filter_temporary_nodes(FileView *view, dir_entry_t *list)
 	}
 
 	dynarray_free(view->dir_entry);
-	view->dir_entry = list;
+	view->dir_entry = entries;
 	view->list_rows = list_size;
 }
 
 int
-local_filter_set(FileView *view, const char filter[])
+local_filter_set(view_t *view, const char filter[])
 {
 	int result;
 	const int current_file_pos = view->local_filter.in_progress
@@ -391,7 +394,7 @@ local_filter_set(FileView *view, const char filter[])
 /* Gets position of an item in dir_entry list at position pos in the unfiltered
  * list.  Returns index on success, otherwise -1 is returned. */
 static int
-get_unfiltered_pos(const FileView *const view, int pos)
+get_unfiltered_pos(const view_t *const view, int pos)
 {
 	const int count = (int)view->local_filter.unfiltered_count;
 	const char *const filename = view->dir_entry[pos].name;
@@ -410,7 +413,7 @@ get_unfiltered_pos(const FileView *const view, int pos)
 /* Loads full list of files into unfiltered list of the view.  Returns position
  * of file under cursor in the unfiltered list. */
 static int
-load_unfiltered_list(FileView *const view)
+load_unfiltered_list(view_t *const view)
 {
 	int current_file_pos = view->list_pos;
 
@@ -426,7 +429,7 @@ load_unfiltered_list(FileView *const view)
 		get_current_full_path(view, sizeof(full_path), full_path);
 
 		filter_clear(&view->local_filter.filter);
-		populate_dir_list(view, 1);
+		(void)populate_dir_list(view, 1);
 
 		/* Resolve current file position in updated list. */
 		entry = entry_from_path(view, view->dir_entry, view->list_rows, full_path);
@@ -458,7 +461,7 @@ load_unfiltered_list(FileView *const view)
 /* Checks whether list of files is incomplete.  Returns non-zero if so,
  * otherwise zero is returned. */
 static int
-list_is_incomplete(FileView *const view)
+list_is_incomplete(view_t *const view)
 {
 	int i;
 
@@ -472,7 +475,7 @@ list_is_incomplete(FileView *const view)
 		return 0;
 	}
 
-	/* Check if there is any directories without leaf nodes.  They aren't counted
+	/* Check if there are any directories without leaf nodes.  They aren't counted
 	 * as filtered out, so need to check separately (or start counting them in
 	 * some way). */
 	for(i = 0; i < view->list_rows; ++i)
@@ -490,7 +493,7 @@ list_is_incomplete(FileView *const view)
 
 /* Adds local filter position (in unfiltered list) to position history. */
 static void
-store_local_filter_position(FileView *const view, int pos)
+store_local_filter_position(view_t *const view, int pos)
 {
 	size_t *const len = &view->local_filter.poshist_len;
 	int *arr = reallocarray(view->local_filter.poshist, *len + 1, sizeof(int));
@@ -508,9 +511,9 @@ store_local_filter_position(FileView *const view, int pos)
  * cleared in unfiltered list.  Returns zero unless addition is performed in
  * which case can return non-zero when all files got filtered out. */
 static int
-update_filtering_lists(FileView *view, int add, int clear)
+update_filtering_lists(view_t *view, int add, int clear)
 {
-	/* filter_temporary_nodes() is similar function. */
+	/* filters_drop_temporaries() is a similar function. */
 
 	size_t i;
 	size_t list_size = 0U;
@@ -643,7 +646,7 @@ reparent_tree_node(dir_entry_t *original, dir_entry_t *filtered)
 /* Use parent_entry to make filtered list not empty, or create such entry (if
  * parent_entry is NULL) and put it to original list. */
 static void
-ensure_filtered_list_not_empty(FileView *view, dir_entry_t *parent_entry)
+ensure_filtered_list_not_empty(view_t *view, dir_entry_t *parent_entry)
 {
 	if(view->list_rows != 0U)
 	{
@@ -669,7 +672,7 @@ ensure_filtered_list_not_empty(FileView *view, dir_entry_t *parent_entry)
 }
 
 void
-local_filter_update_view(FileView *view, int rel_pos)
+local_filter_update_view(view_t *view, int rel_pos)
 {
 	int pos = extract_previously_selected_pos(view);
 
@@ -694,7 +697,7 @@ local_filter_update_view(FileView *view, int rel_pos)
 /* Finds one of previously selected files and updates list of visited files
  * if needed.  Returns file position in current list of files or -1. */
 static int
-extract_previously_selected_pos(FileView *const view)
+extract_previously_selected_pos(view_t *const view)
 {
 	size_t i;
 	for(i = 0; i < view->local_filter.poshist_len; i++)
@@ -714,7 +717,7 @@ extract_previously_selected_pos(FileView *const view)
 
 /* Clears all positions after the pos. */
 static void
-clear_local_filter_hist_after(FileView *const view, int pos)
+clear_local_filter_hist_after(view_t *const view, int pos)
 {
 	view->local_filter.poshist_len -= view->local_filter.poshist_len - (pos + 1);
 }
@@ -722,7 +725,7 @@ clear_local_filter_hist_after(FileView *const view, int pos)
 /* Find nearest filtered neighbour.  Returns index of nearest unfiltered
  * neighbour of the entry initially pointed to by cursor. */
 static int
-find_nearest_neighour(const FileView *const view)
+find_nearest_neighour(const view_t *const view)
 {
 	const int count = view->local_filter.unfiltered_count;
 
@@ -746,7 +749,7 @@ find_nearest_neighour(const FileView *const view)
 }
 
 void
-local_filter_accept(FileView *view)
+local_filter_accept(view_t *view)
 {
 	if(!view->local_filter.in_progress)
 	{
@@ -757,7 +760,7 @@ local_filter_accept(FileView *view)
 
 	local_filter_finish(view);
 
-	cfg_save_filter_history(view->local_filter.filter.raw);
+	hists_filter_save(view->local_filter.filter.raw);
 
 	/* Some of previously selected files could be filtered out, update number of
 	 * selected files. */
@@ -765,7 +768,7 @@ local_filter_accept(FileView *view)
 }
 
 void
-local_filter_apply(FileView *view, const char filter[])
+local_filter_apply(view_t *view, const char filter[])
 {
 	if(view->local_filter.in_progress)
 	{
@@ -774,7 +777,7 @@ local_filter_apply(FileView *view, const char filter[])
 	}
 
 	(void)filter_set(&view->local_filter.filter, filter);
-	cfg_save_filter_history(view->local_filter.filter.raw);
+	hists_filter_save(view->local_filter.filter.raw);
 
 	if(flist_custom_active(view) && view->custom.type != CV_TREE &&
 			view->local_filter.entry_count == 0)
@@ -786,7 +789,7 @@ local_filter_apply(FileView *view, const char filter[])
 }
 
 void
-local_filter_cancel(FileView *view)
+local_filter_cancel(view_t *view)
 {
 	if(!view->local_filter.in_progress)
 	{
@@ -805,7 +808,7 @@ local_filter_cancel(FileView *view)
 
 /* Finishes filtering process and frees associated resources. */
 static void
-local_filter_finish(FileView *view)
+local_filter_finish(view_t *view)
 {
 	dynarray_free(view->local_filter.unfiltered);
 	free(view->local_filter.saved);
@@ -817,7 +820,7 @@ local_filter_finish(FileView *view)
 }
 
 void
-local_filter_remove(FileView *view)
+local_filter_remove(view_t *view)
 {
 	(void)replace_string(&view->local_filter.prev, view->local_filter.filter.raw);
 	filter_clear(&view->local_filter.filter);
@@ -825,17 +828,16 @@ local_filter_remove(FileView *view)
 }
 
 void
-local_filter_restore(FileView *view)
+local_filter_restore(view_t *view)
 {
 	(void)filter_set(&view->local_filter.filter, view->local_filter.prev);
 	(void)replace_string(&view->local_filter.prev, "");
 }
 
 int
-local_filter_matches(FileView *view, const dir_entry_t *entry)
+local_filter_matches(view_t *view, const dir_entry_t *entry)
 {
-	/* FIXME: some very long file names won't be matched against some
-	 * regexps. */
+	/* FIXME: some very long file names won't be matched against some regexps. */
 	char name_with_slash[NAME_MAX + 1 + 1];
 	const char *filename = entry->name;
 	if(fentry_is_dir(entry))
