@@ -38,23 +38,27 @@
 #include "filtering.h"
 #include "types.h"
 
-static void correct_list_pos_down(FileView *view, size_t pos_delta);
-static void correct_list_pos_up(FileView *view, size_t pos_delta);
-static void move_cursor_out_of_scope(FileView *view, entry_predicate pred);
+static void move_cursor_out_of_scope(view_t *view, entry_predicate pred);
+static int get_curr_col(const view_t *view);
+static int get_curr_line(const view_t *view);
+static int get_max_col(const view_t *view);
+static int get_max_line(const view_t *view);
+static int get_column_top_pos(const view_t *view);
+static int get_column_bottom_pos(const view_t *view);
 static const char * get_last_ext(const char name[]);
 static int is_mismatched_entry(const dir_entry_t *entry);
-static int find_next(const FileView *view, entry_predicate pred);
-static int find_prev(const FileView *view, entry_predicate pred);
+static int find_next(const view_t *view, entry_predicate pred);
+static int find_prev(const view_t *view, entry_predicate pred);
 static int file_can_be_displayed(const char directory[], const char filename[]);
 
 int
-find_file_pos_in_list(const FileView *const view, const char file[])
+find_file_pos_in_list(const view_t *const view, const char file[])
 {
 	return flist_find_entry(view, file, NULL);
 }
 
 int
-flist_find_entry(const FileView *view, const char file[], const char dir[])
+flist_find_entry(const view_t *view, const char file[], const char dir[])
 {
 	int i;
 	for(i = 0; i < view->list_rows; ++i)
@@ -72,57 +76,32 @@ flist_find_entry(const FileView *view, const char file[], const char dir[])
 	return -1;
 }
 
-void
-correct_list_pos(FileView *view, ssize_t pos_delta)
-{
-	if(pos_delta > 0)
-	{
-		correct_list_pos_down(view, pos_delta);
-	}
-	else if(pos_delta < 0)
-	{
-		correct_list_pos_up(view, -pos_delta);
-	}
-}
-
 int
-correct_list_pos_on_scroll_down(FileView *view, size_t lines_count)
+correct_list_pos_on_scroll_down(view_t *view, int lines_count)
 {
-	if(!all_files_visible(view))
+	if(!fpos_are_all_files_visible(view))
 	{
-		correct_list_pos_down(view, lines_count*view->column_count);
+		view->list_pos =
+			get_corrected_list_pos_down(view, lines_count*view->run_size);
 		return 1;
 	}
 	return 0;
 }
 
 int
-correct_list_pos_on_scroll_up(FileView *view, size_t lines_count)
+correct_list_pos_on_scroll_up(view_t *view, int lines_count)
 {
-	if(!all_files_visible(view))
+	if(!fpos_are_all_files_visible(view))
 	{
-		correct_list_pos_up(view, lines_count*view->column_count);
+		view->list_pos =
+			get_corrected_list_pos_up(view, lines_count*view->run_size);
 		return 1;
 	}
 	return 0;
 }
 
-/* Tries to move cursor forward by pos_delta positions. */
-static void
-correct_list_pos_down(FileView *view, size_t pos_delta)
-{
-	view->list_pos = get_corrected_list_pos_down(view, pos_delta);
-}
-
-/* Tries to move cursor backwards by pos_delta positions. */
-static void
-correct_list_pos_up(FileView *view, size_t pos_delta)
-{
-	view->list_pos = get_corrected_list_pos_up(view, pos_delta);
-}
-
 void
-flist_set_pos(FileView *view, int pos)
+flist_set_pos(view_t *view, int pos)
 {
 	if(pos < 1)
 	{
@@ -136,7 +115,7 @@ flist_set_pos(FileView *view, int pos)
 
 	if(pos != -1)
 	{
-		FileView *const other = (view == curr_view) ? other_view : curr_view;
+		view_t *const other = (view == curr_view) ? other_view : curr_view;
 
 		view->list_pos = pos;
 		fview_position_updated(view);
@@ -150,7 +129,7 @@ flist_set_pos(FileView *view, int pos)
 }
 
 void
-flist_ensure_pos_is_valid(FileView *view)
+flist_ensure_pos_is_valid(view_t *view)
 {
 	if(view->list_pos >= view->list_rows)
 	{
@@ -159,7 +138,7 @@ flist_ensure_pos_is_valid(FileView *view)
 }
 
 void
-move_cursor_out_of(FileView *view, FileListScope scope)
+move_cursor_out_of(view_t *view, FileListScope scope)
 {
 	/* XXX: this functionality might be unnecessary now that we have directory
 	 *      merging. */
@@ -178,7 +157,7 @@ move_cursor_out_of(FileView *view, FileListScope scope)
 /* Ensures that cursor is moved outside of entries that satisfy the predicate if
  * that's possible. */
 static void
-move_cursor_out_of_scope(FileView *view, entry_predicate pred)
+move_cursor_out_of_scope(view_t *view, entry_predicate pred)
 {
 	/* TODO: if we reach bottom of the list and predicate holds try scanning to
 	 * the top. */
@@ -191,53 +170,247 @@ move_cursor_out_of_scope(FileView *view, entry_predicate pred)
 }
 
 int
-at_first_line(const FileView *view)
+fpos_get_col(const view_t *view, int pos)
 {
-	return view->list_pos/view->column_count == 0;
+	return (fview_is_transposed(view) ? pos/view->run_size : pos%view->run_size);
 }
 
 int
-at_last_line(const FileView *view)
+fpos_get_line(const view_t *view, int pos)
 {
-	const size_t col_count = view->column_count;
-	return view->list_pos/col_count == (view->list_rows - 1)/col_count;
+	return (fview_is_transposed(view) ? pos%view->run_size : pos/view->run_size);
 }
 
 int
-at_first_column(const FileView *view)
+fpos_can_move_left(const view_t *view)
 {
-	return view->list_pos%view->column_count == 0;
+	return (view->list_pos > 0);
 }
 
 int
-at_last_column(const FileView *view)
+fpos_can_move_right(const view_t *view)
 {
-	return view->list_pos%view->column_count == view->column_count - 1;
-}
-
-void
-go_to_start_of_line(FileView *view)
-{
-	view->list_pos = get_start_of_line(view);
+	return (view->list_pos < view->list_rows - 1);
 }
 
 int
-get_start_of_line(const FileView *view)
+fpos_can_move_up(const view_t *view)
 {
-	const int pos = MAX(MIN(view->list_pos, view->list_rows - 1), 0);
-	return ROUND_DOWN(pos, view->column_count);
+	return fview_is_transposed(view) ? view->list_pos > 0
+	                                 : view->list_pos >= view->run_size;
 }
 
 int
-get_end_of_line(const FileView *view)
+fpos_can_move_down(const view_t *view)
 {
-	int pos = MAX(MIN(view->list_pos, view->list_rows - 1), 0);
-	pos += (view->column_count - 1) - pos%view->column_count;
-	return MIN(pos, view->list_rows - 1);
+	return fview_is_transposed(view) ? view->list_pos < view->list_rows - 1
+	                                 : get_curr_line(view) < get_max_line(view);
 }
 
 int
-flist_find_group(const FileView *view, int next)
+at_first_column(const view_t *view)
+{
+	return (get_curr_col(view) == 0);
+}
+
+int
+at_last_column(const view_t *view)
+{
+	return get_curr_col(view) == get_max_col(view);
+}
+
+/* Retrieves column number of cursor.  Returns the number. */
+static int
+get_curr_col(const view_t *view)
+{
+	return fpos_get_col(view, view->list_pos);
+}
+
+/* Retrieves line number of cursor.  Returns the number. */
+static int
+get_curr_line(const view_t *view)
+{
+	return fpos_get_line(view, view->list_pos);
+}
+
+/* Retrieves maximum column number.  Returns the number. */
+static int
+get_max_col(const view_t *view)
+{
+	return fview_is_transposed(view) ? (view->list_rows - 1)/view->run_size
+	                                 : (view->run_size - 1);
+}
+
+/* Retrieves maximum line number.  Returns the number. */
+static int
+get_max_line(const view_t *view)
+{
+	return fview_is_transposed(view) ? (view->run_size - 1)
+	                                 : (view->list_rows - 1)/view->run_size;
+}
+
+int
+get_start_of_line(const view_t *view)
+{
+	return fview_is_transposed(view) ? view->list_pos%view->run_size
+	                                 : ROUND_DOWN(view->list_pos, view->run_size);
+}
+
+int
+get_end_of_line(const view_t *view)
+{
+	if(fview_is_transposed(view))
+	{
+		const int last_top_pos = ROUND_DOWN(view->list_rows - 1, view->run_size);
+		const int pos = last_top_pos + view->list_pos%view->run_size;
+		return (pos < view->list_rows ? pos : pos - view->run_size);
+	}
+
+	return MIN(view->list_rows - 1, get_start_of_line(view) + view->run_size - 1);
+}
+
+int
+fpos_get_hor_step(const struct view_t *view)
+{
+	return (fview_is_transposed(view) ? view->run_size : 1);
+}
+
+int
+fpos_get_ver_step(const struct view_t *view)
+{
+	return (fview_is_transposed(view) ? 1 : view->run_size);
+}
+
+int
+fpos_has_hidden_top(const view_t *view)
+{
+	return (fview_is_transposed(view) ? 0 : can_scroll_up(view));
+}
+
+int
+fpos_has_hidden_bottom(const view_t *view)
+{
+	return (fview_is_transposed(view) ? 0 : can_scroll_down(view));
+}
+
+int
+fpos_get_top_pos(const view_t *view)
+{
+	return get_column_top_pos(view)
+	     + (can_scroll_up(view) ? fpos_get_offset(view) : 0);
+}
+
+int
+fpos_get_middle_pos(const view_t *view)
+{
+	const int top_pos = get_column_top_pos(view);
+	const int bottom_pos = get_column_bottom_pos(view);
+	const int v = (fview_is_transposed(view) ? 1 : view->run_size);
+	return top_pos + (DIV_ROUND_UP(bottom_pos - top_pos, v)/2)*v;
+}
+
+int
+fpos_get_bottom_pos(const view_t *view)
+{
+	return get_column_bottom_pos(view)
+	     - (can_scroll_down(view) ? fpos_get_offset(view) : 0);
+}
+
+int
+fpos_get_offset(const view_t *view)
+{
+	int val;
+
+	if(fview_is_transposed(view))
+	{
+		/* Scroll offset doesn't make much sense for transposed table. */
+		return 0;
+	}
+
+	val = MIN(DIV_ROUND_UP(view->window_rows - 1, 2), MAX(cfg.scroll_off, 0));
+	return val*view->column_count;
+}
+
+int
+fpos_are_all_files_visible(const view_t *view)
+{
+	return view->list_rows <= view->window_cells;
+}
+
+int
+fpos_get_last_visible_cell(const view_t *view)
+{
+	return view->top_line + view->window_cells - 1;
+}
+
+int
+fpos_half_scroll(view_t *view, int down)
+{
+	int new_pos;
+
+	int offset = MAX(view->window_cells/2, view->run_size);
+	offset = ROUND_DOWN(offset, curr_view->run_size);
+
+	if(down)
+	{
+		new_pos = get_corrected_list_pos_down(view, offset);
+		new_pos = MAX(new_pos, view->list_pos + offset);
+
+		if(new_pos >= view->list_rows)
+		{
+			new_pos -= view->column_count*
+				DIV_ROUND_UP(new_pos - (view->list_rows - 1), view->column_count);
+		}
+	}
+	else
+	{
+		new_pos = get_corrected_list_pos_up(view, offset);
+		new_pos = MIN(new_pos, view->list_pos - offset);
+
+		if(new_pos < 0)
+		{
+			new_pos += view->column_count*DIV_ROUND_UP(-new_pos, view->column_count);
+		}
+	}
+
+	scroll_by_files(view, new_pos - view->list_pos);
+	return new_pos;
+}
+
+/* Retrieves position of a file at the top of visible part of current column.
+ * Returns the position. */
+static int
+get_column_top_pos(const view_t *view)
+{
+	const int column_correction = fview_is_transposed(view)
+	  ? ROUND_DOWN(view->list_pos - view->top_line, view->run_size)
+	  : view->list_pos%view->run_size;
+	return view->top_line + column_correction;
+}
+
+/* Retrieves position of a file at the bottom of visible part of current column.
+ * Returns the position. */
+static int
+get_column_bottom_pos(const view_t *view)
+{
+	if(fview_is_transposed(view))
+	{
+		const int top_pos = get_column_top_pos(view);
+		const int last = view->list_rows - 1;
+		return MIN(top_pos + view->window_rows - 1, last);
+	}
+	else
+	{
+		const int last_top_pos =
+			ROUND_DOWN(MIN(fpos_get_last_visible_cell(view), view->list_rows - 1),
+					view->run_size);
+		const int pos = last_top_pos + view->list_pos%view->run_size;
+		return (pos < view->list_rows ? pos : pos - view->run_size);
+	}
+}
+
+int
+flist_find_group(const view_t *view, int next)
 {
 	/* TODO: refactor/simplify this function (flist_find_group()). */
 
@@ -344,7 +517,7 @@ flist_find_group(const FileView *view, int next)
 					return pos;
 				break;
 			case SK_BY_NITEMS:
-				if(entry_get_nitems(view, nentry) != entry_get_nitems(view, pentry))
+				if(fentry_get_nitems(view, nentry) != fentry_get_nitems(view, pentry))
 					return pos;
 				break;
 			case SK_BY_TIME_ACCESSED:
@@ -386,6 +559,10 @@ flist_find_group(const FileView *view, int next)
 				if(nentry->mode != pentry->mode)
 					return pos;
 				break;
+			case SK_BY_INODE:
+				if(nentry->inode != pentry->inode)
+					return pos;
+				break;
 			case SK_BY_PERMISSIONS:
 				{
 					char nperms[16];
@@ -428,7 +605,7 @@ get_last_ext(const char name[])
 }
 
 int
-flist_find_dir_group(const FileView *view, int next)
+flist_find_dir_group(const view_t *view, int next)
 {
 	const int correction = next ? -1 : 0;
 	const int lb = correction;
@@ -452,14 +629,14 @@ flist_find_dir_group(const FileView *view, int next)
 }
 
 int
-flist_first_sibling(const FileView *view)
+flist_first_sibling(const view_t *view)
 {
 	const int parent = view->list_pos - view->dir_entry[view->list_pos].child_pos;
 	return (parent == view->list_pos ? 0 : parent + 1);
 }
 
 int
-flist_last_sibling(const FileView *view)
+flist_last_sibling(const view_t *view)
 {
 	int pos = view->list_pos - view->dir_entry[view->list_pos].child_pos;
 	if(pos == view->list_pos)
@@ -486,7 +663,7 @@ flist_last_sibling(const FileView *view)
 }
 
 int
-flist_next_dir_sibling(const FileView *view)
+flist_next_dir_sibling(const view_t *view)
 {
 	int pos = view->list_pos;
 	const int parent = view->dir_entry[pos].child_pos == 0
@@ -510,7 +687,7 @@ flist_next_dir_sibling(const FileView *view)
 }
 
 int
-flist_prev_dir_sibling(const FileView *view)
+flist_prev_dir_sibling(const view_t *view)
 {
 	int pos = view->list_pos;
 	/* Determine original parent (-1 for top-most entry). */
@@ -541,31 +718,31 @@ flist_prev_dir_sibling(const FileView *view)
 }
 
 int
-flist_next_dir(const FileView *view)
+flist_next_dir(const view_t *view)
 {
 	return find_next(view, &fentry_is_dir);
 }
 
 int
-flist_prev_dir(const FileView *view)
+flist_prev_dir(const view_t *view)
 {
 	return find_prev(view, &fentry_is_dir);
 }
 
 int
-flist_next_selected(const FileView *view)
+flist_next_selected(const view_t *view)
 {
 	return find_next(view, &is_entry_selected);
 }
 
 int
-flist_prev_selected(const FileView *view)
+flist_prev_selected(const view_t *view)
 {
 	return find_prev(view, &is_entry_selected);
 }
 
 int
-flist_next_mismatch(const FileView *view)
+flist_next_mismatch(const view_t *view)
 {
 	return (view->custom.type == CV_DIFF)
 	     ? find_next(view, &is_mismatched_entry)
@@ -573,7 +750,7 @@ flist_next_mismatch(const FileView *view)
 }
 
 int
-flist_prev_mismatch(const FileView *view)
+flist_prev_mismatch(const view_t *view)
 {
 	return (view->custom.type == CV_DIFF)
 	     ? find_prev(view, &is_mismatched_entry)
@@ -588,7 +765,7 @@ is_mismatched_entry(const dir_entry_t *entry)
 	/* To avoid passing view pointer here, we exploit the fact that entry_to_pos()
 	 * checks whether it's argument belongs to the given view. */
 	int pos = entry_to_pos(&lwin, entry);
-	FileView *other = &rwin;
+	view_t *other = &rwin;
 	if(pos == -1)
 	{
 		pos = entry_to_pos(&rwin, entry);
@@ -601,7 +778,7 @@ is_mismatched_entry(const dir_entry_t *entry)
 /* Finds position of the next entry matching the predicate.  Returns new
  * position which isn't changed if no next directory is found. */
 static int
-find_next(const FileView *view, entry_predicate pred)
+find_next(const view_t *view, entry_predicate pred)
 {
 	int pos = view->list_pos;
 	while(++pos < view->list_rows)
@@ -617,7 +794,7 @@ find_next(const FileView *view, entry_predicate pred)
 /* Finds position of the previous entry matching the predicate.  Returns new
  * position which isn't changed if no previous directory is found. */
 static int
-find_prev(const FileView *view, entry_predicate pred)
+find_prev(const view_t *view, entry_predicate pred)
 {
 	int pos = view->list_pos;
 	while(--pos >= 0)
@@ -631,7 +808,7 @@ find_prev(const FileView *view, entry_predicate pred)
 }
 
 int
-ensure_file_is_selected(FileView *view, const char name[])
+ensure_file_is_selected(view_t *view, const char name[])
 {
 	int file_pos;
 	char nm[NAME_MAX + 1];
@@ -652,16 +829,16 @@ ensure_file_is_selected(FileView *view, const char name[])
 	{
 		if(nm[0] == '.')
 		{
-			set_dot_files_visible(view, 1);
+			dot_filter_set(view, 1);
 			file_pos = find_file_pos_in_list(view, nm);
 		}
 
 		if(file_pos < 0)
 		{
-			remove_filename_filter(view);
+			name_filters_remove(view);
 
-			/* remove_filename_filter() postpones list of files reloading. */
-			populate_dir_list(view, 1);
+			/* name_filters_remove() postpones reloading of list files. */
+			(void)populate_dir_list(view, 1);
 
 			file_pos = find_file_pos_in_list(view, nm);
 		}
@@ -684,7 +861,7 @@ file_can_be_displayed(const char directory[], const char filename[])
 }
 
 int
-flist_find_by_ch(const FileView *view, int ch, int backward, int wrap)
+flist_find_by_ch(const view_t *view, int ch, int backward, int wrap)
 {
 	int x;
 	const int upcase = (cfg.case_override & CO_GOTO_FILE)
