@@ -39,6 +39,7 @@
 #include <string.h> /* memcmp() strcmp() strdup() strlen() */
 
 #include "cfg/config.h"
+#include "compat/dtype.h"
 #include "compat/fs_limits.h"
 #include "compat/os.h"
 #include "int/vim.h"
@@ -142,7 +143,7 @@ io_progress_changed(const io_progress_t *const state)
 	/* Don't query for scheduled redraw or input for background operations. */
 	if(!pdata->bg)
 	{
-		redraw = fetch_redraw_scheduled();
+		redraw = stats_redraw_fetch();
 
 		if(!pdata->dialog)
 		{
@@ -436,14 +437,14 @@ fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
 
 	if(nlines < count)
 	{
-		status_bar_errorf("Not enough file names (%d/%d)", nlines, count);
+		ui_sb_errf("Not enough file names (%d/%d)", nlines, count);
 		curr_stats.save_msg = 1;
 		return 0;
 	}
 
 	if(nlines > count)
 	{
-		status_bar_errorf("Too many file names (%d/%d)", nlines, count);
+		ui_sb_errf("Too many file names (%d/%d)", nlines, count);
 		curr_stats.save_msg = 1;
 		return 0;
 	}
@@ -462,9 +463,9 @@ fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
 						strnoscmp(files[i], list[i], list_s - list[i]) != 0)
 				{
 					if(file_s == NULL)
-						status_bar_errorf("Name \"%s\" contains slash", list[i]);
+						ui_sb_errf("Name \"%s\" contains slash", list[i]);
 					else
-						status_bar_errorf("Won't move \"%s\" file", files[i]);
+						ui_sb_errf("Won't move \"%s\" file", files[i]);
 					curr_stats.save_msg = 1;
 					return 0;
 				}
@@ -473,7 +474,7 @@ fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
 
 		if(list[i][0] != '\0' && is_in_string_array(list, i, list[i]))
 		{
-			status_bar_errorf("Name \"%s\" duplicates", list[i]);
+			ui_sb_errf("Name \"%s\" duplicates", list[i]);
 			curr_stats.save_msg = 1;
 			return 0;
 		}
@@ -532,7 +533,7 @@ fops_check_file_rename(const char dir[], const char old[], const char new[],
 		switch(signal_type)
 		{
 			case ST_STATUS_BAR:
-				status_bar_errorf("File \"%s\" already exists", new);
+				ui_sb_errf("File \"%s\" already exists", new);
 				curr_stats.save_msg = 1;
 				break;
 			case ST_DIALOG:
@@ -563,7 +564,7 @@ is_file_name_changed(const char old[], const char new[])
 }
 
 char **
-fops_grab_marked_files(FileView *view, size_t *nmarked)
+fops_grab_marked_files(view_t *view, size_t *nmarked)
 {
 	char **marked = NULL;
 	dir_entry_t *entry = NULL;
@@ -576,13 +577,14 @@ fops_grab_marked_files(FileView *view, size_t *nmarked)
 }
 
 int
-fops_is_dir_entry(const char full_path[], const struct dirent* dentry)
+fops_is_dir_entry(const char full_path[], const struct dirent *dentry)
 {
 #ifndef _WIN32
 	struct stat s;
-	if(dentry->d_type != DT_UNKNOWN)
+	const int type = get_dirent_type(dentry, full_path);
+	if(type != DT_UNKNOWN)
 	{
-		return dentry->d_type == DT_DIR;
+		return type == DT_DIR;
 	}
 	if(os_lstat(full_path, &s) == 0 && s.st_ino != 0)
 	{
@@ -594,18 +596,8 @@ fops_is_dir_entry(const char full_path[], const struct dirent* dentry)
 #endif
 }
 
-void
-fops_fixup_entry_after_rename(FileView *view, dir_entry_t *entry,
-		const char new_fname[])
-{
-	if(entry_to_pos(view, entry) == view->list_pos || flist_custom_active(view))
-	{
-		fentry_rename(view, entry, new_fname);
-	}
-}
-
 int
-fops_enqueue_marked_files(ops_t *ops, FileView *view, const char dst_hint[],
+fops_enqueue_marked_files(ops_t *ops, view_t *view, const char dst_hint[],
 		int to_trash)
 {
 	int nmarked_files = 0;
@@ -683,7 +675,7 @@ fops_get_dst_name(const char src_path[], int from_trash)
 }
 
 int
-fops_can_read_selected_files(FileView *view)
+fops_can_read_marked_files(view_t *view)
 {
 	dir_entry_t *entry;
 
@@ -693,12 +685,14 @@ fops_can_read_selected_files(FileView *view)
 	}
 
 	entry = NULL;
-	while(iter_selected_entries(view, &entry))
+	while(iter_marked_entries(view, &entry))
 	{
-		char full_path[PATH_MAX];
-
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
-		if(os_access(full_path, R_OK) == 0)
+
+		/* We can copy links even when they are broken, so it's OK to don't check
+		 * them (otherwise access() fails for broken links). */
+		if(entry->type == FT_LINK || os_access(full_path, R_OK) == 0)
 		{
 			continue;
 		}
@@ -713,7 +707,7 @@ fops_can_read_selected_files(FileView *view)
 }
 
 int
-fops_check_dir_path(const FileView *view, const char path[], char buf[],
+fops_check_dir_path(const view_t *view, const char path[], char buf[],
 		size_t buf_len)
 {
 	if(path[0] == '/' || path[0] == '~')
@@ -910,7 +904,7 @@ fops_free_bg_args(bg_args_t *args)
 }
 
 void
-fops_prepare_for_bg_task(FileView *view, bg_args_t *args)
+fops_prepare_for_bg_task(view_t *view, bg_args_t *args)
 {
 	dir_entry_t *entry;
 
@@ -928,7 +922,7 @@ fops_prepare_for_bg_task(FileView *view, bg_args_t *args)
 }
 
 void
-fops_append_marked_files(FileView *view, char buf[], char **fnames)
+fops_append_marked_files(view_t *view, char buf[], char **fnames)
 {
 	const int custom_fnames = (fnames != NULL);
 	size_t len = strlen(buf);
@@ -968,7 +962,7 @@ fops_get_cancellation_suffix(void)
 }
 
 int
-fops_view_can_be_changed(const FileView *view)
+fops_view_can_be_changed(const view_t *view)
 {
 	/* TODO: maybe add check whether directory of specific entry is writable for
 	 *       custom views. */
@@ -977,9 +971,9 @@ fops_view_can_be_changed(const FileView *view)
 }
 
 int
-fops_view_can_be_extended(const FileView *view, int at)
+fops_view_can_be_extended(const view_t *view, int at)
 {
-	if(flist_custom_active(view) && view->custom.type != CV_TREE)
+	if(flist_custom_active(view) && !cv_tree(view->custom.type))
 	{
 		show_error_msg("Operation error",
 				"Custom view can't handle this operation.");
@@ -990,9 +984,9 @@ fops_view_can_be_extended(const FileView *view, int at)
 }
 
 const char *
-fops_get_dst_dir(const FileView *view, int at)
+fops_get_dst_dir(const view_t *view, int at)
 {
-	if(flist_custom_active(view) && view->custom.type == CV_TREE)
+	if(flist_custom_active(view) && cv_tree(view->custom.type))
 	{
 		if(at < 0)
 		{
@@ -1026,65 +1020,5 @@ fops_is_dir_writable(DirRole dir_role, const char path[])
 	return 0;
 }
 
-uint64_t
-fops_dir_size(const char path[], int force_update,
-		const struct cancellation_t *cancellation)
-{
-	DIR* dir;
-	struct dirent* dentry;
-	const char* slash = "";
-	uint64_t size;
-
-	dir = os_opendir(path);
-	if(dir == NULL)
-	{
-		return 0U;
-	}
-
-	if(!ends_with_slash(path))
-	{
-		slash = "/";
-	}
-
-	size = 0;
-	while((dentry = os_readdir(dir)) != NULL)
-	{
-		char full_path[PATH_MAX];
-
-		if(is_builtin_dir(dentry->d_name))
-		{
-			continue;
-		}
-
-		snprintf(full_path, sizeof(full_path), "%s%s%s", path, slash,
-				dentry->d_name);
-		if(fops_is_dir_entry(full_path, dentry))
-		{
-			uint64_t dir_size;
-			dcache_get_at(full_path, &dir_size, NULL);
-			if(dir_size == DCACHE_UNKNOWN || force_update)
-			{
-				dir_size = fops_dir_size(full_path, force_update, cancellation);
-			}
-			size += dir_size;
-		}
-		else
-		{
-			size += get_file_size(full_path);
-		}
-
-		if(cancellation_requested(cancellation))
-		{
-			os_closedir(dir);
-			return 0U;
-		}
-	}
-
-	os_closedir(dir);
-
-	(void)dcache_set_at(path, size, DCACHE_UNKNOWN);
-	return size;
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
