@@ -56,7 +56,7 @@ dir_size_args_t;
 
 static int delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash,
 		int nested);
-static const char * get_top_dir(const FileView *view);
+static const char * get_top_dir(const view_t *view);
 static void delete_files_in_bg(bg_op_t *bg_op, void *arg);
 static void delete_file_in_bg(ops_t *ops, const char path[], int use_trash);
 static int prepare_register(int reg);
@@ -67,13 +67,13 @@ TSTATIC const char * gen_clone_name(const char dir[], const char normal_name[]);
 static int clone_file(const dir_entry_t *entry, const char path[],
 		const char clone[], ops_t *ops);
 static void get_group_file_list(char *list[], int count, char buf[]);
-static void go_to_first_file(FileView *view, char *names[], int count);
-static void update_dir_entry_size(const FileView *view, int index, int force);
+static void go_to_first_file(view_t *view, char *names[], int count);
+static void update_dir_entry_size(const view_t *view, int index, int force);
 static void start_dir_size_calc(const char path[], int force);
 static void dir_size_bg(bg_op_t *bg_op, void *arg);
 static void dir_size(bg_op_t *bg_op, char path[], int force);
 static int bg_cancellation_hook(void *arg);
-static void redraw_after_path_change(FileView *view, const char path[]);
+static void redraw_after_path_change(view_t *view, const char path[]);
 #ifndef _WIN32
 static void change_owner_cb(const char new_owner[]);
 static int complete_owner(const char str[], void *arg);
@@ -82,7 +82,7 @@ static int complete_group(const char str[], void *arg);
 #endif
 
 int
-fops_delete(FileView *view, int reg, int use_trash)
+fops_delete(view_t *view, int reg, int use_trash)
 {
 	char undo_msg[COMMAND_GROUP_INFO_LEN];
 	int i;
@@ -151,7 +151,7 @@ fops_delete(FileView *view, int reg, int use_trash)
 	ui_view_reset_selection_and_reload(view);
 	ui_view_schedule_reload(view == curr_view ? other_view : curr_view);
 
-	status_bar_messagef("%d %s %celeted%s", ops->succeeded,
+	ui_sb_msgf("%d %s %celeted%s", ops->succeeded,
 			(ops->succeeded == 1) ? "file" : "files", use_trash ? 'd' : 'D',
 			fops_get_cancellation_suffix());
 
@@ -160,7 +160,7 @@ fops_delete(FileView *view, int reg, int use_trash)
 }
 
 int
-fops_delete_current(FileView *view, int use_trash, int nested)
+fops_delete_current(view_t *view, int use_trash, int nested)
 {
 	char undo_msg[COMMAND_GROUP_INFO_LEN];
 	dir_entry_t *entry;
@@ -201,7 +201,7 @@ fops_delete_current(FileView *view, int use_trash, int nested)
 		ui_views_reload_filelists();
 	}
 
-	status_bar_messagef("%d %s %celeted%s", ops->succeeded,
+	ui_sb_msgf("%d %s %celeted%s", ops->succeeded,
 			(ops->succeeded == 1) ? "file" : "files", use_trash ? 'd' : 'D',
 			fops_get_cancellation_suffix());
 
@@ -276,7 +276,7 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 }
 
 int
-fops_delete_bg(FileView *view, int use_trash)
+fops_delete_bg(view_t *view, int use_trash)
 {
 	char task_desc[COMMAND_GROUP_INFO_LEN];
 	bg_args_t *args;
@@ -304,7 +304,7 @@ fops_delete_bg(FileView *view, int use_trash)
 	if(args->sel_list_len == 0)
 	{
 		fops_free_bg_args(args);
-		status_bar_message("Nothing to delete");
+		ui_sb_msg("Nothing to delete");
 		return 1;
 	}
 
@@ -371,9 +371,9 @@ fops_delete_bg(FileView *view, int use_trash)
 /* Retrieves root directory of file system sub-tree (for regular or tree views).
  * Returns the path or NULL (for custom views). */
 static const char *
-get_top_dir(const FileView *view)
+get_top_dir(const view_t *view)
 {
-	if(flist_custom_active(view) && view->custom.type != CV_TREE)
+	if(flist_custom_active(view) && !cv_tree(view->custom.type))
 	{
 		return NULL;
 	}
@@ -437,7 +437,7 @@ delete_file_in_bg(ops_t *ops, const char path[], int use_trash)
 }
 
 int
-fops_yank(FileView *view, int reg)
+fops_yank(view_t *view, int reg)
 {
 	int nyanked_files;
 	dir_entry_t *entry;
@@ -459,7 +459,7 @@ fops_yank(FileView *view, int reg)
 
 	regs_update_unnamed(reg);
 
-	status_bar_messagef("%d file%s yanked", nyanked_files,
+	ui_sb_msgf("%d file%s yanked", nyanked_files,
 			(nyanked_files == 1) ? "" : "s");
 
 	return 1;
@@ -483,7 +483,7 @@ prepare_register(int reg)
 }
 
 int
-fops_retarget(FileView *view)
+fops_retarget(view_t *view)
 {
 	char full_path[PATH_MAX];
 	char linkto[PATH_MAX];
@@ -502,12 +502,12 @@ fops_retarget(FileView *view)
 
 	if(fentry_is_fake(entry))
 	{
-		status_bar_error("Entry doesn't correspond to a file");
+		ui_sb_err("Entry doesn't correspond to a file");
 		return 1;
 	}
 	if(entry->type != FT_LINK)
 	{
-		status_bar_error("File is not a symbolic link");
+		ui_sb_err("File is not a symbolic link");
 		return 1;
 	}
 
@@ -579,7 +579,7 @@ complete_filename(const char str[], void *arg)
 }
 
 int
-fops_clone(FileView *view, char *list[], int nlines, int force, int copies)
+fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 {
 	int i;
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
@@ -594,7 +594,7 @@ fops_clone(FileView *view, char *list[], int nlines, int force, int copies)
 	ops_t *ops;
 	const char *const curr_dir = flist_get_dir(view);
 
-	if(!fops_can_read_selected_files(view))
+	if(!fops_can_read_marked_files(view))
 	{
 		return 0;
 	}
@@ -710,7 +710,7 @@ fops_clone(FileView *view, char *list[], int nlines, int force, int copies)
 		/* Don't update cursor position if more than one file is cloned. */
 		if(nmarked == 1U)
 		{
-			fops_fixup_entry_after_rename(view, entry, clone_name);
+			fentry_rename(view, entry, clone_name);
 		}
 		ops_advance(ops, err == 0);
 
@@ -724,7 +724,7 @@ fops_clone(FileView *view, char *list[], int nlines, int force, int copies)
 		free_string_array(list, nlines);
 	}
 
-	status_bar_messagef("%d file%s cloned%s", ops->succeeded,
+	ui_sb_msgf("%d file%s cloned%s", ops->succeeded,
 			(ops->succeeded == 1) ? "" : "s", fops_get_cancellation_suffix());
 
 	fops_free_ops(ops);
@@ -740,7 +740,7 @@ is_clone_list_ok(int count, char *list[])
 	{
 		if(path_exists(list[i], NODEREF))
 		{
-			status_bar_errorf("File \"%s\" already exists", list[i]);
+			ui_sb_errf("File \"%s\" already exists", list[i]);
 			return 0;
 		}
 	}
@@ -762,7 +762,8 @@ gen_clone_name(const char dir[], const char normal_name[])
 	copy_str(result, sizeof(result), normal_name);
 	chosp(result);
 
-	copy_str(extension, sizeof(extension), cut_extension(result));
+	copy_str(extension, sizeof(extension),
+			cut_extension(result[0] == '.' ? &result[1] : result));
 
 	len = strlen(result);
 	i = 1;
@@ -817,7 +818,7 @@ clone_file(const dir_entry_t *entry, const char path[], const char clone[],
 }
 
 int
-fops_mkdirs(FileView *view, int at, char **names, int count, int create_parent)
+fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 {
 	char buf[COMMAND_GROUP_INFO_LEN + 1];
 	int i;
@@ -838,19 +839,19 @@ fops_mkdirs(FileView *view, int at, char **names, int count, int create_parent)
 
 		if(is_in_string_array(names, i, names[i]))
 		{
-			status_bar_errorf("Name \"%s\" duplicates", names[i]);
+			ui_sb_errf("Name \"%s\" duplicates", names[i]);
 			return 1;
 		}
 		if(names[i][0] == '\0')
 		{
-			status_bar_errorf("Name #%d is empty", i + 1);
+			ui_sb_errf("Name #%d is empty", i + 1);
 			return 1;
 		}
 
 		to_canonic_path(names[i], dst_dir, full, sizeof(full));
 		if(path_exists(full, NODEREF))
 		{
-			status_bar_errorf("File \"%s\" already exists", names[i]);
+			ui_sb_errf("File \"%s\" already exists", names[i]);
 			return 1;
 		}
 	}
@@ -893,13 +894,13 @@ fops_mkdirs(FileView *view, int at, char **names, int count, int create_parent)
 		go_to_first_file(view, names, count);
 	}
 
-	status_bar_messagef("%d director%s created%s", n, (n == 1) ? "y" : "ies",
+	ui_sb_msgf("%d director%s created%s", n, (n == 1) ? "y" : "ies",
 			fops_get_cancellation_suffix());
 	return 1;
 }
 
 int
-fops_mkfiles(FileView *view, int at, char *names[], int count)
+fops_mkfiles(view_t *view, int at, char *names[], int count)
 {
 	int i;
 	int n;
@@ -918,19 +919,19 @@ fops_mkfiles(FileView *view, int at, char *names[], int count)
 
 		if(is_in_string_array(names, i, names[i]))
 		{
-			status_bar_errorf("Name \"%s\" duplicates", names[i]);
+			ui_sb_errf("Name \"%s\" duplicates", names[i]);
 			return 1;
 		}
 		if(names[i][0] == '\0')
 		{
-			status_bar_errorf("Name #%d is empty", i + 1);
+			ui_sb_errf("Name #%d is empty", i + 1);
 			return 1;
 		}
 
 		to_canonic_path(names[i], dst_dir, full, sizeof(full));
 		if(path_exists(full, NODEREF))
 		{
-			status_bar_errorf("File \"%s\" already exists", names[i]);
+			ui_sb_errf("File \"%s\" already exists", names[i]);
 			return 1;
 		}
 	}
@@ -962,7 +963,7 @@ fops_mkfiles(FileView *view, int at, char *names[], int count)
 		go_to_first_file(view, names, count);
 	}
 
-	status_bar_messagef("%d file%s created%s", n, (n == 1) ? "" : "s",
+	ui_sb_msgf("%d file%s created%s", n, (n == 1) ? "" : "s",
 			fops_get_cancellation_suffix());
 
 	fops_free_ops(ops);
@@ -988,11 +989,11 @@ get_group_file_list(char *list[], int count, char buf[])
 /* Sets view cursor to point at first file found found in supplied list of file
  * names. */
 static void
-go_to_first_file(FileView *view, char *names[], int count)
+go_to_first_file(view_t *view, char *names[], int count)
 {
 	int i;
 
-	load_saving_pos(view, 1);
+	load_saving_pos(view);
 
 	for(i = 0; i < view->list_rows; ++i)
 	{
@@ -1006,7 +1007,7 @@ go_to_first_file(FileView *view, char *names[], int count)
 }
 
 int
-fops_restore(FileView *view)
+fops_restore(view_t *view)
 {
 	int m, n;
 	dir_entry_t *entry;
@@ -1042,13 +1043,12 @@ fops_restore(FileView *view)
 
 	ui_view_schedule_reload(view);
 
-	status_bar_messagef("Restored %d of %d%s", m, n,
-			fops_get_cancellation_suffix());
+	ui_sb_msgf("Restored %d of %d%s", m, n, fops_get_cancellation_suffix());
 	return 1;
 }
 
 void
-fops_size_bg(const FileView *view, int force)
+fops_size_bg(const view_t *view, int force)
 {
 	int i;
 
@@ -1071,7 +1071,7 @@ fops_size_bg(const FileView *view, int force)
 
 /* Initiates background size calculation for view entry. */
 static void
-update_dir_entry_size(const FileView *view, int index, int force)
+update_dir_entry_size(const view_t *view, int index, int force)
 {
 	char full_path[PATH_MAX];
 	const dir_entry_t *const entry = &view->dir_entry[index];
@@ -1096,7 +1096,7 @@ update_dir_entry_size(const FileView *view, int index, int force)
 static void
 start_dir_size_calc(const char path[], int force)
 {
-	char task_desc[PATH_MAX];
+	char task_desc[PATH_MAX + 1];
 	dir_size_args_t *args;
 
 	args = malloc(sizeof(*args));
@@ -1155,7 +1155,7 @@ bg_cancellation_hook(void *arg)
 
 /* Schedules view redraw in case path change might have affected it. */
 static void
-redraw_after_path_change(FileView *view, const char path[])
+redraw_after_path_change(view_t *view, const char path[])
 {
 	if(path_starts_with(view->curr_dir, path) ||
 			flist_custom_active(view))
@@ -1164,6 +1164,61 @@ redraw_after_path_change(FileView *view, const char path[])
 	}
 }
 
+uint64_t
+fops_dir_size(const char path[], int force_update,
+		const cancellation_t *cancellation)
+{
+	struct dirent *dentry;
+	const char *slash;
+	uint64_t size;
+
+	DIR *dir = os_opendir(path);
+	if(dir == NULL)
+	{
+		return 0U;
+	}
+
+	slash = (ends_with_slash(path) ? "" : "/");
+	size = 0U;
+	while((dentry = os_readdir(dir)) != NULL)
+	{
+		char full_path[PATH_MAX + 1];
+
+		if(is_builtin_dir(dentry->d_name))
+		{
+			continue;
+		}
+
+		snprintf(full_path, sizeof(full_path), "%s%s%s", path, slash,
+				dentry->d_name);
+		if(fops_is_dir_entry(full_path, dentry))
+		{
+			uint64_t dir_size;
+			dcache_get_at(full_path, &dir_size, NULL);
+			if(dir_size == DCACHE_UNKNOWN || force_update)
+			{
+				dir_size = fops_dir_size(full_path, force_update, cancellation);
+			}
+			size += dir_size;
+		}
+		else
+		{
+			size += get_file_size(full_path);
+		}
+
+		if(cancellation_requested(cancellation))
+		{
+			os_closedir(dir);
+			return 0U;
+		}
+	}
+
+	os_closedir(dir);
+
+	(void)dcache_set_at(path, size, DCACHE_UNKNOWN);
+	return size;
+}
+
 #ifndef _WIN32
 
 int
@@ -1172,7 +1227,7 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 /* Integer to pointer conversion. */
 #define V(e) (void *)(long)(e)
 
-	FileView *const view = curr_view;
+	view_t *const view = curr_view;
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 	ops_t *ops;
 	dir_entry_t *entry;
@@ -1218,7 +1273,7 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 	}
 	cmd_group_end();
 
-	status_bar_messagef("%d file%s fully processed%s", ops->succeeded,
+	ui_sb_msgf("%d file%s fully processed%s", ops->succeeded,
 			(ops->succeeded == 1) ? "" : "s", fops_get_cancellation_suffix());
 	fops_free_ops(ops);
 
@@ -1251,7 +1306,7 @@ change_owner_cb(const char new_owner[])
 
 	if(get_uid(new_owner, &uid) != 0)
 	{
-		status_bar_errorf("Invalid user name: \"%s\"", new_owner);
+		ui_sb_errf("Invalid user name: \"%s\"", new_owner);
 		curr_stats.save_msg = 1;
 		return;
 	}
@@ -1292,7 +1347,7 @@ change_group_cb(const char new_group[])
 
 	if(get_gid(new_group, &gid) != 0)
 	{
-		status_bar_errorf("Invalid group name: \"%s\"", new_group);
+		ui_sb_errf("Invalid group name: \"%s\"", new_group);
 		curr_stats.save_msg = 1;
 		return;
 	}
