@@ -50,16 +50,17 @@ RenameAction;
 
 static void rename_file_cb(const char new_name[]);
 static int complete_filename_only(const char str[], void *arg);
-static char ** add_files_to_list(const char path[], char *files[], int *len);
-static int perform_renaming(FileView *view, char *files[], char is_dup[],
-		int len, char *dst[]);
+static char ** add_files_to_list(const char base[], const char path[],
+		char *files[], int *len);
+static int perform_renaming(view_t *view, char *files[], char is_dup[], int len,
+		char *dst[]);
 TSTATIC const char * incdec_name(const char fname[], int k);
 static int count_digits(int number);
 static const char * substitute_tr(const char name[], const char pattern[],
 		const char sub[]);
 static RenameAction check_rename(const char old_fname[], const char new_fname[],
 		char **dest, int ndest);
-static int rename_marked(FileView *view, const char desc[], const char lhs[],
+static int rename_marked(view_t *view, const char desc[], const char lhs[],
 		const char rhs[], char **dest);
 static const char * gsubstitute_regexp(regex_t *re, const char src[],
 		const char sub[], regmatch_t matches[]);
@@ -70,7 +71,7 @@ static const char * substitute_regexp(const char src[], const char sub[],
 static char rename_file_ext[NAME_MAX + 1];
 
 void
-fops_rename_current(FileView *view, int name_only)
+fops_rename_current(view_t *view, int name_only)
 {
 	const dir_entry_t *const curr = get_current_entry(view);
 	char filename[strlen(curr->name) + 1];
@@ -123,7 +124,7 @@ rename_file_cb(const char new_name[])
 
 	if(contains_slash(new_name))
 	{
-		status_bar_error("Name can not contain slash");
+		ui_sb_err("Name can not contain slash");
 		curr_stats.save_msg = 1;
 		return;
 	}
@@ -164,7 +165,7 @@ complete_filename_only(const char str[], void *arg)
 }
 
 int
-fops_rename(FileView *view, char *list[], int nlines, int recursive)
+fops_rename(view_t *view, char *list[], int nlines, int recursive)
 {
 	char **files;
 	int nfiles;
@@ -175,7 +176,7 @@ fops_rename(FileView *view, char *list[], int nlines, int recursive)
 	/* Allow list of names in tests. */
 	if(curr_stats.load_stage != 0 && recursive && nlines != 0)
 	{
-		status_bar_error("Recursive rename doesn't accept list of new names");
+		ui_sb_err("Recursive rename doesn't accept list of new names");
 		return 1;
 	}
 	if(!fops_view_can_be_changed(view))
@@ -189,11 +190,11 @@ fops_rename(FileView *view, char *list[], int nlines, int recursive)
 	while(iter_marked_entries(view, &entry))
 	{
 		char path[PATH_MAX + 1];
-		get_short_path_of(view, entry, 0, 0, sizeof(path), path);
+		get_short_path_of(view, entry, NF_NONE, 0, sizeof(path), path);
 
 		if(recursive)
 		{
-			files = add_files_to_list(path, files, &nfiles);
+			files = add_files_to_list(flist_get_dir(view), path, files, &nfiles);
 		}
 		else
 		{
@@ -215,7 +216,7 @@ fops_rename(FileView *view, char *list[], int nlines, int recursive)
 		if(nfiles == 0 ||
 				(list = fops_edit_list(nfiles, files, &nlines, 0)) == NULL)
 		{
-			status_bar_message("0 files renamed");
+			ui_sb_msg("0 files renamed");
 		}
 		else
 		{
@@ -230,8 +231,7 @@ fops_rename(FileView *view, char *list[], int nlines, int recursive)
 		const int renamed = perform_renaming(view, files, is_dup, nfiles, list);
 		if(renamed >= 0)
 		{
-			status_bar_messagef("%d file%s renamed", renamed,
-					(renamed == 1) ? "" : "s");
+			ui_sb_msgf("%d file%s renamed", renamed, (renamed == 1) ? "" : "s");
 		}
 	}
 
@@ -251,19 +251,22 @@ fops_rename(FileView *view, char *list[], int nlines, int recursive)
 /* Appends files inside of the specified path to the list of the length *len.
  * Returns new list pointer. */
 static char **
-add_files_to_list(const char path[], char *files[], int *len)
+add_files_to_list(const char base[], const char path[], char *files[], int *len)
 {
-	DIR* dir;
-	struct dirent* dentry;
-	const char* slash = "";
+	DIR *dir;
+	struct dirent *dentry;
+	const char *slash = "";
 
-	if(!is_dir(path))
+	char full_path[PATH_MAX + 1];
+	to_canonic_path(path, base, full_path, sizeof(full_path));
+
+	if(!is_dir(full_path))
 	{
 		*len = add_to_string_array(&files, *len, 1, path);
 		return files;
 	}
 
-	dir = os_opendir(path);
+	dir = os_opendir(full_path);
 	if(dir == NULL)
 		return files;
 
@@ -274,9 +277,9 @@ add_files_to_list(const char path[], char *files[], int *len)
 	{
 		if(!is_builtin_dir(dentry->d_name))
 		{
-			char buf[PATH_MAX];
+			char buf[PATH_MAX + 1];
 			snprintf(buf, sizeof(buf), "%s%s%s", path, slash, dentry->d_name);
-			files = add_files_to_list(buf, files, len);
+			files = add_files_to_list(base, buf, files, len);
 		}
 	}
 
@@ -288,7 +291,7 @@ add_files_to_list(const char path[], char *files[], int *len)
  * marks elements that are in both lists.  Lengths of all lists must be equal to
  * len.  Returns number of renamed files. */
 static int
-perform_renaming(FileView *view, char *files[], char is_dup[], int len,
+perform_renaming(view_t *view, char *files[], char is_dup[], int len,
 		char *dst[])
 {
 	char undo_msg[MAX(10 + NAME_MAX, COMMAND_GROUP_INFO_LEN) + 1];
@@ -376,8 +379,8 @@ perform_renaming(FileView *view, char *files[], char is_dup[], int len,
 			new_name = get_last_path_component(dst[i]);
 
 			/* For regular views rename file in internal structures for correct
-				* positioning of cursor after reloading.  For custom views rename to
-				* prevent files from disappearing. */
+			 * positioning of cursor after reloading.  For custom views rename to
+			 * prevent files from disappearing. */
 			fentry_rename(view, entry, new_name);
 
 			if(flist_custom_active(view))
@@ -399,7 +402,7 @@ perform_renaming(FileView *view, char *files[], char is_dup[], int len,
 }
 
 int
-fops_incdec(FileView *view, int k)
+fops_incdec(view_t *view, int k)
 {
 	size_t names_len = 0;
 	char **names = NULL;
@@ -495,7 +498,7 @@ fops_incdec(FileView *view, int k)
 			err = 1;
 			break;
 		}
-		fops_fixup_entry_after_rename(view, entry, new_fname);
+		fentry_rename(view, entry, new_fname);
 		++nrenames;
 		++nrenamed;
 	}
@@ -514,17 +517,16 @@ fops_incdec(FileView *view, int k)
 
 	if(nrenames > 0)
 	{
-		ui_view_schedule_full_reload(view);
+		ui_view_schedule_reload(view);
 	}
 
 	if(err > 0)
 	{
-		status_bar_error("Rename error");
+		ui_sb_err("Rename error");
 	}
 	else if(err == 0)
 	{
-		status_bar_messagef("%d file%s renamed", nrenamed,
-				(nrenamed == 1) ? "" : "s");
+		ui_sb_msgf("%d file%s renamed", nrenamed, (nrenamed == 1) ? "" : "s");
 	}
 
 	return 1;
@@ -587,7 +589,7 @@ count_digits(int number)
 }
 
 int
-fops_case(FileView *view, int to_upper)
+fops_case(view_t *view, int to_upper)
 {
 	char **dest;
 	int ndest;
@@ -628,13 +630,13 @@ fops_case(FileView *view, int to_upper)
 
 		if(is_in_string_array(dest, ndest, new_fname))
 		{
-			status_bar_errorf("Name \"%s\" duplicates", new_fname);
+			ui_sb_errf("Name \"%s\" duplicates", new_fname);
 			err = 1;
 			break;
 		}
 		if(path_exists(new_fname, NODEREF) && !is_case_change(new_fname, old_fname))
 		{
-			status_bar_errorf("File \"%s\" already exists", new_fname);
+			ui_sb_errf("File \"%s\" already exists", new_fname);
 			err = 1;
 			break;
 		}
@@ -657,7 +659,7 @@ fops_case(FileView *view, int to_upper)
 }
 
 int
-fops_subst(FileView *view, const char pattern[], const char sub[], int ic,
+fops_subst(view_t *view, const char pattern[], const char sub[], int ic,
 		int glob)
 {
 	regex_t re;
@@ -687,7 +689,7 @@ fops_subst(FileView *view, const char pattern[], const char sub[], int ic,
 
 	if((err = regcomp(&re, pattern, cflags)) != 0)
 	{
-		status_bar_errorf("Regexp error: %s", get_regexp_error(err, &re));
+		ui_sb_errf("Regexp error: %s", get_regexp_error(err, &re));
 		regfree(&re);
 		return 1;
 	}
@@ -753,7 +755,7 @@ fops_subst(FileView *view, const char pattern[], const char sub[], int ic,
 }
 
 int
-fops_tr(FileView *view, const char from[], const char to[])
+fops_tr(view_t *view, const char from[], const char to[])
 {
 	char **dest;
 	int ndest;
@@ -846,22 +848,22 @@ check_rename(const char old_fname[], const char new_fname[], char **dest,
 
 	if(is_in_string_array(dest, ndest, new_fname))
 	{
-		status_bar_errorf("Name \"%s\" duplicates", new_fname);
+		ui_sb_errf("Name \"%s\" duplicates", new_fname);
 		return RA_FAIL;
 	}
 	if(new_fname[0] == '\0')
 	{
-		status_bar_errorf("Destination name of \"%s\" is empty", old_fname);
+		ui_sb_errf("Destination name of \"%s\" is empty", old_fname);
 		return RA_FAIL;
 	}
 	if(contains_slash(new_fname))
 	{
-		status_bar_errorf("Destination name \"%s\" contains slash", new_fname);
+		ui_sb_errf("Destination name \"%s\" contains slash", new_fname);
 		return RA_FAIL;
 	}
 	if(path_exists(new_fname, NODEREF))
 	{
-		status_bar_errorf("File \"%s\" already exists", new_fname);
+		ui_sb_errf("File \"%s\" already exists", new_fname);
 		return RA_FAIL;
 	}
 
@@ -872,7 +874,7 @@ check_rename(const char old_fname[], const char new_fname[], char **dest,
  * rhs can be NULL to omit their printing (both at the same time).  Returns new
  * value for save_msg flag. */
 static int
-rename_marked(FileView *view, const char desc[], const char lhs[],
+rename_marked(view_t *view, const char desc[], const char lhs[],
 		const char rhs[], char **dest)
 {
 	int i;
@@ -902,14 +904,13 @@ rename_marked(FileView *view, const char desc[], const char lhs[],
 		if(fops_mv_file(entry->name, entry->origin, new_fname, entry->origin,
 					OP_MOVE, 1, NULL) == 0)
 		{
-			fops_fixup_entry_after_rename(view, entry, new_fname);
+			fentry_rename(view, entry, new_fname);
 			++nrenamed;
 		}
 	}
 
 	cmd_group_end();
-	status_bar_messagef("%d file%s renamed", nrenamed,
-			(nrenamed == 1) ? "" : "s");
+	ui_sb_msgf("%d file%s renamed", nrenamed, (nrenamed == 1) ? "" : "s");
 
 	return 1;
 }
