@@ -25,17 +25,17 @@
 #include <stddef.h> /* NULL */
 #include <stdio.h> /* remove() snprintf() */
 #include <stdlib.h> /* free() */
-#include <string.h> /* strerror() strlen() */
+#include <string.h> /* strlen() */
 
 #include "../compat/fs_limits.h"
 #include "../compat/os.h"
-#include "../ui/cancellation.h"
 #include "../utils/fs.h"
 #include "../utils/log.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/utils.h"
 #include "../background.h"
+#include "private/ioc.h"
 #include "private/ioe.h"
 #include "private/ioeta.h"
 #include "private/traverser.h"
@@ -67,7 +67,7 @@ rm_visitor(const char full_path[], VisitAction action, void *param)
 	io_args_t *const rm_args = param;
 	VisitResult result = VR_OK;
 
-	if(rm_args->cancellable && ui_cancellation_requested())
+	if(io_cancelled(rm_args))
 	{
 		return VR_CANCELLED;
 	}
@@ -83,7 +83,7 @@ rm_visitor(const char full_path[], VisitAction action, void *param)
 				io_args_t args = {
 					.arg1.path = full_path,
 
-					.cancellable = rm_args->cancellable,
+					.cancellation = rm_args->cancellation,
 					.estim = rm_args->estim,
 
 					.result = rm_args->result,
@@ -98,7 +98,7 @@ rm_visitor(const char full_path[], VisitAction action, void *param)
 				io_args_t args = {
 					.arg1.path = full_path,
 
-					.cancellable = rm_args->cancellable,
+					.cancellation = rm_args->cancellation,
 					.estim = rm_args->estim,
 
 					.result = rm_args->result,
@@ -131,7 +131,7 @@ ior_cp(io_args_t *const args)
 		io_args_t rm_args = {
 			.arg1.path = dst,
 
-			.cancellable = args->cancellable,
+			.cancellation = args->cancellation,
 			.estim = args->estim,
 
 			.result = args->result,
@@ -141,7 +141,7 @@ ior_cp(io_args_t *const args)
 		args->result = rm_args.result;
 		if(result != 0)
 		{
-			if(!args->cancellable || !ui_cancellation_requested())
+			if(!io_cancelled(args))
 			{
 				(void)ioe_errlst_append(&args->result.errors, dst, IO_ERR_UNKNOWN,
 						"Failed to remove");
@@ -172,7 +172,7 @@ ior_mv(io_args_t *const args)
 	if(crs == IO_CRS_FAIL && path_exists(dst, DEREF) && !is_case_change(src, dst))
 	{
 		(void)ioe_errlst_append(&args->result.errors, dst, EEXIST,
-				strerror(EEXIST));
+				"Destination path already exists");
 		return 1;
 	}
 
@@ -181,13 +181,13 @@ ior_mv(io_args_t *const args)
 		if(!is_file(src))
 		{
 			(void)ioe_errlst_append(&args->result.errors, src, EISDIR,
-					strerror(EISDIR));
+					"Can't append when source is not a file");
 			return 1;
 		}
 		if(!is_file(dst))
 		{
 			(void)ioe_errlst_append(&args->result.errors, dst, EISDIR,
-					strerror(EISDIR));
+					"Can't append when destination is not a file");
 			return 1;
 		}
 	}
@@ -209,6 +209,13 @@ ior_mv(io_args_t *const args)
 	switch(errno)
 	{
 		case EXDEV:
+#ifndef _WIN32
+		/* At least SSHFS fails to propagate EXDEV and reports EPERM.  So we try to
+		 * do the copy across mounts anyway, which might actually work despite the
+		 * error code. */
+		case EPERM:
+		case EACCES:
+#endif
 			{
 				int result = ior_cp(args);
 				if(result == 0)
@@ -216,7 +223,7 @@ ior_mv(io_args_t *const args)
 					io_args_t rm_args = {
 						.arg1.path = src,
 
-						.cancellable = args->cancellable,
+						.cancellation = args->cancellation,
 						.estim = args->estim,
 
 						.result = args->result,
@@ -244,7 +251,7 @@ ior_mv(io_args_t *const args)
 				io_args_t rm_args = {
 					.arg1.path = dst,
 
-					.cancellable = args->cancellable,
+					.cancellation = args->cancellation,
 					.estim = args->estim,
 
 					.result = args->result,
@@ -260,7 +267,7 @@ ior_mv(io_args_t *const args)
 				args->result = rm_args.result;
 				if(error != 0)
 				{
-					if(!args->cancellable || !ui_cancellation_requested())
+					if(!io_cancelled(args))
 					{
 						(void)ioe_errlst_append(&args->result.errors, dst, IO_ERR_UNKNOWN,
 								"Failed to remove");
@@ -271,7 +278,7 @@ ior_mv(io_args_t *const args)
 				if(os_rename(src, dst) != 0)
 				{
 					(void)ioe_errlst_append(&args->result.errors, src, errno,
-							strerror(errno));
+							"Rename operation failed");
 					return 1;
 				}
 				return 0;
@@ -284,7 +291,7 @@ ior_mv(io_args_t *const args)
 					io_args_t rm_args = {
 						.arg1.path = dst,
 
-						.cancellable = args->cancellable,
+						.cancellation = args->cancellation,
 						.estim = args->estim,
 
 						.result = args->result,
@@ -294,7 +301,7 @@ ior_mv(io_args_t *const args)
 					args->result = rm_args.result;
 					if(error != 0)
 					{
-						if(!args->cancellable || !ui_cancellation_requested())
+						if(!io_cancelled(args))
 						{
 							(void)ioe_errlst_append(&args->result.errors, dst, IO_ERR_UNKNOWN,
 									"Failed to remove");
@@ -309,7 +316,7 @@ ior_mv(io_args_t *const args)
 
 		default:
 			(void)ioe_errlst_append(&args->result.errors, src, errno,
-					strerror(errno));
+					"Rename operation failed");
 			return errno;
 	}
 }
@@ -342,7 +349,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 	VisitResult result = VR_OK;
 	const char *rel_part;
 
-	if(cp_args->cancellable && ui_cancellation_requested())
+	if(io_cancelled(cp_args))
 	{
 		return VR_CANCELLED;
 	}
@@ -364,7 +371,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 					/* Temporary fake rights so we can add files to the directory. */
 					.arg3.mode = 0700,
 
-					.cancellable = cp_args->cancellable,
+					.cancellation = cp_args->cancellation,
 					.estim = cp_args->estim,
 
 					.result = cp_args->result,
@@ -383,7 +390,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 					/* It's safe to always use fast file cloning on moving files. */
 					.arg4.fast_file_cloning = cp ? cp_args->arg4.fast_file_cloning : 1,
 
-					.cancellable = cp_args->cancellable,
+					.cancellation = cp_args->cancellation,
 					.confirm = cp_args->confirm,
 					.estim = cp_args->estim,
 
@@ -403,7 +410,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 					io_args_t rm_args = {
 						.arg1.path = full_path,
 
-						.cancellable = cp_args->cancellable,
+						.cancellation = cp_args->cancellation,
 						.estim = cp_args->estim,
 
 						.result = cp_args->result,
@@ -419,14 +426,14 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 					if(result == VR_ERROR)
 					{
 						(void)ioe_errlst_append(&cp_args->result.errors, dst_full_path,
-								errno, strerror(errno));
+								errno, "Failed to setup directory permissions");
 					}
 					clone_timestamps(dst_full_path, full_path, &st);
 				}
 				else
 				{
 					(void)ioe_errlst_append(&cp_args->result.errors, full_path, errno,
-							strerror(errno));
+							"Failed to stat() source directory");
 					result = VR_ERROR;
 				}
 				break;
