@@ -18,46 +18,7 @@
 
 #include "ipc.h"
 
-#ifndef ENABLE_REMOTE_CMDS
-
-#include <stddef.h> /* NULL */
-
-int
-ipc_enabled(void)
-{
-	return 0;
-}
-
-char **
-ipc_list(int *len)
-{
-	*len = 0;
-	return NULL;
-}
-
-void
-ipc_init(const char name[], ipc_callback callback_func)
-{
-}
-
-const char *
-ipc_get_name(void)
-{
-	return "";
-}
-
-void
-ipc_check(void)
-{
-}
-
-int
-ipc_send(const char whom[], char *data[])
-{
-	return 1;
-}
-
-#else
+#ifdef ENABLE_REMOTE_CMDS
 
 #if defined(_WIN32) || defined(__CYGWIN__)
 /* Named pipes don't work very well on Cygwin neither directly nor indirectly
@@ -84,13 +45,12 @@ ipc_send(const char whom[], char *data[])
 #include <sys/stat.h> /* mkfifo() stat() */
 #include <dirent.h> /* DIR closedir() opendir() readdir() */
 #include <fcntl.h>
-#include <unistd.h> /* close() open() select() unlink() */
+#include <unistd.h> /* close() open() select() unlink() usleep() */
 
-#include <assert.h> /* assert() */
 #include <errno.h> /* EACCES EEXIST EDQUOT ENOSPC ENXIO errno */
 #include <stddef.h> /* NULL size_t ssize_t */
 #include <stdio.h> /* FILE fclose() fdopen() fread() fwrite() */
-#include <stdlib.h> /* atexit() free() malloc() qsort() snprintf() */
+#include <stdlib.h> /* free() malloc() qsort() snprintf() */
 #include <string.h> /* strcmp() strcpy() strlen() */
 
 #include "utils/fs.h"
@@ -102,6 +62,39 @@ ipc_send(const char whom[], char *data[])
 #include "utils/utils.h"
 #include "status.h"
 
+/**
+ * When IPC is enabled, it exchanges packets between instances which are just
+ * arrays of strings.  Each package looks like this:
+ *
+ *     "version:{...}" -\
+ *     "from:{name}"   --\
+ *     "body:{type}"   ---\
+ *                         some sort of a header that ends on `body:`
+ *     "string #1"     -\
+ *     "string #2"     --\
+ *     {...}           ---\
+ *     "string #N"     ----\
+ *                          payload (might be prepended by this unit)
+ *     "\0"            -\
+ *                       terminator
+ *
+ * Or in sequential form:
+ *     "version:{...}\0body:{type}\0string #1\0string #2\0{...}string #N\0\0"
+ *
+ * {name} is a name of another instance.
+ *
+ * {type} can be:
+ *  - "args" to pass list of arguments, in which case body is prepended with CWD
+ *    unconditionally;
+ *  - "eval" to pass an expression for evaluation in a single line;
+ *  - "eval-result" to communicate result of successful evaluation in a single
+ *    line;
+ *  - "eval-error" to communicate failure of evaluation with no strings.
+ *
+ * On version mismatch or unknown field name, packet is discarded which is
+ * logged.
+ */
+
 /* Prefix for names of all pipes to distinguish them from other pipes. */
 #define PREFIX "vifm-ipc-"
 
@@ -119,16 +112,37 @@ typedef struct
 	char **lst;          /* List of strings. */
 	size_t len;          /* Number of items. */
 	const char *ipc_dir; /* Root of IPC objects. */
+	const ipc_t *ipc;    /* Handle to an IPC instance or NULL. */
 }
 list_data_t;
 
-static void cleanup_at_exit(void);
+/* Storage of data of an instance. */
+struct ipc_t
+{
+	/* Stores callback to report received messages. */
+	ipc_args_cb args_cb;
+	/* Stores callback used for evaluation of expressions. */
+	ipc_eval_cb eval_cb;
+	/* Path to the pipe used by this instance. */
+	char pipe_path[PATH_MAX + 1];
+	/* Opened file of the pipe. */
+	read_pipe_t pipe_file;
+	/* Holds result of expression evaluation or NULL on evaluation error. */
+	char *eval_result;
+};
+
 static read_pipe_t create_pipe(const char name[], char path_buf[], size_t len);
-static char * receive_pkg(void);
+static char * receive_pkg(ipc_t *ipc, int *len);
 static read_pipe_t try_use_pipe(const char path[], int *fatal);
-static void handle_pkg(const char pkg[]);
+static void handle_pkg(ipc_t *ipc, const char pkg[], const char *end);
+static void handle_args(ipc_t *ipc, char ***array, int len);
+static void handle_expr(ipc_t *ipc, const char from[], char *array[], int len);
+static void handle_eval_result(ipc_t *ipc, char *array[], int len);
+static int format_and_send(ipc_t *ipc, const char whom[], char *data[],
+		const char type[]);
 static int send_pkg(const char whom[], const char what[], size_t len);
-static char * get_the_only_target(void);
+static char * get_the_only_target(const ipc_t *ipc);
+static char ** list_servers(const ipc_t *ipc, int *len);
 static int add_to_list(const char name[], const void *data, void *param);
 static const char * get_ipc_dir(void);
 static int sorter(const void *first, const void *second);
@@ -136,94 +150,92 @@ static int sorter(const void *first, const void *second);
 static int pipe_is_in_use(const char path[]);
 #endif
 
-/* Stores callback to report received messages. */
-static ipc_callback callback;
-/* Whether unit was initialized and what's the result (-1 is error, 1 is
- * success). */
-static int initialized;
-/* Path to the pipe used by this instance. */
-static char pipe_path[PATH_MAX];
-/* Opened file of the pipe. */
-static read_pipe_t pipe_file;
+/* Current version string. */
+static const char IPC_VERSION[] = "version:1";
+/* Request to process remote arguments. */
+static const char ARGS_TYPE[] = "args";
+/* Request to process remote expression. */
+static const char EVAL_TYPE[] = "eval";
+/* Reply to remote expression with the result after successful evaluation. */
+static const char EVAL_RESULT_TYPE[] = "eval-result";
+/* Reply to remote expression on error. */
+static const char EVAL_ERROR_TYPE[] = "eval-error";
 
 int
 ipc_enabled(void)
 {
-	return (initialized > 0);
+	return 1;
 }
 
-void
-ipc_init(const char name[], ipc_callback callback_func)
+ipc_t *
+ipc_init(const char name[], ipc_args_cb args_cb, ipc_eval_cb eval_cb)
 {
-	assert(!initialized && "Repeated initialization?");
+	ipc_t *const ipc = malloc(sizeof(*ipc));
+	if(ipc == NULL)
+	{
+		return NULL;
+	}
 
-	callback = callback_func;
+	ipc->args_cb = args_cb;
+	ipc->eval_cb = eval_cb;
 
 	if(name == NULL)
 	{
 		name = "vifm";
 	}
 
-	pipe_file = create_pipe(name, pipe_path, sizeof(pipe_path));
-	if(pipe_file == NULL_READ_PIPE)
+	ipc->pipe_file = create_pipe(name, ipc->pipe_path, sizeof(ipc->pipe_path));
+	if(ipc->pipe_file == NULL_READ_PIPE)
 	{
-		initialized = -1;
-		return;
+		free(ipc);
+		return NULL;
 	}
 
-	atexit(&cleanup_at_exit);
-	initialized = 1;
-}
-
-/* Frees resources used by IPC. */
-static void
-cleanup_at_exit(void)
-{
-#ifndef WIN32_PIPE_READ
-	fclose(pipe_file);
-	unlink(pipe_path);
-#else
-	CloseHandle(pipe_file);
-#endif
-}
-
-const char *
-ipc_get_name(void)
-{
-	if(initialized < 0)
-	{
-		return "";
-	}
-
-	return get_last_path_component(pipe_path) + (sizeof(PREFIX) - 1U);
+	return ipc;
 }
 
 void
-ipc_check(void)
+ipc_free(ipc_t *ipc)
 {
-	char *pkg;
-
-	assert(initialized != 0 && "Wrong IPC unit state.");
-	if(initialized < 0)
+	if(ipc == NULL)
 	{
 		return;
 	}
 
-	pkg = receive_pkg();
-	if(pkg == NULL)
-	{
-		return;
-	}
+#ifndef WIN32_PIPE_READ
+	fclose(ipc->pipe_file);
+	unlink(ipc->pipe_path);
+#else
+	CloseHandle(ipc->pipe_file);
+#endif
+	free(ipc);
+}
 
-	handle_pkg(pkg);
-	free(pkg);
+const char *
+ipc_get_name(const ipc_t *ipc)
+{
+	return get_last_path_component(ipc->pipe_path) + (sizeof(PREFIX) - 1U);
+}
+
+int
+ipc_check(ipc_t *ipc)
+{
+	int len;
+	char *const pkg = receive_pkg(ipc, &len);
+	if(pkg != NULL)
+	{
+		handle_pkg(ipc, pkg, pkg + len);
+		free(pkg);
+		return 1;
+	}
+	return 0;
 }
 
 /* Receives message addressed to this instance.  Returns NULL if there was no
  * message or on failure to read it, otherwise newly allocated string is
  * returned. */
 static char *
-receive_pkg(void)
+receive_pkg(ipc_t *ipc, int *len)
 {
 #ifndef WIN32_PIPE_READ
 	uint32_t size;
@@ -234,25 +246,31 @@ receive_pkg(void)
 	int max_fd;
 	struct timeval ts = { .tv_sec = 0, .tv_usec = 10000 };
 
-	if(fread(&size, sizeof(size), 1U, pipe_file) != 1U || size >= 4294967294U)
+	/* At least on OS X pipe might get into EOF state, so reset it.  This will
+	 * also reset any errors, which is fine with us. */
+	clearerr(ipc->pipe_file);
+
+	if(fread(&size, sizeof(size), 1U, ipc->pipe_file) != 1U ||
+			size >= 4294967294U)
 	{
 		return NULL;
 	}
 
-	pkg = malloc(size + 2U);
+	pkg = malloc(size + 1U);
 	if(pkg == NULL)
 	{
+		LOG_ERROR_MSG("Failed to allocate memory: %lu", (unsigned long)(size + 1));
 		return NULL;
 	}
 
-	max_fd = fileno(pipe_file);
+	max_fd = fileno(ipc->pipe_file);
 	FD_ZERO(&ready);
 	FD_SET(max_fd, &ready);
 
 	p = pkg;
 	while(size != 0U && select(max_fd + 1, &ready, NULL, NULL, &ts) > 0)
 	{
-		const size_t nread = fread(p, 1U, size, pipe_file);
+		const size_t nread = fread(p, 1U, size, ipc->pipe_file);
 		size -= nread;
 		p += nread;
 
@@ -268,12 +286,14 @@ receive_pkg(void)
 	if(size != 0U)
 	{
 		free(pkg);
+		LOG_ERROR_MSG("Failed to read whole packet, left: %lu",
+				(unsigned long)size);
 		return NULL;
 	}
 
-	/* Make sure we have two trailing zeroes. */
-	*p++ = '\0';
+	/* Make sure we have a trailing zero. */
 	*p = '\0';
+	*len = p - pkg;
 
 	return pkg;
 #else
@@ -282,13 +302,13 @@ receive_pkg(void)
 	char *p;
 	DWORD nread;
 
-	if(ReadFile(pipe_file, &size, sizeof(size), &nread, NULL) == FALSE ||
+	if(ReadFile(ipc->pipe_file, &size, sizeof(size), &nread, NULL) == FALSE ||
 			size >= 4294967294U)
 	{
 		return NULL;
 	}
 
-	pkg = malloc(size + 2U);
+	pkg = malloc(size + 1U);
 	if(pkg == NULL)
 	{
 		return NULL;
@@ -301,7 +321,7 @@ receive_pkg(void)
 		 *       inconvenient... */
 		usleep(10000);
 
-		if(ReadFile(pipe_file, p, size, &nread, NULL) == FALSE || nread == 0U)
+		if(ReadFile(ipc->pipe_file, p, size, &nread, NULL) == FALSE || nread == 0U)
 		{
 			break;
 		}
@@ -312,8 +332,8 @@ receive_pkg(void)
 
 	/* Weird requirement for named pipes, need to break and set connection every
 	 * time. */
-	DisconnectNamedPipe(pipe_file);
-	ConnectNamedPipe(pipe_file, NULL);
+	DisconnectNamedPipe(ipc->pipe_file);
+	ConnectNamedPipe(ipc->pipe_file, NULL);
 
 	if(size != 0U)
 	{
@@ -321,9 +341,9 @@ receive_pkg(void)
 		return NULL;
 	}
 
-	/* Make sure we have two trailing zeroes. */
-	*p++ = '\0';
+	/* Make sure we have a trailing zero. */
 	*p = '\0';
+	*len = p - pkg;
 
 	return pkg;
 #endif
@@ -421,28 +441,176 @@ try_use_pipe(const char path[], int *fatal)
 
 /* Parses pkg into array of strings and invokes callback. */
 static void
-handle_pkg(const char pkg[])
+handle_pkg(ipc_t *ipc, const char pkg[], const char *end)
 {
 	char **array = NULL;
 	size_t len = 0U;
+	int in_body = 0;
+	const char *type = NULL;
+	const char *from = NULL;
 
-	while(*pkg != '\0')
+	while(pkg != end)
 	{
-		len = add_to_string_array(&array, len, 1, pkg);
+		if(in_body)
+		{
+			len = add_to_string_array(&array, len, 1, pkg);
+		}
+		else if(starts_with_lit(pkg, "version:"))
+		{
+			if(strcmp(pkg, IPC_VERSION) != 0)
+			{
+				break;
+			}
+		}
+		else if(starts_with_lit(pkg, "from:"))
+		{
+			from = after_first(pkg, ':');
+		}
+		else if(starts_with_lit(pkg, "body:"))
+		{
+			type = after_first(pkg, ':');
+			in_body = 1;
+		}
+		else
+		{
+			break;
+		}
 		pkg += strlen(pkg) + 1;
 	}
-	len = put_into_string_array(&array, len, NULL);
 
-	if(len != 0U)
+	if(pkg != end)
 	{
-		callback(array);
+		LOG_ERROR_MSG("Discarded remote package due to field: `%s`", pkg);
+	}
+	else if(from == NULL)
+	{
+		LOG_ERROR_MSG("Discarded remote package due to missing from field");
+	}
+	else if(type == NULL)
+	{
+		LOG_ERROR_MSG("Discarded remote package due to missing body field");
+	}
+	else if(strcmp(type, ARGS_TYPE) == 0)
+	{
+		handle_args(ipc, &array, len);
+	}
+	else if(strcmp(type, EVAL_TYPE) == 0)
+	{
+		handle_expr(ipc, from, array, len);
+	}
+	else if(strcmp(type, EVAL_RESULT_TYPE) == 0)
+	{
+		handle_eval_result(ipc, array, len);
+	}
+	else if(strcmp(type, EVAL_ERROR_TYPE) == 0)
+	{
+		ipc->eval_result = NULL;
+	}
+	else
+	{
+		LOG_ERROR_MSG("Discarded remote package due to unknown type: `%s`", type);
 	}
 
 	free_string_array(array, len);
 }
 
+/* Handles received message with arguments. */
+static void
+handle_args(ipc_t *ipc, char ***array, int len)
+{
+	if(len == 0U)
+	{
+		return;
+	}
+
+	if(put_into_string_array(array, len, NULL) == len + 1)
+	{
+		ipc->args_cb(*array);
+	}
+}
+
+/* Handles received message with expression to evaluate. */
+static void
+handle_expr(ipc_t *ipc, const char from[], char *array[], int len)
+{
+	char *result;
+
+	if(len != 1U)
+	{
+		LOG_ERROR_MSG("Incorrect number of lines in expr packet: %d", len);
+		return;
+	}
+
+	result = ipc->eval_cb(array[0]);
+	if(result == NULL)
+	{
+		char *data[] = { NULL };
+		if(format_and_send(ipc, from, data, EVAL_ERROR_TYPE) != 0)
+		{
+			LOG_ERROR_MSG("Failed to report evaluation failure");
+		}
+	}
+	else
+	{
+		char *data[] = { result, NULL };
+		if(format_and_send(ipc, from, data, EVAL_RESULT_TYPE) != 0)
+		{
+			LOG_ERROR_MSG("Failed to report evaluation result");
+		}
+		free(result);
+	}
+}
+
+/* Handles answer about successful evaluation of expression. */
+static void
+handle_eval_result(ipc_t *ipc, char *array[], int len)
+{
+	if(len == 1U)
+	{
+		ipc->eval_result = array[0];
+		array[0] = NULL;
+	}
+}
+
 int
-ipc_send(const char whom[], char *data[])
+ipc_send(ipc_t *ipc, const char whom[], char *data[])
+{
+	return format_and_send(ipc, whom, data, ARGS_TYPE);
+}
+
+char *
+ipc_eval(ipc_t *ipc, const char whom[], const char expr[])
+{
+	enum { MAX_USEC = 1000000, MAX_REPEATS = 20 };
+	int repeats;
+
+	char *data[] = { (char *)expr, NULL };
+	if(format_and_send(ipc, whom, data, EVAL_TYPE) != 0)
+	{
+		LOG_ERROR_MSG("Failed to send expression");
+		return NULL;
+	}
+
+	/* Using sleep is just easier than doing read with timeout due to differences
+	 * between platforms... */
+	repeats = 0;
+	while(!ipc_check(ipc))
+	{
+		if(++repeats > MAX_REPEATS)
+		{
+			LOG_ERROR_MSG("Timed out on waiting for --remote-expr response");
+			return NULL;
+		}
+		usleep(MAX_USEC/MAX_REPEATS);
+	}
+
+	return ipc->eval_result;
+}
+
+/* Formats and sends a message of specified type.  The data array should be NULL
+ * terminated.  Returns zero on successful send and non-zero otherwise. */
+static int
+format_and_send(ipc_t *ipc, const char whom[], char *data[], const char type[])
 {
 	/* FIXME: this shouldn't have fixed size.  Or maybe it should be PIPE_BUF to
 	 * guarantee atomic operation. */
@@ -451,29 +619,33 @@ ipc_send(const char whom[], char *data[])
 	char *name = NULL;
 	int ret;
 
-	assert(initialized != 0 && "Wrong IPC unit state.");
-	if(initialized < 0)
-	{
-		return 1;
-	}
+	/* Compose "header". */
+	len = copy_str(pkg, sizeof(pkg), IPC_VERSION);
+	len += MIN(snprintf(pkg + len, sizeof(pkg) - len, "from:%s",
+				ipc_get_name(ipc)) + 1,
+			(int)(sizeof(pkg) - len));
+	len += MIN(snprintf(pkg + len, sizeof(pkg) - len, "body:%s", type) + 1,
+			(int)(sizeof(pkg) - len));
 
-	if(get_cwd(pkg, sizeof(pkg)) == NULL)
+	if(strcmp(type, ARGS_TYPE) == 0)
 	{
-		LOG_ERROR_MSG("Can't get working directory");
-		return 1;
+		if(get_cwd(pkg + len, sizeof(pkg) - len) == NULL)
+		{
+			LOG_ERROR_MSG("Can't get working directory");
+			return 1;
+		}
+		len += strlen(pkg + len) + 1;
 	}
-	len = strlen(pkg) + 1;
 
 	while(*data != NULL)
 	{
 		len += copy_str(pkg + len, sizeof(pkg) - len, *data);
 		++data;
 	}
-	pkg[len++] = '\0';
 
 	if(whom == NULL)
 	{
-		name = get_the_only_target();
+		name = get_the_only_target(ipc);
 		if(name == NULL)
 		{
 			return 1;
@@ -500,15 +672,17 @@ send_pkg(const char whom[], const char what[], size_t len)
 
 	snprintf(path, sizeof(path), "%s/" PREFIX "%s", get_ipc_dir(), whom);
 
-	fd = open(path, O_WRONLY | O_NONBLOCK);
+	fd = open(path, O_WRONLY);
 	if(fd == -1)
 	{
+		LOG_SERROR_MSG(errno, "Failed to open destination pipe");
 		return 1;
 	}
 
 	dst = fdopen(fd, "w");
 	if(dst == NULL)
 	{
+		LOG_SERROR_MSG(errno, "Failed to turn file descriptor into a FILE");
 		close(fd);
 		return 1;
 	}
@@ -517,11 +691,15 @@ send_pkg(const char whom[], const char what[], size_t len)
 	if(fwrite(&size, sizeof(size), 1U, dst) != 1U ||
 			fwrite(what, len, 1U, dst) != 1U)
 	{
-		fclose(dst);
+		LOG_SERROR_MSG(errno, "Failed to write into a pipe");
+		(void)fclose(dst);
 		return 1;
 	}
 
-	fclose(dst);
+	if(fclose(dst) != 0)
+	{
+		LOG_SERROR_MSG(errno, "Failure on close a pipe");
+	}
 	return 0;
 #else
 	char path[PATH_MAX];
@@ -555,11 +733,11 @@ send_pkg(const char whom[], const char what[], size_t len)
 /* Automatically picks target instance to send data to.  Returns newly allocated
  * string or NULL on error (no other instances or memory allocation failure). */
 static char *
-get_the_only_target(void)
+get_the_only_target(const ipc_t *ipc)
 {
 	int len;
 	char *name;
-	char **list = ipc_list(&len);
+	char **list = list_servers(ipc, &len);
 
 	if(len == 0)
 	{
@@ -576,7 +754,16 @@ get_the_only_target(void)
 char **
 ipc_list(int *len)
 {
-	list_data_t data = { .ipc_dir = get_ipc_dir() };
+	return list_servers(NULL, len);
+}
+
+/* Retrieves list with names of servers available for IPC excluding the one
+ * specified by the ipc parameter, which can be NULL.  Returns the list which is
+ * of the *len length. */
+static char **
+list_servers(const ipc_t *ipc, int *len)
+{
+	list_data_t data = { .ipc_dir = get_ipc_dir(), .ipc = ipc };
 
 #ifndef WIN32_PIPE_READ
 	if(enum_dir_content(data.ipc_dir, &add_to_list, &data) != 0)
@@ -611,7 +798,10 @@ ipc_list(int *len)
 	}
 #endif
 
-	qsort(data.lst, data.len, sizeof(*data.lst), &sorter);
+	if(data.len != 0)
+	{
+		qsort(data.lst, data.len, sizeof(*data.lst), &sorter);
+	}
 
 	*len = data.len;
 	return data.lst;
@@ -623,6 +813,7 @@ static int
 add_to_list(const char name[], const void *data, void *param)
 {
 	list_data_t *const list_data = param;
+	const ipc_t *ipc = list_data->ipc;
 
 	if(!starts_with_lit(name, PREFIX))
 	{
@@ -630,7 +821,8 @@ add_to_list(const char name[], const void *data, void *param)
 	}
 
 	/* Skip ourself. */
-	if(stroscmp(name, get_last_path_component(pipe_path)) == 0)
+	if(ipc != NULL &&
+			stroscmp(name, get_last_path_component(ipc->pipe_path)) == 0)
 	{
 		return 0;
 	}
@@ -696,6 +888,58 @@ pipe_is_in_use(const char path[])
 
 #endif
 
+#else
+
+#include <stddef.h> /* NULL */
+
+int
+ipc_enabled(void)
+{
+	return 0;
+}
+
+char **
+ipc_list(int *len)
+{
+	*len = 0;
+	return NULL;
+}
+
+ipc_t *
+ipc_init(const char name[], ipc_args_cb args_cb, ipc_eval_cb eval_cb)
+{
+	return NULL;
+}
+
+void
+ipc_free(ipc_t *ipc)
+{
+}
+
+const char *
+ipc_get_name(const ipc_t *ipc)
+{
+	return "";
+}
+
+int
+ipc_check(ipc_t *ipc)
+{
+	return 0;
+}
+
+int
+ipc_send(ipc_t *ipc, const char whom[], char *data[])
+{
+	return 1;
+}
+
+char *
+ipc_eval(ipc_t *ipc, const char whom[], const char expr[])
+{
+	return NULL;
+}
+
 #endif
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
