@@ -44,7 +44,7 @@ static void set_mark(const char m, const char directory[], const char file[],
 		time_t timestamp, int force);
 static int is_mark_points_to(const mark_t *mark, const char directory[],
 		const char file[]);
-static int navigate_to_mark(FileView *view, const char m);
+static int navigate_to_mark(view_t *view, const char m);
 TSTATIC mark_t * get_mark_by_name(const char mark);
 static mark_t * find_mark(const int index);
 static int is_mark_valid(const mark_t *mark);
@@ -182,7 +182,7 @@ set_user_mark(const char mark, const char directory[], const char file[])
 {
 	if(!is_user_mark(mark))
 	{
-		status_bar_message("Invalid mark name");
+		ui_sb_msg("Invalid mark name");
 		return 1;
 	}
 
@@ -200,7 +200,7 @@ setup_user_mark(const char mark, const char directory[], const char file[],
 	}
 	else
 	{
-		status_bar_errorf("Only user's marks can be loaded, but got: %c", mark);
+		ui_sb_errf("Only user's marks can be loaded, but got: %c", mark);
 	}
 }
 
@@ -255,7 +255,7 @@ is_mark_points_to(const mark_t *mark, const char directory[], const char file[])
 }
 
 int
-check_mark_directory(FileView *view, char m)
+check_mark_directory(view_t *view, char m)
 {
 	int custom;
 	const mark_t *const mark = get_mark_by_name(m);
@@ -267,7 +267,7 @@ check_mark_directory(FileView *view, char m)
 
 	custom = flist_custom_active(view);
 
-	if(custom && view->custom.type == CV_TREE && strcmp(mark->file, "..") == 0)
+	if(custom && cv_tree(view->custom.type) && strcmp(mark->file, "..") == 0)
 	{
 		return flist_find_entry(view, mark->file, mark->directory);
 	}
@@ -292,7 +292,7 @@ check_mark_directory(FileView *view, char m)
 }
 
 int
-goto_mark(FileView *view, char mark)
+goto_mark(view_t *view, char mark)
 {
 	switch(mark)
 	{
@@ -312,7 +312,7 @@ goto_mark(FileView *view, char mark)
 /* Navigates the view to given mark if it's valid.  Returns new value for
  * save_msg flag. */
 static int
-navigate_to_mark(FileView *view, char m)
+navigate_to_mark(view_t *view, char m)
 {
 	const mark_t *const mark = get_mark_by_name(m);
 
@@ -324,15 +324,15 @@ navigate_to_mark(FileView *view, char m)
 
 	if(!char_is_one_of(valid_marks, m))
 	{
-		status_bar_message("Invalid mark name");
+		ui_sb_msg("Invalid mark name");
 	}
 	else if(is_empty(mark))
 	{
-		status_bar_message("Mark is not set");
+		ui_sb_msg("Mark is not set");
 	}
 	else
 	{
-		status_bar_message("Mark is invalid");
+		ui_sb_msg("Mark is invalid");
 	}
 
 	fview_cursor_redraw(view);
