@@ -21,7 +21,7 @@
 #define VIFM__MENUS__ALL_H__
 
 #include "apropos_menu.h"
-#include "bookmarks_menu.h"
+#include "bmarks_menu.h"
 #include "cabbrevs_menu.h"
 #include "colorscheme_menu.h"
 #include "commands_menu.h"
@@ -34,6 +34,7 @@
 #include "jobs_menu.h"
 #include "locate_menu.h"
 #include "map_menu.h"
+#include "marks_menu.h"
 #include "registers_menu.h"
 #include "trash_menu.h"
 #include "trashes_menu.h"
@@ -45,4 +46,4 @@
 #endif /* VIFM__MENUS__ALL_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
