@@ -20,10 +20,20 @@
 #ifndef VIFM__MENUS__DIRHISTORY_MENU_H__
 #define VIFM__MENUS__DIRHISTORY_MENU_H__
 
-#include "../ui/ui.h"
+#include "../utils/test_helpers.h"
+
+struct view_t;
 
 /* Returns non-zero if status bar message should be saved. */
-int show_history_menu(FileView *view);
+int show_history_menu(struct view_t *view);
+
+#ifdef TEST
+#include "../utils/string_array.h"
+#endif
+
+TSTATIC_DEFS(
+	strlist_t list_dir_history(struct view_t *view, int *pos);
+)
 
 #endif /* VIFM__MENUS__DIRHISTORY_MENU_H__ */
 
