@@ -44,15 +44,15 @@
 static const char * form_filetype_menu_entry(assoc_record_t prog,
 		int descr_width);
 static const char * form_filetype_data_entry(assoc_record_t prog);
-static int execute_filetype_cb(FileView *view, menu_data_t *m);
-static KHandlerResponse filetypes_khandler(FileView *view, menu_data_t *m,
+static int execute_filetype_cb(view_t *view, menu_data_t *m);
+static KHandlerResponse filetypes_khandler(view_t *view, menu_data_t *m,
 		const wchar_t keys[]);
 static void fill_menu_from_records(menu_data_t *m,
 		const assoc_records_t *records);
 static int max_desc_len(const assoc_records_t *records);
 
 int
-show_file_menu(FileView *view, int background)
+show_file_menu(view_t *view, int background)
 {
 	static menu_data_t m;
 
@@ -73,7 +73,7 @@ show_file_menu(FileView *view, int background)
 	magic = get_magic_handlers(typed_name);
 	free(typed_name);
 
-	init_menu_data(&m, view, strdup("Filetype associated commands"),
+	menus_init_data(&m, view, strdup("Filetype associated commands"),
 			strdup("No programs set for this filetype"));
 
 	m.execute_handler = &execute_filetype_cb;
@@ -108,7 +108,7 @@ show_file_menu(FileView *view, int background)
 				form_filetype_menu_entry(magic.list[i], max_len));
 	}
 
-	return display_menu(m.state, view);
+	return menus_enter(m.state, view);
 }
 
 /* Returns pointer to a statically allocated buffer */
@@ -131,7 +131,7 @@ form_filetype_menu_entry(assoc_record_t prog, int descr_width)
 	}
 	else
 	{
-		snprintf(result, sizeof(result), "%s", prog.command);
+		copy_str(result, sizeof(result), prog.command);
 	}
 	return result;
 }
@@ -148,7 +148,7 @@ form_filetype_data_entry(assoc_record_t prog)
 /* Callback that is invoked when menu item is selected.  Should return non-zero
  * to stay in menu mode. */
 static int
-execute_filetype_cb(FileView *view, menu_data_t *m)
+execute_filetype_cb(view_t *view, menu_data_t *m)
 {
 	if(get_current_entry(view)->type == FT_DIR && m->pos == 0)
 	{
@@ -172,7 +172,7 @@ execute_filetype_cb(FileView *view, menu_data_t *m)
 /* Menu-specific shortcut handler.  Returns code that specifies both taken
  * actions and what should be done next. */
 static KHandlerResponse
-filetypes_khandler(FileView *view, menu_data_t *m, const wchar_t keys[])
+filetypes_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 {
 	if(wcscmp(keys, L"c") == 0)
 	{
@@ -187,37 +187,37 @@ filetypes_khandler(FileView *view, menu_data_t *m, const wchar_t keys[])
 }
 
 int
-show_fileprograms_menu(FileView *view, const char fname[])
+show_fileprograms_menu(view_t *view, const char fname[])
 {
 	static menu_data_t m;
 
 	assoc_records_t file_programs;
 
-	init_menu_data(&m, view, format_str("Programs that match %s", fname),
+	menus_init_data(&m, view, format_str("Programs that match %s", fname),
 			format_str("No programs match %s", fname));
 
 	file_programs = ft_get_all_programs(fname);
 	fill_menu_from_records(&m, &file_programs);
 	ft_assoc_records_free(&file_programs);
 
-	return display_menu(m.state, view);
+	return menus_enter(m.state, view);
 }
 
 int
-show_fileviewers_menu(FileView *view, const char fname[])
+show_fileviewers_menu(view_t *view, const char fname[])
 {
 	static menu_data_t m;
 
 	assoc_records_t file_viewers;
 
-	init_menu_data(&m, view, format_str("Viewers that match %s", fname),
+	menus_init_data(&m, view, format_str("Viewers that match %s", fname),
 			format_str("No viewers match %s", fname));
 
 	file_viewers = ft_get_all_viewers(fname);
 	fill_menu_from_records(&m, &file_viewers);
 	ft_assoc_records_free(&file_viewers);
 
-	return display_menu(m.state, view);
+	return menus_enter(m.state, view);
 }
 
 /* Fills the menu with commands from association records. */
