@@ -20,10 +20,10 @@
 #ifndef VIFM__MENUS__GREP_MENU_H__
 #define VIFM__MENUS__GREP_MENU_H__
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Returns non-zero if status bar message should be saved. */
-int show_grep_menu(FileView *view, const char args[], int invert);
+int show_grep_menu(struct view_t *view, const char args[], int invert);
 
 #endif /* VIFM__MENUS__GREP_MENU_H__ */
 
