@@ -22,7 +22,7 @@
 
 #include <stddef.h> /* wchar_t */
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Result of handling key sequence by menu-specific shortcut handler. */
 typedef enum
@@ -58,14 +58,14 @@ typedef struct menu_data_t
 
 	/* Menu-specific shortcut handler, can be NULL.  Returns code that specifies
 	 * both taken actions and what should be done next. */
-	KHandlerResponse (*key_handler)(FileView *view, struct menu_data_t *m,
+	KHandlerResponse (*key_handler)(struct view_t *view, struct menu_data_t *m,
 			const wchar_t keys[]);
 
 	/* Callback that is called when menu item is selected.  Should return non-zero
 	 * to stay in menu mode. */
-	int (*execute_handler)(FileView *view, struct menu_data_t *m);
+	int (*execute_handler)(struct view_t *view, struct menu_data_t *m);
 
-	/* Text displayed by display_menu() function in case menu is empty, it can be
+	/* Text displayed by menus_enter() function in case menu is empty, it can be
 	 * NULL if this cannot happen. */
 	char *empty_msg;
 
@@ -84,97 +84,105 @@ typedef struct menu_data_t
 }
 menu_data_t;
 
+/* Menu data management. */
+
 /* Fills fields of menu_data_t structure with some safe values.  empty_msg is
- * text displayed by display_menu() function in case menu is empty, it can be
- * NULL if this cannot happen and will be freed by reset_menu_data(). */
-void init_menu_data(menu_data_t *m, FileView *view, char title[],
+ * text displayed by menus_enter() function in case menu is empty, it can be
+ * NULL if this cannot happen and will be freed by menus_reset_data(). */
+void menus_init_data(menu_data_t *m, struct view_t *view, char title[],
 		char empty_msg[]);
 
+/* Changes active menu data. */
+void menus_replace_data(menu_data_t *m);
+
 /* Frees resources associated with the menu and clears menu window. */
-void reset_menu_data(menu_data_t *m);
+void menus_reset_data(menu_data_t *m);
 
-void setup_menu(void);
+/* Menu entering/reentering and transformation. */
 
-/* Removes current menu item and redraws the menu. */
-void remove_current_item(menu_state_t *ms);
+/* Prepares menu, draws it and switches to the menu mode.  Returns non-zero if
+ * status bar message should be saved. */
+int menus_enter(menu_state_t *m, struct view_t *view);
+
+/* Restore previously saved menu.  Returns non-zero if status bar message should
+ * be saved. */
+int menus_unstash(struct view_t *view);
+
+/* Moves menu items into custom view.  Returns zero on success, otherwise
+ * non-zero is returned. */
+int menus_to_custom_view(menu_state_t *m, struct view_t *view, int very);
+
+/* Either makes a menu or custom view out of command output.  Returns non-zero
+ * if status bar message should be saved. */
+int menus_capture(struct view_t *view, const char cmd[], int user_sh,
+		menu_data_t *m, int custom_view, int very_custom_view);
+
+/* Menu drawing. */
 
 /* Erases current menu item in menu window. */
-void menu_current_line_erase(menu_state_t *m);
+void menus_erase_current(menu_state_t *m);
 
-void move_to_menu_pos(int pos, menu_state_t *m);
+/* Redraws all screen elements used by menus. */
+void menus_full_redraw(menu_state_t *m);
 
-void redraw_menu(menu_state_t *m);
+/* Redraws only menu list itself. */
+void menus_partial_redraw(menu_state_t *m);
 
-void draw_menu(menu_state_t *m);
+/* Menu operations. */
+
+/* Updates current position in the menu. */
+void menus_set_pos(menu_state_t *m, int pos);
+
+/* Removes current menu item and redraws the menu. */
+void menus_remove_current(menu_state_t *ms);
 
 /* Navigates to/open path specification.  Specification can contain colon
  * followed by a line number when try_open is not zero.  Returns zero on
  * successful parsing and performed try to handle the file otherwise non-zero is
  * returned. */
-int goto_selected_file(menu_data_t *m, FileView *view, const char spec[],
+int menus_goto_file(menu_data_t *m, struct view_t *view, const char spec[],
 		int try_open);
 
 /* Navigates to directory from a menu. */
-void goto_selected_directory(FileView *view, const char path[]);
+void menus_goto_dir(struct view_t *view, const char path[]);
 
-/* Forms list of target files/directories in the current view and possibly
- * changes working directory to use relative paths.  On success returns newly
- * allocated string, which should be freed by the caller, otherwise NULL is
- * returned. */
-char * prepare_targets(FileView *view);
-
-/* Runs external command and puts its output to the m menu.  Returns non-zero if
- * status bar message should be saved. */
-int capture_output_to_menu(FileView *view, const char cmd[], int user_sh,
-		menu_state_t *m);
-
-/* Prepares menu, draws it and switches to the menu mode.  Returns non-zero if
- * status bar message should be saved. */
-int display_menu(menu_state_t *m, FileView *view);
-
-/* Restore previously saved menu.  Returns non-zero if status bar message should
- * be saved. */
-int unstash_menu(FileView *view);
-
-/* Predefined key handler for processing keys on elements of file lists.
- * Returns code that specifies both taken actions and what should be done
- * next. */
-KHandlerResponse filelist_khandler(FileView *view, menu_data_t *m,
-		const wchar_t keys[]);
-
-/* Moves menu items into custom view.  Returns zero on success, otherwise
- * non-zero is returned. */
-int menu_to_custom_view(menu_state_t *m, FileView *view, int very);
-
-/* Either makes a menu or custom view out of command output.  Returns non-zero
- * if status bar message should be saved. */
-int capture_output(FileView *view, const char cmd[], int user_sh,
-		menu_data_t *m, int custom_view, int very_custom_view);
-
-/* Performs search in requested direction.  Either continues the previous one or
- * restarts it. */
-void menus_search(menu_state_t *m, int backward);
+/* Menu search. */
 
 /* Performs search of pattern among menu items.  NULL pattern requests use of
  * the last used pattern.  Returns new value for save_msg flag, but when
  * print_errors isn't requested can return -1 to indicate issues with the
  * pattern. */
-int search_menu_list(const char pattern[], menu_data_t *m, int print_errors);
-
-/* Prints results or error message about search operation to the user. */
-void menu_print_search_msg(const menu_state_t *m);
-
-/* Reset search highlight of a menu. */
-void menus_reset_search_highlight(menu_state_t *m);
-
-/* Retrieves number of search matches in the menu.  Returns the number. */
-int menu_get_matches(menu_state_t *m);
+int menus_search(const char pattern[], menu_data_t *m, int print_errors);
 
 /* Resets search state of the menu according to specified parameters. */
-void menu_new_search(menu_state_t *m, int backward, int new_repeat_count);
+void menus_search_reset(menu_state_t *m, int backward, int new_repeat_count);
 
-/* Changes active menu data. */
-void menus_replace_menu(menu_data_t *m);
+/* Reset search highlight of a menu. */
+void menus_search_reset_hilight(menu_state_t *m);
+
+/* Performs search in requested direction.  Either continues the previous one or
+ * restarts it. */
+void menus_search_repeat(menu_state_t *m, int backward);
+
+/* Prints results or error message about search operation to the user. */
+void menus_search_print_msg(const menu_state_t *m);
+
+/* Retrieves number of search matches in the menu.  Returns the number. */
+int menus_search_matched(menu_state_t *m);
+
+/* Auxiliary functions related to menus. */
+
+/* Forms list of target files/directories in the current view and possibly
+ * changes working directory to use relative paths.  On success returns newly
+ * allocated string, which should be freed by the caller, otherwise NULL is
+ * returned. */
+char * menus_get_targets(struct view_t *view);
+
+/* Predefined key handler for processing keys on elements of file lists.
+ * Returns code that specifies both taken actions and what should be done
+ * next. */
+KHandlerResponse menus_def_khandler(struct view_t *view, menu_data_t *m,
+		const wchar_t keys[]);
 
 #endif /* VIFM__MENUS__MENUS_H__ */
 
