@@ -29,19 +29,13 @@
 #include "menus.h"
 
 int
-show_undolist_menu(FileView *view, int with_details)
+show_undolist_menu(view_t *view, int with_details)
 {
-	char **p;
-
 	static menu_data_t m;
-	init_menu_data(&m, view, strdup("Undolist"), strdup("Undolist is empty"));
+	menus_init_data(&m, view, strdup("Undolist"), strdup("Undolist is empty"));
 
 	m.items = undolist(with_details);
-	p = m.items;
-	while(*p++ != NULL)
-	{
-		++m.len;
-	}
+	m.len = count_strings(m.items);
 
 	/* Add additional entry before setting position. */
 	if(m.len > 0)
@@ -49,7 +43,7 @@ show_undolist_menu(FileView *view, int with_details)
 		m.len = add_to_string_array(&m.items, m.len, 1, "list end");
 	}
 
-	move_to_menu_pos(get_undolist_pos(with_details), m.state);
+	menus_set_pos(m.state, get_undolist_pos(with_details));
 
 	/* Add current position mark to menu item. */
 	if(m.len > 0)
@@ -64,7 +58,7 @@ show_undolist_menu(FileView *view, int with_details)
 		}
 	}
 
-	return display_menu(m.state, view);
+	return menus_enter(m.state, view);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
