@@ -31,7 +31,6 @@
 #include <wctype.h> /* iswprint() */
 
 #include "../cfg/config.h"
-#include "../cfg/hist.h"
 #include "../compat/curses.h"
 #include "../compat/fs_limits.h"
 #include "../compat/reallocarray.h"
@@ -48,7 +47,9 @@
 #include "../ui/statusbar.h"
 #include "../ui/statusline.h"
 #include "../ui/ui.h"
+#include "../utils/hist.h"
 #include "../utils/macros.h"
+#include "../utils/matcher.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/test_helpers.h"
@@ -186,7 +187,7 @@ static void cmd_ctrl_xslash(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xa(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xc(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xxc(key_info_t key_info, keys_info_t *keys_info);
-static void paste_short_path(FileView *view);
+static void paste_short_path(view_t *view);
 static void cmd_ctrl_xd(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xxd(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xe(key_info_t key_info, keys_info_t *keys_info);
@@ -194,7 +195,7 @@ static void cmd_ctrl_xxe(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xm(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xr(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xxr(key_info_t key_info, keys_info_t *keys_info);
-static void paste_short_path_root(FileView *view);
+static void paste_short_path_root(view_t *view);
 static void cmd_ctrl_xt(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xxt(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_xequals(key_info_t key_info, keys_info_t *keys_info);
@@ -452,9 +453,12 @@ input_line_changed(void)
 	}
 	else if(prev_mode == MENU_MODE)
 	{
-		menu_redraw();
+		menu_full_redraw();
 	}
 
+	/* Hardware cursor is moved on the screen only on refresh, so refresh status
+	 * bar to force cursor moving there before it becomes visible again. */
+	wrefresh(status_bar);
 	curs_set(1);
 }
 
@@ -471,7 +475,7 @@ handle_empty_input(void)
 		}
 		else
 		{
-			(void)search_menu_list("", sub_mode_ptr, 0);
+			(void)menus_search("", sub_mode_ptr, 0);
 		}
 	}
 
@@ -509,8 +513,8 @@ handle_nonempty_input(void)
 			break;
 		case CLS_MENU_FSEARCH:
 		case CLS_MENU_BSEARCH:
-			result = search_menu_list(mbinput, sub_mode_ptr, 0);
-			update_state(result, menu_get_matches(sub_mode_ptr));
+			result = menus_search(mbinput, sub_mode_ptr, 0);
+			update_state(result, menus_search_matched(sub_mode_ptr));
 			break;
 		case CLS_FILTER:
 			set_local_filter(mbinput);
@@ -585,7 +589,7 @@ enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
 {
 	wchar_t *wcmd;
 	const wchar_t *wprompt;
-	complete_cmd_func complete_func;
+	complete_cmd_func complete_func = NULL;
 
 	if(cl_sub_mode == CLS_FILTER && curr_view->custom.type == CV_DIFF)
 	{
@@ -607,6 +611,7 @@ enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
 	if(sub_mode == CLS_COMMAND || sub_mode == CLS_MENU_COMMAND)
 	{
 		wprompt = L":";
+		complete_func = &complete_cmd;
 	}
 	else if(sub_mode == CLS_FILTER)
 	{
@@ -626,7 +631,6 @@ enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
 		wprompt = L"E";
 	}
 
-	complete_func = (sub_mode == CLS_FILTER) ? NULL : complete_cmd;
 	prepare_cmdline_mode(wprompt, wcmd, complete_func);
 	free(wcmd);
 }
@@ -662,7 +666,7 @@ redraw_cmdline(void)
 {
 	if(prev_mode == MENU_MODE)
 	{
-		menu_redraw();
+		menu_full_redraw();
 	}
 	else
 	{
@@ -698,6 +702,8 @@ prepare_cmdline_mode(const wchar_t prompt[], const wchar_t cmd[],
 	prev_mode = vle_mode_get();
 	vle_mode_set(CMDLINE_MODE, VMT_SECONDARY);
 
+	ui_sb_lock();
+
 	input_stat.line = vifm_wcsdup(cmd);
 	input_stat.initial_line = vifm_wcsdup(input_stat.line);
 	input_stat.index = wcslen(cmd);
@@ -754,7 +760,7 @@ save_view_port(void)
 	}
 	else
 	{
-		save_menu_pos();
+		menu_save_pos();
 	}
 }
 
@@ -764,7 +770,7 @@ set_view_port(void)
 {
 	if(prev_mode == MENU_MODE)
 	{
-		load_menu_pos();
+		menu_restore_pos();
 		return;
 	}
 
@@ -808,7 +814,7 @@ leave_cmdline_mode(void)
 		if(prev_mode == MENU_MODE)
 		{
 			wresize(menu_win, getmaxy(stdscr) - 1, getmaxx(stdscr));
-			update_menu();
+			menu_partial_redraw();
 		}
 	}
 
@@ -824,6 +830,7 @@ leave_cmdline_mode(void)
 		curs_set(0);
 	}
 	curr_stats.save_msg = 0;
+	ui_sb_unlock();
 	ui_sb_clear();
 
 	if(vle_mode_is(CMDLINE_MODE))
@@ -951,9 +958,9 @@ extedit_prompt(const char input[], int cursor_col)
 	}
 	else
 	{
-		cfg_save_prompt_history(input);
+		hists_prompt_save(input);
 
-		status_bar_error("Error querying data from external source.");
+		ui_sb_err("Error querying data from external source.");
 		curr_stats.save_msg = 1;
 	}
 
@@ -1188,8 +1195,8 @@ draw_wild_popup(int *last_pos, int *pos, int *len)
 {
 	const vle_compl_t *const items = vle_compl_get_items();
 	const int count = vle_compl_get_count() - 1;
-	const int max_height = getmaxy(stdscr) - get_required_height() -
-		ui_stat_job_bar_height() - 1;
+	const int max_height = getmaxy(stdscr) - getmaxy(status_bar)
+	                     - ui_stat_job_bar_height() - 1;
 	const int height = MIN(count, MIN(10, max_height));
 	size_t max_title_width;
 	int i, j;
@@ -1201,7 +1208,7 @@ draw_wild_popup(int *last_pos, int *pos, int *len)
 
 	wresize(stat_win, height, getmaxx(stdscr));
 	werase(stat_win);
-	ui_stat_reposition(get_required_height(), 1);
+	ui_stat_reposition(getmaxy(status_bar), 1);
 
 	max_title_width = 0U;
 	for(i = *last_pos, j = 0; i < count && j < height; ++i, ++j)
@@ -1315,9 +1322,8 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 	}
 	else if(!cfg.inc_search || prev_mode == VIEW_MODE || input[0] == '\0')
 	{
-		const char *const pattern = (input[0] == '\0')
-		                          ? cfg_get_last_search_pattern()
-		                          : input;
+		const char *const pattern = (input[0] == '\0') ? hists_search_last()
+		                                               : input;
 
 		switch(sub_mode)
 		{
@@ -1335,7 +1341,7 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 			case CLS_MENU_FSEARCH:
 			case CLS_MENU_BSEARCH:
 				curr_stats.need_update = UT_FULL;
-				curr_stats.save_msg = search_menu_list(pattern, sub_mode_ptr, 1);
+				curr_stats.save_msg = menus_search(pattern, sub_mode_ptr, 1);
 				break;
 
 			default:
@@ -1347,7 +1353,7 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 	{
 		if(prev_mode == MENU_MODE)
 		{
-			menu_print_search_msg(sub_mode_ptr);
+			menus_search_print_msg(sub_mode_ptr);
 			curr_stats.save_msg = 1;
 		}
 		else
@@ -1462,7 +1468,7 @@ save_input_to_history(const keys_info_t *keys_info, const char input[])
 {
 	if(input_stat.search_mode)
 	{
-		cfg_save_search_history(input);
+		hists_search_save(input);
 	}
 	else if(sub_mode == CLS_COMMAND)
 	{
@@ -1470,12 +1476,12 @@ save_input_to_history(const keys_info_t *keys_info, const char input[])
 		const int ignore_input = mapped_input || keys_info->recursive;
 		if(!ignore_input)
 		{
-			cfg_save_command_history(input);
+			hists_commands_save(input);
 		}
 	}
 	else if(sub_mode == CLS_PROMPT)
 	{
-		cfg_save_prompt_history(input);
+		hists_prompt_save(input);
 	}
 }
 
@@ -1725,7 +1731,7 @@ cmd_ctrl_w(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_xslash(key_info_t key_info, keys_info_t *keys_info)
 {
-	paste_str(cfg_get_last_search_pattern(), 0);
+	paste_str(hists_search_last(), 0);
 }
 
 /* Inserts value of automatic filter of active pane into current cursor
@@ -1755,13 +1761,13 @@ cmd_ctrl_xxc(key_info_t key_info, keys_info_t *keys_info)
 /* Pastes short path of the current entry of the view into current cursor
  * position. */
 static void
-paste_short_path(FileView *view)
+paste_short_path(view_t *view)
 {
 	if(flist_custom_active(view))
 	{
-		char short_path[PATH_MAX];
-		get_short_path_of(view, get_current_entry(view), 0, 0, sizeof(short_path),
-				short_path);
+		char short_path[PATH_MAX + 1];
+		get_short_path_of(view, get_current_entry(view), NF_NONE, 0,
+				sizeof(short_path), short_path);
 		paste_str(short_path, 1);
 		return;
 	}
@@ -1806,7 +1812,7 @@ cmd_ctrl_xxe(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_xm(key_info_t key_info, keys_info_t *keys_info)
 {
-	paste_str(curr_view->manual_filter.raw, 0);
+	paste_str(matcher_get_undec(curr_view->manual_filter), 0);
 }
 
 /* Inserts name root of current file of active pane into current cursor
@@ -1828,11 +1834,11 @@ cmd_ctrl_xxr(key_info_t key_info, keys_info_t *keys_info)
 /* Pastes short root of the current entry of the view into current cursor
  * position. */
 static void
-paste_short_path_root(FileView *view)
+paste_short_path_root(view_t *view)
 {
-	char short_path[PATH_MAX];
-	get_short_path_of(view, get_current_entry(view), 0, 0, sizeof(short_path),
-			short_path);
+	char short_path[PATH_MAX + 1];
+	get_short_path_of(view, get_current_entry(view), NF_NONE, 0,
+			sizeof(short_path), short_path);
 	paste_name_part(short_path, 1);
 }
 
@@ -2014,7 +2020,7 @@ cmd_meta_dot(key_info_t key_info, keys_info_t *keys_info)
 
 	stop_regular_completion();
 
-	if(hist_is_empty(&cfg.cmd_hist))
+	if(hist_is_empty(&curr_stats.cmd_hist))
 	{
 		return;
 	}
@@ -2064,9 +2070,10 @@ next_dot_completion(void)
 	char *last;
 	wchar_t *wide;
 
-	if(input_stat.dot_pos <= cfg.cmd_hist.pos)
+	if(input_stat.dot_pos <= curr_stats.cmd_hist.pos)
 	{
-		last = get_last_argument(cfg.cmd_hist.items[input_stat.dot_pos++], 1, &len);
+		last = get_last_argument(curr_stats.cmd_hist.items[input_stat.dot_pos++], 1,
+				&len);
 	}
 	else
 	{
@@ -2359,19 +2366,19 @@ pick_hist(void)
 {
 	if(sub_mode == CLS_COMMAND)
 	{
-		return &cfg.cmd_hist;
+		return &curr_stats.cmd_hist;
 	}
 	if(input_stat.search_mode)
 	{
-		return &cfg.search_hist;
+		return &curr_stats.search_hist;
 	}
 	if(sub_mode == CLS_PROMPT)
 	{
-		return &cfg.prompt_hist;
+		return &curr_stats.prompt_hist;
 	}
 	if(sub_mode == CLS_FILTER)
 	{
-		return &cfg.filter_hist;
+		return &curr_stats.filter_hist;
 	}
 	return NULL;
 }
@@ -2470,8 +2477,7 @@ update_cmdline_size(void)
 	else
 	{
 		wresize(menu_win, getmaxy(stdscr) - required_height, getmaxx(stdscr));
-		update_menu();
-		wrefresh(menu_win);
+		menu_partial_redraw();
 	}
 }
 
@@ -2651,7 +2657,7 @@ stop_regular_completion(void)
 	{
 		if(sub_mode == CLS_MENU_COMMAND)
 		{
-			menu_redraw();
+			menu_full_redraw();
 		}
 		else
 		{
