@@ -41,7 +41,9 @@
 #include "../../utils/fs.h"
 #include "../../utils/macros.h"
 #include "../../utils/path.h"
+#include "../../utils/str.h"
 #include "../../utils/test_helpers.h"
+#include "../../utils/utf8.h"
 #include "../../filelist.h"
 #include "../../flist_sel.h"
 #include "../../ops.h"
@@ -51,16 +53,16 @@
 #include "../wk.h"
 #include "msg_dialog.h"
 
-static const char * get_title(void);
+static char * get_title(int max_width);
 static int is_one_file_selected(int first_file_index);
 static int get_first_file_index(void);
 static int get_selection_size(int first_file_index);
 static void leave_attr_mode(void);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_return(key_info_t key_info, keys_info_t *keys_info);
-TSTATIC void set_perm_string(FileView *view, const int perms[13],
+TSTATIC void set_perm_string(view_t *view, const int perms[13],
 		const int origin_perms[13], int adv_perms[3]);
-static void chmod_file_in_list(FileView *view, int pos, const char *mode,
+static void chmod_file_in_list(view_t *view, int pos, const char *mode,
 		const char *inv_mode, int recurse_dirs);
 static void file_chmod(char *path, const char *mode, const char *inv_mode,
 		int recurse_dirs);
@@ -72,7 +74,7 @@ static void cmd_k(key_info_t key_info, keys_info_t *keys_info);
 static void inc_curr(void);
 static void dec_curr(void);
 
-static FileView *view;
+static view_t *view;
 static int top, bottom;
 static int curr;
 static int permnum;
@@ -122,7 +124,7 @@ init_attr_dialog_mode(void)
 }
 
 void
-enter_attr_mode(FileView *active_view)
+enter_attr_mode(view_t *active_view)
 {
 	mode_t fmode = 0;
 	mode_t diff;
@@ -211,10 +213,8 @@ enter_attr_mode(FileView *active_view)
 void
 redraw_attr_dialog(void)
 {
-	const char *title;
+	char *title;
 	int x, y;
-	size_t title_len;
-	int need_ellipsis;
 
 	werase(change_win);
 	if(file_is_dir)
@@ -277,47 +277,32 @@ redraw_attr_dialog(void)
 	mvwin(change_win, (y - (20 + (file_is_dir != 0)*2))/2, (x - 30)/2);
 	box(change_win, 0, 0);
 
-	x = getmaxx(change_win);
-	title = get_title();
-	title_len = strlen(title);
-	need_ellipsis = (title_len > (size_t)x - 2);
-
-	if(need_ellipsis)
-	{
-		x -= 3;
-		title_len = x;
-	}
-	mvwaddnstr(change_win, 0, (getmaxx(change_win) - title_len)/2, title, x - 2);
-	if(need_ellipsis)
-	{
-		waddstr(change_win, "...");
-	}
+	title = get_title(getmaxx(change_win) - 2);
+	mvwaddstr(change_win, 0, (getmaxx(change_win) - utf8_strsw(title))/2, title);
+	free(title);
 
 	checked_wmove(change_win, curr, col);
 	curs_set(1);
 	wrefresh(change_win);
 }
 
-/* Gets title of the permissions dialog.  Returns pointer to a temporary string
- * of file name in the view or to a statically allocated string. */
-static const char *
-get_title(void)
+/* Composes title for the dialog.  Returns pointer to a newly allocated
+ * string. */
+static char *
+get_title(int max_width)
 {
-	static char title[NAME_MAX + 1];
-
+	char *ellipsed, *title;
 	const int first_file_index = get_first_file_index();
 
-	if(is_one_file_selected(first_file_index))
+	if(!is_one_file_selected(first_file_index))
 	{
-		snprintf(title, sizeof(title), " %s ",
-				view->dir_entry[first_file_index].name);
-	}
-	else
-	{
-		snprintf(title, sizeof(title), " %d files ",
-				get_selection_size(first_file_index));
+		return format_str(" %d files ", get_selection_size(first_file_index));
 	}
 
+	ellipsed = right_ellipsis(view->dir_entry[first_file_index].name,
+			max_width - 2, curr_stats.ellipsis);
+	title = format_str(" %s ", ellipsed);
+	free(ellipsed);
 	return title;
 }
 
@@ -403,7 +388,7 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 }
 
 TSTATIC void
-set_perm_string(FileView *view, const int perms[13], const int origin_perms[13],
+set_perm_string(view_t *view, const int perms[13], const int origin_perms[13],
 		int adv_perms[3])
 {
 	int i = 0;
@@ -465,7 +450,7 @@ set_perm_string(FileView *view, const int perms[13], const int origin_perms[13],
 }
 
 void
-files_chmod(FileView *view, const char mode[], int recurse_dirs)
+files_chmod(view_t *view, const char mode[], int recurse_dirs)
 {
 	char undo_msg[COMMAND_GROUP_INFO_LEN];
 	dir_entry_t *entry;
@@ -504,7 +489,7 @@ files_chmod(FileView *view, const char mode[], int recurse_dirs)
 }
 
 static void
-chmod_file_in_list(FileView *view, int pos, const char *mode,
+chmod_file_in_list(view_t *view, int pos, const char *mode,
 		const char *inv_mode, int recurse_dirs)
 {
 	char path_buf[PATH_MAX];
