@@ -35,6 +35,8 @@
 #include "../../utils/log.h"
 #include "../../utils/macros.h"
 #include "../../utils/path.h"
+#include "../../utils/str.h"
+#include "../../utils/utf8.h"
 #include "../../filelist.h"
 #include "../../flist_sel.h"
 #include "../../status.h"
@@ -58,18 +60,16 @@ enum
 
 static void init(void);
 static void get_attrs(void);
-static const char * get_title(void);
+static char * get_title(int max_width);
 static int is_one_file_selected(int first_file_index);
 static int get_first_file_index(void);
 static int get_selection_size(int first_file_index);
 static void leave_attr_mode(void);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_return(key_info_t key_info, keys_info_t *keys_info);
-static void set_attrs(FileView *view, const int *attrs,
-		const int *origin_attrs);
-static void files_attrib(FileView *view, DWORD add, DWORD sub,
-		int recurse_dirs);
-static void attrib_file_in_list(FileView *view, int pos, DWORD add, DWORD sub,
+static void set_attrs(view_t *view, const int *attrs, const int *origin_attrs);
+static void files_attrib(view_t *view, DWORD add, DWORD sub, int recurse_dirs);
+static void attrib_file_in_list(view_t *view, int pos, DWORD add, DWORD sub,
 		int recurse_dirs);
 static void file_attrib(char *path, DWORD add, DWORD sub, int recurse_dirs);
 static void cmd_G(key_info_t key_info, keys_info_t *keys_info);
@@ -85,7 +85,7 @@ static void draw_curr(void);
 /* properties dialog width */
 static const int WIDTH = 29;
 
-static FileView *view;
+static view_t *view;
 static int top, bottom;
 static int curr;
 static int attr_num;
@@ -154,7 +154,7 @@ init_attr_dialog_mode(void)
 
 /* enters properties change mode */
 void
-enter_attr_mode(FileView *active_view)
+enter_attr_mode(view_t *active_view)
 {
 	if(curr_stats.load_stage < 2)
 		return;
@@ -222,26 +222,23 @@ get_attrs(void)
 	memcpy(origin_attrs, attrs, sizeof(attrs));
 }
 
-/* Gets title of the permissions dialog.  Returns pointer to a temporary string
- * of file name in the view or to a statically allocated string. */
-static const char *
-get_title(void)
+/* Composes title for the dialog.  Returns pointer to a newly allocated
+ * string. */
+static char *
+get_title(int max_width)
 {
-	static char title[NAME_MAX + 1];
-
+	char *ellipsed, *title;
 	const int first_file_index = get_first_file_index();
 
-	if(is_one_file_selected(first_file_index))
+	if(!is_one_file_selected(first_file_index))
 	{
-		snprintf(title, sizeof(title), " %s ",
-				view->dir_entry[first_file_index].name);
-	}
-	else
-	{
-		snprintf(title, sizeof(title), " %d files ",
-				get_selection_size(first_file_index));
+		return format_str(" %d files ", get_selection_size(first_file_index));
 	}
 
+	ellipsed = right_ellipsis(view->dir_entry[first_file_index].name,
+			max_width - 2, curr_stats.ellipsis);
+	title = format_str(" %s ", ellipsed);
+	free(ellipsed);
 	return title;
 }
 
@@ -293,15 +290,12 @@ get_selection_size(int first_file_index)
 	return selection_size;
 }
 
-/* redraws properties change dialog */
 void
 redraw_attr_dialog(void)
 {
-	const char *title;
+	char *title;
 	int i;
 	int x, y;
-	size_t title_len;
-	int need_ellipsis;
 
 	werase(change_win);
 	if(file_is_dir)
@@ -328,21 +322,9 @@ redraw_attr_dialog(void)
 	mvwin(change_win, (y - getmaxy(change_win))/2, (x - getmaxx(change_win))/2);
 	box(change_win, 0, 0);
 
-	x = getmaxx(change_win);
-	title = get_title();
-	title_len = strlen(title);
-	need_ellipsis = (title_len > (size_t)x - 2);
-
-	if(need_ellipsis)
-	{
-		x -= 3;
-		title_len = x;
-	}
-	mvwaddnstr(change_win, 0, (getmaxx(change_win) - title_len)/2, title, x - 2);
-	if(need_ellipsis)
-	{
-		waddstr(change_win, "...");
-	}
+	title = get_title(getmaxx(change_win) - 2);
+	mvwaddstr(change_win, 0, (getmaxx(change_win) - utf8_strsw(title))/2, title);
+	free(title);
 
 	draw_curr();
 }
@@ -378,7 +360,7 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 
 /* sets file properties according to users input. forms attribute change mask */
 static void
-set_attrs(FileView *view, const int *attrs, const int *origin_attrs)
+set_attrs(view_t *view, const int *attrs, const int *origin_attrs)
 {
 	int i;
 	DWORD add_mask = 0;
@@ -404,7 +386,7 @@ set_attrs(FileView *view, const int *attrs, const int *origin_attrs)
 
 /* Changes attributes of files in the view. */
 static void
-files_attrib(FileView *view, DWORD add, DWORD sub, int recurse_dirs)
+files_attrib(view_t *view, DWORD add, DWORD sub, int recurse_dirs)
 {
 	char undo_msg[COMMAND_GROUP_INFO_LEN];
 	dir_entry_t *entry;
@@ -441,7 +423,7 @@ files_attrib(FileView *view, DWORD add, DWORD sub, int recurse_dirs)
 }
 
 /* Changes properties of a single file. */
-static void attrib_file_in_list(FileView *view, int pos, DWORD add, DWORD sub,
+static void attrib_file_in_list(view_t *view, int pos, DWORD add, DWORD sub,
 		int recurse_dirs)
 {
 	char path_buf[PATH_MAX];
