@@ -152,7 +152,7 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 {
-	schedule_redraw();
+	stats_redraw_schedule();
 }
 
 /* Agrees to the query. */
@@ -240,7 +240,7 @@ prompt_error_msg(const char title[], const char message[])
 	char portion[1024];
 	do
 	{
-		snprintf(portion, sizeof(portion), "%s", message);
+		copy_str(portion, sizeof(portion), message);
 		message += strlen(portion);
 		if(prompt_error_msg_internal(title, portion, 1))
 		{
