@@ -112,7 +112,7 @@ static int quit_cmd(const cmd_info_t *cmd_info);
 static int write_cmd(const cmd_info_t *cmd_info);
 static void leave_menu_mode(int reset_selection);
 
-static FileView *view;
+static view_t *view;
 static menu_data_t *menu;
 static int last_search_backward;
 static int was_redraw;
@@ -256,7 +256,7 @@ skip_at_beginning(int id, const char *args)
 }
 
 void
-init_menu_mode(void)
+menu_init_mode(void)
 {
 	int ret_code;
 
@@ -290,7 +290,7 @@ key_handler(wchar_t key)
 }
 
 void
-enter_menu_mode(menu_data_t *m, FileView *active_view)
+menu_enter_mode(menu_data_t *m, view_t *active_view)
 {
 	if(curr_stats.load_stage < 2)
 	{
@@ -311,13 +311,13 @@ enter_menu_mode(menu_data_t *m, FileView *active_view)
 }
 
 void
-reenter_menu_mode(menu_data_t *m)
+menu_reenter_mode(menu_data_t *m)
 {
 	assert(vle_mode_is(MENU_MODE) && "Can't reenter if not in menu mode.");
 	assert(m->len > 0 && "Menu cannot be empty.");
 
-	menus_replace_menu(m);
-	redraw_menu(m->state);
+	menus_replace_data(m);
+	menus_full_redraw(m->state);
 	menu = m;
 }
 
@@ -333,17 +333,17 @@ menu_post(void)
 {
 	if(curr_stats.need_update != UT_NONE)
 	{
-		menu_redraw();
+		menu_full_redraw();
 		curr_stats.need_update = UT_NONE;
 	}
-	status_bar_message(curr_stats.save_msg ? NULL : "");
+	ui_sb_msg(curr_stats.save_msg ? NULL : "");
 }
 
 void
-menu_redraw(void)
+menu_full_redraw(void)
 {
 	was_redraw = 1;
-	redraw_menu(menu->state);
+	menus_full_redraw(menu->state);
 }
 
 static void
@@ -353,7 +353,7 @@ cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info)
 	{
 		const int s = get_effective_menu_scroll_offset(menu);
 		const int off = (getmaxy(menu_win) - 2) - SCROLL_GAP;
-		menu->pos = get_last_visible_line(menu) - off;
+		menu->pos = menu_last_line(menu) - off;
 		change_menu_top(menu, -off);
 		if(cfg.scroll_off > 0 &&
 				menu->top + (getmaxy(menu_win) - 3) - menu->pos < s)
@@ -361,7 +361,7 @@ cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info)
 			menu->pos -= s - (menu->top + (getmaxy(menu_win) - 3) - menu->pos);
 		}
 
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -382,7 +382,7 @@ static void
 cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 {
 	const int s = get_effective_menu_scroll_offset(menu);
-	menu_current_line_erase(menu->state);
+	menus_erase_current(menu->state);
 	menu->top += DIV_ROUND_UP(getmaxy(menu_win) - 3, 2);
 	menu->pos += DIV_ROUND_UP(getmaxy(menu_win) - 3, 2);
 	if(cfg.scroll_off > 0 && menu->pos - menu->top < s)
@@ -390,7 +390,7 @@ cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 		menu->pos += s - (menu->pos - menu->top);
 	}
 
-	update_menu();
+	menu_partial_redraw();
 }
 
 static void
@@ -405,7 +405,7 @@ cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info)
 		}
 
 		++menu->top;
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -423,7 +423,7 @@ cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info)
 			menu->pos += s - (menu->pos - menu->top);
 		}
 
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -431,7 +431,7 @@ cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info)
 static int
 can_scroll_menu_down(const menu_data_t *menu)
 {
-	return get_last_visible_line(menu) < menu->len - 1;
+	return menu_last_line(menu) < menu->len - 1;
 }
 
 /* Moves top line of the menu ensuring that its value is correct. */
@@ -443,7 +443,7 @@ change_menu_top(menu_data_t *const menu, int delta)
 }
 
 int
-get_last_visible_line(const menu_data_t *menu)
+menu_last_line(const menu_data_t *menu)
 {
 	return menu->top + (getmaxy(menu_win) - 2) - 1;
 }
@@ -452,7 +452,7 @@ get_last_visible_line(const menu_data_t *menu)
 static void
 cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 {
-	menu_redraw();
+	menu_full_redraw();
 }
 
 static void
@@ -462,21 +462,21 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 	saved_menu = menu;
-	if(menu->execute_handler != NULL && menu->execute_handler(curr_view, menu))
+	if(menu->execute_handler != NULL && menu->execute_handler(view, menu))
 	{
 		vle_mode_set(MENU_MODE, VMT_PRIMARY);
-		menu_redraw();
+		menu_full_redraw();
 		return;
 	}
 
 	if(!vle_mode_is(MENU_MODE))
 	{
-		reset_menu_data(saved_menu);
+		menus_reset_data(saved_menu);
 	}
 	else if(menu != saved_menu)
 	{
-		reset_menu_data(saved_menu);
-		update_menu();
+		menus_reset_data(saved_menu);
+		menu_partial_redraw();
 	}
 
 	update_ui_on_leaving();
@@ -500,7 +500,7 @@ static void
 cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info)
 {
 	const int s = get_effective_menu_scroll_offset(menu);
-	menu_current_line_erase(menu->state);
+	menus_erase_current(menu->state);
 
 	if(cfg.scroll_off > 0 && menu->top + getmaxy(menu_win) - menu->pos < s)
 	{
@@ -514,7 +514,7 @@ cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info)
 	}
 	menu->pos -= DIV_ROUND_UP(getmaxy(menu_win) - 3, 2);
 
-	update_menu();
+	menu_partial_redraw();
 }
 
 /* Returns scroll offset value for the menu taking menu height into account. */
@@ -536,7 +536,7 @@ cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 		}
 
 		--menu->top;
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -544,7 +544,8 @@ static void
 cmd_slash(key_info_t key_info, keys_info_t *keys_info)
 {
 	last_search_backward = 0;
-	menu_new_search(menu->state, last_search_backward, def_count(key_info.count));
+	menus_search_reset(menu->state, last_search_backward,
+			def_count(key_info.count));
 	enter_cmdline_mode(CLS_MENU_FSEARCH, "", menu);
 }
 
@@ -558,7 +559,7 @@ cmd_percent(key_info_t key_info, keys_info_t *keys_info)
 		return;
 	}
 	line = (key_info.count*menu->len)/100;
-	move_to_menu_pos(line, menu->state);
+	menus_set_pos(menu->state, line);
 	wrefresh(menu_win);
 }
 
@@ -575,7 +576,8 @@ static void
 cmd_qmark(key_info_t key_info, keys_info_t *keys_info)
 {
 	last_search_backward = 1;
-	menu_new_search(menu->state, last_search_backward, def_count(key_info.count));
+	menus_search_reset(menu->state, last_search_backward,
+			def_count(key_info.count));
 	enter_cmdline_mode(CLS_MENU_BSEARCH, "", menu);
 }
 
@@ -594,8 +596,8 @@ cmd_G(key_info_t key_info, keys_info_t *keys_info)
 		key_info.count = menu->len;
 	}
 
-	menu_current_line_erase(menu->state);
-	move_to_menu_pos(key_info.count - 1, menu->state);
+	menus_erase_current(menu->state);
+	menus_set_pos(menu->state, key_info.count - 1);
 	wrefresh(menu_win);
 }
 
@@ -611,8 +613,8 @@ cmd_H(key_info_t key_info, keys_info_t *keys_info)
 
 	top = (menu->top == 0) ? 0 : (menu->top + off);
 
-	menu_current_line_erase(menu->state);
-	move_to_menu_pos(top, menu->state);
+	menus_erase_current(menu->state);
+	menus_set_pos(menu->state, top);
 	wrefresh(menu_win);
 }
 
@@ -641,8 +643,8 @@ cmd_L(key_info_t key_info, keys_info_t *keys_info)
 		top -= off;
 	}
 
-	menu_current_line_erase(menu->state);
-	move_to_menu_pos(top - 3, menu->state);
+	menus_erase_current(menu->state);
+	menus_set_pos(menu->state, top - 3);
 	wrefresh(menu_win);
 }
 
@@ -660,8 +662,8 @@ cmd_M(key_info_t key_info, keys_info_t *keys_info)
 		new_pos = menu->top + DIV_ROUND_UP(getmaxy(menu_win) - 3, 2);
 	}
 
-	menu_current_line_erase(menu->state);
-	move_to_menu_pos(MAX(0, new_pos - 1), menu->state);
+	menus_erase_current(menu->state);
+	menus_set_pos(menu->state, MAX(0, new_pos - 1));
 	wrefresh(menu_win);
 }
 
@@ -671,7 +673,7 @@ cmd_N(key_info_t key_info, keys_info_t *keys_info)
 	key_info.count = def_count(key_info.count);
 	while(key_info.count-- > 0)
 	{
-		menus_search(menu->state, !last_search_backward);
+		menus_search_repeat(menu->state, !last_search_backward);
 	}
 }
 
@@ -686,7 +688,7 @@ cmd_b(key_info_t key_info, keys_info_t *keys_info)
 static void
 dump_into_custom_view(int very)
 {
-	if(menu_to_custom_view(menu->state, view, very) != 0)
+	if(menus_to_custom_view(menu->state, view, very) != 0)
 	{
 		show_error_msg("Menu transformation",
 				"No valid paths discovered in menu content");
@@ -750,8 +752,8 @@ pass_combination_to_khandler(const wchar_t keys[])
 static void
 cmd_gg(key_info_t key_info, keys_info_t *keys_info)
 {
-	menu_current_line_erase(menu->state);
-	move_to_menu_pos(def_count(key_info.count) - 1, menu->state);
+	menus_erase_current(menu->state);
+	menus_set_pos(menu->state, def_count(key_info.count) - 1);
 	wrefresh(menu_win);
 }
 
@@ -760,9 +762,9 @@ cmd_j(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(menu->pos != menu->len - 1)
 	{
-		menu_current_line_erase(menu->state);
+		menus_erase_current(menu->state);
 		menu->pos += def_count(key_info.count);
-		move_to_menu_pos(menu->pos, menu->state);
+		menus_set_pos(menu->state, menu->pos);
 		wrefresh(menu_win);
 	}
 }
@@ -772,9 +774,9 @@ cmd_k(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(menu->pos != 0)
 	{
-		menu_current_line_erase(menu->state);
+		menus_erase_current(menu->state);
 		menu->pos -= def_count(key_info.count);
-		move_to_menu_pos(menu->pos, menu->state);
+		menus_set_pos(menu->state, menu->pos);
 		wrefresh(menu_win);
 	}
 }
@@ -785,7 +787,7 @@ cmd_n(key_info_t key_info, keys_info_t *keys_info)
 	key_info.count = def_count(key_info.count);
 	while(key_info.count-- > 0)
 	{
-		menus_search(menu->state, last_search_backward);
+		menus_search_repeat(menu->state, last_search_backward);
 	}
 }
 
@@ -808,7 +810,7 @@ cmd_v(key_info_t key_info, keys_info_t *keys_info)
 		qf = 0;
 	}
 
-	endwin();
+	ui_shutdown();
 	curr_stats.need_update = UT_FULL;
 
 	vi_cmd = cfg_get_vicmd(&bg);
@@ -864,7 +866,7 @@ cmd_zb(key_info_t key_info, keys_info_t *keys_info)
 		{
 			menu->top = menu->pos - (getmaxy(menu_win) - 3);
 		}
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -874,7 +876,7 @@ cmd_zH(key_info_t key_info, keys_info_t *keys_info)
 	if(menu->hor_pos != 0)
 	{
 		menu->hor_pos = MAX(0, menu->hor_pos - (getmaxx(menu_win) - 4));
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -882,7 +884,7 @@ static void
 cmd_zL(key_info_t key_info, keys_info_t *keys_info)
 {
 	menu->hor_pos += getmaxx(menu_win) - 4;
-	update_menu();
+	menu_partial_redraw();
 }
 
 static void
@@ -891,7 +893,7 @@ cmd_zh(key_info_t key_info, keys_info_t *keys_info)
 	if(menu->hor_pos != 0)
 	{
 		menu->hor_pos = MAX(0, menu->hor_pos - def_count(key_info.count));
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -899,7 +901,7 @@ static void
 cmd_zl(key_info_t key_info, keys_info_t *keys_info)
 {
 	menu->hor_pos += def_count(key_info.count);
-	update_menu();
+	menu_partial_redraw();
 }
 
 static void
@@ -911,7 +913,7 @@ cmd_zt(key_info_t key_info, keys_info_t *keys_info)
 			menu->top = menu->pos;
 		else
 			menu->top = menu->len - (getmaxy(menu_win) - 3 + 1);
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -927,7 +929,7 @@ cmd_zz(key_info_t key_info, keys_info_t *keys_info)
 		else
 			menu->top = menu->pos - DIV_ROUND_UP(getmaxy(menu_win) - 3, 2);
 
-		update_menu();
+		menu_partial_redraw();
 	}
 }
 
@@ -939,10 +941,10 @@ all_lines_visible(const menu_data_t *const menu)
 }
 
 void
-update_menu(void)
+menu_partial_redraw(void)
 {
-	draw_menu(menu->state);
-	move_to_menu_pos(menu->pos, menu->state);
+	menus_partial_redraw(menu->state);
+	menus_set_pos(menu->state, menu->pos);
 	wrefresh(menu_win);
 }
 
@@ -951,8 +953,8 @@ goto_cmd(const cmd_info_t *cmd_info)
 {
 	if(cmd_info->end != NOT_DEF)
 	{
-		menu_current_line_erase(menu->state);
-		move_to_menu_pos(cmd_info->end, menu->state);
+		menus_erase_current(menu->state);
+		menus_set_pos(menu->state, cmd_info->end);
 		wrefresh(menu_win);
 	}
 	return 0;
@@ -962,7 +964,7 @@ goto_cmd(const cmd_info_t *cmd_info)
 static int
 nohlsearch_cmd(const cmd_info_t *cmd_info)
 {
-	menus_reset_search_highlight(menu->state);
+	menus_search_reset_hilight(menu->state);
 	return 0;
 }
 
@@ -987,14 +989,14 @@ write_cmd(const cmd_info_t *cmd_info)
 }
 
 void
-save_menu_pos(void)
+menu_save_pos(void)
 {
 	saved_top = menu->top;
 	saved_pos = menu->pos;
 }
 
 void
-load_menu_pos(void)
+menu_restore_pos(void)
 {
 	menu->top = saved_top;
 	menu->pos = saved_pos;
@@ -1037,7 +1039,7 @@ leave_menu_mode(int reset_selection)
 		return;
 	}
 
-	reset_menu_data(menu);
+	menus_reset_data(menu);
 
 	if(reset_selection)
 	{
@@ -1051,11 +1053,11 @@ leave_menu_mode(int reset_selection)
 }
 
 void
-execute_cmdline_command(const char cmd[])
+menu_run_command(const char cmd[])
 {
-	if(exec_command(cmd, curr_view, CIT_COMMAND) < 0)
+	if(exec_command(cmd, view, CIT_COMMAND) < 0)
 	{
-		status_bar_error("An error occurred while trying to execute command");
+		ui_sb_err("An error occurred while trying to execute command");
 	}
 	init_cmds(0, &cmds_conf);
 }
