@@ -35,18 +35,19 @@
 #include <wchar.h> /* wcscpy() */
 
 #include "../cfg/config.h"
-#include "../cfg/hist.h"
 #include "../compat/curses.h"
 #include "../compat/fs_limits.h"
 #include "../compat/reallocarray.h"
 #include "../engine/keys.h"
 #include "../engine/mode.h"
+#include "../engine/variables.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../ui/cancellation.h"
 #include "../ui/fileview.h"
 #include "../ui/quickview.h"
 #include "../ui/statusbar.h"
 #include "../ui/ui.h"
+#include "../utils/hist.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
@@ -96,7 +97,6 @@ static void cmd_return(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_o(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_r(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info);
-static void scroll_view(ssize_t offset);
 static void cmd_ctrl_wH(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wJ(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wK(key_info_t key_info, keys_info_t *keys_info);
@@ -114,7 +114,7 @@ static void cmd_ctrl_ww(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wx(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wz(key_info_t key_info, keys_info_t *keys_info);
 static int is_left_or_top(void);
-static FileView * pick_view(void);
+static view_t * pick_view(void);
 static void cmd_ctrl_x(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_shift_tab(key_info_t key_info, keys_info_t *keys_info);
@@ -206,7 +206,7 @@ static void cmd_q_equals(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_t(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_u(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_yy(key_info_t key_info, keys_info_t *keys_info);
-static int calc_pick_files_end_pos(const FileView *view, int count);
+static int calc_pick_files_end_pos(const view_t *view, int count);
 static void cmd_y_selector(key_info_t key_info, keys_info_t *keys_info);
 static void yank(key_info_t key_info, keys_info_t *keys_info);
 static void free_list_of_file_indexes(keys_info_t *keys_info);
@@ -227,6 +227,10 @@ static void cmd_lb_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_rb_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_lb_d(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_rb_d(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_lb_r(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_rb_r(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_lb_R(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_rb_R(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_lb_s(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_rb_s(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_lb_z(key_info_t key_info, keys_info_t *keys_info);
@@ -234,7 +238,7 @@ static void cmd_rb_z(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_left_curly_bracket(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_right_curly_bracket(key_info_t key_info,
 		keys_info_t *keys_info);
-static void pick_files(FileView *view, int end, keys_info_t *keys_info);
+static void pick_files(view_t *view, int end, keys_info_t *keys_info);
 static void selector_S(key_info_t key_info, keys_info_t *keys_info);
 static void selector_a(key_info_t key_info, keys_info_t *keys_info);
 static void selector_s(key_info_t key_info, keys_info_t *keys_info);
@@ -398,6 +402,10 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_RB WK_c,       {{&cmd_rb_c}, .descr = "go to next mismatch"}},
 	{WK_LB WK_d,       {{&cmd_lb_d}, .descr = "go to previous dir"}},
 	{WK_RB WK_d,       {{&cmd_rb_d}, .descr = "go to next dir"}},
+	{WK_LB WK_r,       {{&cmd_lb_r}, .descr = "navigate to previous sibling dir"}},
+	{WK_RB WK_r,       {{&cmd_rb_r}, .descr = "navigate to next sibling dir"}},
+	{WK_LB WK_R,       {{&cmd_lb_R}, .descr = "navigate to previous sibling dir (wrap)"}},
+	{WK_RB WK_R,       {{&cmd_rb_R}, .descr = "navigate to next sibling dir (wrap)"}},
 	{WK_LB WK_s,       {{&cmd_lb_s}, .descr = "go to previous selected entry"}},
 	{WK_RB WK_s,       {{&cmd_rb_s}, .descr = "go to next selected entry"}},
 	{WK_LB WK_z,       {{&cmd_lb_z}, .descr = "go to first sibling"}},
@@ -422,7 +430,7 @@ static keys_add_info_t builtin_cmds[] = {
 	{{K(KEY_END)},               {{&cmd_G},       .descr = "go to the last item"}},
 	{{K(KEY_BTAB)},              {{&cmd_shift_tab}, .descr = "switch to view pane"}},
 #else
-	{WK_ESC L"[Z",            {{&cmd_shift_tab}, .descr = "switch to view pane"}},
+	{WK_ESC L"[Z",               {{&cmd_shift_tab}, .descr = "switch to view pane"}},
 #endif /* ENABLE_EXTENDED_KEYS */
 };
 
@@ -501,7 +509,7 @@ cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(can_scroll_up(curr_view))
 	{
-		page_scroll(get_last_visible_cell(curr_view), -1);
+		page_scroll(fpos_get_last_visible_cell(curr_view), -1);
 	}
 }
 
@@ -518,9 +526,10 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_last_line(curr_view))
+	if(fpos_can_move_down(curr_view))
 	{
-		scroll_view(curr_view->window_cells/2);
+		curr_view->list_pos = fpos_half_scroll(curr_view, 1);
+		ui_view_schedule_redraw(curr_view);
 	}
 }
 
@@ -528,7 +537,8 @@ cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(correct_list_pos_on_scroll_down(curr_view, 1))
+	if(fpos_has_hidden_bottom(curr_view) &&
+			correct_list_pos_on_scroll_down(curr_view, 1))
 	{
 		scroll_down(curr_view, 1);
 		redraw_current_view();
@@ -544,14 +554,19 @@ cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info)
 	}
 }
 
-/* Scrolls pane by one view in both directions. The direction should be 1 or
+/* Scrolls pane by one view in both directions.  The direction should be 1 or
  * -1. */
 static void
 page_scroll(int base, int direction)
 {
-	/* Two lines gap. */
-	int offset = (curr_view->window_rows - 1)*curr_view->column_count;
-	curr_view->list_pos = base + direction*offset;
+	enum { HOR_GAP_SIZE = 2, VER_GAP_SIZE = 1 };
+	int old_pos = curr_view->list_pos;
+	int offset = fview_is_transposed(curr_view)
+	    ? (MAX(1, curr_view->column_count - VER_GAP_SIZE))*curr_view->window_rows
+	    : (curr_view->window_rows - HOR_GAP_SIZE)*curr_view->column_count;
+	int new_pos = base + direction*offset
+	            + old_pos%curr_view->run_size - base%curr_view->run_size;
+	curr_view->list_pos = MAX(0, MIN(curr_view->list_rows - 1, new_pos));
 	scroll_by_files(curr_view, direction*offset);
 	redraw_current_view();
 }
@@ -589,6 +604,8 @@ cmd_emarkemark(key_info_t key_info, keys_info_t *keys_info)
 	{
 		strcpy(prefix, ".!");
 	}
+
+	set_count_vars(key_info.count);
 	enter_cmdline_mode(CLS_COMMAND, prefix, NULL);
 }
 
@@ -663,44 +680,44 @@ cmd_ctrl_r(key_info_t key_info, keys_info_t *keys_info)
 	curr_stats.confirmed = 0;
 	ui_cancellation_reset();
 
-	status_bar_message("Redoing...");
+	ui_sb_msg("Redoing...");
 
 	ret = redo_group();
 
 	if(ret == 0)
 	{
 		ui_views_reload_visible_filelists();
-		status_bar_message("Redone one group");
+		ui_sb_msg("Redone one group");
 	}
 	else if(ret == -2)
 	{
 		ui_views_reload_visible_filelists();
-		status_bar_error("Redone one group with errors");
+		ui_sb_err("Redone one group with errors");
 	}
 	else if(ret == -1)
 	{
-		status_bar_message("Nothing to redo");
+		ui_sb_msg("Nothing to redo");
 	}
 	else if(ret == -3)
 	{
-		status_bar_error("Can't redo group, it was skipped");
+		ui_sb_err("Can't redo group, it was skipped");
 	}
 	else if(ret == -4)
 	{
-		status_bar_error("Can't redo what wasn't undone");
+		ui_sb_err("Can't redo what wasn't undone");
 	}
 	else if(ret == -6)
 	{
-		status_bar_message("Group redo skipped by user");
+		ui_sb_msg("Group redo skipped by user");
 	}
 	else if(ret == -7)
 	{
 		ui_views_reload_visible_filelists();
-		status_bar_message("Redoing was cancelled");
+		ui_sb_msg("Redoing was cancelled");
 	}
 	else if(ret == 1)
 	{
-		status_bar_error("Redo operation was skipped due to previous errors");
+		ui_sb_err("Redo operation was skipped due to previous errors");
 	}
 	curr_stats.save_msg = 1;
 }
@@ -709,24 +726,13 @@ cmd_ctrl_r(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_first_line(curr_view))
+	if(fpos_can_move_up(curr_view))
 	{
-		scroll_view(-(ssize_t)curr_view->window_cells/2);
+		curr_view->list_pos = fpos_half_scroll(curr_view, 0);
+		ui_view_schedule_redraw(curr_view);
 	}
 }
 
-/* Scrolls view by specified number of files. */
-static void
-scroll_view(ssize_t offset)
-{
-	offset = ROUND_DOWN(offset, curr_view->column_count);
-	curr_view->list_pos += offset;
-	correct_list_pos(curr_view, offset);
-	go_to_start_of_line(curr_view);
-	scroll_by_files(curr_view, offset);
-	redraw_current_view();
-}
-
 /* Go to bottom-right window. */
 static void
 cmd_ctrl_wb(key_info_t key_info, keys_info_t *keys_info)
@@ -875,7 +881,7 @@ is_left_or_top(void)
 }
 
 /* Picks view to operate on for Ctrl-W set of shortcuts.  Returns the view. */
-static FileView *
+static view_t *
 pick_view(void)
 {
 	if(vle_mode_is(VIEW_MODE))
@@ -891,7 +897,7 @@ static void
 cmd_ctrl_wx(key_info_t key_info, keys_info_t *keys_info)
 {
 	switch_panes();
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
 		change_window();
 		(void)try_switch_into_view_mode();
@@ -917,7 +923,8 @@ cmd_ctrl_x(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(correct_list_pos_on_scroll_up(curr_view, 1))
+	if(fpos_has_hidden_top(curr_view) &&
+			correct_list_pos_on_scroll_up(curr_view, 1))
 	{
 		scroll_up(curr_view, 1);
 		redraw_current_view();
@@ -951,9 +958,9 @@ go_to_other_window(void)
 static int
 try_switch_into_view_mode(void)
 {
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
-		enter_view_mode(other_view, 0);
+		view_enter_mode(other_view, 0);
 		return 1;
 	}
 	return 0;
@@ -1011,12 +1018,11 @@ find_goto(int ch, int count, int backward, keys_info_t *keys_info)
 static void
 cmd_G(key_info_t key_info, keys_info_t *keys_info)
 {
-	int new_pos;
 	if(key_info.count == NO_COUNT_GIVEN)
+	{
 		key_info.count = curr_view->list_rows;
-
-	new_pos = ROUND_DOWN(key_info.count - 1, curr_view->column_count);
-	pick_or_move(keys_info, new_pos);
+	}
+	pick_or_move(keys_info, key_info.count - 1);
 }
 
 /* Calculate size of selected directories ignoring cached sizes. */
@@ -1045,10 +1051,7 @@ cmd_gf(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_gg(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
-
-	pick_or_move(keys_info, key_info.count - 1);
+	pick_or_move(keys_info, def_count(key_info.count) - 1);
 }
 
 /* Goes to parent directory regardless of ls-like state. */
@@ -1084,7 +1087,7 @@ cmd_gs(key_info_t key_info, keys_info_t *keys_info)
 	reg = regs_find(tolower(key_info.reg));
 	if(reg == NULL || reg->nfiles < 1)
 	{
-		status_bar_error(reg == NULL ? "No such register" : "Register is empty");
+		ui_sb_err(reg == NULL ? "No such register" : "Register is empty");
 		curr_stats.save_msg = 1;
 		return;
 	}
@@ -1152,24 +1155,21 @@ cmd_gv(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_H(key_info_t key_info, keys_info_t *keys_info)
 {
-	size_t new_pos = get_window_top_pos(curr_view);
-	pick_or_move(keys_info, new_pos);
+	pick_or_move(keys_info, fpos_get_top_pos(curr_view));
 }
 
 /* Go to the last file in window. */
 static void
 cmd_L(key_info_t key_info, keys_info_t *keys_info)
 {
-	size_t new_pos = get_window_bottom_pos(curr_view);
-	pick_or_move(keys_info, new_pos);
+	pick_or_move(keys_info, fpos_get_bottom_pos(curr_view));
 }
 
 /* Go to middle line of window. */
 static void
 cmd_M(key_info_t key_info, keys_info_t *keys_info)
 {
-	size_t new_pos = get_window_middle_pos(curr_view);
-	pick_or_move(keys_info, new_pos);
+	pick_or_move(keys_info, fpos_get_middle_pos(curr_view));
 }
 
 /* Picks files or moves cursor depending whether key was pressed as a selector
@@ -1331,9 +1331,30 @@ cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 	{
 		snprintf(prefix, ARRAY_LEN(prefix), ".,.+%d", key_info.count - 1);
 	}
+
+	set_count_vars(key_info.count);
 	enter_cmdline_mode(CLS_COMMAND, prefix, NULL);
 }
 
+void
+set_count_vars(int count)
+{
+	/* TODO: move this to a better place someday, nowhere to place right now. */
+
+	var_val_t value;
+	var_t var;
+
+	value.integer = (count == NO_COUNT_GIVEN ? 0 : count);
+	var = var_new(VTYPE_INT, value);
+	setvar("v:count", var);
+	var_free(var);
+
+	value.integer = (count == NO_COUNT_GIVEN ? 1 : count);
+	var = var_new(VTYPE_INT, value);
+	setvar("v:count1", var);
+	var_free(var);
+}
+
 /* Continues navigation to word which starts with specified character in initial
  * direction. */
 static void
@@ -1414,7 +1435,7 @@ cmd_cp(key_info_t key_info, keys_info_t *keys_info)
 }
 
 void
-normal_cmd_cp(FileView *view, key_info_t key_info)
+normal_cmd_cp(view_t *view, key_info_t key_info)
 {
 #ifndef _WIN32
 	char mode[32];
@@ -1433,7 +1454,7 @@ normal_cmd_cp(FileView *view, key_info_t key_info)
 		 * position (when called from visual mode). */
 		redraw_view(view);
 
-		status_bar_errorf("Incorrect file mode: %s", mode);
+		ui_sb_errf("Incorrect file mode: %s", mode);
 		curr_stats.save_msg = 1;
 		return;
 	}
@@ -1548,13 +1569,13 @@ cmd_dp(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_e(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
-		status_bar_error("Another type of file viewing is activated");
+		ui_sb_err("Another type of file viewing is activated");
 		curr_stats.save_msg = 1;
 		return;
 	}
-	enter_view_mode(curr_view, 1);
+	view_enter_mode(curr_view, 1);
 }
 
 /* Navigates to next word which starts with specified character. */
@@ -1575,7 +1596,10 @@ cmd_h(key_info_t key_info, keys_info_t *keys_info)
 
 	if(!ui_view_displays_columns(curr_view))
 	{
-		go_to_prev(key_info, keys_info, 1);
+		if(fpos_can_move_left(curr_view))
+		{
+			go_to_prev(key_info, keys_info, fpos_get_hor_step(curr_view));
+		}
 		return;
 	}
 
@@ -1606,9 +1630,9 @@ cmd_i(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_j(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_last_line(curr_view))
+	if(fpos_can_move_down(curr_view))
 	{
-		go_to_next(key_info, keys_info, curr_view->column_count);
+		go_to_next(key_info, keys_info, fpos_get_ver_step(curr_view));
 	}
 }
 
@@ -1616,9 +1640,9 @@ cmd_j(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_k(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_first_line(curr_view))
+	if(fpos_can_move_up(curr_view))
 	{
-		go_to_prev(key_info, keys_info, curr_view->column_count);
+		go_to_prev(key_info, keys_info, fpos_get_ver_step(curr_view));
 	}
 }
 
@@ -1637,9 +1661,9 @@ cmd_l(key_info_t key_info, keys_info_t *keys_info)
 	{
 		cmd_return(key_info, keys_info);
 	}
-	else
+	else if(fpos_can_move_right(curr_view))
 	{
-		go_to_next(key_info, keys_info, 1);
+		go_to_next(key_info, keys_info, fpos_get_hor_step(curr_view));
 	}
 }
 
@@ -1676,7 +1700,7 @@ search(key_info_t key_info, int backward)
 
 	int found;
 
-	if(hist_is_empty(&cfg.search_hist))
+	if(hist_is_empty(&curr_stats.search_hist))
 	{
 		return;
 	}
@@ -1687,7 +1711,7 @@ search(key_info_t key_info, int backward)
 	found = 0;
 	if(curr_view->matches == 0)
 	{
-		const char *const pattern = cfg_get_last_search_pattern();
+		const char *const pattern = hists_search_last();
 		curr_stats.save_msg = (find_pattern(curr_view, pattern, backward, 1, &found,
 				0) != 0);
 		--key_info.count;
@@ -1823,48 +1847,48 @@ cmd_u(key_info_t key_info, keys_info_t *keys_info)
 	curr_stats.confirmed = 0;
 	ui_cancellation_reset();
 
-	status_bar_message("Undoing...");
+	ui_sb_msg("Undoing...");
 
 	ret = undo_group();
 
 	if(ret == 0)
 	{
 		ui_views_reload_visible_filelists();
-		status_bar_message("Undone one group");
+		ui_sb_msg("Undone one group");
 	}
 	else if(ret == -2)
 	{
 		ui_views_reload_visible_filelists();
-		status_bar_error("Undone one group with errors");
+		ui_sb_err("Undone one group with errors");
 	}
 	else if(ret == -1)
 	{
-		status_bar_message("Nothing to undo");
+		ui_sb_msg("Nothing to undo");
 	}
 	else if(ret == -3)
 	{
-		status_bar_error("Can't undo group, it was skipped");
+		ui_sb_err("Can't undo group, it was skipped");
 	}
 	else if(ret == -4)
 	{
-		status_bar_error("Can't undo what wasn't redone");
+		ui_sb_err("Can't undo what wasn't redone");
 	}
 	else if(ret == -5)
 	{
-		status_bar_error("Operation cannot be undone");
+		ui_sb_err("Operation cannot be undone");
 	}
 	else if(ret == -6)
 	{
-		status_bar_message("Group undo skipped by user");
+		ui_sb_msg("Group undo skipped by user");
 	}
 	else if(ret == -7)
 	{
 		ui_views_reload_visible_filelists();
-		status_bar_message("Undoing was cancelled");
+		ui_sb_msg("Undoing was cancelled");
 	}
 	else if(ret == 1)
 	{
-		status_bar_error("Undo operation was skipped due to previous errors");
+		ui_sb_err("Undo operation was skipped due to previous errors");
 	}
 	curr_stats.save_msg = 1;
 }
@@ -1890,7 +1914,7 @@ cmd_yy(key_info_t key_info, keys_info_t *keys_info)
 /* Calculates end position for pick_files(...) function using cursor position
  * and count of a command.  Considers possible integer overflow. */
 static int
-calc_pick_files_end_pos(const FileView *view, int count)
+calc_pick_files_end_pos(const view_t *view, int count)
 {
 	/* Way of comparing values makes difference!  This way it will work even when
 	 * count equals to INT_MAX.  Don't change it! */
@@ -1940,32 +1964,32 @@ free_list_of_file_indexes(keys_info_t *keys_info)
 static void
 cmd_zM(key_info_t key_info, keys_info_t *keys_info)
 {
-	restore_filename_filter(curr_view);
+	name_filters_restore(curr_view);
 	local_filter_restore(curr_view);
-	set_dot_files_visible(curr_view, 0);
+	dot_filter_set(curr_view, 0);
 }
 
 /* Remove filename filter. */
 static void
 cmd_zO(key_info_t key_info, keys_info_t *keys_info)
 {
-	remove_filename_filter(curr_view);
+	name_filters_remove(curr_view);
 }
 
 /* Show all hidden files. */
 static void
 cmd_zR(key_info_t key_info, keys_info_t *keys_info)
 {
-	remove_filename_filter(curr_view);
+	name_filters_remove(curr_view);
 	local_filter_remove(curr_view);
-	set_dot_files_visible(curr_view, 1);
+	dot_filter_set(curr_view, 1);
 }
 
 /* Toggle dot files visibility. */
 static void
 cmd_za(key_info_t key_info, keys_info_t *keys_info)
 {
-	toggle_dot_files(curr_view);
+	dot_filter_toggle(curr_view);
 }
 
 /* Excludes entries from custom view. */
@@ -1981,7 +2005,7 @@ normal_cmd_zb(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(can_scroll_up(curr_view))
 	{
-		int bottom = get_window_bottom_pos(curr_view);
+		const int bottom = fpos_get_bottom_pos(curr_view);
 		scroll_up(curr_view, bottom - curr_view->list_pos);
 		redraw_current_view();
 	}
@@ -1991,21 +2015,21 @@ normal_cmd_zb(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_zf(key_info_t key_info, keys_info_t *keys_info)
 {
-	filter_selected_files(curr_view);
+	name_filters_add_selection(curr_view);
 }
 
 /* Hide dot files. */
 static void
 cmd_zm(key_info_t key_info, keys_info_t *keys_info)
 {
-	set_dot_files_visible(curr_view, 0);
+	dot_filter_set(curr_view, 0);
 }
 
 /* Show all the dot files. */
 static void
 cmd_zo(key_info_t key_info, keys_info_t *keys_info)
 {
-	set_dot_files_visible(curr_view, 1);
+	dot_filter_set(curr_view, 1);
 }
 
 /* Reset local filter. */
@@ -2079,6 +2103,38 @@ cmd_rb_d(key_info_t key_info, keys_info_t *keys_info)
 	pick_or_move(keys_info, flist_next_dir(curr_view));
 }
 
+/* Navigates to previous sibling directory. */
+static void
+cmd_lb_r(key_info_t key_info, keys_info_t *keys_info)
+{
+	curr_stats.save_msg = (go_to_sibling_dir(curr_view,
+				-def_count(key_info.count), 0) != 0);
+}
+
+/* Navigates to next sibling directory. */
+static void
+cmd_rb_r(key_info_t key_info, keys_info_t *keys_info)
+{
+	curr_stats.save_msg = (go_to_sibling_dir(curr_view,
+				def_count(key_info.count), 0) != 0);
+}
+
+/* Navigates to previous sibling directory with wrapping. */
+static void
+cmd_lb_R(key_info_t key_info, keys_info_t *keys_info)
+{
+	curr_stats.save_msg = (go_to_sibling_dir(curr_view,
+				-def_count(key_info.count), 1) != 0);
+}
+
+/* Navigates to next sibling directory with wrapping. */
+static void
+cmd_rb_R(key_info_t key_info, keys_info_t *keys_info)
+{
+	curr_stats.save_msg = (go_to_sibling_dir(curr_view,
+				def_count(key_info.count), 1) != 0);
+}
+
 /* Go to or pick files until and including previous selected entry or do
  * nothing. */
 static void
@@ -2131,7 +2187,7 @@ normal_cmd_zt(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(can_scroll_down(curr_view))
 	{
-		int top = get_window_top_pos(curr_view);
+		const int top = fpos_get_top_pos(curr_view);
 		scroll_down(curr_view, curr_view->list_pos - top);
 		redraw_current_view();
 	}
@@ -2141,16 +2197,16 @@ normal_cmd_zt(key_info_t key_info, keys_info_t *keys_info)
 void
 normal_cmd_zz(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!all_files_visible(curr_view))
+	if(!fpos_are_all_files_visible(curr_view))
 	{
-		int middle = get_window_middle_pos(curr_view);
+		const int middle = fpos_get_middle_pos(curr_view);
 		scroll_by_files(curr_view, curr_view->list_pos - middle);
 		redraw_current_view();
 	}
 }
 
 static void
-pick_files(FileView *view, int end, keys_info_t *keys_info)
+pick_files(view_t *view, int end, keys_info_t *keys_info)
 {
 	int delta, i, x;
 
@@ -2261,7 +2317,7 @@ selector_s(key_info_t key_info, keys_info_t *keys_info)
 }
 
 int
-find_npattern(FileView *view, const char pattern[], int backward,
+find_npattern(view_t *view, const char pattern[], int backward,
 		int print_errors)
 {
 	const int nrepeats = search_repeat - 1;
