@@ -73,7 +73,7 @@ enum
 };
 
 /* Describes view state and its properties. */
-typedef struct
+struct view_info_t
 {
 	/* Data of the view. */
 	char **lines;     /* List of real lines. */
@@ -98,27 +98,25 @@ typedef struct
 	int search_repeat;        /* Saved count prefix of search commands. */
 
 	/* The rest of the state. */
-	FileView *view; /* File view association with the view. */
+	view_t *view;   /* File view association with the view. */
 	char *filename; /* Full path to the file being viewed. */
 	char *viewer;   /* When non-NULL, specifies custom preview command (no
 	                   implicit %c). */
-	int abandoned;  /* Whether view mode was abandoned. */
-	int graphics;   /* Whether viewer presumably displays graphics. */
+	int detached;   /* Whether view mode was detached. */
+	int graphical;  /* Whether viewer presumably displays graphics. */
 	int wrap;       /* Whether lines are wrapped. */
-}
-view_info_t;
+};
 
 /* View information structure indexes and count. */
 enum
 {
-	VI_QV,    /* Index of view information structure for quickview window. */
 	VI_LWIN,  /* Index of view information structure for left window. */
 	VI_RWIN,  /* Index of view information structure for right window. */
 	VI_COUNT, /* Number of view information structures. */
 };
 
-static int try_ressurect_abandoned(const char full_path[], int explore);
-static void try_redraw_explore_view(const FileView *const view, int vi_index);
+static int try_resurrect_detached(const char full_path[], int explore);
+static void try_redraw_explore_view(const view_t *view);
 static void reset_view_info(view_info_t *vi);
 static void init_view_info(view_info_t *vi);
 static void free_view_info(view_info_t *vi);
@@ -134,7 +132,7 @@ static void cmd_ctrl_wH(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wJ(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wK(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wL(key_info_t key_info, keys_info_t *keys_info);
-static FileView * get_active_view(void);
+static view_t * get_active_view(void);
 static void cmd_ctrl_wb(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wh(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_wj(key_info_t key_info, keys_info_t *keys_info);
@@ -184,14 +182,15 @@ static void cmd_w(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_z(key_info_t key_info, keys_info_t *keys_info);
 static void update_with_win(key_info_t *const key_info);
 static int is_trying_the_same_file(void);
-static int get_file_to_explore(const FileView *view, char buf[],
-		size_t buf_len);
+static int get_file_to_explore(const view_t *view, char buf[], size_t buf_len);
 static int forward_if_changed(view_info_t *vi);
 static int scroll_to_bottom(view_info_t *vi);
 static void reload_view(view_info_t *vi, int silent);
+static view_info_t * view_info_alloc(void);
 
-static view_info_t view_info[VI_COUNT];
-static view_info_t* vi = &view_info[VI_QV];
+/* Points to current (for quick view) or last used (for explore mode)
+ * view_info_t structure. */
+static view_info_t *vi;
 
 static keys_add_info_t builtin_cmds[] = {
 	{WK_C_b,           {{&cmd_b},      .descr = "scroll page up"}},
@@ -291,12 +290,12 @@ static keys_add_info_t builtin_cmds[] = {
 	{{K(KEY_END)},               {{&cmd_G}, .descr = "scroll to the end"}},
 	{{K(KEY_BTAB)},              {{&cmd_q}, .descr = "leave view mode"}},
 #else
-	{WK_ESCAPE L"[Z",            {{&cmd_q}, .descr = "leave view mode"}},
+	{WK_ESC L"[Z",               {{&cmd_q}, .descr = "leave view mode"}},
 #endif /* ENABLE_EXTENDED_KEYS */
 };
 
 void
-init_view_mode(void)
+view_init_mode(void)
 {
 	int ret_code;
 
@@ -304,14 +303,10 @@ init_view_mode(void)
 	assert(ret_code == 0);
 
 	(void)ret_code;
-
-	init_view_info(&view_info[VI_QV]);
-	init_view_info(&view_info[VI_LWIN]);
-	init_view_info(&view_info[VI_RWIN]);
 }
 
 void
-enter_view_mode(FileView *view, int explore)
+view_enter_mode(view_t *view, int explore)
 {
 	char full_path[PATH_MAX];
 
@@ -321,8 +316,8 @@ enter_view_mode(FileView *view, int explore)
 		return;
 	}
 
-	/* Either make use of abandoned view or prune it. */
-	if(try_ressurect_abandoned(full_path, explore) == 0)
+	/* Either make use of detached view or prune it. */
+	if(try_resurrect_detached(full_path, explore) == 0)
 	{
 		ui_views_update_titles();
 		return;
@@ -356,7 +351,7 @@ enter_view_mode(FileView *view, int explore)
 }
 
 void
-make_abandoned_view(FileView *view, const char cmd[])
+view_detached_make(view_t *view, const char cmd[])
 {
 	char full_path[PATH_MAX];
 
@@ -372,7 +367,7 @@ make_abandoned_view(FileView *view, const char cmd[])
 	vi->view = view;
 	vi->viewer = strdup(cmd);
 	vi->filename = strdup(full_path);
-	vi->abandoned = 1;
+	vi->detached = 1;
 
 	if(load_view_data(vi, "File viewing", full_path, NOSILENT) != 0)
 	{
@@ -384,12 +379,13 @@ make_abandoned_view(FileView *view, const char cmd[])
 	view_redraw();
 }
 
-/* Either makes use of abandoned view or prunes it.  Returns zero on success,
+/* Either makes use of detached view or prunes it.  Returns zero on success,
  * otherwise non-zero is returned. */
 static int
-try_ressurect_abandoned(const char full_path[], int explore)
+try_resurrect_detached(const char full_path[], int explore)
 {
-	const int same_file = vi->abandoned
+	const int same_file = vi != NULL
+	                   && vi->detached
 	                   && vi->view == (explore ? curr_view : other_view)
 	                   && vi->filename != NULL
 	                   && stroscmp(vi->filename, full_path) == 0;
@@ -412,7 +408,7 @@ try_ressurect_abandoned(const char full_path[], int explore)
 }
 
 void
-try_activate_view_mode(void)
+view_try_activate_mode(void)
 {
 	if(curr_view->explore_mode)
 	{
@@ -427,7 +423,7 @@ view_pre(void)
 	if(curr_stats.save_msg == 0)
 	{
 		const char *const suffix = vi->auto_forward ? "(auto forwarding)" : "";
-		status_bar_messagef("-- VIEW -- %s", suffix);
+		ui_sb_msgf("-- VIEW -- %s", suffix);
 		curr_stats.save_msg = 2;
 	}
 }
@@ -453,8 +449,8 @@ view_redraw(void)
 {
 	view_info_t *saved_vi = vi;
 
-	try_redraw_explore_view(&lwin, VI_LWIN);
-	try_redraw_explore_view(&rwin, VI_RWIN);
+	try_redraw_explore_view(&lwin);
+	try_redraw_explore_view(&rwin);
 
 	if(!lwin.explore_mode && !rwin.explore_mode)
 	{
@@ -467,17 +463,17 @@ view_redraw(void)
 /* Redraws view in explore mode if view is really in explore mode and is visible
  * on the screen. */
 static void
-try_redraw_explore_view(const FileView *const view, int vi_index)
+try_redraw_explore_view(const view_t *const view)
 {
 	if(view->explore_mode && ui_view_is_visible(view))
 	{
-		vi = &view_info[vi_index];
+		vi = view->vi;
 		redraw();
 	}
 }
 
 void
-leave_view_mode(void)
+view_leave_mode(void)
 {
 	reset_view_info(vi);
 
@@ -502,7 +498,7 @@ leave_view_mode(void)
 }
 
 void
-view_explore_mode_quit(FileView *view)
+view_quit_explore_mode(view_t *view)
 {
 	assert(!vle_mode_is(VIEW_MODE) && "Unexpected mode.");
 	if(!view->explore_mode)
@@ -512,7 +508,7 @@ view_explore_mode_quit(FileView *view)
 
 	view->explore_mode = 0;
 
-	reset_view_info(&view_info[(view == &lwin) ? VI_LWIN : VI_RWIN]);
+	reset_view_info(view->vi);
 
 	redraw_view(view);
 	ui_view_title_update(view);
@@ -628,7 +624,7 @@ draw(void)
 	const int searched = (vi->last_search_backward != -1);
 	esc_state state;
 
-	if(vi->graphics)
+	if(vi->graphical)
 	{
 		const char *cmd = qv_get_viewer(vi->filename);
 		cmd = (cmd != NULL) ? ma_get_clear_cmd(cmd) : NULL;
@@ -671,7 +667,7 @@ draw(void)
 }
 
 int
-find_vwpattern(const char *pattern, int backward)
+view_find_pattern(const char pattern[], int backward)
 {
 	int err;
 
@@ -683,7 +679,7 @@ find_vwpattern(const char *pattern, int backward)
 	vi->last_search_backward = -1;
 	if((err = regcomp(&vi->re, pattern, get_regexp_cflags(pattern))) != 0)
 	{
-		status_bar_errorf("Invalid pattern: %s", get_regexp_error(err, &vi->re));
+		ui_sb_errf("Invalid pattern: %s", get_regexp_error(err, &vi->re));
 		regfree(&vi->re);
 		draw();
 		return 1;
@@ -705,73 +701,51 @@ cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_wH(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(is_right_or_bottom())
-	{
-		view_switch_views();
-	}
 	move_window(get_active_view(), 0, 1);
 }
 
 static void
 cmd_ctrl_wJ(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(is_top_or_left())
-	{
-		view_switch_views();
-	}
 	move_window(get_active_view(), 1, 0);
 }
 
 static void
 cmd_ctrl_wK(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(is_right_or_bottom())
-	{
-		view_switch_views();
-	}
 	move_window(get_active_view(), 1, 1);
 }
 
 static void
 cmd_ctrl_wL(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(is_top_or_left())
-	{
-		view_switch_views();
-	}
 	move_window(get_active_view(), 0, 0);
 }
 
 /* Gets pointer to the currently active view from the view point of the
  * view-mode.  Returns that pointer. */
-static FileView *
+static view_t *
 get_active_view(void)
 {
-	return curr_stats.view ? other_view : curr_view;
+	return (curr_stats.preview.on ? other_view : curr_view);
 }
 
 void
-view_switch_views(void)
+view_panes_swapped(void)
 {
-	view_info_t saved_vi = view_info[VI_LWIN];
-	view_info[VI_LWIN] = view_info[VI_RWIN];
-	view_info[VI_RWIN] = saved_vi;
-
-	if(vi == &view_info[VI_LWIN])
+	if(curr_stats.preview.explore != NULL)
 	{
-		vi = &view_info[VI_RWIN];
-	}
-	else if(vi == &view_info[VI_RWIN])
-	{
-		vi = &view_info[VI_LWIN];
-	}
-	else
-	{
-		view_info[VI_QV].view = curr_view;
+		curr_stats.preview.explore->view = curr_view;
 	}
 
-	view_info[VI_LWIN].view = &lwin;
-	view_info[VI_RWIN].view = &rwin;
+	if(lwin.vi != NULL)
+	{
+		lwin.vi->view = &lwin;
+	}
+	if(rwin.vi != NULL)
+	{
+		rwin.vi->view = &rwin;
+	}
 }
 
 /* Go to bottom-right window. */
@@ -863,7 +837,7 @@ is_right_or_bottom(void)
 static int
 is_top_or_left(void)
 {
-	const FileView *const top_or_left = curr_view->explore_mode ? &lwin : &rwin;
+	const view_t *const top_or_left = curr_view->explore_mode ? &lwin : &rwin;
 	return curr_view == top_or_left;
 }
 
@@ -877,7 +851,7 @@ cmd_ctrl_wv(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_ww(key_info_t key_info, keys_info_t *keys_info)
 {
-	vi->abandoned = 1;
+	vi->detached = 1;
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 	if(curr_view->explore_mode)
 	{
@@ -885,7 +859,7 @@ cmd_ctrl_ww(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	ui_views_update_titles();
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
 		qv_draw(curr_view);
 	}
@@ -895,10 +869,10 @@ cmd_ctrl_ww(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_wx(key_info_t key_info, keys_info_t *keys_info)
 {
-	vi->abandoned = 1;
+	vi->detached = 1;
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 	switch_panes();
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
 		change_window();
 	}
@@ -908,7 +882,7 @@ cmd_ctrl_wx(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_wz(key_info_t key_info, keys_info_t *keys_info)
 {
-	leave_view_mode();
+	view_leave_mode();
 	qv_hide();
 }
 
@@ -940,7 +914,7 @@ cmd_tab(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(!curr_view->explore_mode)
 	{
-		leave_view_mode();
+		view_leave_mode();
 		return;
 	}
 
@@ -959,8 +933,14 @@ cmd_tab(key_info_t key_info, keys_info_t *keys_info)
 static void
 pick_vi(int explore)
 {
-	const int index = !explore ? VI_QV : (curr_view == &lwin ? VI_LWIN : VI_RWIN);
-	vi = &view_info[index];
+	view_info_t **ptr = (explore ? &curr_view->vi : &curr_stats.preview.explore);
+
+	if(*ptr == NULL)
+	{
+		*ptr = view_info_alloc();
+	}
+
+	vi = *ptr;
 }
 
 static void
@@ -1100,13 +1080,13 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 	else
 	{
 		const char *const v = (vi->viewer != NULL) ? vi->viewer : viewer;
-		const int graphics = is_graphics_viewer(v);
-		FileView *const curr = curr_view;
-		curr_view = curr_stats.view ? curr_view
+		const int graphical = is_graphical_viewer(v);
+		view_t *const curr = curr_view;
+		curr_view = curr_stats.preview.on ? curr_view
 		          : (vi->view != NULL) ? vi->view : curr_view;
 		curr_stats.preview_hint = vi->view;
 
-		if(graphics)
+		if(graphical)
 		{
 			/* Wait a bit to let terminal emulator do actual refresh (at least some
 			 * of them need this). */
@@ -1132,9 +1112,9 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 			return 3;
 		}
 
-		if(graphics)
+		if(graphical)
 		{
-			vi->graphics = 1;
+			vi->graphical = 1;
 		}
 
 		ui_cancellation_reset();
@@ -1434,14 +1414,14 @@ get_part(const char line[], int offset, size_t max_len, char part[])
 static void
 display_error(const char error_msg[])
 {
-	status_bar_error(error_msg);
+	ui_sb_err(error_msg);
 	curr_stats.save_msg = 1;
 }
 
 static void
 cmd_q(key_info_t key_info, keys_info_t *keys_info)
 {
-	leave_view_mode();
+	view_leave_mode();
 }
 
 static void
@@ -1480,7 +1460,7 @@ cmd_v(key_info_t key_info, keys_info_t *keys_info)
 	/* In some cases two redraw operations are needed, otherwise TUI is not fully
 	 * redrawn. */
 	update_screen(UT_REDRAW);
-	schedule_redraw();
+	stats_redraw_schedule();
 }
 
 static void
@@ -1522,9 +1502,9 @@ set_from_default_win(key_info_t *const key_info)
 }
 
 int
-draw_abandoned_view_mode(void)
+view_detached_draw(void)
 {
-	if(!vi->abandoned)
+	if(vi == NULL || !vi->detached)
 	{
 		return 0;
 	}
@@ -1560,7 +1540,7 @@ is_trying_the_same_file(void)
 /* Gets full path to the file that will be explored (the current file of the
  * view).  Returns non-zero if file cannot be explored. */
 static int
-get_file_to_explore(const FileView *view, char buf[], size_t buf_len)
+get_file_to_explore(const view_t *view, char buf[], size_t buf_len)
 {
 	const dir_entry_t *const curr = get_current_entry(view);
 	if(fentry_is_fake(curr))
@@ -1596,13 +1576,13 @@ view_check_for_updates(void)
 {
 	int need_redraw = 0;
 
-	need_redraw += forward_if_changed(&view_info[VI_QV]);
-	need_redraw += forward_if_changed(&view_info[VI_LWIN]);
-	need_redraw += forward_if_changed(&view_info[VI_RWIN]);
+	need_redraw += forward_if_changed(curr_stats.preview.explore);
+	need_redraw += forward_if_changed(lwin.vi);
+	need_redraw += forward_if_changed(rwin.vi);
 
 	if(need_redraw)
 	{
-		schedule_redraw();
+		stats_redraw_schedule();
 	}
 }
 
@@ -1613,7 +1593,7 @@ forward_if_changed(view_info_t *vi)
 {
 	filemon_t mon;
 
-	if(!vi->auto_forward)
+	if(vi == NULL || !vi->auto_forward)
 	{
 		return 0;
 	}
@@ -1675,5 +1655,30 @@ reload_view(view_info_t *vi, int silent)
 	}
 }
 
+const char *
+view_detached_get_viewer(void)
+{
+	return (vi == NULL ? NULL : vi->viewer);
+}
+
+/* Allocates and initializes view mode information.  Returns pointer to it. */
+static view_info_t *
+view_info_alloc(void)
+{
+	view_info_t *const vi = malloc(sizeof(*vi));
+	init_view_info(vi);
+	return vi;
+}
+
+void
+view_info_free(view_info_t *vi)
+{
+	if(vi != NULL)
+	{
+		free_view_info(vi);
+		free(vi);
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
