@@ -26,7 +26,6 @@
 #include <string.h>
 
 #include "../cfg/config.h"
-#include "../cfg/hist.h"
 #include "../compat/curses.h"
 #include "../engine/keys.h"
 #include "../engine/mode.h"
@@ -34,6 +33,7 @@
 #include "../ui/fileview.h"
 #include "../ui/statusbar.h"
 #include "../ui/ui.h"
+#include "../utils/hist.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
@@ -67,8 +67,8 @@ typedef enum
 }
 AmendType;
 
-static void backup_selection_flags(FileView *view);
-static void restore_selection_flags(FileView *view);
+static void backup_selection_flags(view_t *view);
+static void restore_selection_flags(view_t *view);
 static void cmd_ctrl_a(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
@@ -140,7 +140,7 @@ static void cmd_y(key_info_t key_info, keys_info_t *keys_info);
 static void accept_and_leave(int save_msg);
 static void reject_and_leave(void);
 static void leave_clearing_selection(int go_to_top, int save_msg);
-static void update_marks(FileView *view);
+static void update_marks(view_t *view);
 static void cmd_zd(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zf(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_left_paren(key_info_t key_info, keys_info_t *keys_info);
@@ -159,17 +159,17 @@ static void cmd_left_curly_bracket(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_right_curly_bracket(key_info_t key_info,
 		keys_info_t *keys_info);
 static void find_goto(int ch, int count, int backward);
-static void select_up_one(FileView *view, int start_pos);
-static void select_down_one(FileView *view, int start_pos);
+static void select_up_one(view_t *view, int start_pos);
+static void select_down_one(view_t *view, int start_pos);
 static void apply_selection(int pos);
 static void revert_selection(int pos);
 static void update(void);
-static int find_update(FileView *view, int backward);
+static int find_update(view_t *view, int backward);
 static void goto_pos_force_update(int pos);
 static void goto_pos(int pos);
 static int move_pos(int pos);
 
-static FileView *view;
+static view_t *view;
 static int start_pos;
 static int last_fast_search_char;
 static int last_fast_search_backward = -1;
@@ -351,7 +351,7 @@ leave_visual_mode(int save_msg, int goto_top, int clear_selection)
 /* Stores current values of selected flags of all items in the directory for
  * future use. */
 static void
-backup_selection_flags(FileView *view)
+backup_selection_flags(view_t *view)
 {
 	int i;
 	for(i = 0; i < view->list_rows; ++i)
@@ -363,7 +363,7 @@ backup_selection_flags(FileView *view)
 /* Restore previous state of selected flags stored by
  * backup_selection_flags(). */
 static void
-restore_selection_flags(FileView *view)
+restore_selection_flags(view_t *view)
 {
 	int i;
 
@@ -392,7 +392,7 @@ cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(can_scroll_up(view))
 	{
-		page_scroll(get_last_visible_cell(view), -1);
+		page_scroll(fpos_get_last_visible_cell(view), -1);
 	}
 }
 
@@ -405,24 +405,16 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_last_line(view))
+	if(fpos_can_move_down(view))
 	{
-		size_t new_pos;
-		size_t offset = view->window_cells/2;
-		offset = ROUND_DOWN(offset, view->column_count);
-		new_pos = get_corrected_list_pos_down(view, offset);
-		new_pos = MAX(new_pos, view->list_pos + offset);
-		new_pos = MIN(new_pos, (size_t)view->list_rows);
-		new_pos = ROUND_DOWN(new_pos, view->column_count);
-		view->top_line += new_pos - view->list_pos;
-		goto_pos(new_pos);
+		goto_pos(fpos_half_scroll(view, 1));
 	}
 }
 
 static void
 cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(can_scroll_down(view))
+	if(fpos_has_hidden_top(view))
 	{
 		int new_pos = get_corrected_list_pos_down(view, view->column_count);
 		scroll_down(view, view->column_count);
@@ -444,11 +436,13 @@ cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info)
 static void
 page_scroll(int base, int direction)
 {
-	int new_pos;
-	/* Two lines gap. */
-	int lines = view->window_rows - 1;
-	int offset = lines*view->column_count;
-	new_pos = base + direction*offset;
+	enum { HOR_GAP_SIZE = 2, VER_GAP_SIZE = 1 };
+	int offset = fview_is_transposed(view)
+	           ? MAX(1, (view->column_count - VER_GAP_SIZE))*view->window_rows
+	           : (view->window_rows - HOR_GAP_SIZE)*view->column_count;
+	int new_pos = base + direction*offset
+	            + view->list_pos%view->run_size - base%view->run_size;
+	new_pos = MAX(0, MIN(view->list_rows - 1, new_pos));
 	scroll_by_files(view, direction*offset);
 	goto_pos(new_pos);
 }
@@ -483,17 +477,9 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_first_line(view))
+	if(fpos_can_move_up(view))
 	{
-		int new_pos;
-		size_t offset = view->window_cells/2;
-		offset = ROUND_DOWN(offset, view->column_count);
-		new_pos = get_corrected_list_pos_up(view, offset);
-		new_pos = MIN(new_pos, view->list_pos - (int)offset);
-		new_pos = MAX(new_pos, 0);
-		new_pos = ROUND_DOWN(new_pos, view->column_count);
-		view->top_line += new_pos - view->list_pos;
-		goto_pos(new_pos);
+		goto_pos(fpos_half_scroll(view, 0));
 	}
 }
 
@@ -519,7 +505,7 @@ call_incdec(int count)
 static void
 cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(can_scroll_up(view))
+	if(fpos_has_hidden_top(view))
 	{
 		int new_pos = get_corrected_list_pos_up(view, view->column_count);
 		scroll_up(view, view->column_count);
@@ -533,7 +519,7 @@ static void
 cmd_C(key_info_t key_info, keys_info_t *keys_info)
 {
 	int save_msg;
-	check_marking(curr_view, 0, NULL);
+	check_marking(view, 0, NULL);
 	save_msg = fops_clone(view, NULL, 0, 0, def_count(key_info.count));
 	accept_and_leave(save_msg);
 }
@@ -557,35 +543,32 @@ cmd_F(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_G(key_info_t key_info, keys_info_t *keys_info)
 {
-	int new_pos;
 	if(key_info.count == NO_COUNT_GIVEN)
+	{
 		key_info.count = view->list_rows;
-	new_pos = ROUND_DOWN(key_info.count - 1, view->column_count);
-	goto_pos(new_pos);
+	}
+	goto_pos(key_info.count - 1);
 }
 
 /* Move to the first line of window, selecting as we go. */
 static void
 cmd_H(key_info_t key_info, keys_info_t *keys_info)
 {
-	size_t new_pos = get_window_top_pos(view);
-	goto_pos(new_pos);
+	goto_pos(fpos_get_top_pos(view));
 }
 
 /* Move to the last line of window, selecting as we go. */
 static void
 cmd_L(key_info_t key_info, keys_info_t *keys_info)
 {
-	size_t new_pos = get_window_bottom_pos(view);
-	goto_pos(new_pos);
+	goto_pos(fpos_get_bottom_pos(view));
 }
 
 /* Move to middle line of the window, selecting from start position to there. */
 static void
 cmd_M(key_info_t key_info, keys_info_t *keys_info)
 {
-	size_t new_pos = get_window_middle_pos(view);
-	goto_pos(new_pos);
+	goto_pos(fpos_get_middle_pos(view));
 }
 
 static void
@@ -674,6 +657,7 @@ static void
 cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 {
 	update_marks(view);
+	set_count_vars(key_info.count);
 	enter_cmdline_mode(CLS_COMMAND, "", NULL);
 }
 
@@ -754,7 +738,7 @@ cmd_f(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_gA(key_info_t key_info, keys_info_t *keys_info)
 {
-	fops_size_bg(curr_view, 1);
+	fops_size_bg(view, 1);
 	accept_and_leave(0);
 }
 
@@ -762,7 +746,7 @@ cmd_gA(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ga(key_info_t key_info, keys_info_t *keys_info)
 {
-	fops_size_bg(curr_view, 0);
+	fops_size_bg(view, 0);
 	accept_and_leave(0);
 }
 
@@ -797,9 +781,7 @@ cmd_cw(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_gg(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
-	goto_pos(key_info.count - 1);
+	goto_pos(def_count(key_info.count) - 1);
 }
 
 static void
@@ -888,9 +870,14 @@ cmd_h(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(!ui_view_displays_columns(view))
 	{
-		go_to_prev(key_info, keys_info, 1, 1);
+		if(fpos_can_move_left(view))
+		{
+			go_to_prev(key_info, keys_info, 1, fpos_get_hor_step(view));
+		}
+		return;
 	}
-	else if(get_current_entry(view)->child_pos != 0)
+
+	if(get_current_entry(view)->child_pos != 0)
 	{
 		const dir_entry_t *entry = get_current_entry(view);
 		key_info.count = def_count(key_info.count);
@@ -900,7 +887,7 @@ cmd_h(key_info_t key_info, keys_info_t *keys_info)
 		}
 		key_info.count = 1;
 		go_to_prev(key_info, keys_info, 1,
-				view->list_pos - entry_to_pos(curr_view, entry));
+				view->list_pos - entry_to_pos(view, entry));
 	}
 }
 
@@ -914,18 +901,18 @@ cmd_i(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_j(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(ui_view_displays_columns(view) || !at_last_line(view))
+	if(fpos_can_move_down(view))
 	{
-		go_to_next(key_info, keys_info, 1, view->column_count);
+		go_to_next(key_info, keys_info, 1, fpos_get_ver_step(view));
 	}
 }
 
 static void
 cmd_k(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(ui_view_displays_columns(view) || !at_first_line(view))
+	if(fpos_can_move_up(view))
 	{
-		go_to_prev(key_info, keys_info, 1, view->column_count);
+		go_to_prev(key_info, keys_info, 1, fpos_get_ver_step(view));
 	}
 }
 
@@ -946,9 +933,9 @@ cmd_l(key_info_t key_info, keys_info_t *keys_info)
 	{
 		cmd_gl(key_info, keys_info);
 	}
-	else
+	else if(fpos_can_move_right(view))
 	{
-		go_to_next(key_info, keys_info, 1, 1);
+		go_to_next(key_info, keys_info, 1, fpos_get_hor_step(view));
 	}
 }
 
@@ -985,14 +972,14 @@ search(key_info_t key_info, int backward, int interactive)
 
 	int found;
 
-	if(hist_is_empty(&cfg.search_hist))
+	if(hist_is_empty(&curr_stats.search_hist))
 	{
 		return;
 	}
 
 	if(view->matches == 0)
 	{
-		const char *const pattern = cfg_get_last_search_pattern();
+		const char *const pattern = hists_search_last();
 		curr_stats.save_msg = find_vpattern(view, pattern, backward, interactive);
 		return;
 	}
@@ -1137,7 +1124,7 @@ leave_clearing_selection(int go_to_top, int save_msg)
 }
 
 static void
-update_marks(FileView *view)
+update_marks(view_t *view)
 {
 	char start_mark, end_mark;
 	const dir_entry_t *start_entry, *end_entry;
@@ -1174,14 +1161,14 @@ update_marks(FileView *view)
 static void
 cmd_zd(key_info_t key_info, keys_info_t *keys_info)
 {
-	flist_custom_exclude(curr_view, key_info.count == 1);
+	flist_custom_exclude(view, key_info.count == 1);
 	accept_and_leave(0);
 }
 
 static void
 cmd_zf(key_info_t key_info, keys_info_t *keys_info)
 {
-	filter_selected_files(view);
+	name_filters_add_selection(view);
 	accept_and_leave(0);
 }
 
@@ -1306,7 +1293,7 @@ find_goto(int ch, int count, int backward)
 
 /* move up one position in the window, adding to the selection list */
 static void
-select_up_one(FileView *view, int start_pos)
+select_up_one(view_t *view, int start_pos)
 {
 	view->list_pos--;
 	if(view->list_pos < 0)
@@ -1330,7 +1317,7 @@ select_up_one(FileView *view, int start_pos)
 
 /* move down one position in the window, adding to the selection list */
 static void
-select_down_one(FileView *view, int start_pos)
+select_down_one(view_t *view, int start_pos)
 {
 	++view->list_pos;
 
@@ -1461,7 +1448,7 @@ update_visual_mode(void)
 }
 
 int
-find_vpattern(FileView *view, const char pattern[], int backward,
+find_vpattern(view_t *view, const char pattern[], int backward,
 		int print_errors)
 {
 	int i;
@@ -1488,7 +1475,7 @@ find_vpattern(FileView *view, const char pattern[], int backward,
 
 /* returns non-zero when it finds something */
 static int
-find_update(FileView *view, int backward)
+find_update(view_t *view, int backward)
 {
 	const int old_pos = view->list_pos;
 	const int found = goto_search_match(view, backward);
