@@ -30,10 +30,10 @@ typedef enum
 	OP_REMOVESL, /* cl */
 	OP_COPY,     /* copy and clone */
 	OP_COPYF,    /* copy with file overwrite */
-	OP_COPYA,    /* copy with file appending file part */
+	OP_COPYA,    /* copy with appending to existing contents of destination */
 	OP_MOVE,     /* move, rename and substitute */
 	OP_MOVEF,    /* move with file overwrite */
-	OP_MOVEA,    /* move with file appending file part */
+	OP_MOVEA,    /* move file appending to existing contents of destination */
 	OP_MOVETMP1, /* multiple files rename */
 	OP_MOVETMP2, /* multiple files rename */
 	OP_MOVETMP3, /* multiple files rename */
@@ -77,16 +77,24 @@ ErrorResolutionPolicy;
  * helps to keep track of progress. */
 typedef struct
 {
-	OPS main_op;          /* Primary operation performed on items. */
-	int total;            /* Total number of items to be processed. */
-	int current;          /* Number of current item. */
-	int succeeded;        /* Number of successfully processed items. */
-	ioeta_estim_t *estim; /* When non-NULL, populated with estimates for items and
-	                         also frees it on ops_free(). */
-	const char *descr;    /* Description of operations. */
-	int shallow_eta;      /* Count only top level items, without recursion. */
-	int bg;               /* Executed in background (no user interaction). */
-	char *errors;         /* Multi-line string of errors. */
+	OPS main_op;           /* Primary operation performed on items. */
+	int total;             /* Total number of items to be processed. */
+	int current;           /* Number of current item. */
+	int succeeded;         /* Number of successfully processed items. */
+	ioeta_estim_t *estim;  /* When non-NULL, populated with estimates for items
+	                          and also frees it on ops_free(). */
+	const char *descr;     /* Description of operations. */
+	int shallow_eta;       /* Count only top level items, without recursion. */
+	int bg;                /* Executed in background (no user interaction). */
+	struct bg_op_t *bg_op; /* Information for background operation. */
+	char *errors;          /* Multi-line string of errors. */
+
+	/* It's unsafe to access global cfg object from threads performing background
+	 * operations, so copy them and use the copies. */
+	char *slow_fs_list;    /* Copy of 'slowfs' option value. */
+	char *delete_prg;      /* Copy of 'deleteprg' option value. */
+	int use_system_calls;  /* Copy of 'syscalls' option value. */
+	int fast_file_cloning; /* Copy of part of 'iooptions' option value. */
 
 	char *base_dir;   /* Base directory in which operation is taking place. */
 	char *target_dir; /* Target directory of the operation (same as base_dir if
