@@ -89,7 +89,10 @@ static const char * to_endpoint(int i, char buffer[]);
 static void init_timefmt(optval_t *val);
 static void init_trashdir(optval_t *val);
 static void init_dotfiles(optval_t *val);
+static void init_lsoptions(optval_t *val);
 static void init_lsview(optval_t *val);
+static void init_milleroptions(optval_t *val);
+static void init_millerview(optval_t *val);
 static void init_shortmess(optval_t *val);
 static void init_sizefmt(optval_t *val);
 static void init_iooptions(optval_t *val);
@@ -102,7 +105,7 @@ static void init_tuioptions(optval_t *val);
 static void init_wordchars(optval_t *val);
 static void load_options_defaults(void);
 static void add_options(void);
-static void load_sort_option_inner(FileView *view, char sort_keys[]);
+static void load_sort_option_inner(view_t *view, char sort_keys[]);
 static void aproposprg_handler(OPT_OP op, optval_t val);
 static void autochpos_handler(OPT_OP op, optval_t val);
 static void caseoptions_handler(OPT_OP op, optval_t val);
@@ -139,7 +142,7 @@ static void laststatus_handler(OPT_OP op, optval_t val);
 static void lines_handler(OPT_OP op, optval_t val);
 static void locateprg_handler(OPT_OP op, optval_t val);
 static void mintimeoutlen_handler(OPT_OP op, optval_t val);
-static void scroll_line_down(FileView *view);
+static void scroll_line_down(view_t *view);
 static void rulerformat_handler(OPT_OP op, optval_t val);
 static void runexec_handler(OPT_OP op, optval_t val);
 static void scrollbind_handler(OPT_OP op, optval_t val);
@@ -155,35 +158,47 @@ static void smartcase_handler(OPT_OP op, optval_t val);
 static void sortnumbers_handler(OPT_OP op, optval_t val);
 static void dotfiles_global(OPT_OP op, optval_t val);
 static void dotfiles_local(OPT_OP op, optval_t val);
+static void lsoptions_global(OPT_OP op, optval_t val);
+static void lsoptions_local(OPT_OP op, optval_t val);
+static void set_lsoptions(int *transposed, optval_t val, OPT_SCOPE scope);
+static void fill_lsoptions(optval_t *val, int transposed);
 static void lsview_global(OPT_OP op, optval_t val);
 static void lsview_local(OPT_OP op, optval_t val);
+static void milleroptions_global(OPT_OP op, optval_t val);
+static void milleroptions_local(OPT_OP op, optval_t val);
+static void set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope);
+static void fill_milleroptions(optval_t *val, int ratios[3]);
+static void millerview_global(OPT_OP op, optval_t val);
+static void millerview_local(OPT_OP op, optval_t val);
 static void number_global(OPT_OP op, optval_t val);
 static void number_local(OPT_OP op, optval_t val);
 static void numberwidth_global(OPT_OP op, optval_t val);
 static void numberwidth_local(OPT_OP op, optval_t val);
-static void set_numberwidth(FileView *view, int *num_width, int width);
+static void set_numberwidth(view_t *view, int *num_width, int width);
+static void previewprg_global(OPT_OP op, optval_t val);
+static void previewprg_local(OPT_OP op, optval_t val);
 static void relativenumber_global(OPT_OP op, optval_t val);
 static void relativenumber_local(OPT_OP op, optval_t val);
-static void update_num_type(FileView *view, NumberingType *num_type,
+static void update_num_type(view_t *view, NumberingType *num_type,
 		NumberingType type, int enable);
 static void sort_global(OPT_OP op, optval_t val);
 static void sort_local(OPT_OP op, optval_t val);
-static void set_sort(FileView *view, char sort_keys[], char order[]);
+static void set_sort(view_t *view, char sort_keys[], char order[]);
 static void sortgroups_global(OPT_OP op, optval_t val);
 static void sortgroups_local(OPT_OP op, optval_t val);
-static void set_sortgroups(FileView *view, char **opt, char value[]);
-static void sorting_changed(FileView *view, int defer_slow);
+static void set_sortgroups(view_t *view, char **opt, char value[]);
+static void sorting_changed(view_t *view, int defer_slow);
 static void sortorder_global(OPT_OP op, optval_t val);
 static void sortorder_local(OPT_OP op, optval_t val);
-static void set_sortorder(FileView *view, int ascending, char sort_keys[]);
+static void set_sortorder(view_t *view, int ascending, char sort_keys[]);
 static void viewcolumns_global(OPT_OP op, optval_t val);
 static void viewcolumns_local(OPT_OP op, optval_t val);
-static void set_viewcolumns(FileView *view, const char view_columns[]);
-static void set_view_columns_option(FileView *view, const char value[],
+static void set_viewcolumns(view_t *view, const char view_columns[]);
+static void set_view_columns_option(view_t *view, const char value[],
 		int update_ui);
 static void add_column(columns_t *columns, column_info_t column_info);
 static int map_name(const char name[], void *arg);
-static void resort_view(FileView * view);
+static void resort_view(view_t * view);
 static void statusline_handler(OPT_OP op, optval_t val);
 static void suggestoptions_handler(OPT_OP op, optval_t val);
 static int read_int(const char line[], int *i);
@@ -227,6 +242,7 @@ static const char *sort_enum[] = {
 	[SK_BY_NITEMS]        = "nitems",
 	[SK_BY_GROUPS]        = "groups",
 	[SK_BY_TARGET]        = "target",
+	[SK_BY_ROOT]          = "root",
 #ifndef _WIN32
 	[SK_BY_GROUP_ID]      = "gid",
 	[SK_BY_GROUP_NAME]    = "gname",
@@ -235,9 +251,13 @@ static const char *sort_enum[] = {
 	[SK_BY_OWNER_NAME]    = "uname",
 	[SK_BY_PERMISSIONS]   = "perms",
 	[SK_BY_NLINKS]        = "nlinks",
+	[SK_BY_INODE]         = "inode",
 #endif
+
+	[SK_NONE]             = "",
+	[SK_BY_ID]            = "",
 };
-ARRAY_GUARD(sort_enum, 1 + SK_COUNT);
+ARRAY_GUARD(sort_enum, SK_TOTAL);
 
 /* Possible values of 'caseoptions'. */
 static const char *caseoptions_vals[][2] = {
@@ -277,16 +297,22 @@ static const char *dotdirs_vals[][2] = {
 };
 ARRAY_GUARD(dotdirs_vals, NUM_DOT_DIRS);
 
+/* Possible keys of 'lsoptions' option. */
+static const char *lsoptions_enum[][2] = {
+	{ "transposed", "fill grid by column instead of by line" },
+};
+
 /* Possible values of 'suggestoptions'. */
 static const char *suggestoptions_vals[][2] = {
-	{ "normal",    "display in normal mode" },
-	{ "visual",    "display in visual mode" },
-	{ "view",      "display in view mode" },
-	{ "otherpane", "use other pane for suggestions, if available" },
-	{ "delay",     "display suggestions after a short delay" },
-	{ "keys",      "include keys suggestions in results" },
-	{ "marks",     "include marks suggestions in results" },
-	{ "registers", "include registers suggestions in results" },
+	{ "normal",      "display in normal mode" },
+	{ "visual",      "display in visual mode" },
+	{ "view",        "display in view mode" },
+	{ "otherpane",   "use other pane for suggestions, if available" },
+	{ "delay",       "display suggestions after a short delay" },
+	{ "keys",        "include keys suggestions in results" },
+	{ "foldsubkeys", "fold multiple keys with common prefix" },
+	{ "marks",       "include marks suggestions in results" },
+	{ "registers",   "include registers suggestions in results" },
 };
 ARRAY_GUARD(suggestoptions_vals, NUM_SUGGESTION_FLAGS);
 
@@ -298,15 +324,16 @@ static const char *iooptions_vals[][2] = {
 /* Possible flags of 'shortmess' and their count. */
 static const char *shortmess_vals[][2] = {
 	{ "Tp", "all shortmess values" },
-	{ "T", "shorten too long status bar messages" },
-	{ "p", "substitute home path with ~ in view title" },
+	{ "T",  "shorten too long status bar messages" },
+	{ "p",  "substitute home path with ~ in view title" },
 };
 
 /* Possible flags of 'tuioptions' and their count. */
 static const char *tuioptions_vals[][2] = {
-	{ "ps", "all tuioptions values" },
-	{ "p",  "use padding in views and preview" },
-	{ "s",  "display side borders" },
+	{ "psu", "all tuioptions values" },
+	{ "p",   "use padding in views and preview" },
+	{ "s",   "display side borders" },
+	{ "u",   "use Unicode characters in the TUI" },
 };
 
 /* Possible values of 'dirsize' option. */
@@ -320,6 +347,13 @@ static const char *fillchars_enum[][2] = {
 	{ "vborder:", "filler of vertical borders" },
 };
 
+/* Possible keys of 'milleroptions' option. */
+static const char *milleroptions_enum[][2] = {
+	{ "lsize:", "proportion of space given to the left column" },
+	{ "csize:", "proportion of space given to the center column" },
+	{ "rsize:", "proportion of space given to the right column" },
+};
+
 /* Possible keys of 'sizefmt' option. */
 static const char *sizefmt_enum[][2] = {
 	{ "units:",     "iec (1024) or si (1000) unit size" },
@@ -399,6 +433,10 @@ static const char *sort_types[][2] = {
 	{ "+mode", "by file mode" },
 	{ "-mode", "by file mode" },
 
+	{ "inode",  "by file inode number" },
+	{ "+inode", "by file inode number" },
+	{ "-inode", "by file inode number" },
+
 	{ "uid",  "by user ID" },
 	{ "+uid", "by user ID" },
 	{ "-uid", "by user ID" },
@@ -728,10 +766,24 @@ options[] = {
 	  OPT_BOOL, 0, NULL, &dotfiles_global, &dotfiles_local,
 	  { .init = &init_dotfiles },
 	},
-	{ "lsview", "", "display items in a table",
+	{ "lsoptions", "", "settings for ls-like view",
+	  OPT_STRLIST, ARRAY_LEN(lsoptions_enum), lsoptions_enum,
+		&lsoptions_global, &lsoptions_local,
+	  { .init = &init_lsoptions },
+	},
+	{ "lsview", "", "display items in a grid",
 	  OPT_BOOL, 0, NULL, &lsview_global, &lsview_local,
 	  { .init = &init_lsview },
 	},
+	{ "milleroptions", "", "settings for miller view",
+	  OPT_STRLIST, ARRAY_LEN(milleroptions_enum), milleroptions_enum,
+		&milleroptions_global, &milleroptions_local,
+	  { .init = &init_milleroptions },
+	},
+	{ "millerview", "", "display cascading lists",
+	  OPT_BOOL, 0, NULL, &millerview_global, &millerview_local,
+	  { .init = &init_millerview },
+	},
 	{ "number", "nu", "display line numbers",
 	  OPT_BOOL, 0, NULL, &number_global, &number_local,
 	  { .init = &init_number },
@@ -740,6 +792,10 @@ options[] = {
 	  OPT_INT, 0, NULL, &numberwidth_global, &numberwidth_local,
 	  { .init = &init_numberwidth },
 	},
+	{ "previewprg", "", "command to use to preview all files",
+	  OPT_STR, 0, NULL, &previewprg_global, &previewprg_local,
+	  { .ref.str_val = &empty },
+	},
 	{ "relativenumber", "rnu", "display relative line numbers",
 	  OPT_BOOL, 0, NULL, &relativenumber_global, &relativenumber_local,
 	  { .init = &init_relativenumber },
@@ -809,7 +865,7 @@ uni_handler(const char name[], optval_t val, OPT_SCOPE scope)
 	{
 		if(strcmp(options[i].name, name) == 0)
 		{
-			FileView *const tmp_view = curr_view;
+			view_t *const tmp_view = curr_view;
 			curr_view = other_view;
 
 			/* Make sure option value remains valid even if updated in the handler.
@@ -974,12 +1030,34 @@ init_dotfiles(optval_t *val)
 	val->bool_val = !curr_view->hide_dot_g;
 }
 
+/* Initializes 'lsoptions' option from global value. */
+static void
+init_lsoptions(optval_t *val)
+{
+	fill_lsoptions(val, curr_view->ls_transposed_g);
+}
+
+/* Initializes 'lsview' option from global value. */
 static void
 init_lsview(optval_t *val)
 {
 	val->bool_val = curr_view->ls_view_g;
 }
 
+/* Initializes 'milleroptions' option from global value. */
+static void
+init_milleroptions(optval_t *val)
+{
+	fill_milleroptions(val, curr_view->miller_ratios_g);
+}
+
+/* Initializes 'millerview' option from global value. */
+static void
+init_millerview(optval_t *val)
+{
+	val->bool_val = curr_view->miller_view_g;
+}
+
 /* Initializes 'shortmess' from current configuration state. */
 static void
 init_shortmess(optval_t *val)
@@ -1043,9 +1121,10 @@ static void
 init_tuioptions(optval_t *val)
 {
 	static char buf[32];
-	snprintf(buf, sizeof(buf), "%s%s",
+	snprintf(buf, sizeof(buf), "%s%s%s",
 			cfg.extra_padding ? "p" : "",
-			cfg.side_borders_visible ? "s" : "");
+			cfg.side_borders_visible ? "s" : "",
+			cfg.use_unicode_characters ? "u" : "");
 	val->str_val = buf;
 }
 
@@ -1091,7 +1170,7 @@ init_wordchars(optval_t *val)
 
 			if(r == l)
 			{
-				snprintf(range, sizeof(range), "%s", to_endpoint(l, left));
+				copy_str(range, sizeof(range), to_endpoint(l, left));
 			}
 			else
 			{
@@ -1148,7 +1227,7 @@ add_options(void)
 }
 
 void
-reset_local_options(FileView *view)
+reset_local_options(view_t *view)
 {
 	optval_t val;
 
@@ -1159,10 +1238,23 @@ reset_local_options(FileView *view)
 	val.int_val = !view->hide_dot_g;
 	set_option("dotfiles", val, OPT_LOCAL);
 
+	view->ls_transposed = view->ls_transposed_g;
+	fill_lsoptions(&val, view->ls_transposed_g);
+	set_option("lsoptions", val, OPT_LOCAL);
+
 	fview_set_lsview(view, view->ls_view_g);
 	val.int_val = view->ls_view_g;
 	set_option("lsview", val, OPT_LOCAL);
 
+	memcpy(view->miller_ratios, view->miller_ratios_g,
+			sizeof(view->miller_ratios));
+	fill_milleroptions(&val, view->miller_ratios_g);
+	set_option("milleroptions", val, OPT_LOCAL);
+
+	fview_set_millerview(view, view->miller_view_g);
+	val.int_val = view->miller_view_g;
+	set_option("millerview", val, OPT_LOCAL);
+
 	view->num_type = view->num_type_g;
 	val.bool_val = view->num_type_g & NT_SEQ;
 	set_option("number", val, OPT_LOCAL);
@@ -1181,10 +1273,14 @@ reset_local_options(FileView *view)
 	replace_string(&view->sort_groups, view->sort_groups_g);
 	val.str_val = view->sort_groups;
 	set_option("sortgroups", val, OPT_LOCAL);
+
+	replace_string(&view->preview_prg, view->preview_prg_g);
+	val.str_val = view->preview_prg;
+	set_option("previewprg", val, OPT_LOCAL);
 }
 
 void
-load_view_options(FileView *view)
+load_view_options(view_t *view)
 {
 	optval_t val;
 
@@ -1205,11 +1301,26 @@ load_view_options(FileView *view)
 	val.bool_val = !view->hide_dot_g;
 	set_option("dotfiles", val, OPT_GLOBAL);
 
+	fill_lsoptions(&val, view->ls_transposed);
+	set_option("lsoptions", val, OPT_LOCAL);
+	fill_lsoptions(&val, view->ls_transposed_g);
+	set_option("lsoptions", val, OPT_GLOBAL);
+
 	val.bool_val = view->ls_view;
 	set_option("lsview", val, OPT_LOCAL);
 	val.bool_val = view->ls_view_g;
 	set_option("lsview", val, OPT_GLOBAL);
 
+	fill_milleroptions(&val, view->miller_ratios);
+	set_option("milleroptions", val, OPT_LOCAL);
+	fill_milleroptions(&val, view->miller_ratios_g);
+	set_option("milleroptions", val, OPT_GLOBAL);
+
+	val.bool_val = view->miller_view;
+	set_option("millerview", val, OPT_LOCAL);
+	val.bool_val = view->miller_view_g;
+	set_option("millerview", val, OPT_GLOBAL);
+
 	val.bool_val = view->num_type & NT_SEQ;
 	set_option("number", val, OPT_LOCAL);
 	val.bool_val = view->num_type_g & NT_SEQ;
@@ -1224,10 +1335,15 @@ load_view_options(FileView *view)
 	set_option("relativenumber", val, OPT_LOCAL);
 	val.bool_val = view->num_type_g & NT_REL;
 	set_option("relativenumber", val, OPT_GLOBAL);
+
+	val.str_val = view->preview_prg;
+	set_option("previewprg", val, OPT_LOCAL);
+	val.str_val = view->preview_prg_g;
+	set_option("previewprg", val, OPT_GLOBAL);
 }
 
 void
-clone_local_options(const FileView *from, FileView *to, int defer_slow)
+clone_local_options(const view_t *from, view_t *to, int defer_slow)
 {
 	const char *sort;
 
@@ -1253,12 +1369,24 @@ clone_local_options(const FileView *from, FileView *to, int defer_slow)
 	memcpy(to->sort_g, from->sort_g, sizeof(to->sort_g));
 	sorting_changed(to, defer_slow);
 
+	to->ls_transposed_g = from->ls_transposed_g;
+
 	to->ls_view_g = from->ls_view_g;
 	fview_set_lsview(to, from->ls_view);
+
+	memcpy(to->miller_ratios_g, from->miller_ratios_g,
+			sizeof(to->miller_ratios_g));
+	memcpy(to->miller_ratios, from->miller_ratios, sizeof(to->miller_ratios));
+
+	to->miller_view_g = from->miller_view_g;
+	fview_set_millerview(to, from->miller_view);
+
+	replace_string(&to->preview_prg, from->preview_prg);
+	replace_string(&to->preview_prg_g, from->preview_prg_g);
 }
 
 void
-load_sort_option(FileView *view)
+load_sort_option(view_t *view)
 {
 	load_sort_option_inner(view, view->sort);
 	load_sort_option_inner(view, view->sort_g);
@@ -1275,7 +1403,7 @@ load_sort_option(FileView *view)
 
 /* Loads sorting related options ("sort" and "sortorder"). */
 static void
-load_sort_option_inner(FileView *view, char sort_keys[])
+load_sort_option_inner(view_t *view, char sort_keys[])
 {
 	/* This approximate maximum length also includes "+" or "-" sign and a
 	 * comma (",") between items. */
@@ -1349,11 +1477,11 @@ process_set_args(const char args[], int global, int local)
 	{
 		vle_tb_append_line(vle_err, "Invalid argument for :set command");
 		/* We just modified text buffer and can't use text_buffer variable. */
-		status_bar_error(vle_tb_get_data(vle_err));
+		ui_sb_err(vle_tb_get_data(vle_err));
 	}
 	else if(text_buffer[0] != '\0')
 	{
-		status_bar_message(text_buffer);
+		ui_sb_msg(text_buffer);
 	}
 
 	return error ? -1 : (text_buffer[0] != '\0');
@@ -1502,7 +1630,7 @@ str_to_classify(const char str[], char type_decs[FT_COUNT][2][9])
 	saveptr = NULL;
 	for(token = str_copy; (token = split_and_get_dc(token, &saveptr));)
 	{
-		FileType type;
+		FileType type = FT_UNK;
 		const char *expr = NULL;
 		const char *suffix = pick_out_decoration(token, &type, &expr);
 
@@ -1974,16 +2102,16 @@ mintimeoutlen_handler(OPT_OP op, optval_t val)
 }
 
 static void
-scroll_line_down(FileView *view)
+scroll_line_down(view_t *view)
 {
-	view->window_rows--;
-	if(view->list_pos == view->top_line + view->window_rows + 1)
+	--view->window_rows;
+	if(view->list_pos == view->top_line + view->window_rows)
 	{
-		view->top_line++;
-		view->curr_line--;
+		++view->top_line;
+		--view->curr_line;
 		draw_dir_list(view);
 	}
-	wresize(view->win, view->window_rows + 1, view->window_width + 1);
+	wresize(view->win, view->window_rows, view->window_cols);
 }
 
 static void
@@ -2178,6 +2306,69 @@ dotfiles_local(OPT_OP op, optval_t val)
 	ui_view_schedule_reload(curr_view);
 }
 
+/* Handles update of ls settings as a global option. */
+static void
+lsoptions_global(OPT_OP op, optval_t val)
+{
+	set_lsoptions(&curr_view->ls_transposed_g, val, OPT_GLOBAL);
+}
+
+/* Handles update of ls settings as a local option. */
+static void
+lsoptions_local(OPT_OP op, optval_t val)
+{
+	set_lsoptions(&curr_view->ls_transposed, val, OPT_LOCAL);
+}
+
+/* Handles update of ls-view settings. */
+static void
+set_lsoptions(int *transposed, optval_t val, OPT_SCOPE scope)
+{
+	char *new_val = strdup(val.str_val);
+	char *part = new_val, *state = NULL;
+
+	int transpose = 0;
+
+	while((part = split_and_get(part, ',', &state)) != NULL)
+	{
+		if(strcmp(part, "transposed") == 0)
+		{
+			transpose = 1;
+		}
+		else
+		{
+			break_at(part, ':');
+			vle_tb_append_linef(vle_err, "Unknown key for 'lsoptions' option: %s",
+					part);
+			break;
+		}
+	}
+	free(new_val);
+
+	if(part == NULL)
+	{
+		*transposed = transpose;
+
+		if(!ui_view_displays_columns(curr_view) &&
+				transposed == &curr_view->ls_transposed)
+		{
+			ui_view_schedule_redraw(curr_view);
+		}
+	}
+
+	fill_lsoptions(&val, *transposed);
+	set_option("lsoptions", val, scope);
+}
+
+/* Loads value of lsoptions as a string. */
+static void
+fill_lsoptions(optval_t *val, int transposed)
+{
+	static char buf[64];
+	copy_str(buf, sizeof(buf), transposed ? "transposed" : "");
+	val->str_val = buf;
+}
+
 /* Handles switch that controls column vs. ls-like view in global option. */
 static void
 lsview_global(OPT_OP op, optval_t val)
@@ -2192,6 +2383,117 @@ lsview_local(OPT_OP op, optval_t val)
 	fview_set_lsview(curr_view, val.bool_val);
 }
 
+/* Handles update of miller columns settings as a global option. */
+static void
+milleroptions_global(OPT_OP op, optval_t val)
+{
+	set_milleroptions(curr_view->miller_ratios_g, val, OPT_GLOBAL);
+}
+
+/* Handles update of miller columns settings as a local option. */
+static void
+milleroptions_local(OPT_OP op, optval_t val)
+{
+	set_milleroptions(curr_view->miller_ratios, val, OPT_LOCAL);
+}
+
+/* Handles update of miller columns settings. */
+static void
+set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope)
+{
+	char *new_val = strdup(val.str_val);
+	char *part = new_val, *state = NULL;
+
+	int lsize = 0, csize = 1, rsize = 0;
+
+	while((part = split_and_get(part, ',', &state)) != NULL)
+	{
+		if(starts_with_lit(part, "lsize:"))
+		{
+			const char *const num = after_first(part, ':');
+			if(!read_int(num, &lsize))
+			{
+				vle_tb_append_linef(vle_err, "Failed to parse \"lsize\" value: %s",
+						num);
+				break;
+			}
+		}
+		else if(starts_with_lit(part, "csize:"))
+		{
+			const char *const num = after_first(part, ':');
+			if(!read_int(num, &csize))
+			{
+				vle_tb_append_linef(vle_err, "Failed to parse \"csize\" value: %s",
+						num);
+				break;
+			}
+			if(csize < 1)
+			{
+				vle_tb_append_linef(vle_err, "\"csize\" can't be less than 1, got: %s",
+						num);
+				break;
+			}
+		}
+		else if(starts_with_lit(part, "rsize:"))
+		{
+			const char *const num = after_first(part, ':');
+			if(!read_int(num, &rsize))
+			{
+				vle_tb_append_linef(vle_err, "Failed to parse \"rsize\" value: %s",
+						num);
+				break;
+			}
+		}
+		else
+		{
+			break_at(part, ':');
+			vle_tb_append_linef(vle_err, "Unknown key for 'milleroptions' option: %s",
+					part);
+			break;
+		}
+	}
+	free(new_val);
+
+	if(part == NULL)
+	{
+		ratios[0] = MAX(0, MIN(100, lsize));
+		ratios[1] = MAX(0, MIN(100, csize));
+		ratios[2] = MAX(0, MIN(100, rsize));
+
+		if(ratios == curr_view->miller_ratios)
+		{
+			ui_view_schedule_redraw(curr_view);
+		}
+	}
+
+	fill_milleroptions(&val, ratios);
+	set_option("milleroptions", val, scope);
+}
+
+/* Loads value of milleroptions as a string. */
+static void
+fill_milleroptions(optval_t *val, int ratios[3])
+{
+	static char buf[64];
+	snprintf(buf, sizeof(buf), "lsize:%d,csize:%d,rsize:%d", ratios[0], ratios[1],
+			ratios[2]);
+	val->str_val = buf;
+}
+
+/* Handles switch that controls cascading lists view in global option. */
+static void
+millerview_global(OPT_OP op, optval_t val)
+{
+	curr_view->miller_view_g = val.bool_val;
+}
+
+/* Handles switch that controls cascading lists view in local option. */
+static void
+millerview_local(OPT_OP op, optval_t val)
+{
+	fview_set_millerview(curr_view, val.bool_val);
+}
+
 /* Handles file numbers displaying toggle in global option. */
 static void
 number_global(OPT_OP op, optval_t val)
@@ -2222,7 +2524,7 @@ numberwidth_local(OPT_OP op, optval_t val)
 
 /* Sets number width for the view. */
 static void
-set_numberwidth(FileView *view, int *num_width, int width)
+set_numberwidth(view_t *view, int *num_width, int width)
 {
 	*num_width = width;
 
@@ -2232,6 +2534,21 @@ set_numberwidth(FileView *view, int *num_width, int width)
 	}
 }
 
+/* Handles change of global value of the 'previewprg' option. */
+static void
+previewprg_global(OPT_OP op, optval_t val)
+{
+	replace_string(&curr_view->preview_prg_g, val.str_val);
+}
+
+/* Handles change of local value of the 'previewprg' option. */
+static void
+previewprg_local(OPT_OP op, optval_t val)
+{
+	replace_string(&curr_view->preview_prg, val.str_val);
+	curr_stats.need_update = UT_REDRAW;
+}
+
 /* Handles relative file numbers displaying toggle in global option. */
 static void
 relativenumber_global(OPT_OP op, optval_t val)
@@ -2249,7 +2566,7 @@ relativenumber_local(OPT_OP op, optval_t val)
 /* Handles toggling of boolean number related option and updates current view if
  * needed. */
 static void
-update_num_type(FileView *view, NumberingType *num_type, NumberingType type,
+update_num_type(view_t *view, NumberingType *num_type, NumberingType type,
 		int enable)
 {
 	const NumberingType old_num_type = *num_type;
@@ -2288,7 +2605,7 @@ sort_local(OPT_OP op, optval_t val)
 
 /* Sets sorting value for the view. */
 static void
-set_sort(FileView *view, char sort_keys[], char order[])
+set_sort(view_t *view, char sort_keys[], char order[])
 {
 	char *part = order, *state = NULL;
 	int key_count = 0;
@@ -2367,7 +2684,7 @@ sortgroups_local(OPT_OP op, optval_t val)
 /* Sets sort_groups fields (*opt) of the view to the value handling malformed
  * values correctly. */
 static void
-set_sortgroups(FileView *view, char **opt, char value[])
+set_sortgroups(view_t *view, char **opt, char value[])
 {
 	OPT_SCOPE scope = (opt == &view->sort_groups) ? OPT_LOCAL : OPT_GLOBAL;
 	int failure = 0;
@@ -2417,7 +2734,7 @@ set_sortgroups(FileView *view, char **opt, char value[])
 
 /* Reacts on changes of view sorting. */
 static void
-sorting_changed(FileView *view, int defer_slow)
+sorting_changed(view_t *view, int defer_slow)
 {
 	/* Reset search results, which might be outdated after resorting. */
 	view->matches = 0;
@@ -2446,7 +2763,7 @@ sortorder_local(OPT_OP op, optval_t val)
 
 /* Updates sorting order for the view. */
 static void
-set_sortorder(FileView *view, int ascending, char sort_keys[])
+set_sortorder(view_t *view, int ascending, char sort_keys[])
 {
 	if((ascending ? +1 : -1)*sort_keys[0] < 0)
 	{
@@ -2476,14 +2793,14 @@ viewcolumns_local(OPT_OP op, optval_t val)
 
 /* Setups view columns for the view. */
 static void
-set_viewcolumns(FileView *view, const char view_columns[])
+set_viewcolumns(view_t *view, const char view_columns[])
 {
 	const int update_columns_ui = ui_view_displays_columns(view);
 	set_view_columns_option(view, view_columns, update_columns_ui);
 }
 
 void
-load_dot_filter_option(const FileView *view)
+load_dot_filter_option(const view_t *view)
 {
 	const optval_t val = { .bool_val = !view->hide_dot };
 	set_option("dotfiles", val, OPT_GLOBAL);
@@ -2491,7 +2808,7 @@ load_dot_filter_option(const FileView *view)
 }
 
 void
-load_view_columns_option(FileView *view, const char value[])
+load_view_columns_option(view_t *view, const char value[])
 {
 	set_view_columns_option(view, value, 1);
 }
@@ -2500,7 +2817,7 @@ load_view_columns_option(FileView *view, const char value[])
  * Doesn't change actual value of the option, which is important for setting
  * sorting order via sort dialog. */
 static void
-set_view_columns_option(FileView *view, const char value[], int update_ui)
+set_view_columns_option(view_t *view, const char value[], int update_ui)
 {
 	const char *new_value = (value[0] == '\0') ? DEFAULT_VIEW_COLUMNS : value;
 	columns_t *columns = update_ui ? view->columns : NULL;
@@ -2561,7 +2878,7 @@ map_name(const char name[], void *arg)
 	/* Handle secondary key (designated by {}). */
 	if(*name == '\0')
 	{
-		const FileView *const view = arg;
+		const view_t *const view = arg;
 		const char *const sort = ui_view_sort_list_get(view, view->sort);
 		return (int)get_secondary_key((SortingKey)abs(sort[0]));
 	}
@@ -2582,7 +2899,7 @@ load_geometry(void)
 }
 
 static void
-resort_view(FileView * view)
+resort_view(view_t * view)
 {
 	resort_dir_list(1, view);
 	ui_view_schedule_redraw(curr_view);
@@ -2607,12 +2924,13 @@ suggestoptions_handler(OPT_OP op, optval_t val)
 
 	while((part = split_and_get(part, ',', &state)) != NULL)
 	{
-		if(strcmp(part, "normal") == 0)         new_value |= SF_NORMAL;
-		else if(strcmp(part, "visual") == 0)    new_value |= SF_VISUAL;
-		else if(strcmp(part, "view") == 0)      new_value |= SF_VIEW;
-		else if(strcmp(part, "otherpane") == 0) new_value |= SF_OTHERPANE;
-		else if(strcmp(part, "keys") == 0)      new_value |= SF_KEYS;
-		else if(strcmp(part, "marks") == 0)     new_value |= SF_MARKS;
+		if(strcmp(part, "normal") == 0)           new_value |= SF_NORMAL;
+		else if(strcmp(part, "visual") == 0)      new_value |= SF_VISUAL;
+		else if(strcmp(part, "view") == 0)        new_value |= SF_VIEW;
+		else if(strcmp(part, "otherpane") == 0)   new_value |= SF_OTHERPANE;
+		else if(strcmp(part, "keys") == 0)        new_value |= SF_KEYS;
+		else if(strcmp(part, "marks") == 0)       new_value |= SF_MARKS;
+		else if(strcmp(part, "foldsubkeys") == 0) new_value |= SF_FOLDSUBKEYS;
 		else if(starts_with_lit(part, "delay:"))
 		{
 			const char *const num = after_first(part, ':');
@@ -2696,6 +3014,10 @@ reset_suggestoptions(void)
 	if(cfg.sug.flags & SF_OTHERPANE) vle_tb_appendf(descr, "%s", "otherpane,");
 	if(cfg.sug.flags & SF_KEYS)      vle_tb_appendf(descr, "%s", "keys,");
 	if(cfg.sug.flags & SF_MARKS)     vle_tb_appendf(descr, "%s", "marks,");
+	if(cfg.sug.flags & SF_FOLDSUBKEYS)
+	{
+		vle_tb_appendf(descr, "%s", "foldsubkeys,");
+	}
 	if(cfg.sug.flags & SF_DELAY)
 	{
 		if(cfg.sug.delay == 500)
@@ -2820,6 +3142,7 @@ tuioptions_handler(OPT_OP op, optval_t val)
 	/* Turn all flags off. */
 	cfg.extra_padding = 0;
 	cfg.side_borders_visible = 0;
+	cfg.use_unicode_characters = 0;
 
 	/* And set the ones present in the value. */
 	p = val.str_val;
@@ -2833,6 +3156,9 @@ tuioptions_handler(OPT_OP op, optval_t val)
 			case 's':
 				cfg.side_borders_visible = 1;
 				break;
+			case 'u':
+				cfg.use_unicode_characters = 1;
+				break;
 
 			default:
 				assert(0 && "Unhandled tuioptions flag.");
@@ -2841,6 +3167,9 @@ tuioptions_handler(OPT_OP op, optval_t val)
 		++p;
 	}
 
+	curr_stats.ellipsis = (cfg.use_unicode_characters ? "…" : "...");
+	columns_set_ellipsis(curr_stats.ellipsis);
+
 	curr_stats.need_update = UT_REDRAW;
 }
 
@@ -3005,7 +3334,7 @@ wrap_handler(OPT_OP op, optval_t val)
 static void
 text_option_changed(void)
 {
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
 		qv_draw(curr_view);
 	}
