@@ -61,15 +61,14 @@ static int compare_entry_names(const dir_entry_t *a, const dir_entry_t *b,
 static int compare_full_file_names(const char s[], const char t[],
 		int ignore_case);
 static int compare_file_names(const char s[], const char t[], int ignore_case);
-static int compare_file_sizes(const dir_entry_t *f, int fdir,
-		const dir_entry_t *s, int sdir);
+static int compare_file_sizes(const dir_entry_t *f, const dir_entry_t *s);
 static int compare_item_count(const dir_entry_t *f, int fdir,
 		const dir_entry_t *s, int sdir);
 static int compare_group(const char f[], const char s[], regex_t *regex);
 static int compare_targets(const dir_entry_t *f, const dir_entry_t *s);
 
 /* View which is being sorted. */
-static FileView* view;
+static view_t* view;
 /* Picked sort array of the view. */
 static const char *view_sort;
 /* Picked sort groups setting of the view. */
@@ -84,7 +83,7 @@ static SortingKey sort_type;
 static void *sort_data;
 
 void
-sort_view(FileView *v)
+sort_view(view_t *v)
 {
 	dir_entry_t *unsorted_list;
 
@@ -99,7 +98,7 @@ sort_view(FileView *v)
 	view_sort_groups = v->sort_groups;
 	custom_view = flist_custom_active(v);
 
-	if(!custom_view || v->custom.type != CV_TREE)
+	if(!custom_view || !cv_tree(v->custom.type))
 	{
 		/* Tree sorting works fine for flat list, but requires a bit more
 		 * resources, so skip it. */
@@ -117,8 +116,16 @@ sort_view(FileView *v)
 
 	unsorted_list = v->dir_entry;
 	v->dir_entry = dynarray_extend(NULL, v->list_rows*sizeof(*v->dir_entry));
-
-	sort_tree_slice(&v->dir_entry[0], unsorted_list, v->list_rows, 1);
+	if(v->dir_entry != NULL)
+	{
+		sort_tree_slice(&v->dir_entry[0], unsorted_list, v->list_rows, 1);
+	}
+	else
+	{
+		/* Just do nothing on memory error. */
+		v->dir_entry = unsorted_list;
+		unsorted_list = NULL;
+	}
 
 	if(filter_is_empty(&v->local_filter.filter))
 	{
@@ -126,7 +133,7 @@ sort_view(FileView *v)
 	}
 	else
 	{
-		filter_temporary_nodes(v, unsorted_list);
+		filters_drop_temporaries(v, unsorted_list);
 	}
 }
 
@@ -168,7 +175,7 @@ sort_tree_slice(dir_entry_t *entries, const dir_entry_t *children,
 }
 
 void
-sort_entries(FileView *v, entries_t entries)
+sort_entries(view_t *v, entries_t entries)
 {
 	if(v->sort_g[0] > SK_LAST)
 	{
@@ -414,7 +421,7 @@ sort_dir_list(const void *one, const void *two)
 			break;
 
 		case SK_BY_SIZE:
-			retval = compare_file_sizes(first, first_is_dir, second, second_is_dir);
+			retval = compare_file_sizes(first, second);
 			break;
 
 		case SK_BY_NITEMS:
@@ -446,6 +453,10 @@ sort_dir_list(const void *one, const void *two)
 			retval = first->mode - second->mode;
 			break;
 
+		case SK_BY_INODE:
+			retval = first->inode - second->inode;
+			break;
+
 		case SK_BY_OWNER_NAME: /* FIXME */
 		case SK_BY_OWNER_ID:
 			retval = first->uid - second->uid;
@@ -485,32 +496,10 @@ sort_dir_list(const void *one, const void *two)
 
 /* Compares two file sizes.  Returns standard -1, 0, 1 for comparisons. */
 static int
-compare_file_sizes(const dir_entry_t *f, int fdir, const dir_entry_t *s,
-		int sdir)
+compare_file_sizes(const dir_entry_t *f, const dir_entry_t *s)
 {
-	uint64_t fsize = f->size;
-	uint64_t ssize = s->size;
-
-	if(fdir)
-	{
-		uint64_t size;
-		dcache_get_of(f, &size, NULL);
-		if(size != DCACHE_UNKNOWN)
-		{
-			fsize = size;
-		}
-	}
-
-	if(sdir)
-	{
-		uint64_t size;
-		dcache_get_of(s, &size, NULL);
-		if(size != DCACHE_UNKNOWN)
-		{
-			ssize = size;
-		}
-	}
-
+	const uint64_t fsize = fentry_get_size(view, f);
+	const uint64_t ssize = fentry_get_size(view, s);
 	return (fsize < ssize) ? -1 : (fsize > ssize);
 }
 
@@ -520,11 +509,11 @@ static int
 compare_item_count(const dir_entry_t *f, int fdir, const dir_entry_t *s,
 		int sdir)
 {
-	/* We don't want to call entry_get_nitems() for files as sorting huge lists
+	/* We don't want to call fentry_get_nitems() for files as sorting huge lists
 	 * of files can call this function a lot of times, thus even small extra
 	 * performance overhead is not desirable. */
-	const uint64_t fsize = fdir ? entry_get_nitems(view, f) : 0U;
-	const uint64_t ssize = sdir ? entry_get_nitems(view, s) : 0U;
+	const uint64_t fsize = fdir ? fentry_get_nitems(view, f) : 0U;
+	const uint64_t ssize = sdir ? fentry_get_nitems(view, s) : 0U;
 	return (fsize > ssize) ? 1 : (fsize < ssize) ? -1 : 0;
 }
 
@@ -585,11 +574,11 @@ compare_targets(const dir_entry_t *f, const dir_entry_t *s)
 static int
 compare_entry_names(const dir_entry_t *a, const dir_entry_t *b, int ignore_case)
 {
-	char a_short_path[PATH_MAX];
-	char b_short_path[PATH_MAX];
+	char a_short_path[PATH_MAX + 1];
+	char b_short_path[PATH_MAX + 1];
 
-	get_short_path_of(view, a, 0, 0, sizeof(a_short_path), a_short_path);
-	get_short_path_of(view, b, 0, 0, sizeof(b_short_path), b_short_path);
+	get_short_path_of(view, a, NF_NONE, 0, sizeof(a_short_path), a_short_path);
+	get_short_path_of(view, b, NF_NONE, 0, sizeof(b_short_path), b_short_path);
 
 	return compare_full_file_names(a_short_path, b_short_path, ignore_case);
 }
@@ -657,6 +646,7 @@ get_secondary_key(SortingKey primary_key)
 		case SK_BY_GROUP_NAME:
 		case SK_BY_GROUP_ID:
 		case SK_BY_MODE:
+		case SK_BY_INODE:
 		case SK_BY_PERMISSIONS:
 		case SK_BY_NLINKS:
 #endif
