@@ -35,6 +35,7 @@
 #include "cfg/config.h"
 #include "compat/fs_limits.h"
 #include "compat/pthread.h"
+#include "compat/reallocarray.h"
 #include "ui/colors.h"
 #include "ui/ui.h"
 #include "utils/env.h"
@@ -45,6 +46,7 @@
 #include "utils/str.h"
 #include "utils/utils.h"
 #include "cmd_completion.h"
+#include "cmd_core.h"
 #include "filelist.h"
 
 /* Environment variables by which application hosted by terminal multiplexer can
@@ -65,8 +67,8 @@ static void determine_fuse_umount_cmd(status_t *stats);
 static void set_gtk_available(status_t *stats);
 static int reset_dircache(void);
 static void set_last_cmdline_command(const char cmd[]);
-static void dcache_get(const char path[], uint64_t *size, uint64_t *nitems,
-		time_t ts);
+static void save_into_history(const char item[], hist_t *hist, int len);
+static void size_updater(void *data, void *arg);
 
 status_t curr_stats;
 
@@ -84,7 +86,7 @@ static fsdata_t *dcache_size;
 static fsdata_t *dcache_nitems;
 
 int
-init_status(config_t *config)
+stats_init(config_t *config)
 {
 	inside_screen = !is_null_or_empty(env_get(SCREEN_ENVVAR));
 	inside_tmux = !is_null_or_empty(env_get(TMUX_ENVVAR));
@@ -95,11 +97,18 @@ init_status(config_t *config)
 	curr_stats.exec_env_type = get_exec_env_type();
 	stats_update_shell_type(config->shell);
 
-	return reset_status(config);
+	(void)hist_init(&curr_stats.cmd_hist, config->history_len);
+	(void)hist_init(&curr_stats.search_hist, config->history_len);
+	(void)hist_init(&curr_stats.prompt_hist, config->history_len);
+	(void)hist_init(&curr_stats.filter_hist, config->history_len);
+
+	hists_resize(config->history_len);
+
+	return stats_reset(config);
 }
 
 /* Initializes most fields of the status structure, some are left to be
- * initialized by the reset_status() function. */
+ * initialized by the stats_reset() function. */
 static void
 load_def_values(status_t *stats, config_t *config)
 {
@@ -124,10 +133,10 @@ load_def_values(status_t *stats, config_t *config)
 	stats->cs = &config->cs;
 	strcpy(stats->color_scheme, "");
 
-	stats->view = 0;
-	stats->graphics_preview = 0;
-	update_string(&stats->preview_cleanup, NULL);
-	stats->clear_preview = 0;
+	stats->preview.on = 0;
+	stats->preview.graphical = 0;
+	update_string(&stats->preview.cleanup_cmd, NULL);
+	stats->preview.clearing = 0;
 
 	stats->msg_head = 0;
 	stats->msg_tail = 0;
@@ -153,6 +162,8 @@ load_def_values(status_t *stats, config_t *config)
 	stats->initial_lines = INT_MIN;
 	stats->initial_columns = INT_MIN;
 
+	stats->ellipsis = "...";
+
 	stats->shell_type = ST_NORMAL;
 
 	stats->fuse_umount_cmd = "";
@@ -169,6 +180,10 @@ load_def_values(status_t *stats, config_t *config)
 
 	stats->global_local_settings = 0;
 
+	stats->history_size = 0;
+
+	stats->ipc = NULL;
+
 #ifdef HAVE_LIBGTK
 	stats->gtk_available = 0;
 #endif
@@ -205,7 +220,7 @@ set_gtk_available(status_t *stats)
 }
 
 int
-reset_status(const config_t *config)
+stats_reset(const config_t *config)
 {
 	set_last_cmdline_command("");
 
@@ -229,13 +244,13 @@ reset_dircache(void)
 }
 
 void
-schedule_redraw(void)
+stats_redraw_schedule(void)
 {
 	pending_redraw = 1;
 }
 
 int
-fetch_redraw_scheduled(void)
+stats_redraw_fetch(void)
 {
 	if(pending_redraw)
 	{
@@ -246,7 +261,7 @@ fetch_redraw_scheduled(void)
 }
 
 void
-set_using_term_multiplexer(int use_term_multiplexer)
+stats_set_use_multiplexer(int use_term_multiplexer)
 {
 	if(!use_term_multiplexer)
 	{
@@ -266,28 +281,6 @@ set_using_term_multiplexer(int use_term_multiplexer)
 	}
 }
 
-void
-update_last_cmdline_command(const char cmd[])
-{
-	if(!curr_stats.restart_in_progress && curr_stats.load_stage == 3)
-	{
-		set_last_cmdline_command(cmd);
-	}
-}
-
-/* Sets last_cmdline_command field of the status structure. */
-static void
-set_last_cmdline_command(const char cmd[])
-{
-	const int err = replace_string(&curr_stats.last_cmdline_command, cmd);
-	if(err != 0)
-	{
-		LOG_ERROR_MSG("replace_string() failed on duplicating: %s", cmd);
-	}
-	assert(curr_stats.last_cmdline_command != NULL &&
-			"The field was not initialized properly");
-}
-
 void
 stats_update_shell_type(const char shell_cmd[])
 {
@@ -364,45 +357,128 @@ stats_save_msg(const char msg[])
 }
 
 void
-dcache_get_at(const char path[], uint64_t *size, uint64_t *nitems)
+hists_resize(int new_size)
 {
-	dcache_get(path, size, nitems, 0);
+	const int old_size = curr_stats.history_size;
+	const int delta = new_size - old_size;
+
+	if(new_size <= 0)
+	{
+		hist_reset(&curr_stats.search_hist, old_size);
+		hist_reset(&curr_stats.cmd_hist, old_size);
+		hist_reset(&curr_stats.prompt_hist, old_size);
+		hist_reset(&curr_stats.filter_hist, old_size);
+		curr_stats.history_size = 0;
+		return;
+	}
+
+	curr_stats.history_size = new_size;
+
+	if(delta < 0)
+	{
+		hist_trunc(&curr_stats.search_hist, new_size, -delta);
+		hist_trunc(&curr_stats.cmd_hist, new_size, -delta);
+		hist_trunc(&curr_stats.prompt_hist, new_size, -delta);
+		hist_trunc(&curr_stats.filter_hist, new_size, -delta);
+	}
+
+	curr_stats.cmd_hist.items = reallocarray(curr_stats.cmd_hist.items, new_size,
+			sizeof(char *));
+	curr_stats.search_hist.items = reallocarray(curr_stats.search_hist.items,
+			new_size, sizeof(char *));
+	curr_stats.prompt_hist.items = reallocarray(curr_stats.prompt_hist.items,
+			new_size, sizeof(char *));
+	curr_stats.filter_hist.items = reallocarray(curr_stats.filter_hist.items,
+			new_size, sizeof(char *));
+
+	if(delta > 0)
+	{
+		const size_t str_item_len = sizeof(char *)*delta;
+
+		memset(curr_stats.cmd_hist.items + old_size, 0, str_item_len);
+		memset(curr_stats.search_hist.items + old_size, 0, str_item_len);
+		memset(curr_stats.prompt_hist.items + old_size, 0, str_item_len);
+		memset(curr_stats.filter_hist.items + old_size, 0, str_item_len);
+	}
 }
 
 void
-dcache_get_of(const dir_entry_t *entry, uint64_t *size, uint64_t *nitems)
+hists_commands_save(const char command[])
 {
-	char full_path[PATH_MAX];
-	get_full_path_of(entry, sizeof(full_path), full_path);
-	dcache_get(full_path, size, nitems, entry->mtime);
+	if(is_history_command(command))
+	{
+		if(!curr_stats.restart_in_progress && curr_stats.load_stage == 3)
+		{
+			set_last_cmdline_command(command);
+		}
+		save_into_history(command, &curr_stats.cmd_hist, curr_stats.history_size);
+	}
 }
 
-/* Retrieves information about the path if data is newer than ts (0 requests to
- * skip the check).  size and/or nitems can be NULL.  On unknown values
- * variables are set to DCACHE_UNKNOWN. */
+/* Sets last_cmdline_command field of the status structure. */
 static void
-dcache_get(const char path[], uint64_t *size, uint64_t *nitems, time_t ts)
+set_last_cmdline_command(const char cmd[])
+{
+	const int err = replace_string(&curr_stats.last_cmdline_command, cmd);
+	if(err != 0)
+	{
+		LOG_ERROR_MSG("replace_string() failed on duplicating: %s", cmd);
+	}
+	assert(curr_stats.last_cmdline_command != NULL &&
+			"The field was not initialized properly");
+}
+
+void
+hists_search_save(const char pattern[])
+{
+	save_into_history(pattern, &curr_stats.search_hist, curr_stats.history_size);
+}
+
+void
+hists_prompt_save(const char input[])
+{
+	save_into_history(input, &curr_stats.prompt_hist, curr_stats.history_size);
+}
+
+void
+hists_filter_save(const char input[])
+{
+	save_into_history(input, &curr_stats.filter_hist, curr_stats.history_size);
+}
+
+/* Adaptor for the hist_add() function, which handles signed history length. */
+static void
+save_into_history(const char item[], hist_t *hist, int len)
+{
+	if(len >= 0)
+	{
+		hist_add(hist, item, len);
+	}
+}
+
+const char *
+hists_search_last(void)
+{
+	return hist_is_empty(&curr_stats.search_hist)
+	     ? ""
+	     : curr_stats.search_hist.items[0];
+}
+
+void
+dcache_get_at(const char path[], uint64_t *size, uint64_t *nitems)
 {
 	/* Initialization to make condition false by default. */
-	dcache_data_t size_data = { .timestamp = ts };
-	dcache_data_t nitems_data;
+	dcache_data_t size_data, nitems_data;
 
 	pthread_mutex_lock(&dcache_size_mutex);
-	if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) != 0 ||
-			(ts != 0 && ts > size_data.timestamp))
+	if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) != 0)
 	{
 		size_data.value = DCACHE_UNKNOWN;
-
-		if(ts != 0 && ts > size_data.timestamp)
-		{
-			fsdata_invalidate(dcache_size, path);
-		}
 	}
 	pthread_mutex_unlock(&dcache_size_mutex);
 
 	pthread_mutex_lock(&dcache_nitems_mutex);
-	if(fsdata_get(dcache_nitems, path, &nitems_data, sizeof(nitems_data)) != 0 ||
-			(ts != 0 && ts > nitems_data.timestamp))
+	if(fsdata_get(dcache_nitems, path, &nitems_data, sizeof(nitems_data)) != 0)
 	{
 		nitems_data.value = DCACHE_UNKNOWN;
 	}
@@ -418,6 +494,61 @@ dcache_get(const char path[], uint64_t *size, uint64_t *nitems, time_t ts)
 	}
 }
 
+void
+dcache_get_of(const dir_entry_t *entry, dcache_result_t *size,
+		dcache_result_t *nitems)
+{
+	dcache_data_t size_data, nitems_data;
+
+	char full_path[PATH_MAX + 1];
+	get_full_path_of(entry, sizeof(full_path), full_path);
+
+	size->value = DCACHE_UNKNOWN;
+	size->is_valid = 0;
+
+	nitems->value = DCACHE_UNKNOWN;
+	nitems->is_valid = 0;
+
+	pthread_mutex_lock(&dcache_size_mutex);
+	if(fsdata_get(dcache_size, full_path, &size_data, sizeof(size_data)) == 0)
+	{
+		size->value = size_data.value;
+		/* We check strictly for less than to handle scenario when multiple changes
+		 * occurred during the same second. */
+		size->is_valid = (entry->mtime < size_data.timestamp);
+	}
+	pthread_mutex_unlock(&dcache_size_mutex);
+
+	pthread_mutex_lock(&dcache_nitems_mutex);
+	if(fsdata_get(dcache_nitems, full_path, &nitems_data,
+				sizeof(nitems_data)) == 0)
+	{
+		nitems->value = nitems_data.value;
+		/* We check strictly for less than to handle scenario when multiple changes
+		 * occurred during the same second. */
+		nitems->is_valid = (entry->mtime < nitems_data.timestamp);
+	}
+	pthread_mutex_unlock(&dcache_nitems_mutex);
+}
+
+void
+dcache_update_parent_sizes(const char path[], uint64_t by)
+{
+	pthread_mutex_lock(&dcache_size_mutex);
+	(void)fsdata_map_parents(dcache_size, path, &size_updater, &by);
+	pthread_mutex_unlock(&dcache_size_mutex);
+}
+
+/* Updates cached value by a fixed amount. */
+static void
+size_updater(void *data, void *arg)
+{
+	const uint64_t *const by = arg;
+	dcache_data_t *const what = data;
+
+	what->value += *by;
+}
+
 int
 dcache_set_at(const char path[], uint64_t size, uint64_t nitems)
 {
