@@ -25,6 +25,7 @@
 
 #include "compat/fs_limits.h"
 #include "ui/color_scheme.h"
+#include "utils/hist.h"
 
 /* Special value foe dcache fields meaning that it wasn't set. */
 #define DCACHE_UNKNOWN ((uint64_t)-1)
@@ -91,23 +92,40 @@ typedef enum
 }
 ShellType;
 
+/* Type of output variables of dcache_get_of(), which represent state of cache
+ * entries. */
+typedef struct
+{
+	uint64_t value; /* Cached value or DCACHE_UNKNOWN. */
+	int is_valid;   /* Whether value is up-to-date. */
+}
+dcache_result_t;
+
+/* Current preview (quickview) parameters. */
+typedef struct
+{
+	char *cleanup_cmd;           /* Cleanup command. */
+	struct view_info_t *explore; /* State of explored quick view. */
+	unsigned int on : 1;         /* Whether preview mode is active. */
+	unsigned int graphical : 1;  /* Whether current preview displays graphics. */
+	unsigned int clearing : 1;   /* Whether in process of clearing preview. */
+}
+preview_t;
+
 typedef struct
 {
 	UpdateType need_update;
 	int last_char;
 	int save_msg; /* zero - don't save, 2 - save after resize, other - save */
 	int use_register;
-	int use_input_bar;
+	int use_input_bar;     /* Whether input bar should be updated. */
 	int curr_register;
 	int register_saved;
 	int number_of_windows;
 	int drop_new_dir_hist; /* Skip recording of new directory history. */
 	int load_stage; /* 0 - no TUI, 1 - part of TUI, 2 - TUI, 3 - all */
 
-	int view;              /* Whether preview mode is active. */
-	int graphics_preview;  /* Whether current preview displays graphics. */
-	char *preview_cleanup; /* Cleanup command for preview. */
-	int clear_preview;     /* Whether we're in process of clearing preview. */
+	preview_t preview; /* State of preview (quickview). */
 
 	/* Describes terminal state with regard to its dimensions. */
 	TermState term_state;
@@ -154,6 +172,8 @@ typedef struct
 	int initial_lines;   /* Initial terminal height in lines. */
 	int initial_columns; /* Initial terminal width in characters. */
 
+	const char *ellipsis; /* String representation of ellipsis. */
+
 	ShellType shell_type; /* Specifies type of shell. */
 
 	const char *fuse_umount_cmd; /* Command to use for fuse unmounting. */
@@ -170,6 +190,14 @@ typedef struct
 
 	int global_local_settings; /* Set local settings globally. */
 
+	int history_size;   /* Number of elements in histories. */
+	hist_t cmd_hist;    /* History of command-line commands. */
+	hist_t search_hist; /* History of search patterns. */
+	hist_t prompt_hist; /* History of prompt input. */
+	hist_t filter_hist; /* History of local filter patterns. */
+
+	struct ipc_t *ipc; /* Handle for IPC mechanism. */
+
 #ifdef HAVE_LIBGTK
 	int gtk_available; /* for mimetype detection */
 #endif
@@ -178,26 +206,24 @@ status_t;
 
 extern status_t curr_stats;
 
-/* Returns non-zero on error. */
-int init_status(struct config_t *config);
+/* Initializes curr_stats from the configuration.  Returns non-zero on error,
+ * otherwise zero is returned. */
+int stats_init(struct config_t *config);
 
 /* Resets some part of runtime status information to its initial values.
  * Returns non-zero on error. */
-int reset_status(const struct config_t *config);
+int stats_reset(const struct config_t *config);
 
 /* Sets internal flag to schedule postponed redraw operation of the UI. */
-void schedule_redraw(void);
+void stats_redraw_schedule(void);
 
 /* Checks for postponed redraw operations of the UI.  Has side effects.  Returns
  * non-zero if redraw operation was scheduled and resets internal flag. */
-int fetch_redraw_scheduled(void);
+int stats_redraw_fetch(void);
 
 /* Updates curr_stats to reflect whether terminal multiplexers support is
  * enabled. */
-void set_using_term_multiplexer(int use_term_multiplexer);
-
-/* Updates last_cmdline_command field of the status structure. */
-void update_last_cmdline_command(const char cmd[]);
+void stats_set_use_multiplexer(int use_term_multiplexer);
 
 /* Updates curr_stats.shell_type field according to passed shell command. */
 void stats_update_shell_type(const char shell_cmd[]);
@@ -229,17 +255,40 @@ int stats_file_choose_action_set(void);
 /* Records status bar message. */
 void stats_save_msg(const char msg[]);
 
+/* Managing histories. */
+
+/* Changes size of histories stored in status_t.  Zero or negative value
+ * disables them. */
+void hists_resize(int new_size);
+
+/* Saves command to command history. */
+void hists_commands_save(const char command[]);
+
+/* Saves pattern to search history. */
+void hists_search_save(const char pattern[]);
+
+/* Saves input to prompt history. */
+void hists_prompt_save(const char input[]);
+
+/* Saves input to local filter history. */
+void hists_filter_save(const char pattern[]);
+
+/* Gets the most recently used search pattern.  Returns the pattern or empty
+ * string if search history is empty. */
+const char * hists_search_last(void);
+
 /* Caching of information about directories. */
 
 /* Retrieves information about the path.  size and/or nitems can be NULL.  On
  * unknown values variables are set to DCACHE_UNKNOWN. */
 void dcache_get_at(const char path[], uint64_t *size, uint64_t *nitems);
 
-/* Retrieves information about the entry.  size and/or nitems can be NULL.  On
- * unknown values variables are set to DCACHE_UNKNOWN.  Values older than entry
- * modification date are considered unknown. */
-void dcache_get_of(const struct dir_entry_t *entry, uint64_t *size,
-		uint64_t *nitems);
+/* Retrieves information about the entry checking whether it's outdated. */
+void dcache_get_of(const struct dir_entry_t *entry, dcache_result_t *size,
+		dcache_result_t *nitems);
+
+/* Updates cached sizes of parents by specified amount. */
+void dcache_update_parent_sizes(const char path[], uint64_t by);
 
 /* Updates information about the path.  Returns zero on success, otherwise
  * non-zero is returned. */
