@@ -73,12 +73,16 @@ const char *tags[] = {
 	"vifm-'lines'",
 	"vifm-'locateprg'",
 	"vifm-'ls'",
+	"vifm-'lsoptions'",
 	"vifm-'lsview'",
+	"vifm-'milleroptions'",
+	"vifm-'millerview'",
 	"vifm-'mintimeoutlen'",
 	"vifm-'nu'",
 	"vifm-'number'",
 	"vifm-'numberwidth'",
 	"vifm-'nuw'",
+	"vifm-'previewprg'",
 	"vifm-'relativenumber'",
 	"vifm-'rnu'",
 	"vifm-'ruf'",
@@ -140,6 +144,7 @@ const char *tags[] = {
 	"vifm---no-configs",
 	"vifm---on-choose",
 	"vifm---remote",
+	"vifm---remote-expr",
 	"vifm---select",
 	"vifm---server-list",
 	"vifm---server-name",
@@ -244,7 +249,9 @@ const char *tags[] = {
 	"vifm-:hi",
 	"vifm-:highlight",
 	"vifm-:his",
+	"vifm-:histnext",
 	"vifm-:history",
+	"vifm-:histprev",
 	"vifm-:if",
 	"vifm-:invert",
 	"vifm-:jobs",
@@ -427,13 +434,17 @@ const char *tags[] = {
 	"vifm-Y",
 	"vifm-ZQ",
 	"vifm-ZZ",
+	"vifm-[R",
 	"vifm-[c",
 	"vifm-[count]",
 	"vifm-[d",
+	"vifm-[r",
 	"vifm-[s",
 	"vifm-[z",
+	"vifm-]R",
 	"vifm-]c",
 	"vifm-]d",
+	"vifm-]r",
 	"vifm-]s",
 	"vifm-]z",
 	"vifm-^",
@@ -506,6 +517,8 @@ const char *tags[] = {
 	"vifm-compare-views",
 	"vifm-configure",
 	"vifm-count",
+	"vifm-count-variable",
+	"vifm-count1-variable",
 	"vifm-cp",
 	"vifm-cpo-f",
 	"vifm-cpo-s",
@@ -708,11 +721,15 @@ const char *tags[] = {
 	"vifm-startup",
 	"vifm-system()",
 	"vifm-t",
+	"vifm-term()",
 	"vifm-to-p",
 	"vifm-to-s",
+	"vifm-to-u",
 	"vifm-trash",
 	"vifm-u",
 	"vifm-v",
+	"vifm-v:count",
+	"vifm-v:count1",
 	"vifm-v:servername",
 	"vifm-v_:",
 	"vifm-v_CTRL-C",
