@@ -22,10 +22,9 @@
 
 #include <assert.h> /* assert() */
 
+#include "../utils/cancellation.h"
 #include "../status.h"
 
-static int ui_cancellation_enabled(void);
-
 /* State of cancellation request processing. */
 typedef enum
 {
@@ -36,10 +35,22 @@ typedef enum
 }
 cancellation_request_state;
 
+static int ui_cancellation_hook(void *arg);
+static int ui_cancellation_enabled(void);
+
+const cancellation_t ui_cancellation_info = { .hook = &ui_cancellation_hook };
+
 /* Whether cancellation was requested.  Used by ui_cancellation_* group of
  * functions. */
 static cancellation_request_state cancellation_state;
 
+/* Implementation of cancellation hook for. */
+static int
+ui_cancellation_hook(void *arg)
+{
+	return ui_cancellation_requested();
+}
+
 void
 ui_cancellation_reset(void)
 {
@@ -59,7 +70,7 @@ ui_cancellation_enable(void)
 
 	/* The check is here for tests, which are running with uninitialized
 	 * curses. */
-	if(curr_stats.load_stage > 0)
+	if(curr_stats.load_stage > 2)
 	{
 		/* Temporary disable raw mode of terminal so that Ctrl-C is handled as
 			* SIGINT signal rather than as regular input character. */
@@ -90,7 +101,7 @@ ui_cancellation_disable(void)
 
 	/* The check is here for tests, which are running with uninitialized
 	 * curses. */
-	if(curr_stats.load_stage > 0)
+	if(curr_stats.load_stage > 2)
 	{
 		/* Restore raw mode of terminal so that Ctrl-C is be handled as regular
 		 * input character rather than as SIGINT signal. */
@@ -112,7 +123,7 @@ ui_cancellation_pause(void)
 
 	/* The check is here for tests, which are running with uninitialized
 	 * curses. */
-	if(curr_stats.load_stage > 0)
+	if(curr_stats.load_stage > 2)
 	{
 		raw();
 	}
@@ -127,7 +138,7 @@ ui_cancellation_resume(int state)
 		assert(ui_cancellation_enabled() && "Invalid cancellation resume");
 		/* The check is here for tests, which are running with uninitialized
 		 * curses. */
-		if(curr_stats.load_stage > 0)
+		if(curr_stats.load_stage > 2)
 		{
 			noraw();
 		}
