@@ -29,6 +29,7 @@
 #include <string.h> /* strcpy() strlen() */
 
 #include "../cfg/config.h"
+#include "../compat/dtype.h"
 #include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../compat/reallocarray.h"
@@ -68,6 +69,7 @@ char *HI_GROUPS[] = {
 	[JOB_LINE_COLOR]     = "JobLine",
 	[SUGGEST_BOX_COLOR]  = "SuggestBox",
 	[MISMATCH_COLOR]     = "CmpMismatch",
+	[AUX_WIN_COLOR]      = "AuxWin",
 };
 ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR);
 
@@ -93,6 +95,7 @@ const char *HI_GROUPS_DESCR[] = {
 	[JOB_LINE_COLOR]     = "job bar",
 	[SUGGEST_BOX_COLOR]  = "suggestion box",
 	[MISMATCH_COLOR]     = "mismatched diff entries",
+	[AUX_WIN_COLOR]      = "auxiliary part of window",
 };
 ARRAY_GUARD(HI_GROUPS_DESCR, ARRAY_LEN(HI_GROUPS));
 
@@ -390,6 +393,7 @@ static const col_attr_t default_cs[] = {
 	[JOB_LINE_COLOR]     = { COLOR_BLACK,   COLOR_WHITE, A_BOLD | A_REVERSE      },
 	[SUGGEST_BOX_COLOR]  = { -1,            -1,          A_BOLD                  },
 	[MISMATCH_COLOR]     = { COLOR_WHITE,   COLOR_RED,   A_BOLD                  },
+	[AUX_WIN_COLOR]      = { -1,            -1,          -1                      },
 };
 ARRAY_GUARD(default_cs, MAXNUM_COLOR);
 
@@ -443,15 +447,19 @@ cs_rename_all(void)
 
 	while((d = os_readdir(dir)) != NULL)
 	{
+#ifndef _WIN32
+		unsigned char type;
+#endif
 		char full_old_path[PATH_MAX + 16 + NAME_MAX];
 		char full_new_path[PATH_MAX + 16 + NAME_MAX];
 		snprintf(full_old_path, sizeof(full_old_path), "%s/%s", cfg.colors_dir,
 				d->d_name);
 		snprintf(full_new_path, sizeof(full_new_path), "%s/%s.vifm", cfg.colors_dir,
 				d->d_name);
+
 #ifndef _WIN32
-		if(d->d_type == DT_REG ||
-				(d->d_type == DT_UNKNOWN && is_regular_file(full_old_path)))
+		type = get_dirent_type(d, full_old_path);
+		if(type == DT_REG || (type == DT_UNKNOWN && is_regular_file(full_old_path)))
 #else
 		if(is_regular_file(full_old_path))
 #endif
@@ -999,18 +1007,33 @@ cs_mix_colors(col_attr_t *base, const col_attr_t *mixup)
 	}
 }
 
-int
+void
 cs_add_file_hi(struct matchers_t *matchers, const col_attr_t *hi)
 {
 	void *p;
 	file_hi_t *file_hi;
 	col_scheme_t *const cs = curr_stats.cs;
 
+	/* Try to find existing record that exactly matches the expression and update
+	 * the record. */
+	int i;
+	const char *expr = matchers_get_expr(matchers);
+	for(i = 0; i < cs->file_hi_count; ++i)
+	{
+		if(strcmp(matchers_get_expr(cs->file_hi[i].matchers), expr) == 0)
+		{
+			matchers_free(matchers);
+			cs->file_hi[i].hi = *hi;
+			return;
+		}
+	}
+
 	p = reallocarray(cs->file_hi, cs->file_hi_count + 1, sizeof(*cs->file_hi));
 	if(p == NULL)
 	{
+		matchers_free(matchers);
 		show_error_msg("Color Scheme File Highlight", "Not enough memory");
-		return 1;
+		return;
 	}
 	cs->file_hi = p;
 
@@ -1020,8 +1043,6 @@ cs_add_file_hi(struct matchers_t *matchers, const col_attr_t *hi)
 	file_hi->hi = *hi;
 
 	++cs->file_hi_count;
-
-	return 0;
 }
 
 const col_attr_t *
@@ -1048,6 +1069,26 @@ cs_get_file_hi(const col_scheme_t *cs, const char fname[], int *hi_hint)
 	return NULL;
 }
 
+int
+cs_del_file_hi(const char matchers_expr[])
+{
+	col_scheme_t *const cs = curr_stats.cs;
+
+	int i;
+	for(i = 0; i < cs->file_hi_count; ++i)
+	{
+		if(strcmp(matchers_get_expr(cs->file_hi[i].matchers), matchers_expr) == 0)
+		{
+			memmove(&cs->file_hi[i], &cs->file_hi[i + 1],
+					sizeof(*cs->file_hi)*((cs->file_hi_count - 1) - i));
+			--cs->file_hi_count;
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 int
 cs_is_color_set(const col_attr_t *color)
 {
