@@ -29,9 +29,6 @@
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
 
-/* Maximum number of ellipsis dots. */
-#define MAX_ELLIPSIS_DOT_COUNT 3U
-
 /* Character used to fill gaps in lines. */
 #define GAP_FILL_CHAR ' '
 
@@ -71,8 +68,7 @@ static void init_new_column(column_t *col, column_info_t info);
 static void mark_for_recalculation(columns_t *cols);
 static column_func get_column_func(int column_id);
 static AlignType decorate_output(const column_t *col, char buf[],
-		size_t max_line_width);
-static void add_ellipsis(AlignType align, char buf[]);
+		size_t buf_len, size_t max_line_width);
 static size_t calculate_max_width(const column_t *col, size_t len,
 		size_t max_line_width);
 static size_t calculate_start_pos(const column_t *col, const char buf[],
@@ -80,7 +76,6 @@ static size_t calculate_start_pos(const column_t *col, const char buf[],
 static void fill_gap_pos(const void *data, size_t from, size_t to);
 static size_t get_width_on_screen(const char str[]);
 static void recalculate_if_needed(columns_t *cols, size_t max_width);
-static int recalculation_is_needed(const columns_t *cols, size_t max_width);
 static void recalculate(columns_t *cols, size_t max_width);
 static void update_widths(columns_t *cols, size_t max_width);
 static int update_abs_and_rel_widths(columns_t *cols, size_t *max_width);
@@ -94,6 +89,8 @@ static size_t col_desc_count;
 static column_desc_t *col_descs;
 /* Column print function. */
 static column_line_print_func print_func;
+/* String to be used in place of ellipsis. */
+static const char *ellipsis = "...";
 
 void
 columns_set_line_print_func(column_line_print_func func)
@@ -101,6 +98,12 @@ columns_set_line_print_func(column_line_print_func func)
 	print_func = func;
 }
 
+void
+columns_set_ellipsis(const char ell[])
+{
+	ellipsis = ell;
+}
+
 int
 columns_add_column_desc(int column_id, column_func func)
 {
@@ -282,7 +285,8 @@ columns_format_line(columns_t *cols, const void *data, size_t max_line_width)
 
 		col->func(col->info.column_id, data, sizeof(col_buffer), col_buffer);
 		strcpy(full_column, col_buffer);
-		align = decorate_output(col, col_buffer, max_line_width);
+		align = decorate_output(col, col_buffer, sizeof(col_buffer),
+				max_line_width);
 		cur_col_start = calculate_start_pos(col, col_buffer, align);
 
 		/* Ensure that we are not trying to draw current column in the middle of a
@@ -320,12 +324,15 @@ columns_format_line(columns_t *cols, const void *data, size_t max_line_width)
 /* Adds decorations like ellipsis to the output.  Returns actual align type used
  * for the column (might not match col->info.align). */
 static AlignType
-decorate_output(const column_t *col, char buf[], size_t max_line_width)
+decorate_output(const column_t *col, char buf[], size_t buf_len,
+		size_t max_line_width)
 {
 	const size_t len = get_width_on_screen(buf);
 	const size_t max_col_width = calculate_max_width(col, len, max_line_width);
 	const int too_long = len > max_col_width;
 	AlignType result;
+	const char *const ell = (col->info.cropping == CT_ELLIPSIS ? ellipsis : "");
+	char *ellipsed;
 
 	if(!too_long)
 	{
@@ -335,71 +342,21 @@ decorate_output(const column_t *col, char buf[], size_t max_line_width)
 	if(col->info.align == AT_LEFT ||
 			(col->info.align == AT_DYN && len <= max_col_width))
 	{
-		const size_t truncate_pos = utf8_strsnlen(buf, max_col_width);
-		buf[truncate_pos] = '\0';
+		ellipsed = right_ellipsis(buf, max_col_width, ell);
 		result = AT_LEFT;
 	}
 	else
 	{
-		int extra_spaces;
-
-		const size_t truncate_pos = utf8_strsnlen(buf, len - max_col_width);
-		const char *new_beginning = buf + truncate_pos;
-
-		extra_spaces = 0;
-		while(get_width_on_screen(new_beginning) > max_col_width)
-		{
-			++extra_spaces;
-			new_beginning += utf8_chrw(new_beginning);
-		}
-
-		memmove(buf + extra_spaces, new_beginning, strlen(new_beginning) + 1);
-		if(extra_spaces != 0)
-		{
-			memset(buf, ' ', extra_spaces);
-		}
-
-		assert(get_width_on_screen(buf) == max_col_width && "Column isn't filled.");
+		ellipsed = left_ellipsis(buf, max_col_width, ell);
 		result = AT_RIGHT;
 	}
 
-	if(col->info.cropping == CT_ELLIPSIS)
-	{
-		add_ellipsis(result, buf);
-	}
+	copy_str(buf, buf_len, ellipsed);
+	free(ellipsed);
 
 	return result;
 }
 
-/* Adds ellipsis to the string in buf not changing enlarging its length (at most
- * three first or last characters are replaced). */
-static void
-add_ellipsis(AlignType align, char buf[])
-{
-	const size_t len = get_width_on_screen(buf);
-	const size_t dot_count = MIN(len, MAX_ELLIPSIS_DOT_COUNT);
-	if(align == AT_LEFT)
-	{
-		const size_t width_limit = len - dot_count;
-		const size_t pos = utf8_strsnlen(buf, width_limit);
-		memset(buf + pos, '.', dot_count);
-		buf[pos + dot_count] = '\0';
-	}
-	else
-	{
-		const char *new_beginning = buf;
-		size_t skipped = 0;
-		while(skipped < dot_count)
-		{
-			skipped += utf8_chrsw(new_beginning);
-			new_beginning += utf8_chrw(new_beginning);
-		}
-
-		memmove(buf + dot_count, new_beginning, strlen(new_beginning) + 1);
-		memset(buf, '.', dot_count);
-	}
-}
-
 /* Calculates maximum width for outputting content of the col. */
 static size_t
 calculate_max_width(const column_t *col, size_t len, size_t max_line_width)
@@ -470,17 +427,16 @@ get_width_on_screen(const char str[])
 static void
 recalculate_if_needed(columns_t *cols, size_t max_width)
 {
-	if(recalculation_is_needed(cols, max_width))
+	if(!columns_matches_width(cols, max_width))
 	{
 		recalculate(cols, max_width);
 	}
 }
 
-/* Checks if recalculation is needed, and returns non-zero if so. */
-static int
-recalculation_is_needed(const columns_t *cols, size_t max_width)
+int
+columns_matches_width(const columns_t *cols, size_t max_width)
 {
-	return cols->max_width != max_width;
+	return cols->max_width == max_width;
 }
 
 /* Recalculates column widths and start offsets. */
