@@ -20,10 +20,9 @@
 #ifndef VIFM__UI__FILEVIEW_H__
 #define VIFM__UI__FILEVIEW_H__
 
-#include <stddef.h> /* size_t */
-
 #include "../utils/test_helpers.h"
-#include "ui.h"
+
+struct view_t;
 
 /* Initialization/termination functions. */
 
@@ -31,97 +30,70 @@
 void fview_init(void);
 
 /* Initializes view once before. */
-void fview_view_init(FileView *view);
+void fview_view_init(struct view_t *view);
 
 /* Resets view state partially. */
-void fview_view_reset(FileView *view);
+void fview_view_reset(struct view_t *view);
 
 /* Resets view state with regard to color schemes. */
-void fview_view_cs_reset(FileView *view);
+void fview_view_cs_reset(struct view_t *view);
 
 /* Appearance related functions. */
 
 /* Redraws directory list and puts inactive mark for the other view. */
-void draw_dir_list(FileView *view);
+void draw_dir_list(struct view_t *view);
 
-/* Redraws directory list without any extra actions that are performed in
- * draw_dir_list(). */
-void draw_dir_list_only(FileView *view);
+/* Redraws directory list without putting inactive mark if view == other. */
+void draw_dir_list_only(struct view_t *view);
 
 /* Updates view (maybe postponed) on the screen (redraws file list and
  * cursor). */
-void redraw_view(FileView *view);
-
-/* Updates view immediately on the screen (redraws file list and cursor). */
-void redraw_view_imm(FileView *view);
+void redraw_view(struct view_t *view);
 
 /* Updates current view (maybe postponed) on the screen (redraws file list and
  * cursor) */
 void redraw_current_view(void);
 
-/* Restores normal appearance of item under the cursor. */
-void erase_current_line_bar(FileView *view);
+/* Adds inactive cursor mark to the view. */
+void put_inactive_mark(struct view_t *view);
 
 /* Redraws cursor of the view on the screen. */
-void fview_cursor_redraw(FileView *view);
-
-/* Viewport related functions. */
-
-/* Checks whether if all files are visible, so no scrolling is needed.  Returns
- * non-zero if so, and zero otherwise. */
-int all_files_visible(const FileView *view);
-
-/* Gets file position of last visible cell in the view.  Value returned may be
- * greater than or equal to the number of files in the view and thus should be
- * treated correctly.  Returns the index. */
-size_t get_last_visible_cell(const FileView *view);
-
-/* Calculates position in list of files that corresponds to window top, which is
- * adjusted according to 'scrolloff' option.  Returns the position. */
-size_t get_window_top_pos(const FileView *view);
-
-/* Calculates position in list of files that corresponds to window middle, which
- * is adjusted according to 'scrolloff' option.  Returns the position. */
-size_t get_window_middle_pos(const FileView *view);
-
-/* Calculates position in list of files that corresponds to window bottom, which
- * is adjusted according to 'scrolloff' option.  Returns the position. */
-size_t get_window_bottom_pos(const FileView *view);
+void fview_cursor_redraw(struct view_t *view);
 
 /* Scrolling related functions. */
 
 /* Checks if view can be scrolled up (there are more files).  Returns non-zero
  * if so, and zero otherwise. */
-int can_scroll_up(const FileView *view);
+int can_scroll_up(const struct view_t *view);
 
 /* Checks if view can be scrolled down (there are more files).  Returns non-zero
  * if so, and zero otherwise. */
-int can_scroll_down(const FileView *view);
+int can_scroll_down(const struct view_t *view);
 
 /* Scrolls view up at least by specified number of files.  Updates both top and
  * cursor positions. */
-void scroll_up(FileView *view, size_t by);
+void scroll_up(struct view_t *view, int by);
 
 /* Scrolls view down at least by specified number of files.  Updates both top
  * and cursor positions. */
-void scroll_down(FileView *view, size_t by);
+void scroll_down(struct view_t *view, int by);
 
 /* Calculates list position corrected for scrolling down.  Returns adjusted
  * position. */
-int get_corrected_list_pos_down(const FileView *view, size_t pos_delta);
+int get_corrected_list_pos_down(const struct view_t *view, int pos_delta);
 
 /* Calculates list position corrected for scrolling up.  Returns adjusted
  * position. */
-int get_corrected_list_pos_up(const FileView *view, size_t pos_delta);
+int get_corrected_list_pos_up(const struct view_t *view, int pos_delta);
 
 /* Updates current and top line of a view according to 'scrolloff' option value.
  * Returns non-zero if redraw is needed. */
-int consider_scroll_offset(FileView *view);
+int consider_scroll_offset(struct view_t *view);
 
 /* Scrolls view down or up at least by specified number of files.  Updates both
  * top and cursor positions.  A wrapper for scroll_up() and scroll_down()
  * functions. */
-void scroll_by_files(FileView *view, ssize_t by);
+void scroll_by_files(struct view_t *view, int by);
 
 /* Recalculates difference of two panes scroll positions. */
 void update_scroll_bind_offset(void);
@@ -129,49 +101,64 @@ void update_scroll_bind_offset(void);
 /* Layout related functions. */
 
 /* Enables/disables ls-like style of the view. */
-void fview_set_lsview(FileView *view, int enabled);
+void fview_set_lsview(struct view_t *view, int enabled);
 
-/* Evaluates number of columns in the view.  Returns the number. */
-size_t calculate_columns_count(FileView *view);
+/* Checks whether view displays grid that's filled by column. */
+int fview_is_transposed(const struct view_t *view);
+
+/* Enables/disables cascading columns style of the view. */
+void fview_set_millerview(struct view_t *view, int enabled);
+
+/* Requests update of view geometry properties (stuff that depends on
+ * dimensions; there is also an implicit dependency on file list, because grid
+ * is defined by longest file name). */
+void fview_update_geometry(struct view_t *view);
 
 /* Callback-like function which triggers some view-specific updates after
  * directory of the view changes. */
-void fview_dir_updated(FileView *view);
+void fview_dir_updated(struct view_t *view);
 
 /* Callback-like function which triggers some view-specific updates after list
  * of files changes. */
-void fview_list_updated(FileView *view);
+void fview_list_updated(struct view_t *view);
 
 /* Callback-like function which triggers some view-specific updates after cursor
  * position in the list changed. */
-void fview_position_updated(FileView *view);
+void fview_position_updated(struct view_t *view);
 
 /* Callback-like function which triggers some view-specific updates after view
  * sorting changed. */
-void fview_sorting_updated(FileView *view);
+void fview_sorting_updated(struct view_t *view);
 
 #ifdef TEST
+#include <stddef.h> /* size_t */
 
-/* Packet set of parameters to pass as user data for processing columns. */
-typedef struct
-{
-	FileView *view;    /* View on which cell is being drawn. */
-	size_t line_pos;   /* File position in the file list (the view). */
-	int line_hi_group; /* Cached line highlight (avoid per-column calculation). */
-	int is_current;    /* Whether this file is selected with the cursor. */
-
-	size_t current_line;  /* Line of the cell. */
-	size_t column_offset; /* Offset in characters of the column. */
-
-	size_t *prefix_len; /* Data prefix length (should be drawn in neutral color).
-	                     * A pointer to allow changing value in const struct.
-	                     * Should be zero first time, then auto reset. */
-}
-column_data_t;
-
+#include "ui.h"
 #endif
 
 TSTATIC_DEFS(
+	/* Packet set of parameters to pass as user data for processing columns. */
+	typedef struct
+	{
+		struct view_t *view; /* View on which cell is being drawn. */
+		dir_entry_t *entry;  /* Entry that is being displayed. */
+		int line_pos;        /* File position in the file list (the view). */
+		int line_hi_group;   /* Line highlight (to avoid per-column calculation). */
+		int current_pos;     /* Position of entry selected with the cursor. */
+		int total_width;     /* Total width available for drawing. */
+		int number_width;    /* Width of line number column (0 when disabled). */
+
+		size_t current_line;  /* Line of the cell within the view window. */
+		size_t column_offset; /* Offset in characters of the column. */
+
+		size_t *prefix_len; /* Data prefix length (should be drawn in neutral
+		                       color).  A pointer to allow changing value in const
+		                       struct.  Should be zero first time, then auto
+		                       reset. */
+		int is_main;        /* Whether this is main file list. */
+	}
+	column_data_t;
+
 	void format_name(int id, const void *data, size_t buf_len, char buf[]);
 )
 
