@@ -98,11 +98,11 @@ static char * expand_viewer_command(const char viewer[]);
 int
 qv_ensure_is_shown(void)
 {
-	if(!curr_stats.view && !qv_can_show())
+	if(!curr_stats.preview.on && !qv_can_show())
 	{
 		return 1;
 	}
-	curr_stats.view = 1;
+	curr_stats.preview.on = 1;
 	return 0;
 }
 
@@ -111,12 +111,12 @@ qv_can_show(void)
 {
 	if(curr_stats.number_of_windows == 1)
 	{
-		status_bar_error("Cannot view files in one window mode");
+		ui_sb_err("Cannot view files in one window mode");
 		return 0;
 	}
 	if(other_view->explore_mode)
 	{
-		status_bar_error("Other view is already used for file viewing");
+		ui_sb_err("Other view is already used for file viewing");
 		return 0;
 	}
 	return 1;
@@ -125,41 +125,41 @@ qv_can_show(void)
 void
 qv_toggle(void)
 {
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
-		curr_stats.view = 0;
+		curr_stats.preview.on = 0;
 
 		if(ui_view_is_visible(other_view))
 		{
 			/* Force cleaning possible leftovers of graphics, otherwise curses
 			 * internal structures don't know that those parts need to be redrawn on
 			 * the screen. */
-			if(curr_stats.preview_cleanup != NULL || curr_stats.graphics_preview)
+			if(curr_stats.preview.cleanup_cmd != NULL || curr_stats.preview.graphical)
 			{
-				qv_cleanup(other_view, curr_stats.preview_cleanup);
+				qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
 			}
 
 			draw_dir_list(other_view);
 			refresh_view_win(other_view);
 		}
 
-		update_string(&curr_stats.preview_cleanup, NULL);
-		curr_stats.graphics_preview = 0;
+		update_string(&curr_stats.preview.cleanup_cmd, NULL);
+		curr_stats.preview.graphical = 0;
 	}
 	else
 	{
-		curr_stats.view = 1;
+		curr_stats.preview.on = 1;
 		qv_draw(curr_view);
 	}
 }
 
 void
-qv_draw(FileView *view)
+qv_draw(view_t *view)
 {
 	const dir_entry_t *curr;
 
 	if(curr_stats.load_stage < 2 || curr_stats.number_of_windows == 1 ||
-	   vle_mode_is(VIEW_MODE) || draw_abandoned_view_mode())
+	   vle_mode_is(VIEW_MODE) || view_detached_draw())
 	{
 		return;
 	}
@@ -222,7 +222,7 @@ view_entry(const dir_entry_t *entry)
 static void
 view_file(const char path[])
 {
-	int graphics = 0;
+	int graphical = 0;
 	const char *viewer;
 	const char *clear_cmd;
 	FILE *fp;
@@ -253,12 +253,12 @@ view_file(const char path[])
 	}
 	else
 	{
-		graphics = is_graphics_viewer(viewer);
+		graphical = is_graphical_viewer(viewer);
 		/* If graphics will be displayed, clear the window and wait a bit to let
 		 * terminal emulator do actual refresh (at least some of them need this). */
-		if(graphics)
+		if(graphical)
 		{
-			qv_cleanup(other_view, curr_stats.preview_cleanup);
+			qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
 			usleep(50000);
 		}
 		fp = qv_execute_viewer(viewer);
@@ -274,14 +274,14 @@ view_file(const char path[])
 
 	/* We want to wipe the view if it was displaying graphics, but won't anymore.
 	 * Do this only if we didn't already cleared the window. */
-	if(!graphics)
+	if(!graphical)
 	{
 		cleanup_for_text();
 	}
-	curr_stats.graphics_preview = graphics;
+	curr_stats.preview.graphical = graphical;
 
 	clear_cmd = (viewer != NULL) ? ma_get_clear_cmd(viewer) : NULL;
-	update_string(&curr_stats.preview_cleanup, clear_cmd);
+	update_string(&curr_stats.preview.cleanup_cmd, clear_cmd);
 
 	wattrset(other_view->win, 0);
 	view_stream(fp, cfg.wrap_quick_view);
@@ -614,28 +614,28 @@ write_message(const char msg[])
 static void
 cleanup_for_text(void)
 {
-	if(curr_stats.preview_cleanup != NULL || curr_stats.graphics_preview)
+	if(curr_stats.preview.cleanup_cmd != NULL || curr_stats.preview.graphical)
 	{
-		qv_cleanup(other_view, curr_stats.preview_cleanup);
+		qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
 	}
-	update_string(&curr_stats.preview_cleanup, NULL);
-	curr_stats.graphics_preview = 0;
+	update_string(&curr_stats.preview.cleanup_cmd, NULL);
+	curr_stats.preview.graphical = 0;
 }
 
 void
 qv_hide(void)
 {
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
 		qv_toggle();
 	}
 	if(lwin.explore_mode)
 	{
-		view_explore_mode_quit(&lwin);
+		view_quit_explore_mode(&lwin);
 	}
 	if(rwin.explore_mode)
 	{
-		view_explore_mode_quit(&rwin);
+		view_quit_explore_mode(&rwin);
 	}
 }
 
@@ -660,14 +660,7 @@ expand_viewer_command(const char viewer[])
 	char *result;
 	if(strchr(viewer, '%') == NULL)
 	{
-		char *escaped;
-		FileView *view = curr_stats.preview_hint;
-		if(view == NULL)
-		{
-			view = curr_view;
-		}
-
-		escaped = shell_like_escape(get_current_file_name(view), 0);
+		char *escaped = shell_like_escape(get_current_file_name(curr_view), 0);
 		result = format_str("%s %s", viewer, escaped);
 		free(escaped);
 	}
@@ -679,9 +672,9 @@ expand_viewer_command(const char viewer[])
 }
 
 void
-qv_cleanup(FileView *view, const char cmd[])
+qv_cleanup(view_t *view, const char cmd[])
 {
-	FileView *const curr = curr_view;
+	view_t *const curr = curr_view;
 	FILE *fp;
 
 	if(cmd == NULL)
@@ -691,9 +684,9 @@ qv_cleanup(FileView *view, const char cmd[])
 	}
 
 	curr_view = view;
-	curr_stats.clear_preview = 1;
+	curr_stats.preview.clearing = 1;
 	fp = qv_execute_viewer(cmd);
-	curr_stats.clear_preview = 0;
+	curr_stats.preview.clearing = 0;
 	curr_view = curr;
 
 	while(fgetc(fp) != EOF);
@@ -705,12 +698,20 @@ qv_cleanup(FileView *view, const char cmd[])
 const char *
 qv_get_viewer(const char path[])
 {
-	char *const typed_fname = is_dir(path)
-	                        ? format_str("%s/", path)
-	                        : strdup(path);
-	const char *const viewer = ft_get_viewer(typed_fname);
-	free(typed_fname);
-	return viewer;
+	if(is_null_or_empty(curr_view->preview_prg))
+	{
+		char *const typed_fname = is_dir(path)
+		                        ? format_str("%s/", path)
+		                        : strdup(path);
+		const char *const viewer = ft_get_viewer(typed_fname);
+		free(typed_fname);
+		return viewer;
+	}
+	else
+	{
+		/* 'previewprg' option has priority over :fileviewer command. */
+		return curr_view->preview_prg;
+	}
 }
 
 void
