@@ -47,10 +47,10 @@
 #include "../filelist.h"
 #include "ui.h"
 
-static void update_stat_window_old(FileView *view, int lazy_redraw);
+static void update_stat_window_old(view_t *view, int lazy_redraw);
 static void refresh_window(WINDOW *win, int lazily);
-TSTATIC char * expand_status_line_macros(FileView *view, const char format[]);
-static char * parse_view_macros(FileView *view, const char **format,
+TSTATIC char * expand_status_line_macros(view_t *view, const char format[]);
+static char * parse_view_macros(view_t *view, const char **format,
 		const char macros[], int opt);
 static int expand_num(char buf[], size_t buf_len, int val);
 static const char * get_tip(void);
@@ -58,7 +58,6 @@ static void check_expanded_str(const char buf[], int skip, int *nexpansions);
 static pthread_spinlock_t * get_job_bar_changed_lock(void);
 static void init_job_bar_changed_lock(void);
 static int is_job_bar_visible(void);
-static void update_job_bar(void);
 static const char * format_job_bar(void);
 static char ** take_job_descr_snapshot(void);
 
@@ -72,7 +71,7 @@ static int job_bar_changed;
 static pthread_spinlock_t job_bar_changed_lock;
 
 void
-update_stat_window(FileView *view, int lazy_redraw)
+ui_stat_update(view_t *view, int lazy_redraw)
 {
 	int x;
 	char *buf;
@@ -115,7 +114,7 @@ update_stat_window(FileView *view, int lazy_redraw)
 /* Formats status line in the "old way" (before introduction of 'statusline'
  * option). */
 static void
-update_stat_window_old(FileView *view, int lazy_redraw)
+update_stat_window_old(view_t *view, int lazy_redraw)
 {
 	const dir_entry_t *const curr = get_current_entry(view);
 	char name_buf[160*2 + 1];
@@ -141,8 +140,9 @@ update_stat_window_old(FileView *view, int lazy_redraw)
 
 	filename = get_current_file_name(view);
 	print_width = utf8_strsnlen(filename, 20 + MAX(0, x - 83));
-	snprintf(name_buf, MIN(sizeof(name_buf), print_width + 1), "%s", filename);
-	friendly_size_notation(curr->size, sizeof(size_buf), size_buf);
+	copy_str(name_buf, MIN(sizeof(name_buf), print_width + 1), filename);
+	friendly_size_notation(fentry_get_size(view, curr), sizeof(size_buf),
+			size_buf);
 
 	get_uid_string(curr, 0, sizeof(id_buf), id_buf);
 	if(id_buf[0] != '\0')
@@ -152,7 +152,7 @@ update_stat_window_old(FileView *view, int lazy_redraw)
 #ifndef _WIN32
 	get_perm_string(perm_buf, sizeof(perm_buf), curr->mode);
 #else
-	snprintf(perm_buf, sizeof(perm_buf), "%s", attr_str_long(curr->attrs));
+	copy_str(perm_buf, sizeof(perm_buf), attr_str_long(curr->attrs));
 #endif
 
 	werase(stat_win);
@@ -199,15 +199,15 @@ refresh_window(WINDOW *win, int lazily)
  * format string.  Returns newly allocated string, which should be freed by the
  * caller, or NULL if there is not enough memory. */
 TSTATIC char *
-expand_status_line_macros(FileView *view, const char format[])
+expand_status_line_macros(view_t *view, const char format[])
 {
-	return expand_view_macros(view, format, "tTfaAugsEdD-lLSz%[]");
+	return expand_view_macros(view, format, "tTfaAugsEdD-xlLSz%[]");
 }
 
 /* Expands possibly limited set of view macros.  Returns newly allocated string,
  * which should be freed by the caller. */
 char *
-expand_view_macros(FileView *view, const char format[], const char macros[])
+expand_view_macros(view_t *view, const char format[], const char macros[])
 {
 	return parse_view_macros(view, &format, macros, 0);
 }
@@ -217,7 +217,7 @@ expand_view_macros(FileView *view, const char format[], const char macros[])
  * calls.  Returns newly allocated string, which should be freed by the
  * caller. */
 static char *
-parse_view_macros(FileView *view, const char **format, const char macros[],
+parse_view_macros(view_t *view, const char **format, const char macros[],
 		int opt)
 {
 	const dir_entry_t *const curr = get_current_entry(view);
@@ -270,7 +270,7 @@ parse_view_macros(FileView *view, const char **format, const char macros[],
 						buf);
 				break;
 			case 't':
-				format_entry_name(curr, sizeof(buf), buf);
+				format_entry_name(curr, NF_FULL, sizeof(buf), buf);
 				break;
 			case 'T':
 				if(curr->type == FT_LINK)
@@ -284,7 +284,7 @@ parse_view_macros(FileView *view, const char **format, const char macros[],
 				}
 				break;
 			case 'f':
-				get_short_path_of(view, curr, 1, 0, sizeof(buf), buf);
+				get_short_path_of(view, curr, NF_FULL, 0, sizeof(buf), buf);
 				break;
 			case 'A':
 #ifndef _WIN32
@@ -300,28 +300,24 @@ parse_view_macros(FileView *view, const char **format, const char macros[],
 				get_gid_string(curr, 0, sizeof(buf), buf);
 				break;
 			case 's':
-				friendly_size_notation(curr->size, sizeof(buf), buf);
+				friendly_size_notation(fentry_get_size(view, curr), sizeof(buf), buf);
 				break;
 			case 'E':
 				{
-					uint64_t size = 0;
-					if(view->selected_files > 0)
+					uint64_t size = 0U;
+
+					typedef int (*iter_f)(view_t *view, dir_entry_t **entry);
+					/* No current element for visual mode, since it can contain truly
+					 * empty selection when cursor is on ../ directory. */
+					iter_f iter = vle_mode_is(VISUAL_MODE) ? &iter_selected_entries
+					                                       : &iter_selection_or_current;
+
+					dir_entry_t *entry = NULL;
+					while(iter(view, &entry))
 					{
-						int i;
-						for(i = 0; i < view->list_rows; i++)
-						{
-							if(view->dir_entry[i].selected)
-							{
-								size += get_file_size_by_entry(view, i);
-							}
-						}
-					}
-					/* Make exception for VISUAL_MODE, since it can contain empty
-					 * selection when cursor is on ../ directory. */
-					else if(!vle_mode_is(VISUAL_MODE))
-					{
-						size = get_file_size_by_entry(view, view->list_pos);
+						size += fentry_get_size(view, entry);
 					}
+
 					friendly_size_notation(size, sizeof(buf), buf);
 				}
 				break;
@@ -332,6 +328,7 @@ parse_view_macros(FileView *view, const char **format, const char macros[],
 				}
 				break;
 			case '-':
+			case 'x':
 				skip = expand_num(buf, sizeof(buf), view->filtered);
 				break;
 			case 'l':
@@ -352,7 +349,7 @@ parse_view_macros(FileView *view, const char **format, const char macros[],
 			case 'D':
 				if(curr_stats.number_of_windows == 1)
 				{
-					FileView *const other = (view == curr_view) ? other_view : curr_view;
+					view_t *const other = (view == curr_view) ? other_view : curr_view;
 					copy_str(buf, sizeof(buf), replace_home_part(other->curr_dir));
 				}
 				break;
@@ -550,11 +547,11 @@ ui_stat_job_bar_add(bg_op_t *bg_op)
 		return;
 	}
 
-	update_job_bar();
+	ui_stat_job_bar_redraw();
 
 	if(ui_stat_job_bar_height() != prev_height)
 	{
-		schedule_redraw();
+		stats_redraw_schedule();
 	}
 }
 
@@ -578,11 +575,11 @@ ui_stat_job_bar_remove(bg_op_t *bg_op)
 
 	if(ui_stat_job_bar_height() != 0)
 	{
-		update_job_bar();
+		ui_stat_job_bar_redraw();
 	}
 	else if(prev_height != 0)
 	{
-		schedule_redraw();
+		stats_redraw_schedule();
 	}
 }
 
@@ -596,12 +593,6 @@ ui_stat_job_bar_changed(bg_op_t *bg_op)
 	pthread_spin_unlock(lock);
 }
 
-void
-ui_stat_job_bar_redraw(void)
-{
-	update_job_bar();
-}
-
 void
 ui_stat_job_bar_check_for_updates(void)
 {
@@ -617,7 +608,7 @@ ui_stat_job_bar_check_for_updates(void)
 
 	if(job_bar_changed_value || getmaxx(job_bar) != prev_width)
 	{
-		update_job_bar();
+		ui_stat_job_bar_redraw();
 	}
 
 	prev_width = getmaxx(job_bar);
@@ -652,9 +643,8 @@ is_job_bar_visible(void)
 	    && ui_stat_job_bar_height() != 0 && !is_in_menu_like_mode();
 }
 
-/* Fills job bar with up-to-date content. */
-static void
-update_job_bar(void)
+void
+ui_stat_job_bar_redraw(void)
 {
 	if(!is_job_bar_visible())
 	{
@@ -705,17 +695,20 @@ format_job_bar(void)
 		                   ? (max_width - width_used)
 		                   : (max_width/nbar_jobs);
 
-		const char *ellipsis = left_ellipsis(descrs[i], width - 2U - reserved);
+		char *const ellipsed = left_ellipsis(descrs[i], width - 2U - reserved,
+				curr_stats.ellipsis);
 
 		if(progress == -1)
 		{
-			snprintf(item_text, sizeof(item_text), "[%s]", ellipsis);
+			snprintf(item_text, sizeof(item_text), "[%s]", ellipsed);
 		}
 		else
 		{
-			snprintf(item_text, sizeof(item_text), "[%s %3d%%]", ellipsis, progress);
+			snprintf(item_text, sizeof(item_text), "[%s %3d%%]", ellipsed, progress);
 		}
 
+		free(ellipsed);
+
 		(void)sstrappend(bar_text, &text_width, sizeof(bar_text), item_text);
 
 		width_used += width;
@@ -762,21 +755,18 @@ ui_stat_draw_popup_line(WINDOW *win, const char item[], const char descr[],
 
 	if(text_width >= win_width)
 	{
-		char *const line = strdup(item);
-		right_ellipsis(line, win_width);
-		wprint(win, line);
-		free(line);
+		char *const whole = right_ellipsis(item, win_width, curr_stats.ellipsis);
+		wprint(win, whole);
+		free(whole);
 		return;
 	}
 
-	left = strdup(item);
-	right_ellipsis(left, win_width - 3);
+	left = right_ellipsis(item, win_width - 3, curr_stats.ellipsis);
 
 	item_width = align_columns ? max_width : utf8_strsw(left);
 	width_left = win_width - 2 - MAX(item_width, utf8_strsw(left));
 
-	right = strdup(descr);
-	right_ellipsis(right, width_left);
+	right = right_ellipsis(descr, width_left, curr_stats.ellipsis);
 
 	line = format_str(fmt, (int)item_width, left, (int)width_left, right);
 	free(left);
