@@ -82,13 +82,13 @@ static WINDOW *ltop_line2;
 static WINDOW *rtop_line1;
 static WINDOW *rtop_line2;
 
-/* Mutexes for views, located out of FileView so that they are never moved nor
+/* Mutexes for views, located out of view_t so that they are never moved nor
  * copied, which would yield undefined behaviour. */
 static pthread_mutex_t lwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t rwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-FileView lwin = { .timestamps_mutex = &lwin_timestamps_mutex };
-FileView rwin = { .timestamps_mutex = &rwin_timestamps_mutex };
+view_t lwin = { .timestamps_mutex = &lwin_timestamps_mutex };
+view_t rwin = { .timestamps_mutex = &rwin_timestamps_mutex };
 
 static void create_windows(void);
 static void update_geometry(void);
@@ -97,31 +97,32 @@ static void clear_border(WINDOW *border);
 static int middle_border_is_visible(void);
 static void update_views(int reload);
 static void reload_lists(void);
-static void reload_list(FileView *view);
-static void update_view(FileView *view);
+static void reload_list(view_t *view);
+static void update_view(view_t *view);
 static void update_window_lazy(WINDOW *win);
 static void update_term_size(void);
 static void update_statusbar_layout(void);
-static int get_ruler_width(FileView *view);
-static char * expand_ruler_macros(FileView *view, const char format[]);
+static int are_statusbar_widgets_visible(void);
+static int get_ruler_width(view_t *view);
+static char * expand_ruler_macros(view_t *view, const char format[]);
 static void switch_panes_content(void);
-static void update_origins(FileView *view, const char *old_main_origin);
+static void update_origins(view_t *view, const char *old_main_origin);
 static void set_splitter(int pos);
+static void refresh_bottom_lines(void);
 static char * path_identity(const char path[]);
-static char * format_view_title(const FileView *view, path_func pf);
-static void print_view_title(const FileView *view, int active_view,
-		char title[]);
-static void fixup_titles_attributes(const FileView *view, int active_view);
+static char * format_view_title(const view_t *view, path_func pf);
+static void print_view_title(const view_t *view, int active_view, char title[]);
+static void fixup_titles_attributes(const view_t *view, int active_view);
+static int is_in_miller_view(const view_t *view);
+static int is_forced_list_mode(const view_t *view);
 static uint64_t get_updated_time(uint64_t prev);
 
 void
-ui_ruler_update(FileView *view, int lazy_redraw)
+ui_ruler_update(view_t *view, int lazy_redraw)
 {
-	const int ruler_is_visible = !vle_mode_is(CMDLINE_MODE)
-	                          && !is_status_bar_multiline();
 	char *expanded;
 
-	if(!ruler_is_visible)
+	if(!are_statusbar_widgets_visible())
 	{
 		/* Do nothing, especially don't update layout because it might be a custom
 		 * layout at the moment. */
@@ -203,7 +204,7 @@ setup_ncurses_interface(void)
 }
 
 /* Initializes all WINDOW variables by calling newwin() to create ncurses
- * windows. */
+ * windows and configures hardware cursor. */
 static void
 create_windows(void)
 {
@@ -237,6 +238,29 @@ create_windows(void)
 	status_bar = newwin(1, 1, 0, 0);
 	ruler_win = newwin(1, 1, 0, 0);
 	input_win = newwin(1, 1, 0, 0);
+
+	leaveok(menu_win, FALSE);
+	leaveok(sort_win, FALSE);
+	leaveok(change_win, FALSE);
+	leaveok(error_win, FALSE);
+	leaveok(lwin.win, FALSE);
+	leaveok(rwin.win, FALSE);
+	leaveok(status_bar, FALSE);
+
+	leaveok(lborder, TRUE);
+	leaveok(lwin.title, TRUE);
+	leaveok(mborder, TRUE);
+	leaveok(ltop_line1, TRUE);
+	leaveok(ltop_line2, TRUE);
+	leaveok(top_line, TRUE);
+	leaveok(rtop_line1, TRUE);
+	leaveok(rtop_line2, TRUE);
+	leaveok(rwin.title, TRUE);
+	leaveok(rborder, TRUE);
+	leaveok(stat_win, TRUE);
+	leaveok(job_bar, TRUE);
+	leaveok(ruler_win, TRUE);
+	leaveok(input_win, TRUE);
 }
 
 void
@@ -276,21 +300,16 @@ ui_char_pressed(wint_t c)
 }
 
 static void
-correct_size(FileView *view)
+correct_size(view_t *view)
 {
-	int x, y;
-
-	getmaxyx(view->win, y, x);
-	view->window_width = x - 1;
-	view->window_rows = y - 1;
-	view->column_count = calculate_columns_count(view);
-	view->window_cells = view->column_count*y;
+	getmaxyx(view->win, view->window_rows, view->window_cols);
+	fview_update_geometry(view);
 }
 
 /* Updates TUI elements sizes and coordinates for single window
  * configuration. */
 static void
-only_layout(FileView *view, int screen_x)
+only_layout(view_t *view, int screen_x)
 {
 	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
 	const int vborder_size_correction = cfg.side_borders_visible ? -2 : 0;
@@ -551,6 +570,12 @@ cv_compare(CVType type)
 	return type == CV_COMPARE || type == CV_DIFF;
 }
 
+int
+cv_tree(CVType type)
+{
+	return type == CV_TREE || type == CV_CUSTOM_TREE;
+}
+
 void
 update_screen(UpdateType update_kind)
 {
@@ -595,9 +620,9 @@ update_screen(UpdateType update_kind)
 		redraw_msg_dialog(0);
 	}
 
-	update_stat_window(curr_view, 0);
+	ui_stat_update(curr_view, 0);
 
-	if(!is_status_bar_multiline())
+	if(!ui_sb_multiline())
 	{
 		if(curr_view->selected_files)
 		{
@@ -620,7 +645,7 @@ update_screen(UpdateType update_kind)
 
 	if(curr_stats.save_msg == 0)
 	{
-		status_bar_message("");
+		ui_sb_msg("");
 	}
 
 	if(vle_mode_is(VIEW_MODE) ||
@@ -693,7 +718,7 @@ reload_lists(void)
 	if(curr_stats.number_of_windows == 2)
 	{
 		ui_view_title_update(other_view);
-		if(curr_stats.view)
+		if(curr_stats.preview.on)
 		{
 			qv_draw(curr_view);
 		}
@@ -706,10 +731,10 @@ reload_lists(void)
 
 /* reloads view on window_reload() call */
 static void
-reload_list(FileView *view)
+reload_list(view_t *view)
 {
 	if(curr_stats.load_stage >= 3)
-		load_saving_pos(view, 1);
+		load_saving_pos(view);
 	else
 		load_dir_list(view,
 				!(cfg.vifm_info&VIFMINFO_SAVEDIRS) || view->list_pos != 0);
@@ -724,10 +749,10 @@ change_window(void)
 
 	if(window_shows_dirlist(other_view))
 	{
-		erase_current_line_bar(other_view);
+		put_inactive_mark(other_view);
 	}
 
-	if(curr_stats.view && !is_dir_list_loaded(curr_view))
+	if(curr_stats.preview.on && !is_dir_list_loaded(curr_view))
 	{
 		/* This view hasn't been loaded since startup yet, do it now. */
 		navigate_to(curr_view, curr_view->curr_dir);
@@ -753,7 +778,7 @@ change_window(void)
 void
 swap_view_roles(void)
 {
-	FileView *const tmp = curr_view;
+	view_t *const tmp = curr_view;
 	curr_view = other_view;
 	other_view = tmp;
 }
@@ -836,13 +861,13 @@ touch_all_windows(void)
 
 /* Updates all parts of file view. */
 static void
-update_view(FileView *view)
+update_view(view_t *view)
 {
 	update_window_lazy(view->title);
 
 	/* If view displays graphics, we don't want to update it or the image will be
 	 * lost. */
-	if(!view->explore_mode && !(curr_stats.view && view == other_view))
+	if(!view->explore_mode && !(curr_stats.preview.on && view == other_view))
 	{
 		update_window_lazy(view->win);
 	}
@@ -941,7 +966,7 @@ redraw_lists(void)
 	redraw_current_view();
 	if(curr_stats.number_of_windows == 2)
 	{
-		if(curr_stats.view)
+		if(curr_stats.preview.on)
 		{
 			qv_draw(curr_view);
 			refresh_view_win(other_view);
@@ -1078,6 +1103,18 @@ resize_for_menu_like(void)
 	return 0;
 }
 
+void
+ui_setup_for_menu_like(void)
+{
+	scrollok(menu_win, FALSE);
+	curs_set(0);
+	werase(menu_win);
+	werase(status_bar);
+	werase(ruler_win);
+	wrefresh(status_bar);
+	wrefresh(ruler_win);
+}
+
 /* Query terminal size from the "device" and pass it to curses library. */
 static void
 update_term_size(void)
@@ -1120,21 +1157,35 @@ update_statusbar_layout(void)
 	ruler_width = get_ruler_width(curr_view);
 	fields_pos = screen_x - (INPUT_WIN_WIDTH + ruler_width);
 
-	wresize(status_bar, 1, fields_pos);
-	mvwin(status_bar, screen_y - 1, 0);
-
 	wresize(ruler_win, 1, ruler_width);
 	mvwin(ruler_win, screen_y - 1, fields_pos + INPUT_WIN_WIDTH);
-	wnoutrefresh(ruler_win);
 
 	wresize(input_win, 1, INPUT_WIN_WIDTH);
 	mvwin(input_win, screen_y - 1, fields_pos);
-	wnoutrefresh(input_win);
+
+	/* We might be in command-line mode in which case we shouldn't change visible
+	 * parts of the layout. */
+	if(are_statusbar_widgets_visible())
+	{
+		wresize(status_bar, 1, fields_pos);
+		mvwin(status_bar, screen_y - 1, 0);
+
+		wnoutrefresh(ruler_win);
+		wnoutrefresh(input_win);
+	}
+}
+
+/* Checks whether ruler and input bar are visible.  Returns non-zero if so, zero
+ * is returned otherwise. */
+static int
+are_statusbar_widgets_visible(void)
+{
+	return !ui_sb_multiline() && !ui_sb_locked();
 }
 
 /* Gets "recommended" width for the ruler.  Returns the width. */
 static int
-get_ruler_width(FileView *view)
+get_ruler_width(view_t *view)
 {
 	char *expanded;
 	int len;
@@ -1157,13 +1208,13 @@ get_ruler_width(FileView *view)
  * string.  Returns newly allocated string, which should be freed by the caller,
  * or NULL if there is not enough memory. */
 static char *
-expand_ruler_macros(FileView *view, const char format[])
+expand_ruler_macros(view_t *view, const char format[])
 {
-	return expand_view_macros(view, format, "-lLS%[]");
+	return expand_view_macros(view, format, "-xlLS%[]");
 }
 
 void
-refresh_view_win(FileView *view)
+refresh_view_win(view_t *view)
 {
 	if(curr_stats.restart_in_progress)
 	{
@@ -1171,20 +1222,14 @@ refresh_view_win(FileView *view)
 	}
 
 	wrefresh(view->win);
-	/* Use getmaxy(...) instead of multiline_status_bar to handle command line
-	 * mode, which doesn't use this module to show multiline messages. */
-	if(cfg.display_statusline && getmaxy(status_bar) > 1)
-	{
-		touchwin(stat_win);
-		wrefresh(stat_win);
-	}
+	refresh_bottom_lines();
 }
 
 void
-move_window(FileView *view, int horizontally, int first)
+move_window(view_t *view, int horizontally, int first)
 {
 	const SPLIT split_type = horizontally ? HSPLIT : VSPLIT;
-	const FileView *const desired_view = first ? &lwin : &rwin;
+	const view_t *const desired_view = first ? &lwin : &rwin;
 	split_view(split_type);
 	if(view != desired_view)
 	{
@@ -1199,11 +1244,11 @@ void
 switch_panes(void)
 {
 	switch_panes_content();
-	try_activate_view_mode();
+	view_try_activate_mode();
 }
 
 void
-ui_view_pick(FileView *view, FileView **old_curr, FileView **old_other)
+ui_view_pick(view_t *view, view_t **old_curr, view_t **old_other)
 {
 	*old_curr = curr_view;
 	*old_other = other_view;
@@ -1217,7 +1262,7 @@ ui_view_pick(FileView *view, FileView **old_curr, FileView **old_other)
 }
 
 void
-ui_view_unpick(FileView *view, FileView *old_curr, FileView *old_other)
+ui_view_unpick(view_t *view, view_t *old_curr, view_t *old_other)
 {
 	if(curr_view != view)
 	{
@@ -1237,15 +1282,10 @@ ui_view_unpick(FileView *view, FileView *old_curr, FileView *old_other)
 static void
 switch_panes_content(void)
 {
-	FileView tmp_view;
+	view_t tmp_view;
 	WINDOW* tmp;
 	int t;
 
-	if(!vle_mode_is(VIEW_MODE))
-	{
-		view_switch_views();
-	}
-
 	tmp = lwin.win;
 	lwin.win = rwin.win;
 	rwin.win = tmp;
@@ -1254,9 +1294,9 @@ switch_panes_content(void)
 	lwin.window_rows = rwin.window_rows;
 	rwin.window_rows = t;
 
-	t = lwin.window_width;
-	lwin.window_width = rwin.window_width;
-	rwin.window_width = t;
+	t = lwin.window_cols;
+	lwin.window_cols = rwin.window_cols;
+	rwin.window_cols = t;
 
 	t = lwin.local_cs;
 	lwin.local_cs = rwin.local_cs;
@@ -1273,12 +1313,14 @@ switch_panes_content(void)
 	update_origins(&lwin, &rwin.curr_dir[0]);
 	update_origins(&rwin, &lwin.curr_dir[0]);
 
+	view_panes_swapped();
+
 	curr_stats.need_update = UT_REDRAW;
 }
 
 /* Updates pointers to main (default) origins in file list entries. */
 static void
-update_origins(FileView *view, const char *old_main_origin)
+update_origins(view_t *view, const char *old_main_origin)
 {
 	char *const new_origin = &view->curr_dir[0];
 	int i;
@@ -1296,7 +1338,7 @@ void
 go_to_other_pane(void)
 {
 	change_window();
-	try_activate_view_mode();
+	view_try_activate_mode();
 }
 
 void
@@ -1348,7 +1390,7 @@ move_splitter(int by, int fact)
 }
 
 void
-ui_view_resize(FileView *view, int to)
+ui_view_resize(view_t *view, int to)
 {
 	int pos;
 
@@ -1376,11 +1418,32 @@ set_splitter(int pos)
 }
 
 void
-format_entry_name(const dir_entry_t *entry, size_t buf_len, char buf[])
+format_entry_name(const dir_entry_t *entry, NameFormat fmt, size_t buf_len,
+		char buf[])
 {
 	const char *prefix, *suffix;
+	char tmp_buf[strlen(entry->name) + 1];
+	const char *name = entry->name;
+
+	if(fmt == NF_NONE)
+	{
+		copy_str(buf, buf_len, entry->name);
+		return;
+	}
+
+	if(fmt == NF_ROOT)
+	{
+		int root_len;
+		const char *ext_pos;
+
+		copy_str(tmp_buf, sizeof(tmp_buf), entry->name);
+		split_ext(tmp_buf, &root_len, &ext_pos);
+		name = tmp_buf;
+	}
+
 	ui_get_decors(entry, &prefix, &suffix);
-	snprintf(buf, buf_len, "%s%s%s", prefix, entry->name, suffix);
+	snprintf(buf, buf_len, "%s%s%s", prefix,
+			(is_root_dir(entry->name) && suffix[0] == '/') ? "" : name, suffix);
 }
 
 void
@@ -1460,20 +1523,37 @@ ui_display_too_small_term_msg(void)
 }
 
 void
-ui_view_win_changed(FileView *view)
+ui_view_win_changed(view_t *view)
 {
 	wnoutrefresh(view->win);
+	refresh_bottom_lines();
+}
+
+/* Makes sure that statusline and statusbar are drawn over view window after
+ * view refresh. */
+static void
+refresh_bottom_lines(void)
+{
+	/* Use getmaxy(...) instead of multiline_status_bar to handle command line
+	 * mode, which doesn't use this module to show multiline messages. */
+	if(cfg.display_statusline && getmaxy(status_bar) > 1)
+	{
+		touchwin(stat_win);
+		wnoutrefresh(stat_win);
+		touchwin(status_bar);
+		wnoutrefresh(status_bar);
+	}
 }
 
 void
-ui_view_reset_selection_and_reload(FileView *view)
+ui_view_reset_selection_and_reload(view_t *view)
 {
 	flist_sel_stash(view);
-	load_saving_pos(view, 1);
+	load_saving_pos(view);
 }
 
 void
-ui_view_reset_search_highlight(FileView *view)
+ui_view_reset_search_highlight(view_t *view)
 {
 	if(view->matches != 0)
 	{
@@ -1485,9 +1565,9 @@ ui_view_reset_search_highlight(FileView *view)
 void
 ui_views_reload_visible_filelists(void)
 {
-	if(curr_stats.view)
+	if(curr_stats.preview.on)
 	{
-		load_saving_pos(curr_view, 1);
+		load_saving_pos(curr_view);
 	}
 	else
 	{
@@ -1498,8 +1578,8 @@ ui_views_reload_visible_filelists(void)
 void
 ui_views_reload_filelists(void)
 {
-	load_saving_pos(curr_view, 1);
-	load_saving_pos(other_view, 1);
+	load_saving_pos(curr_view);
+	load_saving_pos(other_view);
 }
 
 void
@@ -1510,11 +1590,11 @@ ui_views_update_titles(void)
 }
 
 void
-ui_view_title_update(FileView *view)
+ui_view_title_update(view_t *view)
 {
 	char *title;
 	const int gen_view = vle_mode_is(VIEW_MODE) && !curr_view->explore_mode;
-	FileView *selected = gen_view ? other_view : curr_view;
+	view_t *selected = gen_view ? other_view : curr_view;
 
 	if(curr_stats.load_stage < 2)
 	{
@@ -1547,7 +1627,7 @@ path_identity(const char path[])
  * Returns newly allocated string, which should be freed by the caller, or NULL
  * if there is not enough memory. */
 static char *
-format_view_title(const FileView *view, path_func pf)
+format_view_title(const view_t *view, path_func pf)
 {
 	if(view->explore_mode)
 	{
@@ -1555,8 +1635,13 @@ format_view_title(const FileView *view, path_func pf)
 		get_current_full_path(view, sizeof(full_path), full_path);
 		return strdup(pf(full_path));
 	}
-	else if(curr_stats.view && view == other_view)
+	else if(curr_stats.preview.on && view == other_view)
 	{
+		const char *const viewer = view_detached_get_viewer();
+		if(viewer != NULL)
+		{
+			return format_str("Command: %s", viewer);
+		}
 		return format_str("File: %s", get_current_file_name(curr_view));
 	}
 	else if(flist_custom_active(view))
@@ -1573,8 +1658,10 @@ format_view_title(const FileView *view, path_func pf)
 /* Prints view title (which can be changed for printing).  Takes care of setting
  * correct attributes. */
 static void
-print_view_title(const FileView *view, int active_view, char title[])
+print_view_title(const view_t *view, int active_view, char title[])
 {
+	char *ellipsis;
+
 	const size_t title_width = getmaxx(view->title);
 	if(title_width == (size_t)-1)
 	{
@@ -1584,19 +1671,17 @@ print_view_title(const FileView *view, int active_view, char title[])
 	fixup_titles_attributes(view, active_view);
 	werase(view->title);
 
-	if(active_view)
-	{
-		wprint(view->title, left_ellipsis(title, title_width));
-	}
-	else
-	{
-		wprint(view->title, right_ellipsis(title, title_width));
-	}
+	ellipsis = active_view
+	         ? left_ellipsis(title, title_width, curr_stats.ellipsis)
+	         : right_ellipsis(title, title_width, curr_stats.ellipsis);
+
+	wprint(view->title, ellipsis);
+	free(ellipsis);
 }
 
 /* Updates attributes for view titles and top line. */
 static void
-fixup_titles_attributes(const FileView *view, int active_view)
+fixup_titles_attributes(const view_t *view, int active_view)
 {
 	if(active_view)
 	{
@@ -1641,7 +1726,7 @@ ui_view_sort_list_contains(const char sort[SK_COUNT], char key)
 }
 
 void
-ui_view_sort_list_ensure_well_formed(FileView *view, char sort_keys[])
+ui_view_sort_list_ensure_well_formed(view_t *view, char sort_keys[])
 {
 	int found_name_key = 0;
 	int i = -1;
@@ -1673,7 +1758,7 @@ ui_view_sort_list_ensure_well_formed(FileView *view, char sort_keys[])
 }
 
 char *
-ui_view_sort_list_get(const FileView *view, const char sort[])
+ui_view_sort_list_get(const view_t *view, const char sort[])
 {
 	return (flist_custom_active(view) && ui_view_unsorted(view))
 	     ? (char *)view->custom.sort
@@ -1681,31 +1766,23 @@ ui_view_sort_list_get(const FileView *view, const char sort[])
 }
 
 int
-ui_view_displays_numbers(const FileView *const view)
+ui_view_displays_numbers(const view_t *view)
 {
 	return view->num_type != NT_NONE && ui_view_displays_columns(view);
 }
 
 int
-ui_view_is_visible(const FileView *const view)
+ui_view_is_visible(const view_t *view)
 {
 	return curr_stats.number_of_windows == 2 || curr_view == view;
 }
 
-void
-ui_view_clear_history(FileView *const view)
-{
-	cfg_free_history_items(view->history, view->history_num);
-	view->history_num = 0;
-	view->history_pos = 0;
-}
-
 int
-ui_view_displays_columns(const FileView *const view)
+ui_view_displays_columns(const view_t *view)
 {
 	return !view->ls_view
-	    || (flist_custom_active(view) &&
-					(view->custom.type == CV_TREE || cv_compare(view->custom.type)));
+	    || is_in_miller_view(view)
+	    || is_forced_list_mode(view);
 }
 
 FileType
@@ -1725,44 +1802,84 @@ ui_view_entry_target_type(const dir_entry_t *entry)
 }
 
 int
-ui_view_available_width(const FileView *const view)
+ui_view_available_width(const view_t *view)
 {
 	const int correction = cfg.extra_padding ? -2 : 0;
-	return ((int)view->window_width + 1) + correction;
+	return view->window_cols + correction
+	     - ui_view_left_reserved(view) - ui_view_right_reserved(view);
 }
 
 int
-ui_qv_left(const FileView *view)
+ui_view_left_reserved(const view_t *view)
+{
+	const int total = view->miller_ratios[0] + view->miller_ratios[1]
+	                + view->miller_ratios[2];
+	return is_in_miller_view(view)
+	     ? (view->window_cols*view->miller_ratios[0])/total : 0;
+}
+
+int
+ui_view_right_reserved(const view_t *view)
+{
+	dir_entry_t *const entry = get_current_entry(view);
+	const int total = view->miller_ratios[0] + view->miller_ratios[1]
+	                + view->miller_ratios[2];
+	return is_in_miller_view(view)
+	    && fentry_is_dir(entry) && !is_parent_dir(entry->name)
+	     ? (view->window_cols*view->miller_ratios[2])/total
+	     : 0;
+}
+
+/* Whether miller columns should be displayed.  Returns non-zero if so,
+ * otherwise zero is returned. */
+static int
+is_in_miller_view(const view_t *view)
+{
+	return view->miller_view
+	    && !flist_custom_active(view);
+}
+
+/* Whether view must display straight single-column file list.  Returns non-zero
+ * if so, otherwise zero is returned. */
+static int
+is_forced_list_mode(const view_t *view)
+{
+	return flist_custom_active(view)
+	    && (cv_tree(view->custom.type) || cv_compare(view->custom.type));
+}
+
+int
+ui_qv_left(const view_t *view)
 {
 	return cfg.extra_padding ? 1 : 0;
 }
 
 int
-ui_qv_top(const FileView *view)
+ui_qv_top(const view_t *view)
 {
 	return cfg.extra_padding ? 1 : 0;
 }
 
 int
-ui_qv_height(const FileView *view)
+ui_qv_height(const view_t *view)
 {
-	return cfg.extra_padding ? view->window_rows - 1 : view->window_rows + 1;
+	return cfg.extra_padding ? view->window_rows - 2 : view->window_rows;
 }
 
 int
-ui_qv_width(const FileView *view)
+ui_qv_width(const view_t *view)
 {
-	return cfg.extra_padding ? view->window_width - 1 : view->window_width + 1;
+	return cfg.extra_padding ? view->window_cols - 2 : view->window_cols;
 }
 
 const col_scheme_t *
-ui_view_get_cs(const FileView *view)
+ui_view_get_cs(const view_t *view)
 {
 	return view->local_cs ? &view->cs : &cfg.cs;
 }
 
 void
-ui_view_erase(FileView *view)
+ui_view_erase(view_t *view)
 {
 	const col_scheme_t *cs = ui_view_get_cs(view);
 	const int bg = COLOR_PAIR(cs->pair[WIN_COLOR]) | cs->color[WIN_COLOR].attr;
@@ -1771,7 +1888,7 @@ ui_view_erase(FileView *view)
 }
 
 void
-ui_view_wipe(FileView *view)
+ui_view_wipe(view_t *view)
 {
 	int i;
 	int height;
@@ -1796,13 +1913,23 @@ ui_view_wipe(FileView *view)
 }
 
 int
-ui_view_unsorted(const FileView *view)
+ui_view_unsorted(const view_t *view)
 {
 	return cv_unsorted(view->custom.type);
 }
 
 void
-ui_view_schedule_redraw(FileView *view)
+ui_shutdown(void)
+{
+	if(curr_stats.load_stage > 0 && !isendwin())
+	{
+		def_prog_mode();
+		endwin();
+	}
+}
+
+void
+ui_view_schedule_redraw(view_t *view)
 {
 	pthread_mutex_lock(view->timestamps_mutex);
 	view->postponed_redraw = get_updated_time(view->postponed_redraw);
@@ -1810,21 +1937,13 @@ ui_view_schedule_redraw(FileView *view)
 }
 
 void
-ui_view_schedule_reload(FileView *view)
+ui_view_schedule_reload(view_t *view)
 {
 	pthread_mutex_lock(view->timestamps_mutex);
 	view->postponed_reload = get_updated_time(view->postponed_reload);
 	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
-void
-ui_view_schedule_full_reload(FileView *view)
-{
-	pthread_mutex_lock(view->timestamps_mutex);
-	view->postponed_full_reload = get_updated_time(view->postponed_full_reload);
-	pthread_mutex_unlock(view->timestamps_mutex);
-}
-
 /* Gets updated timestamp ensuring that it differs from the previous value.
  * Returns the timestamp. */
 static uint64_t
@@ -1844,18 +1963,22 @@ get_updated_time(uint64_t prev)
 	return new;
 }
 
+void
+ui_view_redrawn(view_t *view)
+{
+	pthread_mutex_lock(view->timestamps_mutex);
+	view->last_redraw = view->postponed_redraw;
+	pthread_mutex_unlock(view->timestamps_mutex);
+}
+
 UiUpdateEvent
-ui_view_query_scheduled_event(FileView *view)
+ui_view_query_scheduled_event(view_t *view)
 {
 	UiUpdateEvent event;
 
 	pthread_mutex_lock(view->timestamps_mutex);
 
-	if(view->postponed_full_reload != view->last_reload)
-	{
-		event = UUE_FULL_RELOAD;
-	}
-	else if(view->postponed_reload != view->last_reload)
+	if(view->postponed_reload != view->last_reload)
 	{
 		event = UUE_RELOAD;
 	}
@@ -1870,7 +1993,6 @@ ui_view_query_scheduled_event(FileView *view)
 
 	view->last_redraw = view->postponed_redraw;
 	view->last_reload = view->postponed_reload;
-	view->postponed_full_reload = view->postponed_reload;
 
 	pthread_mutex_unlock(view->timestamps_mutex);
 
