@@ -33,12 +33,14 @@
 #include <sys/types.h> /* size_t mode_t */
 #include <unistd.h> /* pathconf() readlink() */
 
+#include <ctype.h> /* isalpha() */
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL */
 #include <stdio.h> /* snprintf() remove() */
 #include <stdlib.h> /* free() qsort() */
 #include <string.h> /* strcpy() strdup() strlen() strncmp() strncpy() */
 
+#include "../compat/dtype.h"
 #include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../io/iop.h"
@@ -157,6 +159,7 @@ path_exists_at(const char path[], const char filename[], int deref)
 	{
 		LOG_ERROR_MSG("Passed absolute path where relative one is expected: %s",
 				filename);
+		path = NULL;
 	}
 	return path_exists_internal(path, filename, deref);
 }
@@ -322,8 +325,6 @@ get_link_target_abs(const char link[], const char cwd[], char buf[],
 int
 get_link_target(const char *link, char *buf, size_t buf_len)
 {
-	LOG_FUNC_ENTER;
-
 #ifndef _WIN32
 	char *filename;
 	ssize_t len;
@@ -645,10 +646,10 @@ remove_dir_content(const char path[])
 }
 
 int
-entry_is_link(const char path[], const struct dirent* dentry)
+entry_is_link(const char path[], const struct dirent *dentry)
 {
 #ifndef _WIN32
-	if(dentry->d_type == DT_LNK)
+	if(get_dirent_type(dentry, path) == DT_LNK)
 	{
 		return 1;
 	}
@@ -657,12 +658,13 @@ entry_is_link(const char path[], const struct dirent* dentry)
 }
 
 int
-entry_is_dir(const char full_path[], const struct dirent* dentry)
+entry_is_dir(const char full_path[], const struct dirent *dentry)
 {
 #ifndef _WIN32
-	return (dentry->d_type == DT_UNKNOWN)
+	const unsigned char type = get_dirent_type(dentry, full_path);
+	return (type == DT_UNKNOWN)
 	     ? is_directory(full_path, 0)
-	     : dentry->d_type == DT_DIR;
+	     : type == DT_DIR;
 #else
 	const DWORD MASK = FILE_ATTRIBUTE_REPARSE_POINT | FILE_ATTRIBUTE_DIRECTORY;
 	const DWORD attrs = win_get_file_attrs(full_path);
@@ -672,18 +674,20 @@ entry_is_dir(const char full_path[], const struct dirent* dentry)
 }
 
 int
-is_dirent_targets_dir(const struct dirent *d)
+is_dirent_targets_dir(const char full_path[], const struct dirent *d)
 {
 #ifdef _WIN32
-	return is_dir(d->d_name);
+	return is_dir(full_path);
 #else
-	if(d->d_type == DT_UNKNOWN)
+	const unsigned char type = get_dirent_type(d, full_path);
+
+	if(type == DT_UNKNOWN)
 	{
-		return is_dir(d->d_name);
+		return is_dir(full_path);
 	}
 
-	return  d->d_type == DT_DIR
-	    || (d->d_type == DT_LNK && get_symlink_type(d->d_name) != SLT_UNKNOWN);
+	return  type == DT_DIR
+	    || (type == DT_LNK && get_symlink_type(full_path) != SLT_UNKNOWN);
 #endif
 }
 
@@ -838,7 +842,7 @@ get_cwd(char buf[], size_t size)
 char *
 save_cwd(void)
 {
-	char cwd[PATH_MAX];
+	char cwd[PATH_MAX + 1];
 	if(os_getcwd(cwd, sizeof(cwd)) != cwd)
 	{
 		return NULL;
@@ -886,7 +890,7 @@ win_get_file_attrs(const char path[])
 
 	if(is_path_absolute(path) && !is_unc_path(path))
 	{
-		if(!drive_exists(path[0]))
+		if(isalpha(path[0]) && !drive_exists(path[0]))
 		{
 			return INVALID_FILE_ATTRIBUTES;
 		}
@@ -919,7 +923,7 @@ is_on_fat_volume(const char *path)
 	if(is_unc_path(path))
 	{
 		int i = 4, j = 0;
-		snprintf(buf, sizeof(buf), "%s", path);
+		copy_str(buf, sizeof(buf), path);
 		while(i > 0 && buf[j] != '\0')
 			if(buf[j++] == '/')
 				i--;
