@@ -148,7 +148,7 @@ struct dirent;
 
 /* Uses dentry or full path to check whether target is symbolic link.  Returns
  * non-zero if so, otherwise zero is returned. */
-int entry_is_link(const char path[], const struct dirent* dentry);
+int entry_is_link(const char path[], const struct dirent *dentry);
 
 /* Uses dentry or full path to check file type.  Returns non-zero for
  * directories, otherwise zero is returned.  Symbolic links are _not_
@@ -158,7 +158,7 @@ int entry_is_dir(const char full_path[], const struct dirent* dentry);
 /* Uses dentry or path to check file type.  Assumes that file is located in
  * current working directory.  Returns non-zero for directories, otherwise zero
  * is returned.  Symbolic links are dereferenced. */
-int is_dirent_targets_dir(const struct dirent *d);
+int is_dirent_targets_dir(const char full_path[], const struct dirent *d);
 
 /* Checks that entity pointed to by the path is located under the root
  * directory.  Returns non-zero if so, otherwise zero is returned. */
@@ -191,7 +191,8 @@ int count_dir_items(const char path[]);
 char * get_cwd(char buf[], size_t size);
 
 /* Remembers current working directory.  If path can't be obtained, does
- * nothing.  Result should be passed to restore_cwd(), no checks are needed. */
+ * nothing.  Result should be passed to restore_cwd(), no checks are needed (but
+ * NULL indicates that we failed to get CWD). */
 char * save_cwd(void);
 
 /* Restores previously remembered working directory via save_cwd().  If nothing
