@@ -95,7 +95,7 @@ void ensure_path_well_formed(char *path);
 
 /* Ensures that path to a file is of canonic absolute form.  No trailing slash
  * in the buffer except for the root path.  Canonic paths must be absolute, so
- * if input path isn't base is prepended to it. */
+ * if input path is relative base is prepended to it. */
 void to_canonic_path(const char path[], const char base[], char buf[],
 		size_t buf_len);
 
@@ -142,6 +142,10 @@ void generate_tmp_file_name(const char prefix[], char buf[], size_t buf_len);
  * tmp directory. */
 const char * get_tmpdir(void);
 
+/* Concatenates two paths while making sure to not add extra slash between
+ * them. */
+void build_path(char buf[], size_t buf_len, const char p1[], const char p2[]);
+
 #ifdef _WIN32
 
 int is_unc_path(const char *path);
