@@ -146,6 +146,10 @@ int replace_string(char **str, const char with[]);
  * *str is freed and set to NULL. */
 int update_string(char **str, const char to[]);
 
+/* Replaces *str with the with string, if it's non-NULL.  Returns non-zero if
+ * with is NULL, otherwise zero is returned. */
+int put_string(char **str, char with[]);
+
 /* Adds a character to the end of the string.  Returns the str argument. */
 char * strcatch(char str[], char c);
 
@@ -174,17 +178,15 @@ int sstrappend(char str[], size_t *len, size_t size, const char suffix[]);
 /* Pads buffer pointed to by str to be at least of width "width + 1". */
 void stralign(char str[], size_t width, char pad, int left_align);
 
-/* Ensures that str is of width (in UTF-8 characters) less than or equal to
- * max_width and is left aligned putting ellipsis on the right side if needed.
- * Changes the string in place.  Returns the str unless max_width is zero in
- * which case NULL is returned. */
-char * left_ellipsis(char str[], size_t max_width);
-
-/* Ensures that str is of width (in UTF-8 characters) less than or equal to
+/* Ensures that str is of width (in character positions) less than or equal to
  * max_width and is right aligned putting ellipsis on the left side if needed.
- * Changes the string in place.  Returns the str unless max_width is zero in
- * which case NULL is returned. */
-char * right_ellipsis(char str[], size_t max_width);
+ * Returns newly allocated modified string. */
+char * left_ellipsis(const char str[], size_t max_width, const char ell[]);
+
+/* Ensures that str is of width (in character positions) less than or equal to
+ * max_width and is left aligned putting ellipsis on the right side if needed.
+ * Returns newly allocated modified string. */
+char * right_ellipsis(const char str[], size_t max_width, const char ell[]);
 
 /* "Breaks" single line it two parts (before and after "%=" separator), and
  * re-formats it filling specified width by putting "left part", padded centre
