@@ -54,6 +54,7 @@
 #include "../compat/mntent.h" /* mntent setmntent() getmntent() endmntent() */
 #include "../compat/os.h"
 #include "../compat/reallocarray.h"
+#include "../ui/ui.h"
 #include "../running.h"
 #include "../status.h"
 #include "cancellation.h"
@@ -259,10 +260,10 @@ run_from_fork(int pipe[2], int err_only, char cmd[])
 		_Exit(EXIT_FAILURE);
 	}
 
-	if(dup2(nullfd, STDIN_FILENO) == -1)
-	{
-		_Exit(EXIT_FAILURE);
-	}
+	/* if(dup2(nullfd, STDIN_FILENO) == -1) */
+	/* { */
+	/* 	_Exit(EXIT_FAILURE); */
+	/* } */
 	if(err_only && dup2(nullfd, STDOUT_FILENO) == -1)
 	{
 		_Exit(EXIT_FAILURE);
@@ -566,7 +567,7 @@ read_mnt_entries(unsigned int *nentries)
 		}
 	}
 
-	endmntent(f);
+	(void)endmntent(f);
 
 	return entries;
 }
@@ -1043,7 +1044,7 @@ get_free_space(const char at[])
 		return 0;
 	}
 
-	return (uint64_t)st.f_bsize*st.f_bfree;
+	return (uint64_t)st.f_bsize*st.f_bavail;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
