@@ -355,7 +355,7 @@ is_win_executable(const char name[])
 	char name_buf[NAME_MAX + 1];
 	char ext_buf[16];
 
-	snprintf(name_buf, sizeof(name_buf), "%s", name);
+	copy_str(name_buf, sizeof(name_buf), name);
 	strtoupper(name_buf);
 
 	p = env_get_def("PATHEXT", PATHEXT_EXT_DEF);
@@ -453,8 +453,7 @@ traverse_mount_points(mptraverser client, void *arg)
 		if(drive_exists(c))
 		{
 			const char drive[] = { c, ':', '/', '\0' };
-			struct mntent entry =
-			{
+			struct mntent entry = {
 				.mnt_dir = drive,
 				.mnt_type = "",
 			};
@@ -547,8 +546,7 @@ format_help_cmd(char cmd[], size_t cmd_size)
 void
 display_help(const char cmd[])
 {
-	def_prog_mode();
-	endwin();
+	ui_shutdown();
 	system("cls");
 	if(os_system(cmd) != EXIT_SUCCESS)
 	{
