@@ -40,6 +40,7 @@
 #include "cfg/info.h"
 #include "compat/reallocarray.h"
 #include "engine/autocmds.h"
+#include "engine/parsing.h"
 #include "compat/fs_limits.h"
 #include "engine/cmds.h"
 #include "engine/keys.h"
@@ -53,6 +54,7 @@
 #include "modes/dialogs/msg_dialog.h"
 #include "modes/cmdline.h"
 #include "modes/modes.h"
+#include "modes/normal.h"
 #include "modes/view.h"
 #include "ui/cancellation.h"
 #include "ui/color_manager.h"
@@ -99,8 +101,9 @@ static void move_pair(short int from, short int to);
 static int undo_perform_func(OPS op, void *data, const char src[],
 		const char dst[]);
 static void parse_received_arguments(char *args[]);
-static void remote_cd(FileView *view, const char path[], int handle);
-static void check_path_for_file(FileView *view, const char path[], int handle);
+static char * eval_received_expression(const char expr[]);
+static void remote_cd(view_t *view, const char path[], int handle);
+static void check_path_for_file(view_t *view, const char path[], int handle);
 static int need_to_switch_active_pane(const char lwin_path[],
 		const char rwin_path[]);
 static void load_scheme(void);
@@ -115,7 +118,7 @@ static args_t vifm_args;
 int
 main(int argc, char *argv[])
 {
-	return vifm_main(argc, argv);
+	vifm_exit(vifm_main(argc, argv));
 }
 
 #else
@@ -141,7 +144,7 @@ main()
 	}
 	utf8_argv[i] = NULL;
 
-	return vifm_main(argc, utf8_argv);
+	vifm_exit(vifm_main(argc, utf8_argv));
 }
 
 #endif
@@ -202,7 +205,7 @@ vifm_main(int argc, char *argv[])
 	init_builtin_functions();
 	update_path_env(1);
 
-	if(init_status(&cfg) != 0)
+	if(stats_init(&cfg) != 0)
 	{
 		free_string_array(files, nfiles);
 
@@ -225,10 +228,16 @@ vifm_main(int argc, char *argv[])
 		read_info_file(0);
 	}
 
-	ipc_init(vifm_args.server_name, &parse_received_arguments);
+	curr_stats.ipc = ipc_init(vifm_args.server_name, &parse_received_arguments,
+			&eval_received_expression);
+	if(ipc_enabled() && curr_stats.ipc == NULL)
+	{
+		fputs("Failed to initialize IPC unit", stderr);
+		return -1;
+	}
 	/* Export chosen server name to parsing unit. */
 	{
-		var_val_t value = { .string = (char *)ipc_get_name() };
+		var_val_t value = { .string = (char *)ipc_get_name(curr_stats.ipc) };
 		var_t var = var_new(VTYPE_STRING, value);
 		setvar("v:servername", var);
 		var_free(var);
@@ -290,19 +299,18 @@ vifm_main(int argc, char *argv[])
 
 	curr_stats.load_stage = 1;
 
+	/* Make v:count exist during processing configuration. */
+	set_count_vars(0);
+
 	if(!vifm_args.no_configs)
 	{
 		load_scheme();
 		cfg_load();
 	}
 
-	if(lwin_cv)
+	if(lwin_cv || rwin_cv)
 	{
-		flist_custom_set(&lwin, "-", dir, files, nfiles);
-	}
-	else if(rwin_cv)
-	{
-		flist_custom_set(&rwin, "-", dir, files, nfiles);
+		flist_custom_set(lwin_cv ? &lwin : &rwin, "-", dir, files, nfiles);
 	}
 	free_string_array(files, nfiles);
 
@@ -328,10 +336,16 @@ vifm_main(int argc, char *argv[])
 	flist_hist_save(&rwin, NULL, NULL, -1);
 
 	/* Trigger auto-commands for initial directories. */
-	(void)vifm_chdir(flist_get_dir(&lwin));
-	vle_aucmd_execute("DirEnter", flist_get_dir(&lwin), &lwin);
-	(void)vifm_chdir(flist_get_dir(&rwin));
-	vle_aucmd_execute("DirEnter", flist_get_dir(&rwin), &rwin);
+	if(!lwin_cv)
+	{
+		(void)vifm_chdir(flist_get_dir(&lwin));
+		vle_aucmd_execute("DirEnter", flist_get_dir(&lwin), &lwin);
+	}
+	if(!rwin_cv)
+	{
+		(void)vifm_chdir(flist_get_dir(&rwin));
+		vle_aucmd_execute("DirEnter", flist_get_dir(&rwin), &rwin);
+	}
 
 	update_screen(UT_FULL);
 	modes_update();
@@ -399,17 +413,11 @@ undo_perform_func(OPS op, void *data, const char src[], const char dst[])
 static void
 parse_received_arguments(char *argv[])
 {
-	int argc = 0;
 	args_t args = {};
 
-	while(argv[argc] != NULL)
-	{
-		argc++;
-	}
-
 	(void)vifm_chdir(argv[0]);
 	opterr = 0;
-	args_parse(&args, argc, argv, argv[0]);
+	args_parse(&args, count_strings(argv), argv, argv[0]);
 	args_process(&args, 0);
 
 	exec_startup_commands(&args);
@@ -441,26 +449,27 @@ parse_received_arguments(char *argv[])
 		change_window();
 	}
 
+	/* XXX: why force clearing of the statusbar? */
 	ui_sb_clear();
 	curr_stats.save_msg = 0;
 }
 
 static void
-remote_cd(FileView *view, const char path[], int handle)
+remote_cd(view_t *view, const char path[], int handle)
 {
 	char buf[PATH_MAX];
 
 	if(view->explore_mode)
 	{
-		leave_view_mode();
+		view_leave_mode();
 	}
 
 	if(view == other_view && vle_mode_is(VIEW_MODE))
 	{
-		leave_view_mode();
+		view_leave_mode();
 	}
 
-	if(curr_stats.view && (handle || view == other_view))
+	if(curr_stats.preview.on && (handle || view == other_view))
 	{
 		qv_toggle();
 	}
@@ -475,7 +484,7 @@ remote_cd(FileView *view, const char path[], int handle)
 /* Navigates to/opens (handles) file specified by the path (and file only, no
  * directories). */
 static void
-check_path_for_file(FileView *view, const char path[], int handle)
+check_path_for_file(view_t *view, const char path[], int handle)
 {
 	if(path[0] == '\0' || is_dir(path) || (handle && strcmp(path, "-") == 0))
 	{
@@ -504,6 +513,24 @@ need_to_switch_active_pane(const char lwin_path[], const char rwin_path[])
 	    && curr_view != &lwin;
 }
 
+/* Evaluates expression from a remote instance.  Returns newly allocated string
+ * with the result or NULL on error. */
+static char *
+eval_received_expression(const char expr[])
+{
+	char *result_str;
+
+	var_t result;
+	if(parse(expr, &result) != PE_NO_ERROR)
+	{
+		return NULL;
+	}
+
+	result_str = var_to_string(result);
+	var_free(result);
+	return result_str;
+}
+
 /* Loads color scheme.  Converts old format to the new one if needed. */
 static void
 load_scheme(void)
@@ -522,7 +549,7 @@ load_scheme(void)
 void
 vifm_restart(void)
 {
-	FileView *tmp_view;
+	view_t *tmp_view;
 
 	curr_stats.restart_in_progress = 1;
 
@@ -535,20 +562,12 @@ vifm_restart(void)
 	/* Autocommands. */
 	vle_aucmd_remove(NULL, NULL);
 
-	/* Directory histories. */
-	ui_view_clear_history(&lwin);
-	ui_view_clear_history(&rwin);
-
-	/* All kinds of history. */
-	(void)hist_reset(&cfg.search_hist, cfg.history_len);
-	(void)hist_reset(&cfg.cmd_hist, cfg.history_len);
-	(void)hist_reset(&cfg.prompt_hist, cfg.history_len);
-	(void)hist_reset(&cfg.filter_hist, cfg.history_len);
-	cfg.history_len = 0;
+	/* All kinds of histories. */
+	cfg_resize_histories(0);
 
 	/* Session status.  Must be reset _before_ options, because options take some
 	 * of values from status. */
-	(void)reset_status(&cfg);
+	(void)stats_reset(&cfg);
 
 	/* Options of current pane. */
 	reset_options_to_default();
@@ -656,19 +675,19 @@ vifm_try_leave(int write_info, int cquit, int force)
 	erase();
 	refresh();
 #endif
-	endwin();
+	ui_shutdown();
 
 	vifm_leave(EXIT_SUCCESS, cquit);
 }
 
 void _gnuc_noreturn
-vifm_choose_files(const FileView *view, int nfiles, char *files[])
+vifm_choose_files(const view_t *view, int nfiles, char *files[])
 {
 	int exit_code;
 
 	/* As curses can do something with terminal on shutting down, disable it
 	 * before writing anything to the screen. */
-	endwin();
+	ui_shutdown();
 
 	exit_code = EXIT_SUCCESS;
 	if(vim_write_file_list(view, nfiles, files) != 0)
@@ -699,13 +718,13 @@ vifm_leave(int exit_code, int cquit)
 	}
 
 	term_title_update(NULL);
-	exit(exit_code);
+	vifm_exit(exit_code);
 }
 
 void _gnuc_noreturn
 vifm_finish(const char message[])
 {
-	endwin();
+	ui_shutdown();
 
 	/* Update vifminfo only if we were able to startup, otherwise we can end up
 	 * writing from some intermediate half-initialized state.  One particular
@@ -719,7 +738,14 @@ vifm_finish(const char message[])
 
 	fprintf(stderr, "%s\n", message);
 	LOG_ERROR_MSG("Finishing: %s", message);
-	exit(EXIT_FAILURE);
+	vifm_exit(EXIT_FAILURE);
+}
+
+void _gnuc_noreturn
+vifm_exit(int exit_code)
+{
+	ipc_free(curr_stats.ipc);
+	exit(exit_code);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
