@@ -6,7 +6,7 @@
 
 #include "../../src/engine/cmds.h"
 #include "../../src/engine/completion.h"
-#include "../../src/commands.h"
+#include "../../src/cmd_core.h"
 
 extern struct cmds_conf cmds_conf;
 
@@ -40,9 +40,9 @@ TEST(set)
 	assert_int_equal(0, complete_cmd("se", NULL));
 
 	buf = vle_compl_next();
-	assert_string_equal("set", buf);
+	assert_string_equal("select", buf);
 	free(buf);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
