@@ -19,14 +19,18 @@ extern cmds_conf_t cmds_conf;
 static cmd_info_t cmdi;
 
 static const cmd_add_t commands[] = {
-	{ .name = "",       .abbr = NULL, .handler = dummy_cmd,  .id = -1,    .range = 1,    .cust_sep = 0,
-		.emark = 0,       .qmark = 0,   .expand = 0,           .regexp = 0, .min_args = 0, .max_args = 0, },
-	{ .name = "delete", .abbr = "d",  .handler = delete_cmd, .id = 1,     .range = 1,    .cust_sep = 0,
-		.emark = 1,       .qmark = 0,   .expand = 0,           .regexp = 0, .min_args = 0, .max_args = 1, },
-	{ .name = "yank",   .abbr = "y",  .handler = dummy_cmd,  .id = -10,   .range = 0,    .cust_sep = 0,
-		.emark = 0,       .qmark = 0,   .expand = 0,           .regexp = 0, .min_args = 0, .max_args = 1, },
-	{ .name = "windo",            .abbr = NULL,    .emark = 0,  .id = COM_WINDO,       .range = 0,    .bg = 0, .quote = 0, .regexp = 0,
-		.handler = dummy_cmd,       .qmark = 0,      .expand = 0, .cust_sep = 0,         .min_args = 0, .max_args = NOT_DEF, .select = 0, },
+	{ .name = "",             .abbr = NULL,  .id = -1,           .descr = "descr",
+	  .flags = HAS_RANGE,
+	  .handler = &dummy_cmd,  .min_args = 0, .max_args = 0, },
+	{ .name = "delete",       .abbr = "d",   .id = 1,            .descr = "descr",
+	  .flags = HAS_RANGE | HAS_EMARK,
+	  .handler = &delete_cmd, .min_args = 0, .max_args = 1, },
+	{ .name = "yank",         .abbr = "y",   .id = -10,          .descr = "descr",
+	  .flags = 0,
+	  .handler = &dummy_cmd,  .min_args = 0, .max_args = 1, },
+	{ .name = "windo",        .abbr = NULL,  .id = COM_WINDO,    .descr = "descr",
+	  .flags = 0,
+	  .handler = &dummy_cmd,  .min_args = 0, .max_args = NOT_DEF, },
 };
 
 static int
@@ -276,4 +280,4 @@ TEST(udf_completion)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
