@@ -12,19 +12,19 @@
 
 #include "utils.h"
 
-static void setup_single_entry(FileView *view, const char name[]);
+static void setup_single_entry(view_t *view, const char name[]);
 static uint64_t wait_for_size(const char path[]);
 
 SETUP()
 {
-	init_status(&cfg);
+	stats_init(&cfg);
 }
 
 TEARDOWN()
 {
 	view_teardown(&lwin);
 
-	reset_status(&cfg);
+	stats_reset(&cfg);
 }
 
 TEST(directory_size_is_calculated_in_bg)
@@ -48,7 +48,7 @@ TEST(parent_dir_entry_triggers_calculation_of_current_dir)
 }
 
 static void
-setup_single_entry(FileView *view, const char name[])
+setup_single_entry(view_t *view, const char name[])
 {
 	view->user_selection = 1;
 	view->list_rows = 1;
