@@ -1,20 +1,22 @@
 #include <stic.h>
 
-#include <unistd.h> /* F_OK access() getcwd() */
+#include <sys/types.h>
+#include <unistd.h> /* F_OK access() geteuid() */
 
 #include <stdio.h> /* snprintf() */
 
+#include "../../src/compat/fs_limits.h"
 #include "../../src/io/iop.h"
 #include "../../src/io/ior.h"
 #include "../../src/utils/fs.h"
-#include "../../src/utils/fs_limits.h"
+#include "../../src/utils/path.h"
 
-#define DIR_NAME "dir-to-create"
-#define NESTED_DIR_NAME "dir-to-create/dir-to-create"
+#define DIR_NAME SANDBOX_PATH "/dir-to-create"
+#define NESTED_DIR_NAME SANDBOX_PATH "/dir-to-create/dir-to-create"
 
 static void create_directory(const char path[], const char root[],
 		int create_parents);
-static int has_unix_permissions(void);
+static int non_root_on_unix_like_os(void);
 
 TEST(single_dir_is_created)
 {
@@ -28,12 +30,15 @@ TEST(child_dir_and_parent_are_created)
 
 TEST(create_by_absolute_path)
 {
-	char cwd[PATH_MAX];
-	char full_path[PATH_MAX];
+	if(!is_path_absolute(NESTED_DIR_NAME))
+	{
+		char cwd[PATH_MAX];
+		char full_path[PATH_MAX];
 
-	getcwd(cwd, sizeof(cwd));
-	snprintf(full_path, sizeof(full_path), "%s/%s", cwd, NESTED_DIR_NAME);
-	create_directory(full_path, DIR_NAME, 1);
+		get_cwd(cwd, sizeof(cwd));
+		snprintf(full_path, sizeof(full_path), "%s/%s", cwd, NESTED_DIR_NAME);
+		create_directory(full_path, DIR_NAME, 1);
+	}
 }
 
 static void
@@ -87,7 +92,7 @@ TEST(child_dir_is_not_created)
 	assert_failure(access(NESTED_DIR_NAME, F_OK));
 }
 
-TEST(permissions_are_taken_into_account, IF(has_unix_permissions))
+TEST(permissions_are_taken_into_account, IF(non_root_on_unix_like_os))
 {
 	{
 		io_args_t args = {
@@ -126,7 +131,7 @@ TEST(permissions_are_taken_into_account, IF(has_unix_permissions))
 }
 
 TEST(permissions_are_taken_into_account_for_the_most_nested_only,
-     IF(has_unix_permissions))
+     IF(non_root_on_unix_like_os))
 {
 	{
 		io_args_t args = {
@@ -197,15 +202,18 @@ TEST(permissions_are_taken_into_account_for_the_most_nested_only,
 	}
 }
 
+/* Permissions don't work on non-*nix-like systems and some checks fail for
+ * root user because system allows him to do almost anything and tests are
+ * counting on permissions denial. */
 static int
-has_unix_permissions(void)
+non_root_on_unix_like_os(void)
 {
 #if defined(_WIN32) || defined(__CYGWIN__)
 	return 0;
 #else
-	return 1;
+	return (geteuid() != 0);
 #endif
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
