@@ -9,7 +9,9 @@ void delete_test_file(const char name[]);
 
 int files_are_identical(const char a[], const char b[]);
 
+int not_windows(void);
+
 #endif /* VIFM_TESTS__UTILS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
