@@ -18,6 +18,7 @@
 #include "../../src/engine/variables.h"
 #include "../../src/int/path_env.h"
 #include "../../src/modes/cmdline.h"
+#include "../../src/ui/ui.h"
 #include "../../src/utils/env.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/path.h"
@@ -306,7 +307,7 @@ TEST(help_cmd_escaping)
 	ASSERT_COMPLETION(L"help vifm-", L"help vifm-!!");
 }
 
-TEST(root_is_completed)
+TEST(root_entries_are_completed)
 {
 	make_abs_path(curr_view->curr_dir, sizeof(curr_view->curr_dir),
 			TEST_DATA_PATH, "", saved_cwd);
@@ -554,9 +555,12 @@ TEST(autocmd_name_completion_is_case_insensitive)
 
 TEST(highlight_is_completed)
 {
-	ASSERT_COMPLETION(L"hi ", L"hi Border");
+	ASSERT_COMPLETION(L"hi ", L"hi AuxWin");
 	ASSERT_COMPLETION(L"hi wi", L"hi WildMenu");
 	ASSERT_COMPLETION(L"hi WildMenu cter", L"hi WildMenu cterm");
+
+	assert_success(exec_commands("hi {*.jpg} cterm=none", &lwin, CIT_COMMAND));
+	ASSERT_COMPLETION(L"hi clear ", L"hi clear {*.jpg}");
 }
 
 TEST(case_override_of_paths)
