@@ -12,18 +12,16 @@
 #include "../../src/engine/functions.h"
 #include "../../src/engine/keys.h"
 #include "../../src/modes/modes.h"
-#include "../../src/ui/statusbar.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/env.h"
 #include "../../src/utils/fs.h"
-#include "../../src/utils/matcher.h"
 #include "../../src/utils/path.h"
 #include "../../src/utils/str.h"
 #include "../../src/builtin_functions.h"
 #include "../../src/cmd_core.h"
 #include "../../src/filelist.h"
-#include "../../src/filtering.h"
+#include "../../src/flist_hist.h"
 #include "../../src/ops.h"
 #include "../../src/registers.h"
 #include "../../src/undo.h"
@@ -63,11 +61,6 @@ SETUP()
 	view_setup(&lwin);
 	view_setup(&rwin);
 
-	/* Emulate proper history initialization (must happen after view
-	 * initialization). */
-	cfg_resize_histories(5);
-	cfg_resize_histories(0);
-
 	curr_view = &lwin;
 	other_view = &rwin;
 
@@ -401,7 +394,7 @@ TEST(wincmd_can_switch_views)
 {
 	init_modes();
 	opt_handlers_setup();
-	assert_success(init_status(&cfg));
+	assert_success(stats_init(&cfg));
 
 	curr_view = &rwin;
 	other_view = &lwin;
@@ -436,7 +429,7 @@ TEST(wincmd_can_switch_views)
 	function_reset_all();
 
 	opt_handlers_teardown();
-	assert_success(reset_status(&cfg));
+	assert_success(stats_reset(&cfg));
 	vle_keys_reset();
 }
 
@@ -664,121 +657,59 @@ TEST(map_commands_count_arguments_correctly)
 	vle_keys_reset();
 }
 
-TEST(filter_prints_empty_filters_correctly)
+TEST(hist_next_and_prev)
 {
-	const char *expected = "Filter -- Flags -- Value\n"
-	                       "Local              \n"
-	                       "Name               \n"
-	                       "Auto               ";
+	/* Emulate proper history initialization (must happen after view
+	 * initialization). */
+	cfg_resize_histories(10);
+	cfg_resize_histories(0);
+	cfg_resize_histories(10);
 
-	status_bar_message("");
-	assert_failure(exec_commands("filter?", &lwin, CIT_COMMAND));
-	assert_string_equal(expected, get_last_message());
+	flist_hist_save(&lwin, sandbox, ".", 0);
+	flist_hist_save(&lwin, test_data, ".", 0);
+
+	assert_success(exec_commands("histprev", &lwin, CIT_COMMAND));
+	assert_true(paths_are_same(lwin.curr_dir, sandbox));
+	assert_success(exec_commands("histnext", &lwin, CIT_COMMAND));
+	assert_true(paths_are_same(lwin.curr_dir, test_data));
+
+	cfg_resize_histories(0);
 }
 
-TEST(filter_prints_non_empty_filters)
+TEST(normal_command_does_not_reset_selection)
 {
-	const char *expected = "Filter -- Flags -- Value\n"
-	                       "Local     I        local\n"
-#ifndef _WIN32
-	                       "Name      I        abc\n"
-#else
-	                       "Name      i        abc\n"
-#endif
-	                       "Auto               ";
+	init_modes();
+	opt_handlers_setup();
 
-	assert_success(exec_commands("filter abc", &lwin, CIT_COMMAND));
-	local_filter_apply(&lwin, "local");
+	lwin.list_rows = 2;
+	lwin.list_pos = 0;
+	lwin.dir_entry = dynarray_cextend(NULL,
+			lwin.list_rows*sizeof(*lwin.dir_entry));
+	lwin.dir_entry[0].name = strdup("a");
+	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
+	lwin.dir_entry[0].selected = 1;
+	lwin.dir_entry[1].name = strdup("b");
+	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
+	lwin.dir_entry[1].selected = 0;
+	lwin.selected_files = 1;
 
-	status_bar_message("");
-	assert_failure(exec_commands("filter?", &lwin, CIT_COMMAND));
-	assert_string_equal(expected, get_last_message());
-}
+	assert_success(exec_commands(":normal! t", &lwin, CIT_COMMAND));
+	assert_int_equal(0, lwin.selected_files);
+	assert_false(lwin.dir_entry[0].selected);
+	assert_false(lwin.dir_entry[1].selected);
 
-TEST(filter_with_empty_value_reuses_last_search)
-{
-#ifndef _WIN32
-#define I "I"
-#else
-#define I "i"
-#endif
+	assert_success(exec_commands(":normal! vG\r", &lwin, CIT_COMMAND));
+	assert_int_equal(2, lwin.selected_files);
+	assert_true(lwin.dir_entry[0].selected);
+	assert_true(lwin.dir_entry[1].selected);
 
-	const char *expected = "Filter -- Flags -- Value\n"
-	                       "Local              \n"
-	                       "Name      " I "        pattern\n"
-	                       "Auto               ";
+	assert_success(exec_commands(":normal! t", &lwin, CIT_COMMAND));
+	assert_int_equal(1, lwin.selected_files);
+	assert_true(lwin.dir_entry[0].selected);
+	assert_false(lwin.dir_entry[1].selected);
 
-	cfg_resize_histories(5);
-	cfg_save_search_history("pattern");
-
-	assert_success(exec_commands("filter //" I, &lwin, CIT_COMMAND));
-	status_bar_message("");
-	assert_failure(exec_commands("filter?", &lwin, CIT_COMMAND));
-	assert_string_equal(expected, get_last_message());
-
-	assert_success(exec_commands("filter ''", &lwin, CIT_COMMAND));
-	status_bar_message("");
-	assert_failure(exec_commands("filter?", &lwin, CIT_COMMAND));
-	assert_string_equal(expected, get_last_message());
-
-	assert_success(exec_commands("filter \"\"", &lwin, CIT_COMMAND));
-	status_bar_message("");
-	assert_failure(exec_commands("filter?", &lwin, CIT_COMMAND));
-	assert_string_equal(expected, get_last_message());
-
-#undef I
-}
-
-TEST(filter_without_args_resets_manual_filter)
-{
-	const char *expected = "Filter -- Flags -- Value\n"
-	                       "Local              \n"
-	                       "Name               \n"
-	                       "Auto               ";
-
-	assert_success(exec_commands("filter this", &lwin, CIT_COMMAND));
-	assert_success(exec_commands("filter", &lwin, CIT_COMMAND));
-
-	status_bar_message("");
-	assert_failure(exec_commands("filter?", &lwin, CIT_COMMAND));
-	assert_string_equal(expected, get_last_message());
-}
-
-TEST(filter_can_affect_both_views)
-{
-	assert_string_equal("", curr_view->manual_filter.raw);
-	assert_string_equal("", other_view->manual_filter.raw);
-	curr_view->invert = 1;
-	other_view->invert = 1;
-
-	curr_stats.global_local_settings = 1;
-	assert_success(exec_commands("filter /x/", &lwin, CIT_COMMAND));
-	curr_stats.global_local_settings = 0;
-
-	assert_string_equal("x", curr_view->manual_filter.raw);
-	assert_string_equal("x", other_view->manual_filter.raw);
-	assert_false(curr_view->invert);
-	assert_false(other_view->invert);
-}
-
-TEST(filter_can_setup_inverted_filter)
-{
-	assert_string_equal("", curr_view->manual_filter.raw);
-	curr_view->invert = 0;
-
-	assert_success(exec_commands("filter! /x/", &lwin, CIT_COMMAND));
-
-	assert_string_equal("x", curr_view->manual_filter.raw);
-	assert_true(curr_view->invert);
-}
-
-TEST(filter_can_invert_manual_filter)
-{
-	curr_view->invert = 0;
-	assert_success(exec_commands("filter!", &lwin, CIT_COMMAND));
-	assert_true(curr_view->invert);
-	assert_success(exec_commands("filter!", &lwin, CIT_COMMAND));
-	assert_false(curr_view->invert);
+	opt_handlers_teardown();
+	vle_keys_reset();
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
