@@ -14,7 +14,7 @@
 
 #include "utils.h"
 
-static void add_some_files_to_view(FileView *view);
+static void add_some_files_to_view(view_t *view);
 
 static char cwd[PATH_MAX];
 
@@ -228,21 +228,21 @@ TEST(select_and_unselect_use_last_pattern)
 
 	cfg_resize_histories(5);
 
-	cfg_save_search_history(".*\\.C");
+	hists_search_save(".*\\.C");
 	assert_success(exec_commands("select! //I", &lwin, CIT_COMMAND));
 	assert_int_equal(0, lwin.selected_files);
 	assert_false(lwin.dir_entry[0].selected);
 	assert_false(lwin.dir_entry[1].selected);
 	assert_false(lwin.dir_entry[2].selected);
 
-	cfg_save_search_history(".*\\.c$");
+	hists_search_save(".*\\.c$");
 	assert_success(exec_commands("select //", &lwin, CIT_COMMAND));
 	assert_int_equal(2, lwin.selected_files);
 	assert_true(lwin.dir_entry[0].selected);
 	assert_false(lwin.dir_entry[1].selected);
 	assert_true(lwin.dir_entry[2].selected);
 
-	cfg_save_search_history("a.c");
+	hists_search_save("a.c");
 	assert_success(exec_commands("unselect ////", &lwin, CIT_COMMAND));
 	assert_int_equal(1, lwin.selected_files);
 	assert_false(lwin.dir_entry[0].selected);
@@ -381,7 +381,7 @@ TEST(symlinks_are_not_resolved_in_cwd, IF(not_windows))
 }
 
 static void
-add_some_files_to_view(FileView *view)
+add_some_files_to_view(view_t *view)
 {
 	view->list_rows = 3;
 	view->list_pos = 0;
