@@ -101,8 +101,8 @@ TEST(sync_syncs_filelist)
 
 TEST(sync_removes_leafs_and_tree_data_on_converting_tree_to_cv)
 {
-	lwin.window_rows = 1;
-	rwin.window_rows = 1;
+	lwin.window_rows = 2;
+	rwin.window_rows = 2;
 
 	opt_handlers_setup();
 	assert_success(os_mkdir(SANDBOX_PATH "/dir", 0700));
@@ -146,7 +146,7 @@ TEST(sync_syncs_trees)
 	assert_success(exec_commands("sync! tree", curr_view, CIT_COMMAND));
 	assert_true(flist_custom_active(other_view));
 	curr_stats.load_stage = 2;
-	load_saving_pos(other_view, 1);
+	load_saving_pos(other_view);
 	curr_stats.load_stage = 0;
 
 	assert_int_equal(curr_view->list_rows, other_view->list_rows);
@@ -247,7 +247,7 @@ TEST(tree_syncing_applies_properties_of_destination_view)
 
 	assert_true(flist_custom_active(other_view));
 	curr_stats.load_stage = 2;
-	load_saving_pos(other_view, 1);
+	load_saving_pos(other_view);
 	curr_stats.load_stage = 0;
 
 	assert_int_equal(4, other_view->list_rows);
@@ -301,5 +301,60 @@ TEST(incorrect_parameter_causes_error)
 	assert_failure(exec_commands("sync! nosuchthing", curr_view, CIT_COMMAND));
 }
 
+TEST(sync_syncs_custom_trees)
+{
+	char test_data[PATH_MAX + 1];
+	char path[PATH_MAX + 1];
+
+	opt_handlers_setup();
+
+	columns_add_column_desc(SK_BY_NAME, &format_none);
+	columns_add_column_desc(SK_BY_SIZE, &format_none);
+	columns_set_line_print_func(&column_line_print);
+	other_view->columns = columns_create();
+
+	make_abs_path(test_data, sizeof(test_data), TEST_DATA_PATH, "", saved_cwd);
+
+	flist_custom_start(curr_view, "test");
+	snprintf(path, sizeof(path), "%s/%s", test_data, "compare");
+	flist_custom_add(curr_view, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "read");
+	flist_custom_add(curr_view, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "rename");
+	flist_custom_add(curr_view, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "tree");
+	flist_custom_add(curr_view, path);
+	assert_true(flist_custom_finish(curr_view, CV_REGULAR, 0) == 0);
+
+	assert_success(flist_load_tree(curr_view, test_data));
+
+	curr_view->dir_entry[0].selected = 1;
+	curr_view->selected_files = 1;
+	flist_custom_exclude(curr_view, 1);
+
+	/* As custom trees. */
+
+	assert_success(exec_commands("sync! tree", curr_view, CIT_COMMAND));
+	assert_true(flist_custom_active(other_view));
+	curr_stats.load_stage = 2;
+	load_saving_pos(other_view);
+	curr_stats.load_stage = 0;
+
+	assert_int_equal(CV_CUSTOM_TREE, other_view->custom.type);
+	assert_int_equal(curr_view->list_rows, other_view->list_rows);
+
+	/* As custom views. */
+
+	assert_success(exec_commands("sync! filelist", curr_view, CIT_COMMAND));
+	assert_true(flist_custom_active(other_view));
+	assert_int_equal(CV_VERY, other_view->custom.type);
+
+	columns_free(other_view->columns);
+	other_view->columns = NULL;
+	columns_set_line_print_func(NULL);
+
+	opt_handlers_teardown();
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
