@@ -596,7 +596,7 @@ TEST(filtering_fake_entry_does_nothing)
 
 	lwin.dir_entry[0].selected = 1;
 	lwin.selected_files = 1;
-	filter_selected_files(&lwin);
+	name_filters_add_selection(&lwin);
 
 	assert_int_equal(4, lwin.list_rows);
 	assert_int_equal(4, rwin.list_rows);
@@ -619,7 +619,7 @@ TEST(filtering_updates_two_bound_views)
 
 	rwin.dir_entry[0].selected = 1;
 	rwin.selected_files = 1;
-	filter_selected_files(&rwin);
+	name_filters_add_selection(&rwin);
 
 	basic_panes_check(4);
 	assert_string_equal("same-content-different-name-1", lwin.dir_entry[0].name);
@@ -631,14 +631,14 @@ TEST(filtering_updates_two_bound_views)
 	rwin.dir_entry[1].selected = 1;
 	rwin.dir_entry[2].selected = 1;
 	rwin.selected_files = 3;
-	filter_selected_files(&rwin);
+	name_filters_add_selection(&rwin);
 	basic_panes_check(3);
 
 	lwin.dir_entry[0].selected = 1;
 	lwin.dir_entry[1].selected = 1;
 	lwin.dir_entry[2].selected = 1;
 	lwin.selected_files = 3;
-	filter_selected_files(&lwin);
+	name_filters_add_selection(&lwin);
 	assert_int_equal(CV_REGULAR, lwin.custom.type);
 	assert_int_equal(CV_REGULAR, rwin.custom.type);
 }
