@@ -2,6 +2,8 @@
 
 #include "../../src/cfg/config.h"
 #include "../../src/ui/color_scheme.h"
+#include "../../src/ui/statusbar.h"
+#include "../../src/ui/ui.h"
 #include "../../src/utils/matchers.h"
 #include "../../src/cmd_core.h"
 #include "../../src/status.h"
@@ -110,5 +112,55 @@ TEST(negation)
 			matchers_get_expr(cfg.cs.file_hi[0].matchers));
 }
 
+TEST(highlighting_is_printed_back_correctly)
+{
+	const char *const COMMANDS = "highlight {*.jpg} ctermfg=red";
+	assert_success(exec_commands(COMMANDS, &lwin, CIT_COMMAND));
+
+	ui_sb_msg("");
+	assert_failure(exec_commands("highlight {*.jpg}", &lwin, CIT_COMMAND));
+	assert_string_equal("{*.jpg}    cterm=none ctermfg=red     ctermbg=default",
+			ui_sb_last());
+}
+
+TEST(existing_records_are_updated)
+{
+	const char *const COMMANDS1 = "highlight {*.jpg} ctermfg=red";
+	const char *const COMMANDS2 = "highlight {*.jpg} ctermfg=blue";
+
+	assert_success(exec_commands(COMMANDS1, &lwin, CIT_COMMAND));
+	assert_success(exec_commands(COMMANDS2, &lwin, CIT_COMMAND));
+	assert_int_equal(1, cfg.cs.file_hi_count);
+
+	assert_int_equal(COLOR_BLUE, cfg.cs.file_hi[0].hi.fg);
+}
+
+TEST(all_records_can_be_removed)
+{
+	const char *const COMMANDS1 = "highlight {*.jpg} ctermfg=red";
+	const char *const COMMANDS2 = "highlight {*.avi} cterm=bold";
+	const char *const COMMANDS3 = "highlight clear";
+
+	assert_success(exec_commands(COMMANDS1, &lwin, CIT_COMMAND));
+	assert_success(exec_commands(COMMANDS2, &lwin, CIT_COMMAND));
+	assert_int_equal(2, cfg.cs.file_hi_count);
+	assert_success(exec_commands(COMMANDS3, &lwin, CIT_COMMAND));
+	assert_int_equal(0, cfg.cs.file_hi_count);
+}
+
+TEST(records_can_be_removed)
+{
+	const char *const COMMANDS1 = "highlight {*.jpg} ctermfg=red";
+	const char *const COMMANDS2 = "highlight clear {*.avi}";
+	const char *const COMMANDS3 = "highlight clear {*.jpg}";
+
+	assert_success(exec_commands(COMMANDS1, &lwin, CIT_COMMAND));
+	assert_int_equal(1, cfg.cs.file_hi_count);
+	assert_failure(exec_commands(COMMANDS2, &lwin, CIT_COMMAND));
+	assert_int_equal(1, cfg.cs.file_hi_count);
+	assert_success(exec_commands(COMMANDS3, &lwin, CIT_COMMAND));
+	assert_int_equal(0, cfg.cs.file_hi_count);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
