@@ -1,7 +1,7 @@
 #include <stic.h>
 
 #include <stdlib.h> /* free() */
-#include <string.h> /* strdup() */
+#include <string.h> /* strcpy() strdup() */
 
 #include "../../src/cfg/config.h"
 #include "../../src/compat/fs_limits.h"
@@ -11,20 +11,24 @@
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/filter.h"
 #include "../../src/utils/fs.h"
+#include "../../src/utils/matcher.h"
 #include "../../src/utils/str.h"
 #include "../../src/cmd_core.h"
 #include "../../src/filelist.h"
 #include "../../src/filtering.h"
 #include "../../src/opt_handlers.h"
+#include "../../src/sort.h"
 #include "../../src/status.h"
 
 #include "utils.h"
 
-#define assert_hidden(view, name, dir) \
-	assert_false(filters_file_is_visible(&view, name, dir))
+#define assert_hidden(view, name, is_dir) \
+	assert_false( \
+			filters_file_is_visible(&view, flist_get_dir(&view), name, is_dir, 1))
 
-#define assert_visible(view, name, dir) \
-	assert_true(filters_file_is_visible(&view, name, dir))
+#define assert_visible(view, name, is_dir) \
+	assert_true( \
+			filters_file_is_visible(&view, flist_get_dir(&view), name, is_dir, 1))
 
 static char cwd[PATH_MAX + 1];
 
@@ -35,7 +39,8 @@ SETUP_ONCE()
 
 SETUP()
 {
-	cfg.slow_fs_list = strdup("");
+	update_string(&cfg.fuse_home, "no");
+	update_string(&cfg.slow_fs_list, "");
 
 	cfg.filter_inverted_by_default = 1;
 
@@ -43,6 +48,7 @@ SETUP()
 
 	lwin.list_rows = 7;
 	lwin.list_pos = 2;
+	strcpy(lwin.curr_dir, "/some/path");
 	lwin.dir_entry = dynarray_cextend(NULL,
 			lwin.list_rows*sizeof(*lwin.dir_entry));
 	lwin.dir_entry[0].name = strdup("with(round)");
@@ -117,6 +123,7 @@ SETUP()
 TEARDOWN()
 {
 	update_string(&cfg.slow_fs_list, NULL);
+	update_string(&cfg.fuse_home, NULL);
 
 	view_teardown(&lwin);
 	view_teardown(&rwin);
@@ -128,7 +135,7 @@ TEARDOWN()
 TEST(filtering)
 {
 	assert_int_equal(7, lwin.list_rows);
-	filter_selected_files(&lwin);
+	name_filters_add_selection(&lwin);
 	assert_int_equal(1, lwin.list_rows);
 
 	assert_string_equal("withnonodots", lwin.dir_entry[0].name);
@@ -143,7 +150,7 @@ TEST(filtering_file_does_not_filter_dir)
 	rwin.selected_files = 1;
 
 	assert_int_equal(8, rwin.list_rows);
-	filter_selected_files(&rwin);
+	name_filters_add_selection(&rwin);
 	assert_int_equal(7, rwin.list_rows);
 
 	assert_hidden(rwin, name, 0);
@@ -161,7 +168,7 @@ TEST(filtering_dir_does_not_filter_file)
 	rwin.selected_files = 1;
 
 	assert_int_equal(8, rwin.list_rows);
-	filter_selected_files(&rwin);
+	name_filters_add_selection(&rwin);
 	assert_int_equal(7, rwin.list_rows);
 
 	assert_hidden(rwin, name, 1);
@@ -172,7 +179,7 @@ TEST(filtering_dir_does_not_filter_file)
 
 TEST(filtering_files_does_not_filter_dirs)
 {
-	(void)filter_set(&rwin.manual_filter, "^.*\\.d$");
+	(void)replace_matcher(&rwin.manual_filter, "^.*\\.d$");
 
 	assert_visible(rwin, rwin.dir_entry[0].name, 1);
 	assert_visible(rwin, rwin.dir_entry[1].name, 1);
@@ -187,7 +194,7 @@ TEST(filtering_files_does_not_filter_dirs)
 
 TEST(filtering_dirs_does_not_filter_files)
 {
-	(void)filter_set(&rwin.manual_filter, "^.*\\.d/$");
+	(void)replace_matcher(&rwin.manual_filter, "^.*\\.d/$");
 
 	assert_hidden(rwin, rwin.dir_entry[0].name, 1);
 	assert_hidden(rwin, rwin.dir_entry[1].name, 1);
@@ -202,7 +209,7 @@ TEST(filtering_dirs_does_not_filter_files)
 
 TEST(filtering_files_and_dirs)
 {
-	(void)filter_set(&rwin.manual_filter, "^.*\\.d/?$");
+	(void)replace_matcher(&rwin.manual_filter, "^.*\\.d/?$");
 
 	assert_hidden(rwin, rwin.dir_entry[0].name, 1);
 	assert_hidden(rwin, rwin.dir_entry[1].name, 1);
@@ -231,7 +238,7 @@ TEST(file_after_directory_is_hidden)
 
 	lwin.dir_entry[1].selected = 1;
 	lwin.selected_files = 1;
-	filter_selected_files(&lwin);
+	name_filters_add_selection(&lwin);
 
 	assert_int_equal(1, lwin.list_rows);
 }
@@ -319,9 +326,9 @@ TEST(cursor_is_moved_to_nearest_neighbour)
 TEST(removed_filename_filter_is_stored)
 {
 	assert_success(filter_set(&lwin.auto_filter, "a"));
-	assert_success(filter_set(&lwin.manual_filter, "b"));
+	assert_success(replace_matcher(&lwin.manual_filter, "b"));
 
-	remove_filename_filter(&lwin);
+	name_filters_remove(&lwin);
 
 	assert_string_equal("a", lwin.prev_auto_filter);
 	assert_string_equal("b", lwin.prev_manual_filter);
@@ -330,10 +337,10 @@ TEST(removed_filename_filter_is_stored)
 TEST(filename_filter_can_removed_at_most_once)
 {
 	assert_success(filter_set(&lwin.auto_filter, "a"));
-	assert_success(filter_set(&lwin.manual_filter, "b"));
+	assert_success(replace_matcher(&lwin.manual_filter, "b"));
 
-	remove_filename_filter(&lwin);
-	remove_filename_filter(&lwin);
+	name_filters_remove(&lwin);
+	name_filters_remove(&lwin);
 
 	assert_string_equal("a", lwin.prev_auto_filter);
 	assert_string_equal("b", lwin.prev_manual_filter);
@@ -342,37 +349,158 @@ TEST(filename_filter_can_removed_at_most_once)
 TEST(filename_filter_can_be_cleared)
 {
 	assert_success(filter_set(&lwin.auto_filter, "a"));
-	assert_success(filter_set(&lwin.manual_filter, "b"));
+	assert_success(replace_matcher(&lwin.manual_filter, "b"));
 
-	filename_filter_clear(&lwin);
+	name_filters_drop(&lwin);
 
 	assert_true(filter_is_empty(&lwin.auto_filter));
-	assert_true(filter_is_empty(&lwin.manual_filter));
+	assert_true(matcher_is_empty(lwin.manual_filter));
 
-	assert_true(filename_filter_is_empty(&lwin));
+	assert_true(name_filters_empty(&lwin));
 }
 
 TEST(filename_filter_can_be_restored)
 {
 	assert_success(filter_set(&lwin.auto_filter, "a"));
-	assert_success(filter_set(&lwin.manual_filter, "b"));
+	assert_success(replace_matcher(&lwin.manual_filter, "b"));
 
-	remove_filename_filter(&lwin);
-	restore_filename_filter(&lwin);
+	name_filters_remove(&lwin);
+	name_filters_restore(&lwin);
 
 	assert_string_equal("a", lwin.auto_filter.raw);
-	assert_string_equal("b", lwin.manual_filter.raw);
+	assert_string_equal("b", matcher_get_expr(lwin.manual_filter));
 }
 
 TEST(filename_filter_is_not_restored_from_empty_state)
 {
 	assert_success(filter_set(&lwin.auto_filter, "a"));
-	assert_success(filter_set(&lwin.manual_filter, "b"));
+	assert_success(replace_matcher(&lwin.manual_filter, "b"));
 
-	restore_filename_filter(&lwin);
+	name_filters_restore(&lwin);
 
 	assert_string_equal("a", lwin.auto_filter.raw);
-	assert_string_equal("b", lwin.manual_filter.raw);
+	assert_string_equal("b", matcher_get_expr(lwin.manual_filter));
+}
+
+TEST(filename_filter_can_match_full_paths)
+{
+	assert_success(replace_matcher(&lwin.manual_filter, "///some/path/b$//"));
+	assert_visible(lwin, "a", 0);
+	assert_visible(lwin, "a", 0);
+	assert_hidden(lwin, "b", 0);
+	assert_visible(lwin, "b", 1);
+
+	assert_success(replace_matcher(&lwin.manual_filter, "{{/some/path/b}}"));
+	assert_visible(lwin, "a", 0);
+	assert_visible(lwin, "a", 0);
+	assert_hidden(lwin, "b", 0);
+	assert_visible(lwin, "b", 1);
+
+	assert_success(replace_matcher(&lwin.manual_filter, "///other/path/b//"));
+	assert_visible(lwin, "b", 0);
+	assert_visible(lwin, "b", 1);
+
+	assert_success(replace_matcher(&lwin.manual_filter, "{{/other/path/b}}"));
+	assert_visible(lwin, "b", 0);
+	assert_visible(lwin, "b", 1);
+
+	assert_success(replace_matcher(&lwin.manual_filter,
+				"//^/some/path/[^/]*\\.png$//"));
+	assert_hidden(lwin, "a.png", 0);
+	assert_visible(lwin, "a.png", 1);
+	strcat(lwin.curr_dir, "/nested");
+	assert_visible(lwin, "a.png", 0);
+	assert_visible(lwin, "a.png", 1);
+}
+
+TEST(custom_tree_can_restore_files_after_local_filter_interactive)
+{
+	char test_data[PATH_MAX + 1];
+	char path[PATH_MAX + 1];
+
+	make_abs_path(test_data, sizeof(test_data), TEST_DATA_PATH, "", cwd);
+
+	flist_custom_start(&lwin, "test");
+	snprintf(path, sizeof(path), "%s/%s", test_data, "compare");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "read");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "rename");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "tree");
+	flist_custom_add(&lwin, path);
+	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
+
+	assert_success(flist_load_tree(&lwin, test_data));
+	assert_int_equal(5, lwin.list_rows);
+
+	assert_int_equal(0, local_filter_set(&lwin, "t"));
+	local_filter_accept(&lwin);
+	assert_int_equal(2, lwin.list_rows);
+
+	assert_int_equal(0, local_filter_set(&lwin, ""));
+	local_filter_accept(&lwin);
+	assert_int_equal(5, lwin.list_rows);
+}
+
+TEST(custom_tree_can_restore_files_after_local_filter_non_interactive)
+{
+	char test_data[PATH_MAX + 1];
+	char path[PATH_MAX + 1];
+
+	make_abs_path(test_data, sizeof(test_data), TEST_DATA_PATH, "", cwd);
+
+	flist_custom_start(&lwin, "test");
+	snprintf(path, sizeof(path), "%s/%s", test_data, "compare");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "read");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "rename");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "tree");
+	flist_custom_add(&lwin, path);
+	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
+
+	assert_success(flist_load_tree(&lwin, test_data));
+	assert_int_equal(5, lwin.list_rows);
+
+	local_filter_apply(&lwin, "t");
+	load_dir_list(&lwin, 1);
+	assert_int_equal(2, lwin.list_rows);
+
+	local_filter_apply(&lwin, "");
+	load_dir_list(&lwin, 1);
+	assert_int_equal(5, lwin.list_rows);
+}
+
+TEST(sorting_tree_preserves_parent_dot_dir)
+{
+	char test_data[PATH_MAX + 1];
+	char path[PATH_MAX + 1];
+
+	cfg.dot_dirs = DD_NONROOT_PARENT;
+
+	make_abs_path(test_data, sizeof(test_data), TEST_DATA_PATH, "", cwd);
+
+	flist_custom_start(&lwin, "test");
+	snprintf(path, sizeof(path), "%s/%s", test_data, "compare");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "read");
+	flist_custom_add(&lwin, path);
+	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
+
+	assert_success(flist_load_tree(&lwin, test_data));
+	assert_int_equal(4, lwin.list_rows);
+
+	local_filter_apply(&lwin, "/");
+	load_dir_list(&lwin, 1);
+	assert_int_equal(4, lwin.list_rows);
+
+	assert_string_equal("..", lwin.dir_entry[0].name);
+	sort_view(&lwin);
+	assert_string_equal("..", lwin.dir_entry[0].name);
+
+	cfg.dot_dirs = 0;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
