@@ -18,6 +18,7 @@
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/filter.h"
 #include "../../src/utils/fs.h"
+#include "../../src/utils/matcher.h"
 #include "../../src/utils/path.h"
 #include "../../src/utils/str.h"
 #include "../../src/cmd_core.h"
@@ -33,7 +34,7 @@
 
 static void column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align, const char full_column[]);
-static void setup_custom_view(FileView *view, int very);
+static void setup_custom_view(view_t *view, int very);
 static int filenames_can_include_newline(void);
 
 static char cwd[PATH_MAX + 1];
@@ -125,28 +126,6 @@ TEST(custom_view_replaces_custom_view_fine)
 	assert_true(flist_custom_active(&lwin));
 }
 
-TEST(reload_considers_local_filter)
-{
-	filters_view_reset(&lwin);
-
-	assert_false(flist_custom_active(&lwin));
-
-	flist_custom_start(&lwin, "test");
-	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/a");
-	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/b");
-	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
-
-	local_filter_apply(&lwin, "b");
-
-	load_dir_list(&lwin, 1);
-
-	assert_int_equal(1, lwin.list_rows);
-	assert_string_equal("b", lwin.dir_entry[0].name);
-
-	filter_dispose(&lwin.manual_filter);
-	filter_dispose(&lwin.auto_filter);
-}
-
 TEST(reload_does_not_remove_broken_symlinks, IF(not_windows))
 {
 	char test_file[PATH_MAX];
@@ -202,26 +181,6 @@ TEST(symlinks_to_dirs_are_recognized_as_dirs, IF(not_windows))
 	assert_success(remove("dir-link"));
 }
 
-TEST(locally_filtered_files_are_not_lost_on_reload)
-{
-	filters_view_reset(&lwin);
-
-	assert_false(flist_custom_active(&lwin));
-
-	flist_custom_start(&lwin, "test");
-	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/a");
-	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/b");
-	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
-
-	local_filter_apply(&lwin, "b");
-
-	load_dir_list(&lwin, 1);
-	assert_int_equal(1, lwin.filtered);
-
-	filter_dispose(&lwin.manual_filter);
-	filter_dispose(&lwin.auto_filter);
-}
-
 TEST(register_macros_are_expanded_relatively_to_orig_dir)
 {
 	char *expanded;
@@ -445,6 +404,7 @@ TEST(custom_view_does_not_reset_local_state)
 		assert_int_equal(very ? SK_NONE : -SK_BY_TARGET, lwin.sort[0]);
 		assert_false(filter_is_empty(&lwin.local_filter.filter));
 
+		cdt.entry = &lwin.dir_entry[0];
 		cdt.line_hi_group = 1;
 		columns_format_line(lwin.columns, &cdt, MAX_WIDTH);
 
@@ -512,6 +472,34 @@ TEST(files_with_newline_in_names, IF(filenames_can_include_newline))
 	}
 }
 
+TEST(current_directory_can_be_added_via_dot)
+{
+	assert_success(chdir(SANDBOX_PATH));
+	assert_non_null(get_cwd(lwin.curr_dir, sizeof(lwin.curr_dir)));
+
+#ifndef _WIN32
+	replace_string(&cfg.shell, "/bin/sh");
+#else
+	replace_string(&cfg.shell, "cmd");
+#endif
+	stats_update_shell_type(cfg.shell);
+
+	assert_success(output_to_custom_flist(&lwin, "echo ../misc", 0, 0));
+
+	stats_update_shell_type("/bin/sh");
+	update_string(&cfg.shell, NULL);
+
+	assert_true(flist_custom_active(&lwin));
+	assert_int_equal(1, lwin.list_rows);
+	if(lwin.list_rows > 0)
+	{
+		char full_path[PATH_MAX + 1];
+		get_full_path_of(&lwin.dir_entry[0], sizeof(full_path), full_path);
+
+		assert_string_equal(flist_get_dir(&lwin), full_path);
+	}
+}
+
 TEST(can_set_very_cv_twice_in_a_row)
 {
 	fview_init();
@@ -604,29 +592,6 @@ TEST(cv_flist_title_is_preserved_on_next_cv_load_failure)
 	assert_string_equal("test", lwin.custom.title);
 }
 
-TEST(applying_local_filter_saves_custom_list)
-{
-	filters_view_reset(&lwin);
-
-	assert_false(flist_custom_active(&lwin));
-
-	flist_custom_start(&lwin, "test");
-	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/a");
-	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/b");
-	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
-
-	local_filter_apply(&lwin, "b");
-	load_dir_list(&lwin, 1);
-	assert_int_equal(1, lwin.list_rows);
-
-	local_filter_remove(&lwin);
-	load_dir_list(&lwin, 1);
-	assert_int_equal(2, lwin.list_rows);
-
-	filter_dispose(&lwin.manual_filter);
-	filter_dispose(&lwin.auto_filter);
-}
-
 TEST(excluded_entries_do_not_return)
 {
 	filters_view_reset(&lwin);
@@ -648,7 +613,8 @@ TEST(excluded_entries_do_not_return)
 	load_dir_list(&lwin, 1);
 	assert_int_equal(1, lwin.list_rows);
 
-	filter_dispose(&lwin.manual_filter);
+	matcher_free(lwin.manual_filter);
+	lwin.manual_filter = NULL;
 	filter_dispose(&lwin.auto_filter);
 }
 
@@ -675,11 +641,12 @@ TEST(excluding_entries_does_not_affect_local_filter_list)
 	load_dir_list(&lwin, 1);
 	assert_int_equal(2, lwin.list_rows);
 
-	filter_dispose(&lwin.manual_filter);
+	matcher_free(lwin.manual_filter);
+	lwin.manual_filter = NULL;
 	filter_dispose(&lwin.auto_filter);
 }
 
-TEST(failed_loadin_of_cv_does_not_override_saved_list)
+TEST(failed_loading_of_cv_does_not_override_saved_list)
 {
 	filters_view_reset(&lwin);
 
@@ -701,7 +668,8 @@ TEST(failed_loadin_of_cv_does_not_override_saved_list)
 	load_dir_list(&lwin, 1);
 	assert_int_equal(2, lwin.list_rows);
 
-	filter_dispose(&lwin.manual_filter);
+	matcher_free(lwin.manual_filter);
+	lwin.manual_filter = NULL;
 	filter_dispose(&lwin.auto_filter);
 }
 
@@ -757,7 +725,8 @@ TEST(cursor_is_positioned_close_to_disappeared_file)
 	/* Both 2 and 1 are valid answers, list_pos is automatically corrected. */
 	assert_true(lwin.list_pos == 1 || lwin.list_pos == 2);
 
-	filter_dispose(&lwin.manual_filter);
+	matcher_free(lwin.manual_filter);
+	lwin.manual_filter = NULL;
 	filter_dispose(&lwin.auto_filter);
 
 	restore_cwd(saved_cwd);
@@ -769,6 +738,19 @@ TEST(cursor_is_positioned_close_to_disappeared_file)
 	assert_success(unlink("5"));
 }
 
+TEST(last_directory_is_correctly_recorded_in_cv)
+{
+	char old_path[PATH_MAX + 1];
+
+	setup_custom_view(&lwin, 0);
+	strcpy(old_path, flist_get_dir(&lwin));
+
+	assert_success(change_directory(&lwin, test_data));
+	navigate_back(&lwin);
+
+	assert_true(paths_are_same(lwin.curr_dir, old_path));
+}
+
 static void
 column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align, const char full_column[])
@@ -778,7 +760,7 @@ column_line_print(const void *data, int column_id, const char buf[],
 }
 
 static void
-setup_custom_view(FileView *view, int very)
+setup_custom_view(view_t *view, int very)
 {
 	assert_false(flist_custom_active(view));
 	snprintf(view->curr_dir, sizeof(view->curr_dir), "%s/..", test_data);
