@@ -24,20 +24,22 @@
 
 #include "utils.h"
 
-static int load_tree(FileView *view, const char path[]);
+static int load_tree(view_t *view, const char path[]);
 static void verify_tree_node(column_data_t *cdt, int idx,
 		const char expected[]);
 static void column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align, const char full_column[]);
-static int remove_selected(FileView *view, const dir_entry_t *entry, void *arg);
-static void validate_tree(const FileView *view);
+static int remove_selected(view_t *view, const dir_entry_t *entry, void *arg);
+static void validate_tree(const view_t *view);
 static void validate_parents(const dir_entry_t *entries, int nchildren);
+static void load_view(view_t *view);
 
-static char cwd[PATH_MAX + 1];
+static char cwd[PATH_MAX + 1], test_data[PATH_MAX + 1];
 
 SETUP_ONCE()
 {
 	assert_non_null(get_cwd(cwd, sizeof(cwd)));
+	make_abs_path(test_data, sizeof(test_data), TEST_DATA_PATH, "", cwd);
 }
 
 SETUP()
@@ -134,7 +136,7 @@ TEST(tree_accounts_for_auto_filter)
 	assert_int_equal(12, lwin.list_rows);
 	lwin.dir_entry[11].selected = 1;
 	lwin.selected_files = 1;
-	filter_selected_files(&lwin);
+	name_filters_add_selection(&lwin);
 
 	assert_success(load_tree(&lwin, TEST_DATA_PATH "/tree"));
 	assert_int_equal(11, lwin.list_rows);
@@ -143,7 +145,7 @@ TEST(tree_accounts_for_auto_filter)
 
 TEST(tree_accounts_for_manual_filter)
 {
-	(void)filter_set(&lwin.manual_filter, "^\\.hidden$");
+	(void)replace_matcher(&lwin.manual_filter, "^\\.hidden$");
 	assert_success(load_tree(&lwin, TEST_DATA_PATH "/tree"));
 	assert_int_equal(11, lwin.list_rows);
 	validate_tree(&lwin);
@@ -165,9 +167,7 @@ TEST(leafs_are_suppressed_by_local_filtering)
 	validate_tree(&lwin);
 
 	/* ".." shouldn't appear after reload. */
-	curr_stats.load_stage = 2;
-	load_saving_pos(&lwin, 1);
-	curr_stats.load_stage = 0;
+	load_view(&lwin);
 	assert_int_equal(5, lwin.list_rows);
 	validate_tree(&lwin);
 }
@@ -187,9 +187,7 @@ TEST(leafs_are_not_matched_by_local_filtering)
 	validate_tree(&lwin);
 
 	/* ".." shouldn't appear after reload. */
-	curr_stats.load_stage = 2;
-	load_saving_pos(&lwin, 1);
-	curr_stats.load_stage = 0;
+	load_view(&lwin);
 	assert_int_equal(1, lwin.list_rows);
 	validate_tree(&lwin);
 
@@ -212,9 +210,7 @@ TEST(leafs_are_returned_if_local_filter_is_emptied)
 
 	/* Reload file list to make sure that its state is "clear" (leafs aren't
 	 * loaded in this case). */
-	curr_stats.load_stage = 2;
-	load_saving_pos(&lwin, 1);
-	curr_stats.load_stage = 0;
+	load_view(&lwin);
 
 	/* ".." should appear after filter is emptied. */
 	assert_int_equal(0, local_filter_set(&lwin, ""));
@@ -235,9 +231,7 @@ TEST(reloading_does_not_count_as_location_change)
 	validate_tree(&lwin);
 
 	(void)filter_set(&lwin.local_filter.filter, "dir");
-	curr_stats.load_stage = 2;
-	load_saving_pos(&lwin, 1);
-	curr_stats.load_stage = 0;
+	load_view(&lwin);
 	assert_int_equal(5, lwin.list_rows);
 	validate_tree(&lwin);
 
@@ -288,10 +282,10 @@ TEST(tree_is_reloaded_automatically_with_file_updates)
 	assert_success(load_tree(&lwin, SANDBOX_PATH));
 	assert_int_equal(2, lwin.list_rows);
 
-	check_if_filelist_have_changed(&lwin);
+	check_if_filelist_has_changed(&lwin);
 	ui_view_query_scheduled_event(&lwin);
 	assert_success(remove(SANDBOX_PATH "/b"));
-	check_if_filelist_have_changed(&lwin);
+	check_if_filelist_has_changed(&lwin);
 
 	curr_stats.load_stage = 2;
 	assert_true(process_scheduled_updates_of_view(&lwin));
@@ -328,10 +322,10 @@ TEST(nested_directory_change_detection)
 		assert_success(load_tree(&lwin, SANDBOX_PATH));
 		assert_int_equal(3, lwin.list_rows);
 
-		check_if_filelist_have_changed(&lwin);
+		check_if_filelist_has_changed(&lwin);
 		ui_view_query_scheduled_event(&lwin);
 		assert_success(remove(SANDBOX_PATH "/nested-dir/b"));
-		check_if_filelist_have_changed(&lwin);
+		check_if_filelist_has_changed(&lwin);
 
 		curr_stats.load_stage = 2;
 		assert_true(process_scheduled_updates_of_view(&lwin));
@@ -491,7 +485,7 @@ TEST(local_filter_does_not_block_visiting_directories)
 	validate_tree(&lwin);
 
 	/* Set manual filter to make sure that local filter doesn't dominate it. */
-	(void)filter_set(&lwin.manual_filter, "2");
+	(void)replace_matcher(&lwin.manual_filter, "2");
 
 	(void)filter_set(&lwin.local_filter.filter, "file");
 	load_dir_list(&lwin, 1);
@@ -714,14 +708,14 @@ TEST(dotdirs_do_not_mess_up_change_detection)
 	assert_success(load_tree(&lwin, TEST_DATA_PATH "/tree"));
 
 	/* Discard results of the first check. */
-	check_if_filelist_have_changed(&lwin);
+	check_if_filelist_has_changed(&lwin);
 	(void)ui_view_query_scheduled_event(&lwin);
 
-	check_if_filelist_have_changed(&lwin);
+	check_if_filelist_has_changed(&lwin);
 	assert_int_equal(UUE_NONE, ui_view_query_scheduled_event(&lwin));
-	check_if_filelist_have_changed(&lwin);
+	check_if_filelist_has_changed(&lwin);
 	assert_int_equal(UUE_NONE, ui_view_query_scheduled_event(&lwin));
-	check_if_filelist_have_changed(&lwin);
+	check_if_filelist_has_changed(&lwin);
 	assert_int_equal(UUE_NONE, ui_view_query_scheduled_event(&lwin));
 
 	cfg.dot_dirs = 0;
@@ -768,9 +762,7 @@ TEST(leafs_are_treated_correctly_on_reloading_saving_pos)
 
 	lwin.list_pos = 3;
 	create_file(SANDBOX_PATH "/dir/subdir/subsubdir/file");
-	curr_stats.load_stage = 2;
-	load_saving_pos(&lwin, 1);
-	curr_stats.load_stage = 0;
+	load_view(&lwin);
 	assert_int_equal(4, lwin.list_pos);
 
 	assert_success(remove(SANDBOX_PATH "/dir/file"));
@@ -792,17 +784,46 @@ TEST(cursor_is_set_on_previous_file)
 	assert_int_equal(8, lwin.list_pos);
 }
 
-static int
-load_tree(FileView *view, const char path[])
+TEST(tree_out_of_cv_with_single_element)
 {
-	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), path, "", cwd);
-	return flist_load_tree(&lwin, lwin.curr_dir);
+	char path[PATH_MAX];
+	snprintf(path, sizeof(path), "%s/%s", test_data, "existing-files/a");
+	flist_custom_start(&lwin, "test");
+	flist_custom_add(&lwin, path);
+	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
+
+	assert_success(load_tree(&lwin, TEST_DATA_PATH));
+	assert_int_equal(1, lwin.list_rows);
+}
+
+TEST(tree_out_of_cv_with_two_elements)
+{
+	char path[PATH_MAX];
+
+	flist_custom_start(&lwin, "test");
+	snprintf(path, sizeof(path), "%s/%s", test_data, "existing-files/a");
+	flist_custom_add(&lwin, path);
+	snprintf(path, sizeof(path), "%s/%s", test_data, "read/dos-eof");
+	flist_custom_add(&lwin, path);
+	assert_true(flist_custom_finish(&lwin, CV_REGULAR, 0) == 0);
+
+	assert_success(load_tree(&lwin, TEST_DATA_PATH));
+	assert_int_equal(5, lwin.list_rows);
+}
+
+static int
+load_tree(view_t *view, const char path[])
+{
+	char abs_path[PATH_MAX + 1];
+	make_abs_path(abs_path, sizeof(abs_path), path, "", cwd);
+	return flist_load_tree(view, abs_path);
 }
 
 static void
 verify_tree_node(column_data_t *cdt, int idx, const char expected[])
 {
 	char name[NAME_MAX];
+	cdt->entry = &cdt->view->dir_entry[idx];
 	cdt->line_pos = idx;
 	format_name(-1, cdt, sizeof(name), name);
 	assert_string_equal(expected, name);
@@ -816,13 +837,13 @@ column_line_print(const void *data, int column_id, const char buf[],
 }
 
 static int
-remove_selected(FileView *view, const dir_entry_t *entry, void *arg)
+remove_selected(view_t *view, const dir_entry_t *entry, void *arg)
 {
 	return !entry->selected;
 }
 
 static void
-validate_tree(const FileView *view)
+validate_tree(const view_t *view)
 {
 	int i;
 	for(i = 0; i < view->list_rows; ++i)
@@ -862,5 +883,13 @@ validate_parents(const dir_entry_t *entries, int nchildren)
 	}
 }
 
+static void
+load_view(view_t *view)
+{
+	curr_stats.load_stage = 2;
+	load_saving_pos(view);
+	curr_stats.load_stage = 0;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
