@@ -12,6 +12,7 @@
 #include "../../src/menus/menus.h"
 #include "../../src/modes/modes.h"
 #include "../../src/ui/statusbar.h"
+#include "../../src/ui/ui.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/string_array.h"
 
@@ -21,7 +22,7 @@ static menu_data_t m;
 
 SETUP()
 {
-	init_menu_data(&m, &lwin, strdup("test"), strdup("No matches"));
+	menus_init_data(&m, &lwin, strdup("test"), strdup("No matches"));
 
 	m.len = add_to_string_array(&m.items, m.len, 1, "a");
 	m.len = add_to_string_array(&m.items, m.len, 1, "b");
@@ -30,7 +31,7 @@ SETUP()
 
 TEARDOWN()
 {
-	reset_menu_data(&m);
+	menus_reset_data(&m);
 }
 
 TEST(can_navigate_to_broken_symlink, IF(not_windows))
@@ -52,48 +53,48 @@ TEST(can_navigate_to_broken_symlink, IF(not_windows))
 	/* Were trying to open broken link, which will fail, but the parsing part
 	 * should succeed. */
 	restore_cwd(saved_cwd);
-	assert_success(goto_selected_file(&m, &lwin, buf, 1));
+	assert_success(menus_goto_file(&m, &lwin, buf, 1));
 
 	assert_success(remove(SANDBOX_PATH "/broken-link"));
 }
 
 TEST(nothing_is_searched_if_no_pattern)
 {
-	menus_search(m.state, 0);
-	assert_int_equal(0, menu_get_matches(m.state));
+	menus_search_repeat(m.state, 0);
+	assert_int_equal(0, menus_search_matched(m.state));
 }
 
 TEST(nothing_is_searched_for_wrong_pattern)
 {
-	menu_new_search(m.state, 0, 1);
-	assert_true(search_menu_list("*a", &m, 1));
-	assert_int_equal(0, menu_get_matches(m.state));
+	menus_search_reset(m.state, 0, 1);
+	assert_true(menus_search("*a", &m, 1));
+	assert_int_equal(0, menus_search_matched(m.state));
 }
 
 TEST(search_via_menu_search)
 {
-	menu_new_search(m.state, 0, 1);
-	assert_true(search_menu_list("[abc]", &m, 1));
+	menus_search_reset(m.state, 0, 1);
+	assert_true(menus_search("[abc]", &m, 1));
 	assert_int_equal(1, m.pos);
-	menus_search(m.state, 0);
+	menus_search_repeat(m.state, 0);
 	assert_int_equal(2, m.pos);
 }
 
 TEST(ok_to_print_message_if_there_is_no_pattern)
 {
-	menu_print_search_msg(m.state);
+	menus_search_print_msg(m.state);
 }
 
 TEST(ok_to_print_message_for_wrong_pattern)
 {
-	assert_true(search_menu_list("*", &m, 1));
-	menu_print_search_msg(m.state);
+	assert_true(menus_search("*", &m, 1));
+	menus_search_print_msg(m.state);
 }
 
 TEST(forward_found_no_wrap)
 {
 	cfg.wrap_scan = 0;
-	assert_true(search_menu_list("c", &m, 1));
+	assert_true(menus_search("c", &m, 1));
 	assert_int_equal(2, m.pos);
 }
 
@@ -101,43 +102,43 @@ TEST(forward_found_wrap)
 {
 	m.pos = 1;
 	cfg.wrap_scan = 1;
-	assert_true(search_menu_list("a", &m, 1));
+	assert_true(menus_search("a", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
 TEST(forward_not_found_no_wrap)
 {
 	cfg.wrap_scan = 0;
-	assert_true(search_menu_list("d", &m, 1));
+	assert_true(menus_search("d", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
 TEST(forward_not_found_wrap)
 {
 	cfg.wrap_scan = 1;
-	assert_true(search_menu_list("d", &m, 1));
+	assert_true(menus_search("d", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
 TEST(forward_find_next_no_wrap)
 {
 	cfg.wrap_scan = 0;
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(1, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(2, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(2, m.pos);
 }
 
 TEST(forward_find_next_wrap)
 {
 	cfg.wrap_scan = 1;
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(1, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(2, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
@@ -145,32 +146,32 @@ TEST(backward_found_no_wrap)
 {
 	m.pos = 2;
 	cfg.wrap_scan = 0;
-	menu_new_search(m.state, 1, 1);
-	assert_true(search_menu_list("a", &m, 1));
+	menus_search_reset(m.state, 1, 1);
+	assert_true(menus_search("a", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
 TEST(backward_found_wrap)
 {
 	cfg.wrap_scan = 1;
-	menu_new_search(m.state, 1, 1);
-	assert_true(search_menu_list("c", &m, 1));
+	menus_search_reset(m.state, 1, 1);
+	assert_true(menus_search("c", &m, 1));
 	assert_int_equal(2, m.pos);
 }
 
 TEST(backward_not_found_no_wrap)
 {
 	cfg.wrap_scan = 0;
-	menu_new_search(m.state, 1, 1);
-	assert_true(search_menu_list("d", &m, 1));
+	menus_search_reset(m.state, 1, 1);
+	assert_true(menus_search("d", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
 TEST(backward_not_found_wrap)
 {
 	cfg.wrap_scan = 1;
-	menu_new_search(m.state, 1, 1);
-	assert_true(search_menu_list("d", &m, 1));
+	menus_search_reset(m.state, 1, 1);
+	assert_true(menus_search("d", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
@@ -178,12 +179,12 @@ TEST(backward_find_next_no_wrap)
 {
 	m.pos = 2;
 	cfg.wrap_scan = 0;
-	menu_new_search(m.state, 1, 1);
-	assert_true(search_menu_list(".", &m, 1));
+	menus_search_reset(m.state, 1, 1);
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(1, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(0, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(0, m.pos);
 }
 
@@ -191,21 +192,21 @@ TEST(backward_find_next_wrap)
 {
 	m.pos = 2;
 	cfg.wrap_scan = 1;
-	menu_new_search(m.state, 1, 1);
-	assert_true(search_menu_list(".", &m, 1));
+	menus_search_reset(m.state, 1, 1);
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(1, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(0, m.pos);
-	assert_true(search_menu_list(".", &m, 1));
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(2, m.pos);
 }
 
 TEST(null_pattern_causes_pattern_reuse)
 {
-	menu_new_search(m.state, 0, 1);
-	assert_true(search_menu_list(".", &m, 1));
+	menus_search_reset(m.state, 0, 1);
+	assert_true(menus_search(".", &m, 1));
 	assert_int_equal(1, m.pos);
-	assert_true(search_menu_list(NULL, &m, 1));
+	assert_true(menus_search(NULL, &m, 1));
 	assert_int_equal(2, m.pos);
 }
 
@@ -213,14 +214,14 @@ TEST(empty_mappings_menu_is_not_displayed)
 {
 	init_modes();
 
-	status_bar_message("");
+	ui_sb_msg("");
 	assert_failure(show_map_menu(&lwin, "normal", NORMAL_MODE, L"nonsense"));
-	assert_string_equal("No mappings found", get_last_message());
+	assert_string_equal("No mappings found", ui_sb_last());
 
 	vle_keys_user_add(L"this", L"that", NORMAL_MODE, 0);
-	status_bar_message("");
+	ui_sb_msg("");
 	assert_failure(show_map_menu(&lwin, "normal", NORMAL_MODE, L"nonsense"));
-	assert_string_equal("No mappings found", get_last_message());
+	assert_string_equal("No mappings found", ui_sb_last());
 
 	vle_keys_reset();
 }
