@@ -6,6 +6,7 @@
 #include "../../src/cfg/config.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/fs.h"
+#include "../../src/utils/matcher.h"
 #include "../../src/utils/str.h"
 #include "../../src/filelist.h"
 #include "../../src/filtering.h"
@@ -14,8 +15,8 @@
 
 #include "utils.h"
 
-static void init_view(FileView *view);
-static void free_view(FileView *view);
+static void init_view(view_t *view);
+static void free_view(view_t *view);
 
 SETUP()
 {
@@ -48,10 +49,12 @@ TEARDOWN()
 }
 
 static void
-init_view(FileView *view)
+init_view(view_t *view)
 {
+	char *error;
+
 	filter_init(&view->local_filter.filter, 1);
-	filter_init(&view->manual_filter, 1);
+	assert_non_null(view->manual_filter = matcher_alloc("", 0, 0, "", &error));
 	filter_init(&view->auto_filter, 1);
 
 	view->dir_entry = NULL;
@@ -60,13 +63,13 @@ init_view(FileView *view)
 	view->custom.entry_count = 0;
 	view->custom.entries = NULL;
 
-	view->window_rows = 1;
+	view->window_rows = 2;
 	view->sort[0] = SK_NONE;
 	ui_view_sort_list_ensure_well_formed(view, view->sort);
 }
 
 static void
-free_view(FileView *view)
+free_view(view_t *view)
 {
 	int i;
 
@@ -83,7 +86,8 @@ free_view(FileView *view)
 	dynarray_free(view->custom.entries);
 
 	filter_dispose(&view->local_filter.filter);
-	filter_dispose(&view->manual_filter);
+	matcher_free(view->manual_filter);
+	view->manual_filter = NULL;
 	filter_dispose(&view->auto_filter);
 }
 
