@@ -37,11 +37,9 @@ SETUP()
 	ui_view_sort_list_ensure_well_formed(&lwin, lwin.sort);
 	lwin.columns = columns_create();
 	lwin.view_columns = strdup("");
-	lwin.num_width_g = 4;
-	lwin.num_width = 4;
-	lwin.ls_view = 0;
-	lwin.hide_dot = 1;
-	lwin.hide_dot_g = 1;
+	lwin.num_width_g = lwin.num_width = 4;
+	lwin.ls_view_g = lwin.ls_view = 0;
+	lwin.hide_dot_g = lwin.hide_dot = 1;
 	update_string(&lwin.sort_groups, "");
 	update_string(&lwin.sort_groups_g, "");
 
@@ -52,11 +50,9 @@ SETUP()
 	ui_view_sort_list_ensure_well_formed(&rwin, rwin.sort);
 	rwin.columns = columns_create();
 	rwin.view_columns = strdup("");
-	rwin.num_width_g = 4;
-	rwin.num_width = 4;
-	rwin.ls_view = 0;
-	rwin.hide_dot_g = 1;
-	rwin.hide_dot = 1;
+	rwin.num_width_g = rwin.num_width = 4;
+	rwin.ls_view_g = rwin.ls_view = 0;
+	rwin.hide_dot_g = rwin.hide_dot = 1;
 
 	/* Name+size matches default column view setting ("-{name},{}"). */
 	columns_add_column_desc(SK_BY_NAME, &format_none);
@@ -316,10 +312,10 @@ TEST(suggestoptions_all_values)
 	cfg.sug.delay = 0;
 
 	assert_success(exec_commands("set suggestoptions=normal,visual,view,otherpane"
-				",delay,keys,marks,registers", &lwin, CIT_COMMAND));
+				",delay,keys,marks,registers,foldsubkeys", &lwin, CIT_COMMAND));
 
 	assert_int_equal(SF_NORMAL | SF_VISUAL | SF_VIEW | SF_OTHERPANE | SF_DELAY |
-			SF_KEYS | SF_MARKS | SF_REGISTERS, cfg.sug.flags);
+			SF_KEYS | SF_MARKS | SF_REGISTERS | SF_FOLDSUBKEYS, cfg.sug.flags);
 	assert_int_equal(5, cfg.sug.maxregfiles);
 	assert_int_equal(500, cfg.sug.delay);
 }
@@ -356,7 +352,7 @@ TEST(suggestoptions_empty_value)
 TEST(suggestoptions_registers_number)
 {
 	cfg.sug.flags = SF_NORMAL | SF_VISUAL | SF_VIEW | SF_OTHERPANE | SF_DELAY |
-					SF_KEYS | SF_MARKS | SF_REGISTERS;
+					SF_KEYS | SF_MARKS | SF_REGISTERS | SF_FOLDSUBKEYS;
 	cfg.sug.maxregfiles = 4;
 
 	assert_failure(exec_commands("set suggestoptions=registers:-4", &lwin,
@@ -623,5 +619,115 @@ TEST(values_in_sizefmt_are_deduplicated)
 			vle_tb_get_data(vle_err));
 }
 
+TEST(millerview)
+{
+	assert_success(exec_commands("se millerview", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("se invmillerview", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("setl millerview", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("setl invmillerview", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("setg millerview", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("setg invmillerview", &lwin, CIT_COMMAND));
+}
+
+TEST(milleroptions_handles_wrong_input)
+{
+	assert_failure(exec_commands("se milleroptions=msi:1", &lwin, CIT_COMMAND));
+
+	assert_failure(exec_commands("se milleroptions=lsize:a", &lwin, CIT_COMMAND));
+	assert_failure(exec_commands("se milleroptions=csize:a", &lwin, CIT_COMMAND));
+	assert_failure(exec_commands("se milleroptions=rsize:a", &lwin, CIT_COMMAND));
+
+	assert_failure(exec_commands("se milleroptions=csize:0", &lwin, CIT_COMMAND));
+}
+
+TEST(milleroptions_accepts_correct_input)
+{
+	assert_success(exec_commands("set milleroptions=csize:33,rsize:12", &lwin,
+				CIT_COMMAND));
+
+	vle_tb_clear(vle_err);
+	assert_success(set_options("milleroptions?", OPT_GLOBAL));
+	assert_string_equal("  milleroptions=lsize:0,csize:33,rsize:12",
+			vle_tb_get_data(vle_err));
+}
+
+TEST(milleroptions_normalizes_input)
+{
+	assert_success(exec_commands("set milleroptions=lsize:-10,csize:133", &lwin,
+				CIT_COMMAND));
+
+	vle_tb_clear(vle_err);
+	assert_success(set_options("milleroptions?", OPT_GLOBAL));
+	assert_string_equal("  milleroptions=lsize:0,csize:100,rsize:0",
+			vle_tb_get_data(vle_err));
+}
+
+TEST(lsoptions_empty_input)
+{
+	assert_success(exec_commands("set lsoptions=", &lwin, CIT_COMMAND));
+
+	vle_tb_clear(vle_err);
+	assert_success(set_options("lsoptions?", OPT_GLOBAL));
+	assert_string_equal("  lsoptions=", vle_tb_get_data(vle_err));
+}
+
+TEST(lsoptions_handles_wrong_input)
+{
+	assert_failure(exec_commands("se lsoptions=transposed:yes", &lwin,
+				CIT_COMMAND));
+	assert_failure(exec_commands("se lsoptions=transpose", &lwin,
+				CIT_COMMAND));
+}
+
+TEST(lsoptions_accepts_correct_input)
+{
+	assert_success(exec_commands("set lsview lsoptions=transposed", &lwin,
+				CIT_COMMAND));
+
+	vle_tb_clear(vle_err);
+	assert_success(set_options("lsoptions?", OPT_GLOBAL));
+	assert_string_equal("  lsoptions=transposed", vle_tb_get_data(vle_err));
+}
+
+TEST(lsoptions_normalizes_input)
+{
+	assert_success(exec_commands("set lsoptions=transposed,transposed", &lwin,
+				CIT_COMMAND));
+
+	vle_tb_clear(vle_err);
+	assert_success(set_options("lsoptions?", OPT_GLOBAL));
+	assert_string_equal("  lsoptions=transposed", vle_tb_get_data(vle_err));
+}
+
+TEST(previewprg_updates_state_of_view)
+{
+	assert_success(exec_commands("setg previewprg=gcmd", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("setl previewprg=lcmd", &lwin, CIT_COMMAND));
+
+	vle_tb_clear(vle_err);
+	assert_success(set_options("previewprg?", OPT_GLOBAL));
+	assert_success(set_options("previewprg?", OPT_LOCAL));
+	assert_string_equal("  previewprg=gcmd\n  previewprg=lcmd",
+			vle_tb_get_data(vle_err));
+}
+
+TEST(tuioptions)
+{
+	assert_success(set_options("tuioptions=", OPT_GLOBAL));
+	assert_false(cfg.extra_padding);
+	assert_false(cfg.side_borders_visible);
+	assert_false(cfg.use_unicode_characters);
+
+	assert_success(set_options("tuioptions=pu", OPT_GLOBAL));
+	assert_true(cfg.extra_padding);
+	assert_false(cfg.side_borders_visible);
+	assert_true(cfg.use_unicode_characters);
+
+	assert_success(set_options("tuioptions+=s", OPT_GLOBAL));
+	assert_true(cfg.extra_padding);
+	assert_true(cfg.side_borders_visible);
+	assert_true(cfg.use_unicode_characters);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
