@@ -13,7 +13,6 @@
 #include "../../src/utils/matchers.h"
 #include "../../src/utils/path.h"
 #include "../../src/utils/str.h"
-#include "../../src/background.h"
 #include "../../src/filelist.h"
 #include "../../src/filetype.h"
 #include "../../src/running.h"
@@ -21,11 +20,6 @@
 
 static int prog_exists(const char name[]);
 
-SETUP_ONCE()
-{
-	bg_init();
-}
-
 SETUP()
 {
 #ifndef _WIN32
