@@ -2,7 +2,7 @@
 
 #include "../../src/cfg/config.h"
 #include "../../src/ui/ui.h"
-#include "../../src/commands.h"
+#include "../../src/cmd_core.h"
 
 SETUP()
 {
@@ -21,4 +21,4 @@ TEST(wrong_user_defined_command_name_causes_error)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
