@@ -1,8 +1,6 @@
 #ifndef VIFM_TESTS__UTILS_H__
 #define VIFM_TESTS__UTILS_H__
 
-#include "../../src/ui/ui.h"
-
 /* Executable suffixes. */
 #if defined(__CYGWIN__) || defined(_WIN32)
 #define EXE_SUFFIX ".exe"
@@ -12,6 +10,8 @@
 #define EXE_SUFFIXW L""
 #endif
 
+struct view_t;
+
 /* Prepares option handler for use in tests. */
 void opt_handlers_setup(void);
 
@@ -25,10 +25,10 @@ void undo_setup(void);
 void undo_teardown(void);
 
 /* Initializes view with safe defaults. */
-void view_setup(FileView *view);
+void view_setup(struct view_t *view);
 
 /* Frees resources of the view. */
-void view_teardown(FileView *view);
+void view_teardown(struct view_t *view);
 
 /* Creates file at the path. */
 void create_file(const char path[]);
@@ -47,6 +47,24 @@ void copy_file(const char src[], const char dst[]);
  * returned. */
 int not_windows(void);
 
+struct matcher_t;
+
+/* Changes *matcher to have the value of the expr.  The operation is assumed to
+ * succeed, but it's not guaranteed. */
+int replace_matcher(struct matcher_t **matcher, const char expr[]);
+
+struct view_t;
+
+/* Setups a grid of specified dimentions for the view. */
+void setup_grid(struct view_t *view, int column_count, int list_rows, int init);
+
+/* Setups transposed grid of specified dimentions for the view. */
+void setup_transposed_grid(struct view_t *view, int column_count, int list_rows,
+		int init);
+
+/* Waits termination of all background tasks. */
+void wait_for_bg(void);
+
 #endif /* VIFM_TESTS__UTILS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
