@@ -355,6 +355,22 @@ TEST(after_fake_equal_sign_completion_fail)
 	free(completed);
 }
 
+TEST(after_equal_sign_and_comma_completion_fail)
+{
+	const char *start;
+	char *completed;
+
+	optval_t val = { .str_val = "/home/tmp" };
+	set_option("cdpath", val, OPT_GLOBAL);
+
+	vle_compl_reset();
+	complete_options("cdpath=a,", &start, OPT_GLOBAL);
+
+	completed = vle_compl_next();
+	assert_string_equal("a,", completed);
+	free(completed);
+}
+
 TEST(all_completion_ok)
 {
 	const char *start;
@@ -368,6 +384,25 @@ TEST(all_completion_ok)
 	free(completed);
 }
 
+TEST(completion_of_second_option)
+{
+	const char *str = "all no";
+	const char *start;
+	char *completed;
+
+	vle_compl_reset();
+	complete_options(str, &start, OPT_GLOBAL);
+	assert_string_equal(str + 6, start);
+
+	completed = vle_compl_next();
+	assert_string_equal("fastrun", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("sortorder", completed);
+	free(completed);
+}
+
 TEST(charset_completion_skips_entered_elements)
 {
 	const char *start;
