@@ -12,13 +12,18 @@ static var_t dummy(const call_info_t *call_info);
 
 SETUP_ONCE()
 {
-	static const function_t function_a = { "a", 1, &dummy };
-	static const function_t function_b = { "b", 2, &dummy };
-	static const function_t function_c = { "c", 0, &dummy };
+	static const function_t function_a = { "a", "adescr", 1, &dummy };
+	static const function_t function_b = { "b", "bdescr", 2, &dummy };
+	static const function_t function_c = { "c", "cdescr", 0, &dummy };
 
-	assert_int_equal(0, function_register(&function_a));
-	assert_int_equal(0, function_register(&function_b));
-	assert_int_equal(0, function_register(&function_c));
+	assert_success(function_register(&function_a));
+	assert_success(function_register(&function_b));
+	assert_success(function_register(&function_c));
+}
+
+TEARDOWN_ONCE()
+{
+	function_reset_all();
 }
 
 static var_t
@@ -70,6 +75,16 @@ TEST(concatenation_as_arg_ok)
 	ASSERT_OK("a('a'.'b')", "");
 }
 
+TEST(or_as_arg_ok)
+{
+	ASSERT_OK("a('a'||'b')", "");
+}
+
+TEST(and_as_arg_ok)
+{
+	ASSERT_OK("a('a'&&'b')", "");
+}
+
 TEST(no_args_ok)
 {
 	ASSERT_OK("c()", "");
@@ -85,5 +100,16 @@ TEST(chars_after_function_call_fail)
 	ASSERT_FAIL("a()a", PE_INVALID_EXPRESSION);
 }
 
+TEST(very_long_function_name)
+{
+	const char *const input = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
+		"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
+		"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
+		"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaafffffffffffffffffffffffff"
+		"fffffffff";
+
+	ASSERT_FAIL(input, PE_INVALID_EXPRESSION);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
