@@ -14,13 +14,19 @@ vifm_try_leave(int write_info, int cquit, int force)
 }
 
 void
-vifm_choose_files(const FileView *view, int nfiles, char *files[])
+vifm_choose_files(const view_t *view, int nfiles, char *files[])
 {
 	/* Do nothing. */
 }
 
 void
-vifm_finish(const FileView *view, int nfiles, char *files[])
+vifm_finish(const view_t *view, int nfiles, char *files[])
+{
+	/* Do nothing. */
+}
+
+void
+vifm_exit(int exit_code)
 {
 	/* Do nothing. */
 }
