@@ -81,5 +81,26 @@ TEST(do_not_complete_removed_variables)
 	free(completed);
 }
 
+TEST(builtinvars_completion)
+{
+	char buf[] = "v:";
+	const char *start;
+	char *completed;
+
+	assert_success(setvar("v:test1", var_from_bool(1)));
+	assert_success(setvar("v:test2", var_from_bool(1)));
+
+	complete_variables(buf, &start);
+	assert_string_equal(&buf[0], start);
+
+	completed = vle_compl_next();
+	assert_string_equal("v:test1", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("v:test2", completed);
+	free(completed);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
